/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package de.desy.mst.common.util.autoclient;

import de.desy.mst.libs.util.helper.LogProvider;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashSet;
import java.util.List;
import java.util.Set;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 *
 * @author herb
 * @author <a href="marcus(dot)walla(at)desy(dot)de">walla</a>
 */
public class ServerList {
	private final static Logger log = LogProvider.getLogger(ServerList.class);
	
	private final String contextName; 
	private final Set<ServerData> lstServers;

	public ServerList(String context) {
		contextName = context;
		lstServers = new HashSet<ServerData>();
	}

	@Override
	public String toString() { return contextName; }

	public boolean addServer(final String serverName, final String hostName, EServerType serverType, final String binaryName) {
		return this.lstServers.add(new ServerData(serverName,hostName, serverType, binaryName, binaryName));
	}

//	public boolean addServer(final String serverName, final String hostName, EServerType serverType, final String binaryName, final String transportCDIAddressScript) {
//		return this.lstServers.add(new ServerData(serverName,hostName, serverType, binaryName, transportCDIAddressScript));
//	}
	
	public String[] getServerList() {
		if (this.lstServers.isEmpty()) return new String[] { "none" };

		final List<String> servers = new ArrayList<String>();
		for (final ServerData data: this.lstServers)
			servers.add(data.getServerName());
		Collections.sort(servers);
		return servers.toArray(new String[0]);
	}
	public String getHostName(String serverName) {
		try { return searchData(serverName).getHostName(); }
		catch(final UnknownElementException exception) {
			log.log(Level.WARNING, "unknown element", exception);
			return "none";
		}
	}
	public String getBinName(String serverName) {
		try { return searchData(serverName).getBinaryName(); }
		catch(final UnknownElementException exception) {
			log.log(Level.WARNING, "unknown element", exception);
			return "none";
		}
	}
	public EServerType getServerType(final String serverName) {
		try { return searchData(serverName).getServerType(); }
		catch(final UnknownElementException exception) {
			log.log(Level.WARNING, "unknown element", exception);
			return EServerType.NONE;
		}
	}
	
	public ServerData getServerData(final String serverName) {
		try { return searchData(serverName); }
		catch(final UnknownElementException exception) {
			log.log(Level.WARNING, "unknown element", exception);
			return null;
		}
	}

	private ServerData searchData(final String serverName) throws UnknownElementException {
		 for (final ServerData data: this.lstServers)
			 if (data.getServerName().equals(serverName)) return data;
		 throw new UnknownElementException(serverName);
	}
}

