/**
 * File: EServiceElement.java
 * Date: Mar 30, 2010
 */
package de.desy.mst.libs.accelerator;

import de.desy.mst.libs.accelerator.exception.EnumerationNotUsedException;
import de.desy.mst.libs.accelerator.exception.NotImplementedException;
import de.desy.mst.libs.framework.mstapp.logging.Logger;

/**
 * <p>Service elements. Currently this includes the following enumeration 
 * elements:</p>
 * 
 * <table cellspacing="2" cellpadding="4">
 * <tr><th>Enumeration</th><th>Description</th></tr>
 * <tr bgcolor="#A0A0A0"><td>{@link #PERFORMANCE_DATA}</td><td>Contains the value of the modulator data.</td></tr>
 * <tr bgcolor="#E0E0E0"><td>{@link #MINIMUM}</td><td>Contains the minimum value of the modulator data.</td></tr>
 * <tr bgcolor="#A0A0A0"><td>{@link #MAXIMUM}</td><td>Contains the maximum value of the modulator data.</td></tr>
 * <tr bgcolor="#E0E0E0"><td>{@link #UNIT}</td><td>Contains the EGU unit value of the modulator data.</td></tr>
 * </table>
 *
 * @author <a href="mailto:marcus.walla(at)desy.de">walla</a>
 * @version $Id: EServiceElement.java 34784 2010-04-01 16:23:29Z walla $
 */
public enum EServiceElement implements IEEnum
{
	PERFORMANCE_DATA("pd", "performance data"),
	MINIMUM("min", "minimum"),
	MAXIMUM("max", "maximum"),
	UNIT("unit", "unit");
	
	// --- CLASS FIELDS --------------------------------------------------------
	
	/** Class name. */
	public final static String CLAZZ = EServiceElement.class.getSimpleName();
	
	// --- INSTANCE FIELDS -----------------------------------------------------
	
	private final String _prefixProperty;
	private final String _element;
	
	// --- CONSTRUCTOR ---------------------------------------------------------
	
	/**
	 * Constructor.
	 * 
	 * @param prefixProperty_ Prefix of the TINE property name.
	 * @param element_ Element description.
	 */
	EServiceElement(final String prefixProperty_, final String element_)
	{
		validate(prefixProperty_, element_);
		
		_prefixProperty = prefixProperty_;
		_element = element_;
	}

	// --- INSTANCE METHODS ----------------------------------------------------
	
	/**
	 * Validating.
	 * 
	 * @param prefixProperty_ Prefix of the TINE property name.
	 * @param element_ Element.
	 */
	private void validate(final String prefixProperty_, final String element_)
	{
		if (prefixProperty_ == null)
			Logger.SWApp.error(EServiceElement.CLAZZ + " - prefix property of " + this + " is null.");
		if (prefixProperty_.isEmpty())
			Logger.SWApp.error(EServiceElement.CLAZZ + " - prefix property of " + this + " is empty.");

	
		if (element_ == null)
			Logger.SWApp.error(EServiceElement.CLAZZ + " - element of " + this + " is null.");
		if (element_.isEmpty())
			Logger.SWApp.error(EServiceElement.CLAZZ + " - element of " + this + " is empty.");
	}
	
	// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
	
	/**
	 * Returns the TINE prefix property name.
	 * 
	 * @return TINE prefix property name.
	 */
	public String getPrefixProperty()
	{
		return _prefixProperty;
	}
	
	/**
	 * Returns the element.
	 * 
	 * @return Element.
	 */
	public String getElement()
	{
		return _element;
	}

	@Override
	public String getName(final EName description_) 
			throws EnumerationNotUsedException
	{
		throw new NotImplementedException();
	}

	@Override
	public void putName(EName description, String name)
			throws EnumerationNotUsedException
	{
		throw new NotImplementedException();
	}
}
/* end of file */