/*
 *
 * File: IEDataAddress.java
 * Date: Jan 28, 2008; 1:42:48 PM
 */
package de.desy.mst.libs.accelerator;

/**
 * This interface allows to get from a data value its crate, address and
 * sub-address.
 *
 * @author <a href="mailto:marcus.walla(at)desy.de">walla</a>
 * @version $Id: IEDataAddress.java 12066 2008-05-29 08:35:57Z walla $
 */
public interface IEDataAddress
{
	
	/**
	 * <p>Returns the SEDAC crate address to the caller. If no number was set 
	 * before a {@code -1} is returned.</p>
	 * 
	 * @return The SEDAC crate address is returned to the caller.
	 */
	public abstract int crate();
	
	/**
	 * <p>Returns the SEDAC address to the caller. If no number was set before a 
	 * {@code -1} is returned.</p>
	 * 
	 * @return The SEDAC address is returned to the caller.
	 */
	public abstract int address();
	
	/**
	 * <p>Returns the SEDAC sub-address to the caller. This is calculated from
	 * the address argument by the following algorithm:</p>
	 *
	 * <pre>
	 * 
	 * sub-address = address - address % 16
	 * 
	 * </pre>
	 * 
	 * <p>If no number was set before a {@code -1} is returned.</p>
	 * 
	 * @return Returns the sub-address of the corresponding address.
	 */
	public abstract int subaddress();
}
/* end of file */