/*
 *
 * File: IEDataSetValues.java
 * Date: Jan 28, 2008; 1:10:21 PM
 */
package de.desy.mst.libs.accelerator;

/**
 * It is also possible to set values for each TINE property. In this interface
 * are all put methods included to set a value of it.
 *
 * @author <a href="mailto:marcus.walla(at)desy.de">walla</a>
 * @version $Id: IEDataSetValues.java 12066 2008-05-29 08:35:57Z walla $
 */
public interface IEDataSetValues
{
	/** 
	 * <p>Puts the minimum range value to an enumeration element.</p>
	 * 
	 * <p><b><u>ATTENTION:</u></b> This method should only be used by the abstract
	 * class of {@code de.desy.mst.libs.accelerator.model.ACDIDevices} in this 
	 * library!</p>
	 * 
	 * @param minimum_ The minimum value.
	 */
	public abstract void putMinimum(final float minimum_);

	/** 
	 * <p>Puts the maximum range value to an enumeration element.</p>
	 * 
	 * <p><b><u>ATTENTION:</u></b> This method should only be used by the abstract
	 * class of {@code de.desy.mst.libs.accelerator.model.ACDIDevices} in this 
	 * library!</p>
	 * 
	 * @param maximum_ The maximum value.
	 */
	public abstract void putMaximum(final float maximum_);
	
	/**
	 * Sets the send conversion factor value as a {@code float}. This is
	 * the simplified value for transforming the value from the client to the 
	 * server.
	 * 
	 * <p><b><u>ATTENTION:</u></b> This method should only be used by the abstract
	 * class of {@code de.desy.mst.libs.accelerator.model.ACDIDevices} in this 
	 * library!</p>
	 * 
	 * @param sendFactor_ The send conversion factor is set as a {@code float} 
	 * 		value.
	 */
	public abstract void putSendFactor(final float sendFactor_);

	/**
	 * Sets the receive conversion factor value as a {@code float}. This is
	 * the simplified value for transforming the value from the server to the 
	 * client.
	 * 
	 * <p><b><u>ATTENTION:</u></b> This method should only be used by the abstract
	 * class of {@code de.desy.mst.libs.accelerator.model.ACDIDevices} in this 
	 * library!</p>
	 * 
	 * @param recvFactor_ The receive conversion factor is set as a 
	 * 		{@code float} value.
	 */
	public abstract void putRecvFactor(final float recvFactor_);
	
	/**
	 * <p>Puts the SEDAC crate address to this enumeration element.</p>
	 * 
	 * <p><b><u>ATTENTION:</u></b> This method should only be used by the abstract
	 * class of {@code de.desy.mst.libs.accelerator.model.ACDIDevices} in this 
	 * library!</p>
	 * 
	 * @param crate_ Is the SEDAC crate address.
	 */
	public abstract void putCrate(final int crate_);
	
	/**
	 * <p>Puts the SEDAC address to this enumeration element.</p>
	 * 
	 * <p><b><u>ATTENTION:</u></b> This method should only be used by the abstract
	 * class of {@code de.desy.mst.libs.accelerator.model.ACDIDevices} in this 
	 * library!</p>
	 * 
	 * @param address_ Is the SEDAC address.
	 */
	public abstract void putAddress(final int address_);
	
	/**
	 * Will set the adjusting value for this data component. This is related to
	 * the {@link IEData#isAdjusting()} method. For more information read there.
	 * 
	 * @param value_ Sets the logical value in this data component.
	 */
	public abstract void putAdjusting(final boolean value_);
	
	/**
	 * <p>This method will set the fire event value according to the content of
	 * the parameter. This method decides if this enumeration element will fire
	 * an event to the hardware ({@code true}) or not ({@code false}). The 
	 * getter method is {@link IEData#isFireEvent()}.</p>
	 * 
	 * @param value_ Sets the fire event value.
	 */
	public abstract void putFireEvent(final boolean value_);
}
/* end of file */