/*
 *
 * File: IObserver.java
 * Date: Sep 27, 2006; 5:02:57 PM
 */
package de.desy.mst.libs.accelerator;

/**
 * The observer is the current data model in the observer design pattern. It is
 * called every time by its {@link #update(IDeviceSubject, Object)} method from
 * the subject class {@link IDeviceSubject}.
 *
 * @author <a href="marcus.walla(at)desy.de">walla</a>
 * @version $Id: IObserver.java 19706 2008-12-14 12:41:25Z walla $
 */
public interface IObserver
{
	/**
	 * The update method is responsible for updating (for example) the control
	 * system or for different GUI elements on the display. It carries the
	 * subject of the observer design pattern and an object which holds the
	 * data value to be updated.
	 *
	 * @param subject_ Is the subject of the observer design pattern. It is the
	 * 		controller of every observer instance.
	 * @param obj_ Is the data object.
	 */
	public abstract void update(IDeviceSubject subject_, Object obj_);
}
/* end of file */