/*
 *
 * File: IRunnableExtension.java
 * Date: Feb 13, 2008; 5:58:23 PM
 */
package de.desy.mst.libs.accelerator;

import de.desy.tine.types.TCompoundDataObject;

/**
 * This interface extends the {@link IRunnable} interface. It includes, in a
 * more expressive way, a set value with a TINE data type and the a method
 * which returns the error status of the runnable.
 *   
 * @author <a href="mailto:marcus.walla(at)desy.de">walla</a>
 * @version $Id: IRunnableExtension.java 12066 2008-05-29 08:35:57Z walla $
 */
public interface IRunnableExtension extends IRunnable
{

	/**
	 * The value will be set to the corresponding value of 
	 * {@link de.desy.tine.types.TCompoundDataObject}.
	 * 
	 * @param value_ Sets the interface to the corresponding value.
	 */
	public abstract void setValue(TCompoundDataObject value_);
	
	/**
	 * <p>Queries if an error has occurred on this thread by running it with the
	 * method of {@link Runnable#run()}.</p>
	 *  
	 * <p><u><b>ATTENTION:</b></u> Possibly this query will wait until the 
	 * <tt>run()</tt> method has been executed. So there can be (depending
	 * on the wait-times of the constructor how long this method will wait.
	 * This should possibly done by a <b>public field</b> in the concrete class.</p>  
	 *  
	 * @return A boolean value is returned, which provides the error of this
	 * 		thread.
	 */
	public abstract boolean isError();

}
/* end of file */