/*
 *
 * File: InvalidDeviceNameException.java
 * Date: Nov 29, 2007; 12:21:07 PM
 */
package de.desy.mst.libs.accelerator.exception;

/**
 * <p>This exception is thrown at runtime when the device name is invalid.</p>
 * <p>For example, you wanted to use for the Linac2 RF modulators from 1 to 12. 
 * This is normally done by constructing a TINE path with the following 
 * introduction. The context and device server will be {@code /HARDWARE/RFMod}
 * and the modulators will be counted from 1 to 12: {@code modulator01},
 * {@code modulator02}, ..., {@code modulator12}. If one of these will not
 * end with a number it is invalid and this exception should be thrown.</p>
 * 
 * @author <a href="mailto:marcus.walla(at)desy.de">walla</a>
 * @version $Id: InvalidDeviceNameException.java 14522 2008-08-05 12:50:22Z walla $
 */
public class InvalidDeviceNameException extends RuntimeException
{
	/** Is the serial version UID to unify this class. */
	public final static long serialVersionUID = 4103859591924958130l;
	
	/**
	 * Constructs an invalid device name exception.
	 */
	public InvalidDeviceNameException()
	{
	}

	/**
	 * Constructs an invalid device name exception.
	 * 
	 * @param message_ Is the message.
	 */
	public InvalidDeviceNameException(String message_)
	{
		super(message_);
	}

	/**
	 * Constructs an invalid device name exception.
	 *
	 * @param cause_ Is the throw able cause of this exception.
	 */
	public InvalidDeviceNameException(Throwable cause_)
	{
		super(cause_);
	}

	/**
	 * Constructs an invalid device name exception.
	 * 
	 * @param message_ Is the message.
	 * @param cause_ Is the throw able cause of this exception.
	 */
	public InvalidDeviceNameException(String message_, Throwable cause_)
	{
		super(message_, cause_);
	}

}
/* end of file */