/*
 *
 * File: NotInitializedException.java
 * Date: Nov 28, 2006; 10:39:07 AM
 */
package de.desy.mst.libs.accelerator.exception;

/**
 * <p>The initialization of the instance can not done because some of the 
 * precondition fails.</p>
 * 
 * <p>For example, the transmitter status of Petra3 RF expects different status 
 * values via TINE from the TINE address called {@code /MHF/PERF.ML/Read/TransmitterStatus_L} 
 * or {@code .../TransmitterStatus_R}. If one of the pre defined string values 
 * in the monitor class doesn't fit with the delivered values from TINE this 
 * exception is thrown, to show that there is misleading definition of the pre
 * defined values.</p>
 *
 * @author <a href="marcus.walla at desy.de">walla</a>
 * @version $Id: NotInitializedException.java 22024 2009-02-20 15:08:02Z walla $
 */
public class NotInitializedException extends IllegalArgumentException
{

	/** Is the serial version UID to unify this class. */
	public static final long serialVersionUID = 4829909234954139983l;
	
	/**
	 * Constructor.
	 */
	public NotInitializedException()
	{
	}

	/**
	 * Constructor.
	 * 
	 * @param message_ Message of the {@code NotInitializedException} instance.
	 */
	public NotInitializedException(String message_)
	{
		super(message_);
	}

	/**
	 * Constructor.
	 * 
	 * @param cause_ Is the throwable cause of this exception.
	 */
	public NotInitializedException(Throwable cause_)
	{
		super(cause_);
	}

	/**
	 * Constructor.
	 * 
	 * @param message_ Message of the {@code NotInitializedException} instance.
	 * @param cause_ Is the throwable cause of this exception.
	 */
	public NotInitializedException(String message_, Throwable cause_)
	{
		super(message_, cause_);
	}

}
/* end of file */