/*
 *
 * File: TineException.java
 * Date: Sep 27, 2006; 3:57:59 PM
 */
package de.desy.mst.libs.accelerator.exception;

/**
 * This is a TINE exception which is called during the runtime of a project. It
 * is a more general usage of this exception. 
 *  
 * @author <a href="marcus.walla(at)desy.de">walla</a>
 * @version $Id: TineException.java 12066 2008-05-29 08:35:57Z walla $
 */
public class TineException extends Exception
{

	/** Is the serial version UID to unify this class. */
	public static final long serialVersionUID = 2455204066034123485l;
	
	private final int _error;
	
	/**
	 * The constructor of this exception.
	 * 
	 * @param error_ The error code value. This is in most cases the TINE error 
	 * 		code.
	 */
	public TineException(final int error_)
	{
		_error = error_;
	}

	/**
	 * The constructor of this exception.
	 * 
	 * @param error_ The error code value. This is in most cases the TINE error 
	 * 		code.
	 * @param message_ Gets a message as its description.
	 */
	public TineException(final int error_, final String message_)
	{
		super(message_);
		_error = error_;
	}

	/**
	 * The constructor of this exception.
	 * 
	 * @param error_ The error code value. This is in most cases the TINE error 
	 * 		code.
	 * @param cause_ Is the throwable cause of this exception.
	 */
	public TineException(final int error_, final Throwable cause_)
	{
		super(cause_);
		_error = error_;
	}

	/**
	 * The constructor of this exception.
	 * 
	 * @param error_ The error code value. This is in most cases the TINE error 
	 * 		code.
	 * @param message_ Gets a message as its description.
	 * @param cause_ Is the throwable cause of this exception.
	 */
	public TineException(final int error_, final String message_, final Throwable cause_)
	{
		super(message_, cause_);
		_error = error_;
	}
	
	/**
	 * Returns the error code from the constructor. This is mostly the TINE
	 * error code, which can be resolved to a string value by calling 
	 * {@code TErrorList.getErrorString(int)}.
	 * 
	 * @return The error code is returned.
	 */
	public final int getErrorCode()
	{
		return _error;
	}

}
/* end of file */