/*
 *
 * File: TinePathException.java
 * Date: Mar 22, 2007; 6:11:03 PM
 */
package de.desy.mst.libs.accelerator.exception;

/**
 * <p>This exception is thrown if some parameters in the TINE link path string
 * are not correct or an error occurred on build this path with a call
 * to {@code de.desy.tine.client.TLink}.</p>
 * <p>With a TINE path here is meant the device context, device server and
 * the device name with (more or less) the property name. Each element is separated
 * with a slash: <tt>/&lt;device context&gt;/&lt;device server&gt;/&lt;device name&gt;/
 * &lt;property&gt;</tt>. For more information about TINE see at the 
 * <a href="http://adweb.desy.de/mst/tine/index.html">TINE documentation</a>.</p> 
 * 
 * @author <a href="marcus.walla(at)desy.de">walla</a>
 * @version $Id: TinePathException.java 12066 2008-05-29 08:35:57Z walla $
 */
public class TinePathException extends TineException
{
	/** Is the serial version UID to unify this class. */
	public static final long serialVersionUID = 660200123996792523l;
	
	/**
	 * This is the parameterless constructor of this exception.
	 * 	
	 * @param error_ The error code value. This is in most cases the TINE error 
	 * 		code.
	 */
	public TinePathException(final int error_)
	{
		super(error_);
	}

	/**
	 * This is the constructor of this exception. As its only argument it
	 * will take the message as a {@code java.lang.String} value.
	 * 
	 * @param error_ The error code value. This is in most cases the TINE error 
	 * 		code.
	 * @param message_ Is the message of this exception.
	 */
	public TinePathException(final int error_, final String message_)
	{
		super(error_, message_);
	}

	/**
	 * This is the constructor of this exception. As its only argument it
	 * will take the cause as a {@code java.lang.Throwable} value.
	 * 
	 * @param error_ The error code value. This is in most cases the TINE error 
	 * 		code.
	 * @param cause_ Is the cause of this exception.
	 */
	public TinePathException(final int error_, final Throwable cause_)
	{
		super(error_, cause_);
	}

	/**
	 * This is a constructor of this exception.
	 * 
	 * @param error_ The error code value. This is in most cases the TINE error 
	 * 		code.
	 * @param message_ Is the message of this exception.
	 * @param cause_ Is the cause of this exception.
	 */
	public TinePathException(final int error_, final String message_, 
			final Throwable cause_)
	{
		super(error_, message_, cause_);
	}

}
/* end of file */