/*
 *
 * File: ValueNotSetException.java
 * Date: Feb 20, 2008; 3:28:34 PM
 */
package de.desy.mst.libs.accelerator.exception;

/**
 * If the current value is not set in the device server or hardware these
 * exception can be thrown.
 *
 * @author <a href="mailto:marcus.walla(at)desy.de">walla</a>
 * @version $Id: ValueNotSetException.java 12066 2008-05-29 08:35:57Z walla $
 */
public final class ValueNotSetException extends Exception
{
	/** Is the unique serial version UID */
	final static long serialVersionUID = 3205949843412593014l;
	
	/** Is the holder of the current number. */
	private final Number _currentValue;
	
	/**
	 * The constructor of this exception.
	 * 
	 */
	public ValueNotSetException()
	{
		super();
		_currentValue = null;
	}
	
	/**
	 * The constructor of this exception.
	 *
	 * @param currentValue_ Is the current value you wanted to set.
	 */
	public ValueNotSetException(final Number currentValue_)
	{		
		super("The current value was not set: " + currentValue_);
		
		_currentValue = currentValue_;
	}

	/**
	 * The constructor of this exception.
	 * 
	 * @param message_ Gets a message as its description.
	 * @param currentValue_ Is the current value you wanted to set.
	 */
	public ValueNotSetException(final String message_, 
			final Number currentValue_)
	{
		super(message_);

		_currentValue = currentValue_;
	}

	/**
	 * The constructor of this exception.
	 * 
	 * @param cause_ Is the throwable cause of this exception.
	 * @param currentValue_ Is the current value you wanted to set.
	 */
	public ValueNotSetException(final Throwable cause_,
			final Number currentValue_)
	{
		super(cause_);

		_currentValue = currentValue_;
	}

	/**
	 * The constructor of this exception.
	 * 
	 * @param message_ Gets a message as its description.
	 * @param cause_ Is the throwable cause of this exception.
	 * @param currentValue_ Is the current value you wanted to set.
	 */
	public ValueNotSetException(final String message_, final Throwable cause_,
			final Number currentValue_)
	{
		super(message_, cause_);

		_currentValue = currentValue_;
	}
	
	/**
	 * Returns the current number.
	 * 
	 * @return Returns the current number.
	 */
	public Number getCurrentValue()
	{
		return _currentValue;
	}

}
/* end of file */