/*
 *
 * File: IExecute.java
 * Date: Nov 28, 2006; 9:47:50 AM
 */
package de.desy.mst.libs.accelerator.model;

import de.desy.mst.libs.accelerator.exception.TineException;

/**
 * <p>This is the interface for controlling the execution of a synchronous link 
 * via TINE. There exists three methods for doing a link. The first one will
 * be initialize, which will initialize the link. The second one will call the TINE
 * link every time you call the method {@code IExecute.call()}. The third and
 * last method, {@code IExecute.cancel()} will destroy the current link and
 * set the its internal field to be <tt>false</tt>.</p>
 * 
 * <p>Some extrapolated subclasses will use the method {@code IExecute.initialize()}
 * another way. They will initialize the link by internal calling the link with 
 * the execute method which will be destroyed after doing its job. The internal
 * field that this method has been called before is here set to <tt>true</tt>
 * so that for further class on this class can therefore be done with the
 * {@code IExecute.call()} method.</p>
 * 
 * @author <a href="marcus.walla(at)desy.de">walla</a>
 * @version $Id: IExecute.java 32894 2010-02-08 12:53:07Z walla $
 */
@Deprecated
public interface IExecute
{
	/**
	 * <p>This will initialize the current executing TINE link and put it into
	 * an internal associated map and after doing so calling the TINE link with
	 * the synchronous executing call. An internal field is set to be 
	 * <tt>true<tt>.</p>  
	 * 
	 * <p>For further description of this call see the interface description.</p> 
	 *
	 * @throws TineException A TINE exception is thrown if an error occurred on
	 * 		the link connection with the server.
	 */
	@Deprecated
	public abstract void initialize() throws TineException;
	
	/**
	 * Will call the executing method of the TINE link. This is a synchronous link
	 * between the client and server.
	 * 
	 * @throws TineException A TINE exception is thrown if an error occurred on
	 * 		the link connection with the server.
	 */
	@Deprecated
	public abstract void call() throws TineException;
	
	/**
	 * This will cancel all included links and destroy them from the internal
	 * associated map. The internal field is set to be <tt>false</tt>.
	 * 
	 * @throws TineException A TINE exception is thrown if an error occurred on
	 * 		the link connection with the server.
	 */
	@Deprecated
	public abstract void cancel() throws TineException;
	
	
	/**
	 * This method ask the interface if this class has been initialized. If not
	 * you should at first call the method {@link #initialize()} before calling
	 * other methods.
	 *  
	 * @return Returns a boolean value. It is <tt>true</tt> if this interface has
	 * 		been initialized otherwise a <tt>false</tt> is returned.
	 */
	@Deprecated
	public abstract boolean isInitialized();
}
/* end of file */