/*
 *
 * File: ICavityData.java
 * Date: Jan 27, 2008; 5:49:14 PM
 */
package de.desy.mst.libs.accelerator.model.cavity;

import java.util.Collection;

import de.desy.mst.libs.accelerator.IEData;

/**
 * This is a list, or better a map, of all cavity view elements with its
 * corresponding data element.
 *
 * @author <a href="mailto:marcus.walla(at)desy.de">walla</a>
 * @version $Id: ICavityData.java 19491 2008-12-07 15:18:19Z walla $
 */
public interface ICavityData {
	/**
	 * This will set the unique cavity view element with the data element. If
	 * the view element is {@code null} then a {@code NullPointerException} is
	 * thrown.
	 *
	 * @param view_ Is the cavity view element which can be its GENERAL,
	 * 		ELEMENT, or SWITCH view enumeration class with the
	 * 		corresponding enumeration.
	 * @param data_ Is the data element.
	 */
	public abstract void put(final IECavityElement view_, final IEData data_);

	/**
	 * Returns all set <tt>IEData</tt> elements as a collection.
	 *
	 * @return Returns all set <tt>IEData</tt> elements as a collection.
	 */
	public abstract Collection<IEData> getAllElements();

	/**
	 * Returns the data element corresponding to the input argument.
	 *
	 * @param view_ Is the viewable cavity element.
	 * @return Returns the data element of the view one.
	 */
	public abstract IEData get(final IECavityElement view_);

	/**
	 * This will validate the map. To do this it gets all enumeration elements
	 * from the enumeration interface cavity view and checks if:
	 * <ul>
	 * <li>... it is in the map, and</li>
	 * <li>... the cavity data element isn't {@code null}.</li>
	 * </ul>
	 * If all of the elements return {@code true} then it will leave
	 * without any exception. Otherwise an {@code IllegalArgumentException} is
	 * thrown with a list of all elements which should be set in the correct
	 * way.
	 */
	public abstract void validateElements();
}
/* end of file */