/*
 *
 * File: DBoolean.java
 * Date: Dec 1, 2006; 11:09:56 AM
 */
package de.desy.mst.libs.accelerator.number;

/**
 * This will encapsulate a single boolean value.
 *
 * @author <a href="marcus.walla at desy.de">walla</a>
 * @version $Id: DBoolean.java 22583 2009-03-10 11:22:31Z walla $
 * @see INumber
 * @see DShort
 * @see DByte
 * @see DInteger
 * @see DLong
 * @see DFloat
 * @see DDouble
 *
 */
public final class DBoolean implements INumber
{
	
	/** The internal boolean variable. */
	private Boolean _bool = false;
	
	/**
	 * Constructor of a the field boolean. The boolean value is set to false.
	 */
	public DBoolean()
	{
		_bool = false;
	}
	
	/**
	 * The constructor of the field boolean. Will get one argument as a boolean value.
	 * 
	 * @param bool_ Gets one argument as a boolean value.
	 */
	public DBoolean(boolean bool_)
	{
		_bool = bool_;
	}

	/*
	 * (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.number.INumber#booleanValue()
	 */
	public boolean booleanValue()
	{
		return _bool;
	}
	
	/* (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.number.INumber#byteValue()
	 */
	public byte byteValue()
	{
		return (byte) (_bool==true ? 1 : 0);
	}

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.number.INumber#doubleValue()
	 */
	public double doubleValue()
	{
		return (_bool==true ? 1.0 : 0.0);
	}

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.number.INumber#floatValue()
	 */
	public float floatValue()
	{
		return (_bool==true ? 1.0f : 0.0f);
	}

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.number.INumber#intValue()
	 */
	public int intValue()
	{
		return (_bool==true ? 1 : 0);
	}

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.number.INumber#longValue()
	 */
	public long longValue()
	{
		return (_bool==true ? 1 : 0);
	}

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.number.INumber#shortValue()
	 */
	public short shortValue()
	{
		return (short) (_bool==true ? 1 : 0);
	}
	
	// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.number.INumber#setValue(byte)
	 */
	public void setValue(boolean bool_)
	{
		_bool = bool_;
	}
	
	/* (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.number.INumber#setValue(byte)
	 */
	public void setValue(byte byte_)
	{
		_bool = byte_ == 1 ? true : false;
	}

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.number.INumber#setValue(short)
	 */
	public void setValue(short short_)
	{
		_bool = short_ == 1 ? true : false;
	}

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.number.INumber#setValue(int)
	 */
	public void setValue(int int_)
	{
		_bool = int_ == 1 ? true : false;
	}

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.number.INumber#setValue(long)
	 */
	public void setValue(long long_)
	{
		_bool = long_ == 1 ? true : false;
	}

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.number.INumber#setValue(float)
	 */
	public void setValue(float float_)
	{
		_bool = float_ == 1.0f ? true : false;
	}

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.number.INumber#setValue(double)
	 */
	public void setValue(double double_)
	{
		_bool = double_ == 1.0 ? true : false;
	}
	
	// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

	/*
	 * (non-Javadoc)
	 * @see java.lang.Object#equals(java.lang.Object)
	 */
	public boolean equals(final Object object_)
	{
		return _bool.equals(object_);
	}
	
	/*
	 * (non-Javadoc)
	 * @see java.lang.Object#hashCode()
	 */
	public int hashCode()
	{
		return _bool.hashCode();
	}
	
	/*
	 * (non-Javadoc)
	 * @see java.lang.Object#toString()
	 */
	public String toString()
	{
		return _bool.toString();
	}
}
/* end of file */