/*
 *
 * File: IGenericPanel.java
 * Date: Jan 26, 2007; 3:56:25 PM
 */
package de.desy.mst.libs.accelerator.view;

import javax.swing.DefaultListModel;
import javax.swing.JComponent;
import javax.swing.JPanel;
import javax.swing.JTabbedPane;

import de.desy.mst.libs.accelerator.IEViewGeneric;
import de.desy.mst.libs.accelerator.exception.EnumerationException;


/**
 * A generic panel interface of GUI elements.
 *
 * @author <a href="marcus.walla(at)desy.de">walla</a>
 * @version $Id: IGenericPanel.java 51676 2012-09-05 08:14:32Z walla $
 */
public interface IGenericPanel
{

	/**
	 * Returns the current panel of the delegated container. This can also
	 * return a {@code null} value if no panel exists.
	 *
	 * @return Returns the current panel.
	 */
	public abstract JPanel fetchPanel();

	/**
	 * Returns the current tab panel of the the delegated container. This
	 * can also return a {@code null} value. There can also be returned a
	 * {@code null} value.
	 *
	 * @return Returns the current tab panel value.
	 */
	public abstract JTabbedPane fetchTabbedPane();

	/**
	 * Fetches the component of the delivered view. The view is here the
	 * enumeration interface called {@link IEViewGeneric}. If the container
	 * does not exist there can also be a null returned.
	 *
	 * @param view_ Is the view you wanted to return.
	 * @return Is the component of the delivered view.
	 */
	public abstract JComponent fetchChildPanel(IEViewGeneric view_);

	/**
	 * Returns the container of switch within the view.
	 *
	 * @param view_ The view generic enumeration interface.
	 * @return Returns the view container as an interface.
	 * @throws EnumerationException Throws an enumeration exception if an
	 * 		error occurred on the enumeration (possibly it is not defined and
	 * 		so on).
	 */
	public abstract IViewContainer fetchContainer(IEViewGeneric view_)
		throws EnumerationException;

	/**
	 * Fetches the current list model of the generic panel. If the concrete
	 * panel doesn't contain a model it will return a {@code null} value.
	 *
	 * @param switch_ Is the viewable generic enumeration interface.
	 * @return Is the data list model.
	 * @throws EnumerationException If the corresponding enumeration doesn't
	 * 		exist.
	 */
	public abstract DefaultListModel fetchListModel(IEViewGeneric switch_)
		throws EnumerationException;

}
/* end of file */