/*
 *
 * File: IPolygonComponent.java
 * Date: Nov 2, 2006; 1:48:31 PM
 */
package de.desy.mst.libs.accelerator.view;

import java.awt.event.ActionListener;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;

/**
 * <p>This is the interface of all <b>polygon component</b>. It will carry a 
 * polygon button and listeners and is able to return them. Moreover
 * you can also specify an Action and an action listener. These are for
 * controlling the action and response for an action occurred.<p>
 * <p>The current component will implement this interface:</p>
 * <pre>
 * public final class PolygonContainer implements IPolygonComponent
 * {
 *     ...
 * }
 * </pre>
 *  
 * @author <a href="marcus.walla(at)desy.de">walla</a>
 * @version $Id: IPolygonComponent.java 32894 2010-02-08 12:53:07Z walla $
 * @see IAction
 * @see APolygonButton 
 * @see IPolygonListener
 */
@Deprecated
public interface IPolygonComponent
{
	/**
	 * Returns the button. This is contained in the container. It is a polygon.
	 * 
	 * @return The polygon button is returned. There can also be returned a 
	 * {@code null} value.
	 */
	@Deprecated
	public abstract APolygonButton fetchButton();
	
	/**
	 * This will return the polygon listener.
	 * 
	 * @return Returns the polygon listener. There can also be returned a 
	 * {@code null} value.
	 */
	@Deprecated
	public abstract IPolygonListener fetchListener();

	/**
	 * Returns the mouse listener. This interface is take from the java world and
	 * can be found under {@code java.awt.event.MouseListener}.
	 * 
	 * @return Returns the mouse listener. There can also be returned a 
	 * 		{@code null} value.
	 */
	@Deprecated
	public abstract MouseListener fetchMouseListener();
	
	/**
	 * Returns the mouse motion listener. This is an interface taken from the 
	 * java world and can be found under {@code java.awt.event.MouseMotionListener}.
	 * 
	 * @return Returns the mouse motion listener. There can also be returned a 
	 * 		{@code null} value.
	 */
	@Deprecated
	public abstract MouseMotionListener fetchMouseMotionListener();
	
	/**
	 * This will return an action listener. This is an interface taken from the
	 * java world and can be found under {@code java.awt.event.ActionListener}.
	 * 
	 * @return Returns an action listener. There can also be returned a {@code null}
	 * 		value.
	 */
	@Deprecated
	public abstract ActionListener fetchActionListener();

	/**
	 * Returns the action. This action is performed if one has pressed inside the
	 * button. 
	 * 
	 * @return Returns the current action of the button.  There can also be 
	 * 		returned a {@code null} value.
	 */
	@Deprecated
	public abstract IAction fetchAction();
}
/* end of file */