/*
 *
 * File: RFDialogVersionInfo.java
 * Date: Nov 11, 2008; 12:58:47 PM
 */
package de.desy.mst.libs.accelerator.view;

import java.awt.Frame;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;
import java.util.concurrent.atomic.AtomicBoolean;

import javax.swing.BorderFactory;
import javax.swing.GroupLayout;
import javax.swing.Icon;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.LayoutStyle;
import javax.swing.SwingConstants;
import javax.swing.border.TitledBorder;

import de.desy.mst.libs.accelerator.Utility;
import de.desy.mst.libs.accelerator.view.RFDialog.IDialog;
import de.desy.mst.libs.accelerator.view.RFDialog.IVersion;

/**
 * Represents the versions in a separate dialog box for the user. The version
 * are declared outside of this class and reported in the constructor with
 * an array of {@link IVersion}. The developer of the project is taken from
 * the {@link Utility} class (see for example the argument of
 * {@link Utility#APP_AUTHOR}). The concrete developer is set with the special
 * application by calling from the main method the static method of
 * {@code setApplicationVersion(Class):void}. The dialog box is modal.
 *
 * @author <a href="mailto:marcus.walla(at)desy.de">walla</a>
 * @version $Id: RFDialogVersionInfo.java 33041 2010-02-10 14:30:08Z walla $
 */
public final class RFDialogVersionInfo extends JDialog implements IDialog, WindowListener
{
	/** The internal serial version UID. */
	private static final long serialVersionUID = 8736081212863020705L;

	// -------------------------------------------------------------------------

	public static final int GAP_VERSION = 4;
	public static final int GAP_SEPARATOR = 4;
	public static final int GAP_BORDER = 5;
	public static final int GAP_HEADER = 10;
	public static final int GAP_FOOTER = 10;

	// -------------------------------------------------------------------------

	public static final String MENU_ITEM_INFO = "Informationen zum Programm";

	// -------------------------------------------------------------------------

	private final AtomicBoolean _isActive = new AtomicBoolean(false);

	private final Frame _parent;

	// -------------------------------------------------------------------------

	/**
	 * Constructor. Builds a new dialog box of the versions and a short
	 * description about the developer.
	 *
	 * @param parent_ The parent frame. This can also be {@code null}.
	 * @param versions_ An array of versions.
	 */
	public RFDialogVersionInfo(final Frame parent_, final IVersion... versions_)
	{
		super(parent_, true);
		validate(parent_, versions_);
		_parent = parent_;
		initialize(versions_);
	}

	// -------------------------------------------------------------------------

	private void validate(final Frame parent_, final IVersion[] versions_)
	{
		if (parent_ == null)
			throw new NullPointerException("Frame is null.");
		if (versions_ == null)
			throw new NullPointerException("Version is null.");
		if (versions_.length == 0)
			throw new IllegalArgumentException("Number of version is null.");
	}

	private void initialize(final IVersion[] versions_)
	{
		final Icon icon;
		if (_parent.getIconImages() != null	&& _parent.getIconImages().size() > 0)
			icon = new ImageIcon(_parent.getIconImages().get(0));
		else
			icon = null;

		String appName = "UNBEKANNT";
		if (System.getProperty(Utility.APP_APPLICATION_NAME) != null
				&& !System.getProperty(Utility.APP_APPLICATION_NAME).equals(Utility.UNKNOWN)
				&& System.getProperty(Utility.APP_VERSION) != null
				&& !System.getProperty(Utility.APP_VERSION).equals(Utility.UNKNOWN))
				appName = System.getProperty(Utility.APP_APPLICATION_NAME);

		// ---------------------------------------------------------------------

		JLabel lblAuthor = new JLabel("Autor:");
		JLabel lblAuthorHolder = new JLabel(System.getProperty(Utility.APP_DEVELOPER));
		JLabel lblBuilding = new JLabel("Gebude:");
		JLabel lblBuildingHolder = new JLabel (System.getProperty(Utility.APP_BUILDING) + "/" + System.getProperty(Utility.APP_ROOM));
		JLabel lblGroup = new JLabel("Gruppe:");
		JLabel lblGroupHolder = new JLabel(System.getProperty(Utility.APP_GROUP));
		JLabel lblEMail = new JLabel("E-Mail:");
		JLabel lblEMailHolder = new JLabel(System.getProperty(Utility.APP_EMAIL));
		JLabel lblPhone = new JLabel("Telefon:");
		JLabel lblPhoneHolder = new JLabel(System.getProperty(Utility.APP_PHONE));

		JPanel pnlVersion = null;
		if (versions_.length > 0)
		{
			pnlVersion = new JPanel();
			GroupLayout layoutVersion = new GroupLayout(pnlVersion);
			pnlVersion.setLayout(layoutVersion);
			pnlVersion.setBorder(BorderFactory.createTitledBorder(
					BorderFactory.createLineBorder(CColor.BLACK),
					"Version(en)",
					TitledBorder.DEFAULT_JUSTIFICATION,
					TitledBorder.DEFAULT_POSITION,
					CFrame.FONT_TITLE_BORDER));

			layoutVersion.setAutoCreateContainerGaps(true);
			GroupLayout.ParallelGroup horizontalGrp = layoutVersion.createParallelGroup();
			GroupLayout.SequentialGroup verticalGrp = layoutVersion.createSequentialGroup();

			final JLabel lblJavaVersion = new JLabel("Java - Version " + System.getProperty("java.version","UNKNOWN") + " (JVM " + System.getProperty("java.vm.name","UNKNOWN") + " - Version " + System.getProperty("java.vm.version","UNKNOWN") + ')');
			horizontalGrp.addComponent(lblJavaVersion);
			verticalGrp.addComponent(lblJavaVersion);
			for(int i = 0; i < versions_.length; i++)
			{
				JLabel lblVersion = new JLabel(versions_[i].toString());
				horizontalGrp.addComponent(lblVersion);
				verticalGrp.addGap(5).addComponent(lblVersion);
			}

			JLabel lblHeaderVersion = new JLabel("Version:");
			layoutVersion.setHorizontalGroup(
					layoutVersion.createParallelGroup()
					.addGroup(layoutVersion.createSequentialGroup()
							.addComponent(lblHeaderVersion)
							.addGap(10)
							.addGroup(horizontalGrp)
					)
			);
			layoutVersion.setVerticalGroup(
					layoutVersion.createSequentialGroup()
					.addGap(GAP_SEPARATOR)
					.addGroup(layoutVersion.createParallelGroup()
							.addComponent(lblHeaderVersion)
							.addGroup(verticalGrp)
					)
			);
		}

		// ---------------------------------------------------------------------

		JPanel pnlDeveloper = new JPanel();
		GroupLayout layoutDeveloper = new GroupLayout(pnlDeveloper);
		pnlDeveloper.setLayout(layoutDeveloper);
		pnlDeveloper.setBorder(BorderFactory.createTitledBorder(
				BorderFactory.createLineBorder(CColor.BLACK),
				"Entwicklerin/Entwicker",
				TitledBorder.DEFAULT_JUSTIFICATION,
				TitledBorder.DEFAULT_POSITION,
				CFrame.FONT_TITLE_BORDER));

		layoutDeveloper.setAutoCreateContainerGaps(true);
		// create the horizontal group -------------------------------------
		GroupLayout.ParallelGroup horizontalGrp = layoutDeveloper.createParallelGroup();
		horizontalGrp
			.addGroup(layoutDeveloper.createSequentialGroup()
					.addComponent(lblAuthor)
					.addGap(5)
					.addComponent(lblAuthorHolder)
					.addPreferredGap(LayoutStyle.ComponentPlacement.RELATED,
		                     GroupLayout.PREFERRED_SIZE, Short.MAX_VALUE)
					.addComponent(lblPhone)
					.addGap(5)
					.addComponent(lblPhoneHolder))
			.addGroup(layoutDeveloper.createSequentialGroup()
					.addComponent(lblGroup)
					.addGap(5)
					.addComponent(lblGroupHolder)
					.addPreferredGap(LayoutStyle.ComponentPlacement.RELATED,
		                     GroupLayout.PREFERRED_SIZE, Short.MAX_VALUE)
					.addComponent(lblBuilding)
					.addGap(5)
					.addComponent(lblBuildingHolder))
			.addGroup(layoutDeveloper.createSequentialGroup()
					.addPreferredGap(LayoutStyle.ComponentPlacement.RELATED,
							GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
					.addComponent(lblEMail)
					.addGap(5)
					.addComponent(lblEMailHolder)
			);

		// create the vertical group ---------------------------------------
		GroupLayout.SequentialGroup verticalGrp = layoutDeveloper.createSequentialGroup();
		verticalGrp
			.addGroup(layoutDeveloper.createParallelGroup()
					.addComponent(lblAuthor)
					.addComponent(lblAuthorHolder)
					.addComponent(lblPhone)
					.addComponent(lblPhoneHolder))
			.addGap(10)
			.addGroup(layoutDeveloper.createParallelGroup()
					.addComponent(lblGroup)
					.addComponent(lblGroupHolder)
					.addComponent(lblBuilding)
					.addComponent(lblBuildingHolder))
			.addGap(10)
			.addGroup(layoutDeveloper.createParallelGroup()
					.addComponent(lblEMail)
					.addComponent(lblEMailHolder));

		layoutDeveloper.setHorizontalGroup(horizontalGrp);
		layoutDeveloper.setVerticalGroup(verticalGrp);

		// ---------------------------------------------------------------------

		JLabel lblHeader = new JLabel(MENU_ITEM_INFO, icon, SwingConstants.CENTER);
		lblHeader.setIconTextGap(10);

		JButton btnOK = new JButton("OK");
		btnOK.addActionListener(new ActionListener() {

			public void actionPerformed(ActionEvent e)
			{
				close();
			}

		});

		// ---------------------------------------------------------------------

		JPanel panel = new JPanel();
		GroupLayout layout = new GroupLayout(panel);
		panel.setLayout(layout);

		layout.setAutoCreateContainerGaps(true);
		layout.setHorizontalGroup(layout.createParallelGroup(GroupLayout.Alignment.CENTER)
				.addComponent(lblHeader)
				.addComponent(pnlDeveloper)
				.addComponent(pnlVersion)
				.addComponent(btnOK));
		layout.setVerticalGroup(layout.createSequentialGroup()
				.addComponent(lblHeader)
				.addGap(20)
				.addComponent(pnlDeveloper)
				.addGap(10)
				.addComponent(pnlVersion)
				.addGap(10)
				.addComponent(btnOK));

		// ---------------------------------------------------------------------

		this.setDefaultCloseOperation(JDialog.DO_NOTHING_ON_CLOSE);
		this.addWindowListener(this);
		this.setResizable(false);
		this.setTitle(MENU_ITEM_INFO + " " + appName);
		this.setContentPane(panel);
		this.pack();
	}

	// -------------------------------------------------------------------------

	/*
	 * (non-Javadoc)
	 * @see java.awt.Window#isActive()
	 */
	@Override public boolean isActive()
	{
		return _isActive.get();
	}

	/*
	 * (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.view.RFDialog.IDialog#showDialog()
	 */
	public void showDialog()
	{
		this.setLocationRelativeTo(_parent);
        this.setVisible(true);

		_isActive.set(true);
	}

	/*
	 * (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.view.RFDialog.IDialog#close()
	 */
	public void close()
	{
		this.setVisible(false);
		_isActive.set(false);
	}

	/*
	 * (non-Javadoc)
	 * @see java.awt.event.WindowListener#windowClosing(java.awt.event.WindowEvent)
	 */
	public void windowClosing(WindowEvent e)
	{
		close();
	}

	/*
	 * (non-Javadoc)
	 * @see java.awt.event.WindowListener#windowActivated(java.awt.event.WindowEvent)
	 */
	public void windowActivated(WindowEvent e)
	{
	}

	/*
	 * (non-Javadoc)
	 * @see java.awt.event.WindowListener#windowClosed(java.awt.event.WindowEvent)
	 */
	public void windowClosed(WindowEvent e)
	{
	}

	/*
	 * (non-Javadoc)
	 * @see java.awt.event.WindowListener#windowDeactivated(java.awt.event.WindowEvent)
	 */
	public void windowDeactivated(WindowEvent e)
	{
	}

	/*
	 * (non-Javadoc)
	 * @see java.awt.event.WindowListener#windowDeiconified(java.awt.event.WindowEvent)
	 */
	public void windowDeiconified(WindowEvent e)
	{
	}

	/*
	 * (non-Javadoc)
	 * @see java.awt.event.WindowListener#windowIconified(java.awt.event.WindowEvent)
	 */
	public void windowIconified(WindowEvent e)
	{
	}

	/*
	 * (non-Javadoc)
	 * @see java.awt.event.WindowListener#windowOpened(java.awt.event.WindowEvent)
	 */
	public void windowOpened(WindowEvent e)
	{
	}
}
/* end of file */