/*
 *
 * File: RFList.java
 * Date: Oct 15, 2008; 12:32:00 PM
 */
package de.desy.mst.libs.accelerator.view.generic;

import java.awt.Color;
import java.awt.event.ComponentListener;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.beans.PropertyChangeListener;
import java.util.EventListener;

import javax.swing.BorderFactory;
import javax.swing.JList;
import javax.swing.ListModel;
import javax.swing.border.Border;
import javax.swing.event.ListSelectionListener;

import de.desy.mst.libs.accelerator.CGeneralConstant;
import de.desy.mst.libs.accelerator.EDebug;
import de.desy.mst.libs.accelerator.EGraphicComponent;
import de.desy.mst.libs.accelerator.ESeverity;
import de.desy.mst.libs.accelerator.IEData;
import de.desy.mst.libs.accelerator.view.CColor;
import de.desy.mst.libs.accelerator.view.IViewComponent;
import de.desy.mst.libs.framework.mstapp.logging.Logger;

/**
 * New list component which implements the {@code IViewCompoent} interface.
 *
 * @author <a href="mailto:marcus.walla(at)desy.de">walla</a>
 * @version $Id: RFList.java 51593 2012-08-22 15:37:13Z walla $
 */
@SuppressWarnings("rawtypes")
public final class RFList extends JList implements IViewComponent
{
	// --- CLASS FIELDS --------------------------------------------------------
	
	/** This is the current unique serial version UID. It helps the application
	 * to control serialization compatibility between versions. */
	private static final long serialVersionUID = 9219928894191767480L;

	/** Class name. */
	public static final String CLAZZ = RFList.class.getSimpleName();
	
	// --- INSTANCE FIELDS _----------------------------------------------------
	
    /** The severity of the label component. This has no direct effect on the
     * GUI element. */
    private ESeverity _severity = ESeverity.NORMAL;

    // --- CONSTRUCTOR ---------------------------------------------------------
    
	/**
	 * Constructor.
	 */
	public RFList()
	{
    	debug();
	}

	/**
	 * Constructor.
	 * 
	 * @param dataModel_ List model.
	 */
	@SuppressWarnings("unchecked")
	public RFList(final ListModel dataModel_)
	{
		super(dataModel_);
    	debug();
	}

	// --- INSTANCE METHODS ----------------------------------------------------
	
	/*
	 * (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.view.IViewComponent#debug()
	 */	
	public void debug()
	{
		if (CGeneralConstant.isDebug(EDebug.DEBUG_GRAPHIC))
			super.setBorder(BorderFactory.createLineBorder(CColor.RED));
	}

	/*
	 * (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.view.IViewComponent#fetchGraphicComponent()
	 */
	public EGraphicComponent fetchGraphicComponent()
	{
		// FIXME Auto-generated method stub
		throw new RuntimeException("This method is not implemented!");
	}

	/*
	 * (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.view.IViewComponent#fetchSeverity()
	 */
	public ESeverity fetchSeverity()
	{
		return _severity;
	}

	/*
	 * (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.view.IViewComponent#fetchText(de.desy.mst.libs.accelerator.IEData)
	 */
	public String fetchText(IEData view_)
	{
		throw new RuntimeException("This method is not implemented!");
	}

	/*
	 * (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.view.IViewComponent#hasOpaque()
	 */
	public boolean hasOpaque()
	{
		return this.isOpaque();
	}

	/*
	 * (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.view.IViewComponent#isEnable()
	 */
	public boolean isEnable()
	{
		return this.isEnabled();
	}

	/*
	 * (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.view.IViewComponent#isValueAdjusting(de.desy.mst.libs.accelerator.IEData)
	 */
	public boolean isValueAdjusting(IEData view_)
	{
		throw new RuntimeException("This method is not implemented!");
	}

	/*
	 * (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.view.IViewComponent#isVisual()
	 */
	public boolean isVisual()
	{
		return this.isVisible();
	}

	/*
	 * (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.view.IViewComponent#putBackground(java.awt.Color)
	 */
	public void putBackground(Color background_)
	{
		this.setBackground(background_);
	}

	/*
	 * (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.view.IViewComponent#putBorder(javax.swing.border.Border)
	 */
	public void putBorder(Border border_)
	{
		this.setBorder(border_);
	}

	/*
	 * (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.view.IViewComponent#putEnable(boolean)
	 */
	public void putEnable(boolean enable_)
	{
		this.setEnabled(enable_);
	}

	/*
	 * (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.view.IViewComponent#putForeground(java.awt.Color)
	 */
	public void putForeground(Color foreground_)
	{
		this.setForeground(foreground_);
	}

	/*
	 * (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.view.IViewComponent#putListener(java.util.EventListener)
	 */
	public void putListener(EventListener listener_)
	{
		if (listener_ instanceof ListSelectionListener)
			this.addListSelectionListener((ListSelectionListener) (listener_));
		else if (listener_ instanceof ComponentListener)
			this.addComponentListener((ComponentListener) listener_);
		else if (listener_ instanceof MouseListener)
			this.addMouseListener((MouseListener) listener_);
		else if (listener_ instanceof MouseMotionListener)
			this.addMouseMotionListener((MouseMotionListener) listener_);
		else if (listener_ instanceof PropertyChangeListener)
			this.addPropertyChangeListener((PropertyChangeListener) listener_);
		Logger.SWApp.warn("Listener of " + listener_.getClass().getSimpleName()
				+ " is unrecognized in the RFList class.");
	}

	/*
	 * (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.view.IViewComponent#putNumber(java.lang.Number, de.desy.mst.libs.accelerator.IEData)
	 */
	public void putNumber(Number number_, IEData view_)
	{
		throw new RuntimeException("This method is not implemented!");
	}

	/*
	 * (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.view.IViewComponent#putOpaque(boolean)
	 */
	public void putOpaque(boolean opaque_)
	{
		this.setOpaque(opaque_);
	}

	/*
	 * (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.view.IViewComponent#putSelected(boolean)
	 */
	public void putSelected(boolean isSelected_)
	{
		throw new RuntimeException("This method is not implemented!");
	}

	/*
	 * (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.view.IViewComponent#putSeverity(de.desy.mst.libs.accelerator.ESeverity)
	 */
	public void putSeverity(ESeverity severity_)
	{
		_severity = severity_;
	}

	/*
	 * (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.view.IViewComponent#putText(java.lang.String, de.desy.mst.libs.accelerator.IEData)
	 */
	public void putText(String label_, IEData view_)
	{
		throw new RuntimeException("This method is not implemented!");
	}

	/*
	 * (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.view.IViewComponent#putTooltip(java.lang.String)
	 */
	public void putTooltip(String tooltip_)
	{
		this.setToolTipText(tooltip_);
	}

	/*
	 * (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.view.IViewComponent#putVisual(boolean)
	 */
	public void putVisual(boolean visual_)
	{
		this.setVisible(visual_);
	}
	
	@Override
	public String toString()
	{
		return CLAZZ + "[size" + this.getModel().getSize() + ']';
	}
}
/* end of file */