/*
 *
 * File: RFListModel.java
 * Date: Jun 22, 2007; 12:36:31 PM
 */
package de.desy.mst.libs.accelerator.view.generic;

import java.util.ArrayList;
import java.util.List;

import javax.swing.AbstractListModel;

import de.desy.mst.libs.accelerator.view.IListModel;

/**
 * This will create a RF list model and provide methods on it for adding or
 * removing elements from the list.
 * 
 * @author <a href="mailto:marcus.walla(at)desy.de">walla</a>
 * @version $Id: RFListModel.java 51593 2012-08-22 15:37:13Z walla $
 */
@SuppressWarnings("rawtypes")
public class RFListModel<E> extends AbstractListModel implements IListModel<E>
{
	// --- CLASS FIELDS --------------------------------------------------------
	
	/** This is the current unique serial version UID. It helps the application
	 * to control serialization compatibility between versions. */
	public static final long serialVersionUID = 5102006043959123451l;

	/** Class name. */
	public static final String CLAZZ = RFListModel.class.getSimpleName();
	
	// --- INSTANCE FIELDS -----------------------------------------------------
	
	/** Is the internal storage of the list model. */
	private List<E> _delegate;

	// --- CONSTRUCTOR ---------------------------------------------------------
	
	/**
	 * Creates a new RF list model. The elements are stored in a 
	 * {@code java.util.List}.
	 */
	public RFListModel()
	{
		_delegate = new ArrayList<E>();
		
	}

	// --- INSTANCE METHODS ----------------------------------------------------
	
	/* (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.view.IListModel#add(int, java.lang.Object)
	 */
	public void add(int index_, E element_)
	{
		_delegate.add(index_, element_);
		fireIntervalAdded(this, index_, index_);
	}

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.view.IListModel#addElement(java.lang.Object)
	 */
	public void addElement(E element_)
	{
		_delegate.add(element_);
		fireIntervalAdded(this, _delegate.size() - 1, _delegate.size());
	}

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.view.IListModel#clear()
	 */
	public void clear()
	{
		int size = _delegate.size();
		_delegate.clear();
		fireIntervalRemoved(this, 0, size);
	}

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.view.IListModel#contains(java.lang.Object)
	 */
	public boolean contains(E element_)
	{
		return _delegate.contains(element_);
	}

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.view.IListModel#get(int)
	 */
	public E get(int index_)
	{
		return _delegate.get(index_);
	}

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.view.IListModel#isEmpty()
	 */
	public boolean isEmpty()
	{
		return _delegate.isEmpty();
	}

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.view.IListModel#remove(java.lang.Object)
	 */
	public boolean remove(E element_)
	{
		int size = _delegate.size();
		boolean b = _delegate.remove(element_);
		if (b)
			fireIntervalRemoved(this, 0, size);
		return b;
	}

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.view.IListModel#remove(int)
	 */
	public E remove(int index_)
	{
		int size = _delegate.size();
		E element = _delegate.remove(index_);
		if (element != null)
			fireIntervalRemoved(this, 0, size);
		return element;
	}

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.accelerator.view.IListModel#size()
	 */
	public int size()
	{
		return _delegate.size();
	}

	/* (non-Javadoc)
	 * @see javax.swing.ListModel#getElementAt(int)
	 */
	public E getElementAt(int index_)
	{
		return get(index_);
	}

	/* (non-Javadoc)
	 * @see javax.swing.ListModel#getSize()
	 */
	public int getSize()
	{
		return _delegate.size();
	}
	
	/*
	 * (non-Javadoc)
	 * @see javax.swing.AbstractListModel#fireContentsChanged(java.lang.Object, int, int)
	 */
	@Override
	protected void fireContentsChanged(Object source_, int index0_, int index1_)
	{
		super.fireContentsChanged(source_, index0_, index1_);
	}
	
	/*
	 * (non-Javadoc)
	 * @see javax.swing.AbstractListModel#fireIntervalAdded(java.lang.Object, int, int)
	 */
	@Override
	protected void fireIntervalAdded(Object source_, int index0_, int index1_)
	{
		super.fireIntervalAdded(source_, index0_, index1_);
	}
	
	/*
	 * (non-Javadoc)
	 * @see javax.swing.AbstractListModel#fireIntervalRemoved(java.lang.Object, int, int)
	 */
	@Override
	protected void fireIntervalRemoved(Object source_, int index0_, int index1_)
	{
		super.fireIntervalRemoved(source_, index0_, index1_);
	}

	@Override
	public String toString()
	{
		return CLAZZ + "[size=" + this.getSize() + ']';
	}
	
}
/* end of file */