/*
 *
 * File: MCSTabbedPaneUI.java
 * Date: Jul 10, 2009; 3:27:53 PM
 */
package de.desy.mst.libs.accelerator.view.thirdParty;

import java.awt.Dimension;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Insets;
import java.awt.Rectangle;
import java.awt.RenderingHints;
import java.awt.Toolkit;

import javax.swing.Icon;
import javax.swing.JComponent;
import javax.swing.SwingUtilities;
import javax.swing.plaf.ComponentUI;
import javax.swing.plaf.basic.BasicTabbedPaneUI;

import de.desy.mst.libs.accelerator.view.CColor;

/**
 * <p>New user defined UI classes for the tabbed pane. Defines the about 90 degree
 * rotated tab entries on the left or right side of the tabbed pane.</p>
 * 
 * <p>To initialize this UI class you should set in the constructor this class
 * as the actual UI class:
 * <pre>
 * this.setUI(MCSTabbedPaneUI.createUI());
 * </pre>
 * </p>
 * 
 * <p>For more information about the here realized UI class you should have a
 * look at Sun's forums web page: 
 * <a href="http://forums.sun.com/thread.jspa?threadID=249148&forumID=57">Real Vertical Tabs on JTabbedPane! solution</a>.
 * </p>
 *
 * @author <a href="mailto:marcus.walla(at)desy.de">walla</a>
 * @version $Id: MCSTabbedPaneUI.java 34928 2010-04-11 11:47:19Z walla $
 */
public class MCSTabbedPaneUI extends BasicTabbedPaneUI 
{
	/**
	 * Returns a new instance of the {@code MCSTabbedPaneUI} class.
	 * 
	 * @param c JComponent (What's the sense of the component? Don't know so I
	 * 		deprecated this method!) 
	 * @return Returns a new instance of the {@code MCSTabbedPaneUI} class.
	 */
	@Deprecated
    public static ComponentUI createUI(final JComponent c) 
    {
        return new MCSTabbedPaneUI();
    }
	
	/**
	 * Returns a new instance of the {@code MCSTabbedPaneUI} class.
	 * 
	 * @return Returns a new instance of the {@code MCSTabbedPaneUI} class.
	 */
	public static ComponentUI createUI()
	{
        return new MCSTabbedPaneUI();
	}
 
	protected void paintText(Graphics g, int tabPlacement,
                             Font font, FontMetrics metrics, int tabIndex,
                             String title, Rectangle textRect,
                             boolean isSelected) 
	{
		Graphics2D g2d = (Graphics2D)g;
    	
		// >>> --- WALLA --- 10.07.2009 --- ADDED ------------------------------
    	// HINT: erikb2 from the Java forum mentioned in its posting that you
    	// should activate the anti aliasing of the characters to have a much
    	// nicer displaying of the look and feel. 
    	// For more information you should have a look at the following web 
    	// page: http://forums.sun.com/thread.jspa?messageID=1038918#1038918
		g2d.setRenderingHint(
    			RenderingHints.KEY_ANTIALIASING, 
    			RenderingHints.VALUE_ANTIALIAS_ON); 
		// <<< --- WALLA --- 10.07.2009 --- ADDED ------------------------------

		// >>> --- WALLA --- 10.07.2009 --- ADDED ------------------------------
		// ATTENTION: 
		textRect.x += 8;
		// <<< --- WALLA --- 10.07.2009 --- ADDED ------------------------------
		
		java.awt.geom.AffineTransform tt = g2d.getTransform();
        g2d.setFont(font);
		g2d.rotate(-Math.PI / 2.0);
 
		/*	
		View v;
		if (htmlViews != null 
			&& (v = (View)htmlViews.elementAt(tabIndex)) != null) 
		{
			    v.paint(g, textRect);
		} 
		else 
		{
		*/
		
	    if (tabPane.isEnabled() && tabPane.isEnabledAt(tabIndex)) 
	    {
	    	// tab enabled
			
	    	g2d.setColor(tabPane.getForegroundAt(tabIndex));
	    	//g.drawString(title, textRect.x,textRect.y + metrics.getAscent());
	    	//g.drawString(title, -(int)((textRect.x + textRect.height )),textRect.x+metrics.getAscent());
			g.drawString(title, -(textRect.y + textRect.height)+10,textRect.x+metrics.getAscent());
	    } 
	    else 
	    { 
	    	// tab disabled
			g.setColor(tabPane.getBackgroundAt(tabIndex).brighter());
			//g.drawString(title, textRect.x, textRect.y + metrics.getAscent());
			g.drawString(title, -(textRect.y + SwingUtilities.computeStringWidth(metrics, title))+textRect.height ,textRect.x+metrics.getAscent());
			g.setColor(tabPane.getBackgroundAt(tabIndex).darker());
			//g.drawString(title, textRect.x - 1, textRect.y + metrics.getAscent() - 1);
			g.drawString(title, -(textRect.y + SwingUtilities.computeStringWidth(metrics, title)-1)+textRect.height ,textRect.x+metrics.getAscent()-1);
	    }
	    
		g2d.setTransform( tt ) ;
    }
	
	protected void paintIcon(
			final Graphics g,
			final int tabPlacement,
			final int tabIndex,
			final Icon icon,
			final Rectangle iconRect,
			boolean isSelected)
	{
        if (icon != null) 
        {
        	// >>> --- WALLA --- 13.07.2009 --- ADDED --------------------------
        	iconRect.x += 6;
        	iconRect.y -= 44;
        	// <<< --- WALLA --- 13.07.2009 --- ADDED --------------------------

            icon.paintIcon(tabPane, g, iconRect.x, iconRect.y);
        }
	}
	
	protected void paintTabBackground(
			final Graphics g,
			final int tabPlacement,
			final int tabIndex,
			final int x,
			final int y,
			final int w,
			final int h,
			final boolean isSelected)
	{
		// >>> WALLA --- 13.07.2009 --- CHANGED --------------------------------
        g.setColor(!isSelected ? 
                CColor.GREY9 : CColor.LIGHT_BLUE);
		// <<< WALLA --- 13.07.2009 --- CHANGED --------------------------------

        switch(tabPlacement) {
        case LEFT:
        	g.fillRect(x+1, y+1, w-1, h-3);
        	break;
        case RIGHT:
        	g.fillRect(x, y+1, w-2, h-3);
        	break;
        case BOTTOM:
        	g.fillRect(x+1, y, w-3, h-1);
        	break;
        case TOP:
        default:
        	g.fillRect(x+1, y+1, w-3, h-1);
        }
	}
	
    protected int calculateTabHeight(int tabPlacement, int tabIndex, int fontHeight) 
    {
        Icon icon = getIconForTab(tabIndex);
        FontMetrics metrics = getFontMetrics();
        Insets tabInsets = getTabInsets(tabPlacement, tabIndex);
		int width = 0;
 
        if (icon != null) 
        {
            width += icon.getIconHeight() + textIconGap;
        }
        
        /*
        View v;

		if ((htmlViews!=null) &&
	    	((v = (View)htmlViews.elementAt(tabIndex)) != null)) {
	    	width += (int)v.getPreferredSpan(View.X_AXIS);
		} 
		else 
		{
		*/
	    String title = tabPane.getTitleAt(tabIndex);
	    width += SwingUtilities.computeStringWidth(metrics, title);
	    //}
	    
	    Dimension d = Toolkit.getDefaultToolkit().getScreenSize();
		float ratio = (float)d.width / d.height;
		int height = (int)(ratio * width);
		height += tabInsets.top + tabInsets.bottom + 2;
        return height;
    }
    
    protected int calculateTabWidth(int tabPlacement, int tabIndex, FontMetrics metrics) 
    {
    	int height = 0;
		//	View v;
		//	if ( (htmlViews!=null) &&
		//	     ((v = (View)htmlViews.elementAt(tabIndex)) != null)) {
		//	    height += (int)v.getPreferredSpan(View.Y_AXIS);
		//	} else {
	    height += metrics.getHeight();
	    //	}
	    
        Icon icon = getIconForTab(tabIndex);
        Insets tabInsets = getTabInsets(tabPlacement, tabIndex);
 
        if (icon != null) {
            height = Math.max(height, icon.getIconWidth());
        }
	    Dimension d = Toolkit.getDefaultToolkit().getScreenSize();
		float ratio = (float)d.height / d.width;
		int width = (int)(ratio * height);
 
        width += tabInsets.left + tabInsets.right + 3;
        return width;
    }
    
    protected void layoutLabel(int tabPlacement,
                               FontMetrics metrics, int tabIndex,
                               String title, Icon icon,
                               Rectangle tabRect, Rectangle iconRect,
                               Rectangle textRect, boolean isSelected ) 
    {
        textRect.x = 0;
        textRect.y = 0;
        iconRect.x = 0; 
        iconRect.y = 0;
        
        SwingUtilities.layoutCompoundLabel(tabPane,
                                           metrics, title, icon,
                                           SwingUtilities.CENTER,
                                           SwingUtilities.CENTER,
                                           SwingUtilities.CENTER,
                                           SwingUtilities.TRAILING,
                                           tabRect,
                                           iconRect,
                                           textRect,
                                           textIconGap);
 
        int xNudge = getTabLabelShiftX(tabPlacement, tabIndex, isSelected);
        int yNudge = getTabLabelShiftY(tabPlacement, tabIndex, isSelected);
        iconRect.x += xNudge;
        iconRect.y += yNudge;
 
		textRect.setBounds(tabRect); //should add some insets here to make text rect sit inside tab rect
        textRect.x += xNudge;
        textRect.y += yNudge;
    }
}
/* end of file */