/**
 * Jan 29, 2008 - Jan 30, 2008	ws:	Creation from scratch
 * Jan 13, 2009 - Jan 13, 2009	ws:	added getHelpMgr
 */
package de.desy.mst.libs.framework.mstapp;

import de.desy.mst.libs.framework.mstapp.hilfe.HelpMgr;
import de.desy.mst.libs.framework.mstapp.menu.IPopupMenuMgr;
import de.desy.mst.libs.framework.mstapp.menu.PopUpMenuMgr;
import de.desy.mst.libs.framework.mstapp.mode.MaschineMgr;

/**
 * IMstAppKernel bevor aufwendige Berechnungen gemacht werden. MstAppKernel
 * ist ein MstApp Surrogat bis MstApp existiert.
 * danach wird auf diesen Kernel setMstApp(existentes MstApp) aufgerufen um
 * insbesondere den guten PopUpManager zu nutzen
 * @author ws
 * @version 1.0.0
 */
class MstAppKernel implements IMstAppKernel {
	private final ParamDecoder paramDecoder;
	private IMstAppKernel realMstApp = null;			//synch on this
	private IPopupMenuMgr popUpMenuMgr = null;	//synch on this
//=============================================================================
	/**
	 * @param  paramDecoder not null
	 * @throws NullPointerException
	 */
	public MstAppKernel(final ParamDecoder paramDecoder) {
		super();
		if(paramDecoder == null)
			throw new NullPointerException("ParamDecoder darf nicht null sein.");
		this.paramDecoder = paramDecoder;
	}
//=============================================================================
	/* (non-Javadoc)
	 * @see de.desy.mst.libs.framework.mstapp.IMstAppKernel#getMaschineMgr()
	 */
	@Override synchronized public MaschineMgr getMaschineMgr() {
		return this.paramDecoder.getMaschineMgr();
	}

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.framework.mstapp.IMstAppKernel#getPopUpMenuMgr()
	 */
	@Override synchronized public IPopupMenuMgr getPopUpMenuMgr() {
		if(hasMstApp())
			return getMstApp().getPopUpMenuMgr();

		if(this.popUpMenuMgr == null)
			 this.popUpMenuMgr = new PopUpMenuMgr();
		return this.popUpMenuMgr ;
	}

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.framework.mstapp.IMstAppKernel#getStartParameter()
	 */
	@Override synchronized public ParamDecoder getStartParameter() {
		return this.paramDecoder;
	}
	/* (non-Javadoc)
	 * @see de.desy.mst.libs.framework.mstapp.IMstAppKernel#getHelpMgr()
	 */
	@Override public synchronized HelpMgr getHelpMgr() {
		if(hasMstApp())
			return getMstApp().getHelpMgr();
		return new HelpMgr(getStartParameter().applicationInfo);//selten abgefragt
	}

	synchronized IMstAppKernel getMstApp() 	{return this.realMstApp;}
	synchronized boolean hasMstApp()		{return getMstApp() != null;}
	synchronized void setMstApp(final IMstAppKernel mstApp){
		if(mstApp.getStartParameter() != this.getStartParameter()) {
			final String msg = "mstApp muss den gleichen Startparameter"
							 + " (Paramdecoder) haben.";
			throw new IllegalArgumentException(msg);
		}
		this.realMstApp = mstApp;
		this.popUpMenuMgr = null;	//not needed anymore
	}
}
