 /**
 * Jan  6, 2009 - Jan  7, 2009	ws:	Creation from scratch/DialogShowingURI
 * Apr 21, 2010 - Apr 28, 2010	ws:	now show KiHi vorausgefuellt in createButtonLogbookPrinter
 */
package de.desy.mst.libs.framework.mstapp;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Container;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseListener;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.ScrollPaneConstants;

import de.desy.mst.libs.framework.mstapp.io.Mail;
import de.desy.mst.libs.framework.mstapp.io.MstPrintForm;
import de.desy.mst.libs.framework.mstapp.io.PrintDialogForm;
import de.desy.mst.libs.framework.mstapp.logging.gui.ViewerDialog;
import de.desy.mst.libs.framework.mstapp.util.gui.JDialogWithCloseOnEscape;

/**
 * Replaces the simplistic JOptionPane MsgBox to show the users an unrecoverable
 * crash. (Is not shown for servers.) Some customized actions might be taken
 * @author ws
 * @version 1.0.0
 */
class UncaughtExceptionDialog extends JDialogWithCloseOnEscape {
	private static final long serialVersionUID = 1L;
//=============================================================================
	private final JTextArea textArea;
//=============================================================================
	public UncaughtExceptionDialog(final IMstAppKernel mstAppK, final JFrame owner, final String title, final String msg) {
		super(owner, title, /*modal = */ true);
		this.textArea = new JTextArea();
		initialize(mstAppK, msg);
	}
//=============================================================================
	private void initialize(final IMstAppKernel mstAppK, final String msg) {
		final MouseListener standardPopUpMouseListener = mstAppK.getPopUpMenuMgr().getStandardPopUpMouseListener();
		this.addMouseListener(standardPopUpMouseListener);

		final Container pane = getRootPane();
		pane.setLayout(new BorderLayout());

		pane.add(createContent(msg, standardPopUpMouseListener), BorderLayout.CENTER);
		pane.add(createButtonPanel(mstAppK), BorderLayout.SOUTH);

		this.pack();
		Main.centerToMstFrameMain(this);
	}
	private Component createContent(final String msg, final MouseListener standardPopUpMouseListener) {
		final JTextArea textArea = getTextArea();
		textArea.setText(msg);
		textArea.setEditable(false);
		textArea.addMouseListener(standardPopUpMouseListener);

		final JScrollPane scrollPane = new JScrollPane(textArea, ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED, ScrollPaneConstants.HORIZONTAL_SCROLLBAR_AS_NEEDED);

		final JPanel panel = new JPanel(new BorderLayout());
		panel.add(scrollPane, BorderLayout.CENTER);
		return panel;
	}
	private Component createButtonPanel(final IMstAppKernel mstAppK) {
		final JPanel panel = new JPanel();
		panel.setLayout(new BoxLayout(panel, BoxLayout.X_AXIS));
		panel.setAlignmentX(Component.RIGHT_ALIGNMENT);

		final boolean showInBrowser = false;

		panel.add(createButtonDefaultPrinter());
		panel.add(createButtonSendMail());
		panel.add(createButtonLogbookPrinter(mstAppK));
		panel.add(Box.createHorizontalStrut(50));

		panel.add(Box.createHorizontalGlue());

		panel.add(createButtonShowProtokoll(mstAppK));
		panel.add(createButtonShowAbout(mstAppK, showInBrowser));
		panel.add(Box.createHorizontalStrut(50));
		panel.add(createButtonOK());

		return panel;
	}
	private JButton createButtonShowAbout(final IMstAppKernel mstAppK, final boolean showInBrowser) {
		final boolean modal = true;
		return createButton_FromMenu(mstAppK.getHelpMgr().createHilfePfadMenuesAbout(showInBrowser, modal));
	}
	private JButton createButtonDefaultPrinter() {
		return createButton_FromMenu(MstPrintForm.getMenuPrintDefaultPrinter(this));
	}
	private JButton createButtonLogbookPrinter(final IMstAppKernel mstAppK) {
		final JButton button = new JButton("Drucken/Benachrichtigen ...");
		button.setToolTipText("Der Drucken/Benachrichtigen Dialog wird angezeigt.");
		button.addActionListener(new ActionListener(){
			@Override public void actionPerformed(final ActionEvent evt) {
				final boolean modal = true;
				final boolean errorInfo = modal;
				new PrintDialogForm(UncaughtExceptionDialog.this, modal)
							.showdialog(getTitle(), getMsgBody(), errorInfo);
			}
		});
		return button;
	}
	private JButton createButton_FromMenu(final JMenuItem menu) {
		final JButton button = new JButton(menu.getText());
		button.setToolTipText(menu.getToolTipText());
		final ActionListener[] listeners = menu.getActionListeners();
		if(listeners.length > 0)
			button.addActionListener(listeners[0]);
		button.setEnabled(menu.isEnabled() && listeners.length > 0);
		return button;
	}
	private JButton createButtonSendMail() {
		final JButton button = new JButton("     eMail     ");
		button.setToolTipText("eMail an den Entwickler schicken");
		button.setMnemonic('e');
		button.setAlignmentX(Component.RIGHT_ALIGNMENT);
		button.addActionListener(new ActionListener(){
			@Override public void actionPerformed(final ActionEvent e) {
				sendMail(getTitle(), getMsgBody());
			}
		});
		button.requestFocusInWindow();
		return button;
	}
	private JButton createButtonOK() {
		final JButton button = new JButton("Schliessen");
		button.setToolTipText("Schliessen und damit die Applikation beenden. (Ctrl + Escape)");
		button.setMnemonic('S');
		button.setAlignmentX(Component.RIGHT_ALIGNMENT);
		button.addActionListener(new ActionListener(){
			@Override public void actionPerformed(final ActionEvent e) {
				UncaughtExceptionDialog.this.dispose();
			}
		});
		button.requestFocusInWindow();
		return button;
	}
	private JButton createButtonShowProtokoll(final IMstAppKernel mstAppK) {
		final JButton button = new JButton("Protokoll anzeigen");
		button.setMnemonic('o');
		button.setAlignmentX(Component.RIGHT_ALIGNMENT);
		button.addActionListener(new ActionListener(){
			@Override public void actionPerformed(final java.awt.event.ActionEvent e) {
				final boolean modal = true;
				ViewerDialog.createGeneralLoggingDialog(UncaughtExceptionDialog.this, mstAppK, modal)
				            .setVisible(true);
			}
		});
		return button;
	}
	private void sendMail(final String title, final String msg) {
		final String entryAutor = "";
		final String[] to   = null;
		final String   from = null;
		new Mail().sendMail(title, msg, entryAutor, to, from);
	}
	private final JTextArea getTextArea() {return this.textArea;}
	private String getMsgBody() {return getTextArea().getText();}
}