/**
 *
 * Last changed: 2010-4-22
 * By: ws added @Override's
 * URL: $HeadURL$
 * $Revision$
 *
 */
package de.desy.mst.libs.framework.mstapp.appinf.server;


import java.io.IOException;

import de.desy.mst.libs.framework.mstapp.appinf.AApplicationInterfaceFactory;
import de.desy.mst.libs.framework.mstapp.appinf.IApp;
import de.desy.mst.libs.framework.mstapp.logging.Logger;

/**
 * @author labudda
 *
 */
public final class ServerFactory extends AApplicationInterfaceFactory {
	private final static java.util.concurrent.atomic.AtomicReference<IApp> appRef = new java.util.concurrent.atomic.AtomicReference<IApp>(null);

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.IApplikationInterfaceFactory#getApplicationInformation()
	 */
	@Override public IApp getApplicationInformation() throws IllegalStateException {
		final java.util.jar.JarFile file = ServerFactory.getMainClassJarFile();
		return this.getApplicationInformation(file);
	}

	/**
	 * @param mainClassJarFile
	 * @return interface to get applications informations
	 * @throws IllegalStateException
	 */
	public IApp getApplicationInformation(final java.util.jar.JarFile mainClassJarFile) throws IllegalStateException {
		if (this.isValidKind()) {
			System.out.println("detected serverapp");
			synchronized (ServerFactory.class) {
				if (ServerFactory.appRef.get() == null) {
					try {
						ServerFactory.appRef.set(new ServerApp(mainClassJarFile));
					} catch (final IOException e) {
						throw new IllegalStateException("Manifest not found", e);
					}

				} else {
					// already done
				}
			}
			return ServerFactory.appRef.get();
		} else {
			throw new IllegalStateException("This is not a server applikation");
		}
	}

	/**
	 * @return first element of classpath if it's a jar file. Else: exception
	 * @throws IllegalStateException
	 */
	private static java.util.jar.JarFile getMainClassJarFile() throws IllegalStateException {
		final String classpath = System.getProperty("java.class.path", null);
		if (classpath == null) {
			throw new IllegalStateException("Classpath not found");
		} else {
			final String mainElement = classpath.split(System.getProperty("path.separator"))[0];
			if (mainElement.toLowerCase().endsWith(".jar")) {
				try {
					return new java.util.jar.JarFile(mainElement);
				} catch (final IOException e) {
					throw new IllegalStateException("Jar not found", e);
				}
			} else {
				throw new IllegalStateException("Not a jar file");
			}
		}
	}


	/**
	 * @return first element of classpath if it's a jar file. Else: exception
	 * @throws IllegalStateException
	 */
	/**
	 * @param mainClassName als Name der Klasse, die vom MstApp als Main ubergeben wird
	 * Hier wird der Name als Parameter uebergeben, damit die vorherige Funktionalitaet 
	 * als deprecated fallback genutzt werden kann
	 * @return mains jar file containing manifest in caser of server
	 * @throws IllegalStateException
	 * 
	 * Ist so leider nicht moeglich, weil Main auf diese Klasse zugreift 
	 * bevor getAppName definiert ist 
	 * 
	 */
/*	private static java.util.jar.JarFile getMainClassJarFile() throws IllegalStateException {
		try {
			final Class<?> mainClass = Class.forName(Main.getAppName());
			URL location = mainClass.getResource('/' + mainClass.getName().replace('.', '/') + ".class");
			final java.util.jar.JarFile jar = new java.util.jar.JarFile(new java.io.File(location.toURI()));
			Logger.SWFW.debug(jar.getName() +  " successfully detect as main jar file");
			return jar;
		} catch (ClassNotFoundException | IOException | URISyntaxException e) {
			e.printStackTrace();
			return ServerFactory.getMainClassJarFile();
		}
	}
*/
	/* (non-Javadoc)
	 * @see de.desy.mst.libs.framework.mstapp.appinf.AApplicationInterfaceFactory#areKindSpecificRequirementsFulfilled()
	 */
	@Override protected boolean areKindSpecificRequirementsFulfilled(){
		
		try {
			java.util.jar.JarFile jar = ServerFactory.getMainClassJarFile();
			return jar.getManifest() != null;
		} catch (IOException e) {
			Logger.SWFW.info("error accessing jar file while checking for server", e);
			return false;
		} catch (IllegalStateException e) {
			// no jar, no server
			return false;
		}
	}
}
