/*
 * Created on 	Jun 14, 2005 - Jun 14, 2005
 * Aug 15, 2005
 * Nov  7, 2006 - Nov  9, 2006	ws: added iniFileUIMgrUserPref ...
 * ws: Jul  4, 2007 - Jul  4, 2007	added: doPrefInit(...)
 * Aug 29, 2007 - Aug 29, 2007 	ws: String appName -> ApplicationInfo appInfo
 * May 15, 2008 - May 15, 2008	ws: added: timeOut
 * Aug 01, 2015 - Aug 01, 2015	ws:	readAll() enforce app reading (wst needs it)
 */
package de.desy.mst.libs.framework.mstapp.ini;

import javax.swing.JMenu;

import de.desy.mst.libs.framework.mstapp.ApplicationInfo;
import de.desy.mst.libs.framework.mstapp.Main;
import de.desy.mst.libs.framework.mstapp.logging.Logger;
import de.desy.mst.libs.framework.mstapp.messages.ISetMessage;
import de.desy.mst.libs.framework.mstapp.preferences.IPreferences;
import de.desy.mst.libs.framework.mstapp.preferences.implementation.PrefManager;

/**
 * Manages Initialisation Files
 * etc.
 *
 * @author ws
 * @version 1.0.0
 */
public class MasterMgrForIniFiles {
    static final long TIMEOUT_READ_SYSTEM_MS = 20000L;	//20sek
	private static final String MENUOPTIONS_NULL
    				= "The menuOptions may not be null.";
    private static final String BAD_INIT_JMENU_STATE
    				= "You tried to initAndRegisterAppKonfig after it already "
    				  + "has been initialised.";
    private static final String MENU_SETTINGS_TEXT = "Einstellungen";

    private final ISetMessage  messageRcv;
    private final IniFileUIMgr iniFileUIMgrSystem ;
    private final IniFileUIMgr iniFileUIMgrApplication;
    private final IniFileUIMgr iniFileUIMgrUser;
    private final IniFileUIMgr iniFileUIMgrUserPref;
    private final PrefStorage prefStorage;

    private final boolean aMenuWasInitialised = false;

//=============================================================================

    public MasterMgrForIniFiles(final ApplicationInfo appInfo,final ISetMessage  messageReceiver) {
        super();
        if(appInfo         == null){throw new NullPointerException("The appName may not be null.");}
        if(messageReceiver == null){throw new NullPointerException("ISetMessage may not be null.");}

        this.messageRcv = messageReceiver;

        this.iniFileUIMgrSystem 		= createFileUIMgr(new BaseDataForSystem());
        this.iniFileUIMgrApplication = createFileUIMgr(new BaseDataForAnApplication(appInfo));
        this.iniFileUIMgrUser 		= createFileUIMgr(new BaseDataForTheUser(appInfo));
		final IniFileIOMgr iniFileIOMgrUserPref = new IniFileIOMgr
				(new BaseDataForTheUserPreferences(appInfo), messageReceiver);
        this.prefStorage		= new PrefStorage(appInfo, iniFileIOMgrUserPref);
		this.iniFileUIMgrUserPref	= new IniFileUIMgr(iniFileIOMgrUserPref);
    }

//=============================================================================

	public final IIniFileObserverRegistrar getSystemRegistrar()     {return this.iniFileUIMgrSystem;}
    public final IIniFileObserverRegistrar getApplicationRegistrar(){return this.iniFileUIMgrApplication;}
    public final IIniFileObserverRegistrar getUserRegistrar()       {return this.iniFileUIMgrUser;}
    public final IIniFileObserverRegistrar getUserPrefRegistrar()   {return this.iniFileUIMgrUserPref;}
	public final PrefStorage getPrefStorage() 						{return this.prefStorage;}

//=============================================================================

    /**
     * @param   menuOptions != null
     * @throws  NullPointerException,  if menuOptions == null
     * @throws  IllegalStateException, if called twice
     */
    public final void initAndRegisterJMenuForIniFiles(final JMenu menuOptions) {
        if(this.aMenuWasInitialised){throw new IllegalStateException(BAD_INIT_JMENU_STATE);}
        if(menuOptions == null){throw new NullPointerException(MENUOPTIONS_NULL);}

        final JMenu menuSettings = new JMenu(MENU_SETTINGS_TEXT);
        menuOptions.add(menuSettings);
        this.iniFileUIMgrUser.initAndRegisterJMenu(menuSettings);
        this.iniFileUIMgrApplication.initAndRegisterJMenu(menuSettings);
        this.iniFileUIMgrSystem.initAndRegisterJMenu(menuSettings);
    } 
    public void readAll() {
    	if(Main.mstwebUsageSettings.isUse_mstweb4ini()) {
	        getIniFileUIMgrSystem().read(TIMEOUT_READ_SYSTEM_MS);//erster lang
	        getIniFileUIMgrApplication().read();				 //hatten schon Zeit
	        getIniFileUIMgrUser().read();
	        //getIniFileUIMgrUserPref().read();					//in XTor PrefStorage
    	} else {
	        getIniFileUIMgrApplication().read(TIMEOUT_READ_SYSTEM_MS);//Wst still needs it
    		final String msg = this.getClass().getSimpleName() + ".readAll() ignored except application"
    				         + ", since Main.mstwebUsageSettings.isUse_mstweb4ini() == "
    				         + Main.mstwebUsageSettings.isUse_mstweb4ini();
			Logger.SWFW_.info(msg);
    	}
    }
	public void doPrefInit(final PrefManager pm, final IPreferences einstellungen) {
        pm.register(new PrefIniDateien(einstellungen, this.iniFileUIMgrApplication.getIniFileIOMgr()));
        pm.register(new PrefIniDateien(einstellungen, this.iniFileUIMgrUser.getIniFileIOMgr()));
        pm.register(new PrefIniDateien(einstellungen, this.iniFileUIMgrSystem.getIniFileIOMgr()));
	}

//=============================================================================

    private final IniFileUIMgr createFileUIMgr(final ABaseDataOfMstIni baseData)
    {
        final IniFileIOMgr iniFileIOMgr = new IniFileIOMgr(baseData, this.messageRcv);
        return new IniFileUIMgr(iniFileIOMgr);
    }
 
    private final IniFileUIMgr getIniFileUIMgrSystem()     {return this.iniFileUIMgrSystem;}
    private final IniFileUIMgr getIniFileUIMgrApplication(){return this.iniFileUIMgrApplication;}
    private final IniFileUIMgr getIniFileUIMgrUser()       {return this.iniFileUIMgrUser;}
    //private final IniFileUIMgr getIniFileUIMgrUserPref()   {return iniFileUIMgrUserPref;}
}
