/*
 * Created on 	Mar  9, 2005 - Jun 27, 2005
 * 				Aug 12, 2005 - Aug 17, 2005 ... implements IIniFileRoot, XTor via ABaseDataOfMstIni
 * 				Oct 25, 2005 - ...	new logging system
 * 	Aug  6, 2009 - Aug  6, 2009	ws:	Eclipse asked for removing unnessary casts, so I did
 */
package de.desy.mst.libs.framework.mstapp.ini;

import de.desy.mst.libs.framework.mstapp.logging.Logger;
import de.desy.mst.libs.framework.mstapp.tabellendaten.TabellenDatenIni;

/**
 * Root Klasse des Initialisierungsbaumes <br>
 * Anwendungsprogrammierer setzen hier ihren eignen Baum rein. <br>
 * Ferner home fuer alle strukturell generellen datenmaesig spezifischen MstApp
 * Ini Daten.
 * 
 * @author ws
 * @version 1.0.0
 */
public class MstIni 
       implements IIniFileObserver , IIniFileRoot{ 
	/**Metadata for this ini file*/
	private FileDataIni fileData;
    /**Welche Tabellen Daten sollen geladen werden.*/
    private TabellenDatenIni tabellenDatenIni;
	/**Data for the derived (1) applications, (2)the system or (3) the current user*/
	private Object contentRoot;

//=============================================================================	
    
    /**
     * @param  baseDataOfMstIni
    /**@throws NullpointerException*/
    public MstIni(final ABaseDataOfMstIni baseDataOfMstIni){
    	super();
        this.fileData = new FileDataIni(1L, baseDataOfMstIni);
    }
    /**
     * @deprecated DO NOT USE, it is only for the XMLDecoder.  
     */
    @Deprecated public MstIni() {
        super();
    }

//=============================================================================	
		

    /* (non-Javadoc)
     * @see de.desy.mst.mstapp.ini.IIniFileObserver#getDefaultIniBaumRoot()
     */
    @Override public Object getDefaultIniBaumRoot() {
        return null;
    }
	/* (non-Javadoc)
	 * @see de.desy.mst.mstapp.ini.IPrepareForExport#onPrepareForExport()
	 */
    @Override public Object onPrepareForExport() {
		delegatePrepareTo(this.fileData);
		if(this.contentRoot instanceof IIniFileObserver){
            try {
    		    delegatePrepareTo((IIniFileObserver)this.contentRoot);//just to support old internal iinifileobserver
			} catch (final Exception e) {
				Logger.SWFW.warn("Unexpected application error on onPrepareForExport.",e);
			}            
        }
        return this;
	}
    /* (non-Javadoc)
     * @see de.desy.mst.mstapp.ini.IIniFileObserver#onNewIniAvailable(java.lang.Object)
     */
    @Override public void onNewIniAvailable(final Object iniBaumRoot) {
        //polymorphism does not help due to cast of the caller to IIniFileObserver 
        if(iniBaumRoot instanceof MstIni){
            onNewMstIniAvailable((MstIni) iniBaumRoot);
        }else if(iniBaumRoot instanceof FileDataIni){
            onNewFileDataIniAvailable((FileDataIni) iniBaumRoot);
        }else{
            onNewContentRootAvailable(iniBaumRoot);
        } 
    }

//-----------------------------------------------------------------------------	
	
	/* (non-Javadoc)
	 * @see de.desy.mst.mstapp.ini.IIniFileRoot#getContentRoot()
	 */
    @Override public final Object getContentRoot() {
		return this.contentRoot;
	}
	/* (non-Javadoc)
	 * @see de.desy.mst.mstapp.ini.IIniFileRoot#setContentRoot(java.lang.Object)
	 */
    @Override public final void setContentRoot(final Object contentRoot) {
		this.contentRoot = contentRoot;
	}
    
	public final FileDataIni getFileData() {
		return this.fileData;
	}
	public final void setFileData(final FileDataIni fileData) {
		this.fileData = fileData;
	}
    public final TabellenDatenIni getTabellenDatenIni() {
        return this.tabellenDatenIni;
    }
    public final void setTabellenDatenIni(final TabellenDatenIni tabellenDatenIni) {
        this.tabellenDatenIni = tabellenDatenIni;
    }
	
    public boolean inSeinenWertenGleich(final MstIni otherMstIni) {
    	if(this == otherMstIni)	{return true;}
    	if(otherMstIni == null)	{return false;}
    	if(!insSeinenWertenGleich(getTabellenDatenIni(),otherMstIni.getTabellenDatenIni()))
    							{return false;}
    	if(!insSeinenWertenGleich(getFileData(), otherMstIni.getFileData()))
    							{return false;}
    	if(!insSeinenWertenGleich(getContentRoot(), otherMstIni.getContentRoot()))
    							{return false;}
    	return true;
    }

//=============================================================================	
	
    /*checks for null and delegates the check for export */
    private void delegatePrepareTo(final IIniFileObserver forExport) {
        try {
            if(forExport != null){forExport.onPrepareForExport();}
        } catch (final Exception e) {
            Logger.SWFW.warn("Probleme mit dem Aufruf von onPrepareForExport().", e);
        } 
    }

    /**not null*/
    private void onNewMstIniAvailable(final MstIni mstIni) {
        onNewFileDataIniAvailable(mstIni.getFileData());
        onNewContentRootAvailable(mstIni.getContentRoot());
    }
    /**not null*/
    private void onNewFileDataIniAvailable(final FileDataIni fileDataIni) {
        setFileData(fileDataIni);
        fileDataIni.onNewIniAvailable(fileDataIni);
    }
    /**maybe null*/
    private void onNewContentRootAvailable(final Object contentRoot) {
        setContentRoot(contentRoot);
        if(contentRoot == null) {return;}
        
        if(contentRoot instanceof MstIni)    {throw new AssertionError("Can not be MstIni due to less general polymorphic function.");} 
        if(contentRoot instanceof FileDataIni)  {throw new AssertionError("Can not be FileDataIni due to less general polymorphic function.");}
        if(contentRoot instanceof IIniFileObserver){
            try {
                ((IIniFileObserver) contentRoot).onNewIniAvailable(contentRoot);
            } catch (final Exception e) {
				Logger.SWFW.error("Unexpected application error on fireOnNewIniAvailable.", e);
            }
        }
    }

//=============================================================================	
	
    private boolean insSeinenWertenGleich(final Object lhs, final Object rhs) {
    	if(lhs == rhs)	{return true;}
    	if(lhs == null)	{return false;}
    	return lhs.equals(rhs);
    }
    private boolean insSeinenWertenGleich(final FileDataIni lhs, final FileDataIni rhs) {
    	if(lhs == rhs)	{return true;}
    	if(lhs == null)	{return false;}
    	return lhs.insSeinenWertenGleich(rhs);
    }
}
