/**
 * May  8, 2007 - May 10, 2007	ws:	creation from scratch
 * Sep  9, 2008 - Sep  9, 2008	ws:	support extra Protokoll Cache
 * Sep 24, 2008 - Sep 30, 2008	ws: support fuer abweichendes Protokoll
 * Jan  5, 2010 - Jan  5, 2010	ws:	fixed in createJBOeffnen() callBack
 */
package de.desy.mst.libs.framework.mstapp.ini;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Desktop;
import java.awt.Dimension;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.io.File;
import java.util.Arrays;
import java.util.List;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.SwingConstants;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;

import de.desy.mst.libs.framework.mstapp.logging.Logger;
import de.desy.mst.libs.framework.mstapp.preferences.IPrefContainer;
import de.desy.mst.libs.framework.mstapp.preferences.IPreferences;
import de.desy.mst.libs.framework.mstapp.preferences.helper.APrefClientPanelEinfach;
import de.desy.mst.libs.framework.mstapp.preferences.implementation.PrefContainer;
import de.desy.mst.libs.framework.mstapp.preferences.implementation.PrefManager;
import de.desy.mst.libs.framework.mstapp.util.gui.GUIFactory;

/**
 * Einstellen der lokalen Cache Optionen (Grafischer Teil)
 * @author ws
 * version 1.0.0
 */
public class PrefCachePanel extends APrefClientPanelEinfach {
	/**pp*/ static final String GESPERRT = "###GESPERRT###";
	private static final long serialVersionUID = 1L;
//-----------------------------------------------------------------------------
	/**Puts two Components in BorderLayouted Panel as name suggestes, may be null*/
	static JPanel createWestCenterEastPanel(final Component west, final Component center, final Component east) {
		final JPanel panel = new JPanel(new BorderLayout());
		if(west != null)
			panel.add(west, BorderLayout.WEST);
		if(center != null)
			panel.add(center, BorderLayout.CENTER);
		if(east != null)
			panel.add(east, BorderLayout.EAST);
		return panel;
	}
//=============================================================================

    private final JButton jBOeffnen;
    private final JCheckBox jCBIsChecked;
    private final JLabel jLMessages;
    private final JTextField jTFVerzeichnis;

    private final List<JComponent> followersOfCachingErlaubt;
	private final PrefManager prefManager;
	private final IPreferences prefAbweichendesProtokoll;
//=============================================================================
	@Deprecated /**only for visual editor*/
	public PrefCachePanel() {this(new PrefContainer(null), null, null);}
	//-------------------------------------------------------------------------
	public PrefCachePanel(final IPrefContainer prefContainer
						, final PrefManager    prefManager
						, final IPreferences   prefAbweichendesProtokoll) {
		super(prefContainer);
		this.prefManager = prefManager;
		this.prefAbweichendesProtokoll = prefAbweichendesProtokoll;

		this.setLayout(new BorderLayout());
		final JPanel panelOuter = new JPanel(new BorderLayout());
		this.add(panelOuter, BorderLayout.NORTH);
		panelOuter.add(new JLabel("   "), BorderLayout.WEST);
		panelOuter.add(new JLabel("   "), BorderLayout.EAST);

		final JPanel panel = new JPanel();
		panelOuter.add(panel, BorderLayout.CENTER);
		panel.setLayout(new BoxLayout(panel, BoxLayout.Y_AXIS));

		panel.add(createWestEastPanel(createJLBasisVerzeichnis(), null));
		panel.add(Box.createVerticalStrut(20));

		this.jTFVerzeichnis = createTFVerzeichnisEditierbar();
		panel.add(PrefCachePanel.createWestCenterEastPanel(null, this.jTFVerzeichnis, createTFVerzeichnisReadOnly()));
		panel.add(Box.createVerticalStrut(10));

		final JButton jBDurchsuchen = createJBDurchsuchen();;
		this.jBOeffnen = createJBOeffnen();
        panel.add(createWestEastPanel(jBDurchsuchen, this.jBOeffnen));
		panel.add(Box.createVerticalStrut(10));

        this.jCBIsChecked = createCBCachingErlaubt();
        panel.add(createWestEastPanel(this.jCBIsChecked, null));
		panel.add(Box.createVerticalStrut(10));

        this.jLMessages = createJLMessages();
        panel.add(this.jLMessages);

        this.followersOfCachingErlaubt = createList(this.jTFVerzeichnis
    											  , jBDurchsuchen
    											  , this.jBOeffnen);
		initFromSystem();
	}
	private JTextField createTFVerzeichnisReadOnly() {
		final JTextField textField = new JTextField(getTFVReadOnlyText());
		textField.setEnabled(false);
		return textField;
	}
	private JLabel createJLBasisVerzeichnis() {
		final JLabel label = new JLabel();
		label.setFont(GUIFactory.createHeaderFont());
		label.setText(getLabelText());
		label.setToolTipText(getLabelTTText());
		label.getAccessibleContext().setAccessibleName(getLabelText());
		return label;
	}
	private String getLabelText() {return "Basis-Verzeichnis";}
	/**pp*/ String getTFVReadOnlyText() {return File.separator + LocalHome.HOME_DIR_NAME + File.separator;}
	/**pp*/ String getLabelTTText() {
		return "Unter welchem Verzeichnis ist der lokale Cache angelegt (PC Weit und User spezifisch). Bitte beachten: die Applikationen braucht Schreibrechte.";
	}
	/**pp*/ String getCBText() {return "Lokales Caching ist erlaubt.";}
	/**pp*/ String getCBTTText() {
		return "Nur wenn lokales Caching erlaubt ist, wird auch von konformen Applikationen in das Wurzelverzeichnis geschrieben.";
	}
    /**pp*/ boolean isChecked_System(){return static_isCachingErlaubt_System();}
    /**pp*/ String getVerzeichnis_System(){
		return isChecked_System() ? LocalHome.getHome() : GESPERRT;
    }
    /**pp*/ void setSystem(final String verzeichnis, final boolean isCachingErlaubt){
    	LocalHome.setHome(verzeichnis, isCachingErlaubt);
    	initFromSystem();
    }

	private JTextField createTFVerzeichnisEditierbar() {
		final JTextField tf = new JTextField();
        tf.setFont(GUIFactory.createButtonFont());
        tf.setText(stripVerzeichnis(getVerzeichnis_System()));
        tf.setToolTipText("Das Basis-Verzeichnis kann von Hand (mit abschlieendem enter) oder mittels Durchsuchen eingegeben werden. Die Applikationen brauchen hier eine Schreibberechtigung.");
        tf.setHorizontalAlignment(SwingConstants.RIGHT);
        tf.setEnabled(isChecked_System());
        tf.getDocument().addDocumentListener(new DocumentListener(){
        	@Override public void changedUpdate(final DocumentEvent evt) {updateFromGuiState();}
        	@Override public void insertUpdate (final DocumentEvent evt) {updateFromGuiState();}
        	@Override public void removeUpdate (final DocumentEvent evt) {updateFromGuiState();}
        });
		return tf;
	}
	private JButton createJBDurchsuchen() {
		final JButton button = new JButton();
        button.setText("Durchsuchen");
        button.setToolTipText("Hierbei kann interaktiv ein neues Wurzelverzeichnis ausgewhlt werden.");
        button.addActionListener(new ActionListener() {
        	@Override public void actionPerformed(final ActionEvent evt) {
                setVerzeichnis_GUI(LocalHome.askTheUser(getVerzeichnis_GUI()));
        	}
        });
		return button;
	}
	private JButton createJBOeffnen() {
		final JButton button = new JButton();
        button.setText("\u00d6ffnen");
        button.setToolTipText("Das Wurzelverzeichnis wird geffnet.");
        button.addActionListener(new ActionListener() {
        	@Override public void actionPerformed(final ActionEvent evt) {
        		final String verzeichnis = getVerzeichnis_GUI();
        		if(dirExists(verzeichnis))
        			show(verzeichnis);
        		else if(dirExists(stripVerzeichnis(verzeichnis)))
        			show(stripVerzeichnis(verzeichnis));
        		else{
	        		PrefCachePanel.this.jLMessages.setText("Das Verzeichnis '" + verzeichnis
	        				+ "' existiert nicht. Ignoriere die Aktion.");
	        		PrefCachePanel.this.jLMessages.setVisible(true);
        		}
        	}
			private void show(final String verzeichnis) {
				try {Desktop.getDesktop().open(new File(verzeichnis));}
				catch (final Exception ex) {
					final String msg = "Kann das Verzeichnis '" + verzeichnis
									 + "' nicht im anzeigen.";
					Logger.SWFW.warn(msg, ex);
				}
			}
        });
        return button;
	}
	/**Puts two Components in BorderLayouted Panel as name suggestes, may be null*/
	private JPanel createWestEastPanel(final Component west, final Component east) {
		final JPanel panel = new JPanel(new BorderLayout());
		if(west != null)
			panel.add(west, BorderLayout.WEST);
		if(east != null)
			panel.add(east, BorderLayout.EAST);
		return panel;
	}
	private JCheckBox createCBCachingErlaubt() {
		final JCheckBox cb = new JCheckBox();
        cb.setFont(GUIFactory.createButtonFont());
        cb.setText(getCBText());
        cb.setToolTipText(getCBTTText());
        cb.setBorder(BorderFactory.createEmptyBorder(0, 0, 0, 0));
        cb.setMargin(new Insets(0, 0, 0, 0));
        cb.setSelected(isChecked_System());
        cb.addItemListener(new ItemListener(){
        	@Override public void itemStateChanged(final ItemEvent evt) {
				updateFromGuiState();
			}
        });
		return cb;
	}
	private JLabel createJLMessages() {
		final JLabel label = new JLabel();
		label.setFont(GUIFactory.createButtonFont());
		label.setForeground(Color.red);
		final String dummyTxt = "Hier koennen Messages angezeigt werden.";
		label.setText(dummyTxt + dummyTxt + dummyTxt + dummyTxt);
		label.setVerticalAlignment(SwingConstants.TOP);
		label.setAutoscrolls(true);
		label.setPreferredSize(new Dimension(300, 100));
		return label;
	}
	private List<JComponent> createList(final JComponent ... cmpts) {
		return Arrays.asList(cmpts);
	}

//=============================================================================
	@Override public void onDispose() {}
//-----------------------------------------------------------------------------
	/* (non-Javadoc)
	 * @see de.desy.mst.libs.framework.mstapp.preferences.IPrefClient#onReset()
	 */
	@Override public void onReset() {initFromSystem();}
	/* (non-Javadoc)
	 * @see de.desy.mst.libs.framework.mstapp.preferences.IPrefClient#onSet()
	 */
	@Override public void onSet() throws Exception {
		setSystem(getVerzeichnis_GUI(), isChecked_GUI());
		updateFromGuiState();
	}
//=============================================================================
	/**schreibe System Werte in die Gui, Voraussetzung GUI Created*/
	/**pp*/ final void initFromSystem() {
		this.jLMessages.setVisible(false);
		setIsChecked_GUI	(     isChecked_System());
		setVerzeichnis_GUI	(getVerzeichnis_System());
		updateUI();
	}
    private void updateFromGuiState() {
		this.getPrefContainer().setChanged(isChanged());
		final String verzeichnis = getVerzeichnis_GUI();
		final boolean dirExists = dirExists(stripVerzeichnis(verzeichnis));
		this.jTFVerzeichnis.setForeground(dirExists ? Color.black : Color.blue);
		final boolean cachingErlaubt_GUI = isChecked_GUI();
		updateFollowersOfCachingErlaubt(cachingErlaubt_GUI);
		this.jBOeffnen.setEnabled(dirExists && cachingErlaubt_GUI);
		PrefCachePanel.manageAbweichendesProtokoll(getPrefManager(), getPrefAbweichendesProtokoll());
	}
    private static boolean static_isCachingErlaubt_System(){
		return (!LocalHome.noLocalHomeWanted());
    }
    /**pp*/ static void manageAbweichendesProtokoll(final PrefManager pm, final IPreferences prefAbweichendesProtokoll){
    	if(pm == null || prefAbweichendesProtokoll == null)
    		return;
    	if(PrefCachePanel.static_isCachingErlaubt_System())
    		pm.register(prefAbweichendesProtokoll);
    	else
    		pm.deRegister(prefAbweichendesProtokoll);
    } 
	private PrefManager getPrefManager() {return this.prefManager;}
    private IPreferences getPrefAbweichendesProtokoll() {
		return this.prefAbweichendesProtokoll;
	}
    private boolean isChecked_GUI(){return this.jCBIsChecked.isSelected();}
    private void setIsChecked_GUI(final boolean isChecked){
    	if(isChecked == isChecked_GUI())
    		return;

    	this.jCBIsChecked.setSelected(isChecked);
		updateFollowersOfCachingErlaubt(isChecked);
    	updateFromGuiState();
    }
    private String getVerzeichnis_GUI(){
		return isChecked_GUI() ? transfVerz(this.jTFVerzeichnis.getText())
							   : getVerzeichnis_System();
    }
    /**get it into canonical form*/
    private String transfVerz(String verzeichnis) {
    	verzeichnis = verzeichnis.trim();
    	if(verzeichnis.contains("###"))
    		return verzeichnis;

    	if(!verzeichnis.endsWith(File.separator)){
    		verzeichnis = verzeichnis + File.separator;
    	}
		final String sollEnde = LocalHome.HOME_DIR_NAME + File.separator;
    	if(!verzeichnis.endsWith(sollEnde)){
			verzeichnis = verzeichnis + sollEnde;
    	}
		return verzeichnis;
	}
	private void setVerzeichnis_GUI(final String verzeichnis){
		final String verzeichnis_GUI_vorher = getVerzeichnis_GUI();// .../mst_app/
		if(verzeichnis.equals(verzeichnis_GUI_vorher))
			return;
		this.jTFVerzeichnis.setText(stripVerzeichnis(verzeichnis));
    	updateFromGuiState();
    }
	/**pp*/ String stripVerzeichnis(String verzeichnis) {
		if(verzeichnis == null)
			return "";
		verzeichnis = stripVerzeichnis(verzeichnis, File.separator);
		verzeichnis = stripVerzeichnis(verzeichnis, LocalHome.HOME_DIR_NAME);
		verzeichnis = stripVerzeichnis(verzeichnis, File.separator);
		return verzeichnis;
	}
	/**pp*/ final String stripVerzeichnis(final String verzeichnis, final String suffix) {
		final int stripLength = suffix.length();
		final int length = verzeichnis.length();
		if(length <= stripLength)
			return "";

		if(verzeichnis.endsWith(suffix)) {
			return verzeichnis.substring(0, length - stripLength);
		}

		return verzeichnis;
	}
	private boolean isChanged(){
		final boolean isChanged =
				(isChecked_System() != isChecked_GUI())
			|| 	(!getVerzeichnis_System().equals(getVerzeichnis_GUI()));
		return isChanged;
	}

	private final List<JComponent> getFollowersOfCachingErlaubt() {
		return this.followersOfCachingErlaubt;
	}
	private void updateFollowersOfCachingErlaubt(final boolean enabled) {
		for (final JComponent component : getFollowersOfCachingErlaubt())
			component.setEnabled(enabled);
	}

//=============================================================================

    private boolean dirExists(final String verzeichnis) {
		try {
			return new File(verzeichnis).exists();
		} catch (final Exception e) {
			Logger.SWFW.warn( "PrefCachePanel: Kann das Verzeichnis '"
							+ verzeichnis
							+ "' nicht oeffnen. No action taken."
							);
		}
		return false;
	}
}