/*
 * Created on Jun 17, 2004
 *
 * To change the template for this generated file go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
package de.desy.mst.libs.framework.mstapp.io;


import java.io.*;
import java.util.ArrayList;
import java.awt.print.*;

import de.desy.mst.libs.framework.mstapp.logging.Logger;

/**
 * @author K. Hinsch -mst1-
 * &task some methods to access files
 */
public class FileAccess {
	private RandomAccessFile in;
	private RandomAccessFile out;
	private DataError error = new DataError(0,"success");
	
	/**
	 * Ermittelt die Anzahl der Zeilen eines Files
	 * @param fname
	 * @throws PrinterException
	 * @throws IOException
	 * &author K. Hinsch -mcs1-
	 */
	/*
	 * Vers		Date				Description
	 *  01	Mar 19, 2008	Methode erstellt
	 *  02	Dec 14, 2010	finally close dort
	 */
	public int getNumberOfLines(final String fname )
	throws PrinterException, IOException
	{
		String line = null;
		int LineCounter = 0;
		this.error = new DataError(0,"success");
		try {
			this.in = new RandomAccessFile(fname, "r");
			line = this.in.readLine();
			while (line != null) {
				LineCounter++;
				line = this.in.readLine();	
			}
			return LineCounter;
			
		} catch (final FileNotFoundException e) {
			this.error = new DataError(22, e.toString());
			Logger.SWFW.error("getNumberOfLines(): File nicht gefunden", e);
			return 0;
		}finally{
			try{
				if(this.in != null){
					this.in.close();
				}
			}catch(final Throwable e){
				Logger.SWFW.error("getNumberOfLines(): finally, kann File nicht schlieen", e);
			}
		}
	}
	/**
	 * Ermittelt die Lnge eines Files
	 * @param fname
	 * @throws PrinterException
	 * @throws IOException
	 * &author K. Hinsch -mcs1-
	 */
	/*
	 * Vers		Date				Description
	 *  01	Mar 19, 2008	Methode erstellt
	 *  01	Dec 14, 2010	finally close dort
	 */
	public long getBinaryFileLen(final String fname )
	throws PrinterException, IOException
	{
		long fileLen = 0;
		this.error = new DataError(0,"success");
		try {
			this.in = new RandomAccessFile(fname, "r");
			fileLen=this.in.length();
			return fileLen;
			
		} catch (final FileNotFoundException e) {
			this.error = new DataError(22, e.toString());
			Logger.SWFW.error("getBinaryFileLen(): File nicht gefunden", e);
			return 0;
		}finally{
			try{
				if(this.in != null){
					this.in.close();
				}
			}catch(final Throwable e){
				Logger.SWFW.error("getBinaryFileLen: finally, kann File nicht schlieen", e);
			}
		}
	}
	/**
	 * List ein File mit der Extension .txt oder .csv
	 * @param fname
	 * @param fileBuffer
	 * @throws PrinterException
	 * @throws IOException
	 * &author K. Hinsch -mcs1-
	 */
	/*
	 * Vers		Date				Description
	 *  01	Mar 19, 2008	Methode erstellt
	 *  02	Dec 14, 2010	finally close dort
	 */
	public boolean readTextFile(final String fname, final ArrayList<String> fileBuffer)
	throws PrinterException, IOException
	{
		String line = null;
		this.error = new DataError(0,"success");
		try {
			this.in = new RandomAccessFile(fname, "r");
			line = this.in.readLine();
			while (line != null) {
				fileBuffer.add(line);
				line = this.in.readLine();	
			}
				
			return true;
			
		} catch (final FileNotFoundException e) {
			this.error = new DataError(22, e.toString());
			Logger.SWFW.error("readTextFile(): File nicht gefunden", e);
			return false;
		}finally{
			try{
				if(this.in != null){
					this.in.close();
				}
			}catch(final Throwable e){
				Logger.SWFW.error("readTextFile(): finally, kann File nicht schlieen", e);
			}
		}

	}
	/**
	 * List ein binary File
	 * @param fname
	 * @param b
	 * @param off
	 * @param len
	 * @throws PrinterException
	 * @throws IOException
	 * &author K. Hinsch -mcs1-
	 */
	/*
	 * Vers		Date				Description
	 *  01	Mar 19, 2008	Methode erstellt
	 *  02	Dec 14, 2010	finally close dort
	 */
	public boolean readBinaryFile(final String fname, final byte b[], final int off, final int len)
	throws PrinterException, IOException
	{
		this.error = new DataError(0,"success");
		try {
			this.in = new RandomAccessFile(fname, "r");
			this.in.readFully(b, off, len);
			return true;
			
		} catch (final FileNotFoundException e) {
			this.error = new DataError(22, e.toString());
			Logger.SWFW.error("readBinaryFile(): File nicht gefunden", e);
			return false;
		}finally{
			try{
				if(this.in != null){
					this.in.close();
				}
			}catch(final Throwable e){
				Logger.SWFW.error("readBinaryFile(): finally, kann File nicht schlieen", e);
			}
		}

	}
	/**
	 * Schreibt ein File mit der Extension .txt oder .csv
	 * @param fname
	 * @param fileBuffer
	 * @throws PrinterException
	 * @throws IOException
	 * &author K. Hinsch -mcs1-
	 */
	/*
	 * Vers		Date				Description
	 *  01	Mar 19, 2008	Methode erstellt
	 *  02	Dec 14, 2010	finally close dort
	 */
	public boolean writeTextFile(final String fname, final ArrayList<String> fileBuffer)
	throws PrinterException, IOException
	{
		this.error = new DataError(0,"success");
		try {
			this.out = new RandomAccessFile(fname, "rw");
			for(int i = 0; i < fileBuffer.size(); i++){
				final String line = fileBuffer.get(i);
				if(i == fileBuffer.size() - 1){
					this.out.writeBytes(line);
				}else{
					this.out.writeBytes(line + "\r\n");
				}
			}
			return true;
			
		} catch (final FileNotFoundException e) {
			this.error = new DataError(22, e.toString());
			Logger.SWFW.error("writeTextFile(): File nicht gefunden", e);
			return false;
		}finally{
			try{
				if(this.out != null){
					this.out.close();
				}
			}catch(final Throwable e){
				Logger.SWFW.error("writeTextFile(): finally, kann File nicht schlieen", e);
			}
		}
	}
	  //---------------------------------------------------------------------------
	  /**
	   * Schreibt den Inhalt des ByteArrays in das angegebene File
	   * @param pFileName
	   * @param pByteArr
	   * @throws PrinterException
	   * @throws IOException
	   * &author J. Maa -mcs1-
	   */
	  /*
	   * Vers		Date				Description
	   *  01  Oct 10, 2007  Methode erstellt
	   *  02  Oct 11, 2007  'BufferedOutputStream' benutzen, da sonst ein schon
	   *                    vorhandenes File nur zum Teil ueberschrieben wird.
	   *  03  Nov 16, 2007  kein File schreiben wenn das ByteArray leer ist;
	   *  04  Nov 20, 2007  Positive Fehlernummern mit Enumeration 'ErrorDefs';
	   *  05  Dec 14, 2010	finally close dort
	   */
	  //---------------------------------------------------------------------------
	  public DataError writeBinaryFile(final String pFileName, final byte[] pByteArr) {
	    BufferedOutputStream out = null;
	    this.error = new DataError(0,"success");
	    if (pByteArr!=null) {
	      try {
	        out = new BufferedOutputStream(new FileOutputStream(pFileName));
	        out.write(pByteArr,0,pByteArr.length);
	      } catch (final FileNotFoundException e) {
	        this.error = new DataError(22, e.toString());
			Logger.SWFW.error("writeBinaryFile(): File nicht gefunden", e);
	        return this.error;
	      } catch (final IOException e) {
	        this.error = new DataError(2, e.toString());
			Logger.SWFW.error("writeBinaryFile(): I-/O-Fehler", e);
	        return this.error;
		  }finally{
			try{
				if(out != null){
					out.close();
				}
			}catch(final Throwable e){
				Logger.SWFW.error("writeBinaryFile(): finally, kann File nicht schlieen", e);
			}
		  }
	    } else {
	    	this.error = new DataError(3, "Databuffer is empty");
	    }
	    return this.error;
	  }
	public FileAccess() {
		super();
	}
	  //---------------------------------------------------------------------------
	  /**
	   * Liest ein File und liefert den Fileinhalt in einem ByteArray zurueck. Wenn
	   * der Returnwert=null ist, so gab es einen Fehler beim lesen des Files. Ein
	   * Fehlercode kann mit 'getError' abgefragt werden.
	   * @param pFileName Name des zu lesenden Files
	   * @return ByteArray mit dem Inhalt des Files.
	   * &author J. Maa -mcs1-
	   */
	  /*
	   * Vers		Date				Description
	   *  01	Sep 19, 2007	Methode erstellt
	   *  02  Nov 20, 2007  Positive Fehlernummern mit Enumeration 'ErrorDefs';   */
	  //---------------------------------------------------------------------------
	  public byte[] readFile(final String pFileName) {
	    byte[] fileContents = null;
	    this.error = new DataError(0,"success");
//	  Filelaenge lesen
	    int lng = 0;
	    try {
			//$ANALYSIS-IGNORE
	      lng = (int) Math.min(getBinaryFileLen(pFileName), Integer.MAX_VALUE);
	      if (lng<=0) { this.error = getLastError(); }
	    } catch (final PrinterException e) {
	      e.printStackTrace();
	      this.error = new DataError(4, e.toString());
	    } catch (final IOException e) {
	      e.printStackTrace();
	      this.error = new DataError(2, e.toString());
	    }

	    if (lng>0) {
//	    Bytearray erzeugen und File lesen
	      fileContents = new byte[lng];
	      try {
	        this.readBinaryFile(pFileName, fileContents, 0, lng);
	      } catch (final PrinterException e) {
	        e.printStackTrace();
	        this.error = new DataError(4, e.toString());
	      } catch (final IOException e) {
	        e.printStackTrace();
	        this.error = new DataError(2, e.toString());
	      }
	    }
	    return fileContents;
	  }
	  //---------------------------------------------------------------------------
	  /**
	   * @return Returns the error.
	   * &author J. Maa -mcs1-
	   */
	  /*
	   * Vers		Date				Description
	   *  01  Sep 19, 2007  Methode erstellt
	   *  02  Feb 21, 2008  Neuer ReturnType 'DataError'
	   */
	  //---------------------------------------------------------------------------
	  public DataError getLastError() {
	    return this.error;
	  }
}
