/**
 *  Created on 	Oct 20, 2005 - Oct 27, 2005
 *  			Jan 11, 2006 - Jan 11, 2006	runId hinzugefuegt
 *  			Jun 02, 2006 - Jun 02, 2006	synchronized msgCount -> Atomic msgCount
 *  			Aug 09, 2006 - Aug 09, 2006	ws: toString mit etwas mehr Bedeutung
 * Dec 6, 2007 - Dec 7, 2007	ws:	added .equals(...), hash(),
 * 									pp complete XTor
 * 									toString weniger Output bei throwable
 * Jun 03, 2006 - Jun 12, 2009	ws:	support for Tags
 */
package de.desy.mst.libs.framework.mstapp.logging;

import java.util.Arrays;
import java.util.Date;
import java.util.UUID;
import java.util.concurrent.atomic.AtomicLong;
import de.desy.mst.libs.framework.mstapp.Main;
import de.desy.mst.libs.framework.mstapp.security.MstSecurity;

/**
 * Eine Message fuer das Logging System.
 * Diese Klasse ist immutable
 * @author ws
 * @version 1.0.0
 */
public final class LogMsg {
	private static final LMTag[] EMPTY_TAGS = new LMTag[0];
	/**eindeutige Identifizierung jedes Programmlaufes, UUID dieses Programmlaufes*/
	public static final UUID THIS_RUN_RUNID = UUID.randomUUID();
	//-------------------------------------------------------------------------
	/**please access/modify only by threadsafe createNr(), getNr()*/
	private static AtomicLong amsgCount = new AtomicLong(0);
	//-------------------------------------------------------------------------
	public final long   nr;		//local count um Messages nach Erzeugung sortieren zu koennen
	public final LogLvl level;
	public final Date	date;
	public final String applikation;
	public final String pcName;
	public final Logger kategorie;
	public final String message;
	public final Throwable throwable;
	public final UUID   runId;	//UUID dieser Message
	/**<b>Kontrakt:</b> Werte dieses Arrays duerfen NICHT gendert werden!*/
	private final LMTag[] tags;

//=============================================================================

	/**
	 * pp<br>
	 * basic and complete constructor for LogMsg Decoder only (nr will
	 * be arbitrarily set
	 * @deprecated only for system use
	 */
	@Deprecated LogMsg(final long 		nr
					 , final LogLvl 	level
					 , final Date 		date
					 , final String 	applikation
					 , final String 	pcName
					 , final Logger 	kategorie
					 , final String 	message
					 , final Throwable 	throwable
					 , final UUID		runId
					 , final LMTag ...	tags
	) {
		final MstSecurity mstSecurity = MstSecurity.get();
		this.nr             = nr;
		this.level 			= (level		== null ? LogLvl.info 				: level);
		this.date 			= (date			== null ? new Date		  	()		: date);
		this.applikation	= (applikation	== null ? Main.getAppName	()		: applikation);
		this.pcName 		= (pcName		== null ? mstSecurity.getHostName()	: pcName);
		this.kategorie 		= (kategorie	== null ? Logger.Any 				: kategorie);
		this.message 		= (message		== null ? ""		 				: message);
		this.throwable 		= throwable;
		this.runId 			= runId;
		this.tags			= tags;
	}

	/**basic and complete constructor for LogMsg'es of this application
	 * @deprecated only for system use
	 */
	@Deprecated public LogMsg(final LogLvl 	level
							, final Date 	date
							, final String 	applikation
							, final String 	pcName
							, final Logger 	kategorie
							, final String 	message
							, final Throwable throwable
							, final UUID	runId
							, final LMTag ...tags
	) {
		this(LogMsg.createNr()	//synchronized on class
		   , level
		   , date
		   , applikation
		   , pcName
		   , kategorie
		   , message
		   , throwable
		   , runId
		   , tags
			);
	}
	/**application level complete constructor*/
	public LogMsg(final LogLvl 	level
			    , final Date 	date
			    , final String 	applikation
			    , final String 	pcName
			    , final Logger 	kategorie
			    , final String 	message
			    , final Throwable throwable
			    , final LMTag ...tags
			     ) {
		this(level, date, applikation, pcName, kategorie, message,  throwable, LogMsg.THIS_RUN_RUNID, tags);
	}
	/**basic errorless constructor*/
	public LogMsg(final LogLvl 	level
			    , final Date 	date
			    , final String 	applikation
			    , final String 	pcName
			    , final Logger 	kategorie
			    , final String 	message
			    , final LMTag ...tags
			     ) {
		this(level,	date, applikation, pcName, kategorie, message, null, tags);
	}
	/**minimal constructor*/
	public LogMsg(final LogLvl 	level
			    , final Logger 	kategorie
			    , final String 	message
			    , final LMTag ...tags
			     ) {
		this(level, kategorie, message, null, tags);
	}
	/**minimal constructor with error*/
	public LogMsg(final LogLvl 	level
			    , final Logger 	kategorie
			    , final Throwable throwable
			    , final LMTag ...tags
			     ) {
		this(level, kategorie, "", throwable, tags);
	}
	/**minimal constructor with msg and error*/
	public LogMsg(final LogLvl 	level
			    , final Logger 	kategorie
			    , final String 	message
			    , final Throwable throwable
			    , final LMTag ...tags
			     ) {
		this(level, null, null, null, kategorie, message, throwable, tags);
	}

	/**copies original Message and tries to replace null's by good defaults*/
	public LogMsg(final LogMsg logMsg){
		this(logMsg.level, 	logMsg.date, 		logMsg.applikation, logMsg.pcName
		   , logMsg.kategorie, 	logMsg.message, logMsg.throwable,   logMsg.runId);
	}

//=============================================================================

	/**Threadsafe: msgCount++*/
	public static final long createNr() {return amsgCount.incrementAndGet();}
	/**Threadsafe: laufende Message Nummer dieses Applikations-Laufs*/
	public static final long getMsgCount() {return amsgCount.longValue();}

//-----------------------------------------------------------------------------
	public final long 	getNr() 		{return this.nr;}
	public final LogLvl getLevel() 		{return this.level;}
	public final Date 	getDate() 		{return this.date;}
	public final String getApplikation(){return this.applikation;}
	public final String getPcName() 	{return this.pcName;}
	public final Logger getKategorie() 	{return this.kategorie;}
	public final String getMessage() 	{return this.message;}
	public final Throwable getThrowable(){return this.throwable;}
	public final UUID 	getRunId() 		{return this.runId;}
	public final LMTag[]  getTags()		{return (this.tags == null) ? EMPTY_TAGS : this.tags.clone();}
	/**trivial cases false, refTag.inverted wird nicht speziell behandelt*/
	public final boolean hasTag(final LMTag refTag)	{
		if(this.tags == null || this.tags.length == 0 || refTag == null)
			return false;

		for (final LMTag tag : this.tags)
			if(refTag.equals(tag))
				return true;

		return false;
	}
//-----------------------------------------------------------------------------
	@Override public String toString() {
		return super.toString()
             + ": '"
             + EinfacherLogFormatter.createNopadNothrowableFormatter().format(this)
             + "'"
             ;
	}
	@Override public int hashCode() {return (int)(this.nr ^ (this.nr >>> 32));}
	@Override public boolean equals(final Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		final LogMsg other = (LogMsg) obj;
		if (this.nr != other.nr)
			return false;
		if (this.kategorie == null) {
			if (other.kategorie != null)
				return false;
		} else if (!this.kategorie.equals(other.kategorie))
			return false;
		if (this.level == null) {
			if (other.level != null)
				return false; 
		} else if (!this.level.equals(other.level))
			return false;
		if (this.date == null) {
			if (other.date != null)
				return false;
		} else if (!this.date.equals(other.date))
			return false;
		if (this.message == null) {
			if (other.message != null)
				return false;
		} else if (!this.message.equals(other.message))
			return false;
		if (this.applikation == null) {
			if (other.applikation != null)
				return false;
		} else if (!this.applikation.equals(other.applikation))
			return false;
		if (this.pcName == null) {
			if (other.pcName != null)
				return false;
		} else if (!this.pcName.equals(other.pcName))
			return false;
		if (this.runId == null) {
			if (other.runId != null)
				return false;
		} else if (!this.runId.equals(other.runId))
			return false;
		if (this.throwable == null) {
			if (other.throwable != null)
				return false;
		} else if (!this.throwable.equals(other.throwable))
			return false;
		if (!Arrays.equals(this.tags, other.tags))
			return false;
		return true;
	}
}
