/**
 *  Created on 	Nov 02, 2005 - Nov 02, 2005
 *	Jan 06, 2006 - Jan 06, 2006
 *	Aug 20, 2006 - Aug 20, 2006	ws: Buffered Messages Implementation
 *  Apr 12, 2007 - ...			ws: added support for XLogHandler
 *  Sep 12, 2007 - Nov  7, 2007 ws: added dateiAppender
 *  Jan 10, 2008 - Jan 11, 2008	ws: added getDateiAppender(), flush()
 *  								cleanup
 *	Feb  7, 2008 - Feb  7, 2008 ws: respect NoLocalHomeWanted()
 * 									in dateiAppender
 * 	Aug 15, 2008 - Aug 15, 2008	ws: dateiAppender kann erst bei bekanntem ApplicationInfo registriert werden
 * 	Oct 14, 2008 - Oct 14, 2008	ws:	dateiFilter + Spiel mit MstApp expliziter
 * 	Oct 20, 2008 - Oct 27, 2008	ws:	registerPreference fuer ConsolAppender, DateiAppender
 * 	Dec 15, 2008 - Dec 15, 2008	ws:	try to improve threading properties
 * 	Jun 19, 2008 - Jul  3, 2008	ws:	LogAppenderFile use Factory, temp Files,
 * 									create onApplicationInfo Mechanismus
 * 	Aug 21, 2009 - Aug 21, 2009	ws: added getFirstLogMsg()
 * 	Dec 15, 2009 - Dec 15, 2009	ws:	implemented: logFiles() switch semantics
 * 	May  3, 2012 - May  3, 2012	ws:	use other {@link LogAppenderTine}
 */
package de.desy.mst.libs.framework.mstapp.logging;

import java.util.List;
import java.util.concurrent.Callable;

import de.desy.mst.libs.framework.mstapp.ApplicationInfo;
import de.desy.mst.libs.framework.mstapp.Main;
import de.desy.mst.libs.framework.mstapp.MstApp;
import de.desy.mst.libs.framework.mstapp.ParamDecoder;
import de.desy.mst.libs.framework.mstapp.ini.LocalHome;
import de.desy.mst.libs.framework.mstapp.logging.appender.ILogAppender;
import de.desy.mst.libs.framework.mstapp.logging.appender.LogAppenderBuffer;
import de.desy.mst.libs.framework.mstapp.logging.appender.LogAppenderJavaKonsole;
import de.desy.mst.libs.framework.mstapp.logging.appender.file.LogAppenderFile;
import de.desy.mst.libs.framework.mstapp.logging.filter.FilterModus;
import de.desy.mst.libs.framework.mstapp.logging.filter.HauptLogFilter;
import de.desy.mst.libs.framework.mstapp.logging.filter.ALogTaggedFilter.TagState;
import de.desy.mst.libs.framework.mstapp.logging.map_tinelogsrv.MstLog4TineLog_Facade;
import de.desy.mst.libs.framework.mstapp.preferences.IPrefManager;
import de.desy.mst.libs.framework.mstapp.preferences.IPreferences;
import de.desy.mst.libs.framework.mstapp.preferences.helper.PrefGenericParent;
import de.desy.mst.libs.framework.mstapp.util.threads.Sleep;

/**
 * Package Private Implementation des Loggers.<br>
 * Rein statische Klasse
 * @author ws
 * @version 1.0.0
 */
class LoggerImpl {
	private static final long TIMEOUT_MS = 60L*1000L;
	private static final long   START_MS = System.currentTimeMillis();
//	private final static List<LogMsg> logMsgs = Collections.synchronizedList(new LinkedList<LogMsg>());
	private final    static LogAppenderBuffer		systemAppender;		//never null, Buffered ALLE Logging Eintraege
																		//Managed alle registrierten Observer
	private final    static ILogAppender  			consolAppender;		//never null
	private volatile static LogAppenderFile			dateiAppender;		//may be null at beginning
	private volatile static List<LogAppenderFile> 	tempDateiAppender;	//may be null at beginning
	private volatile static ILogAppender 			tineLogAppender;	//may be null
	private final    static XLogHandler 			xLogHandler;

	private volatile static MstApp mstApp;				//(no Synch on class necessary)
	private volatile static IPreferences preference; 	//(no Synch on class necessary)
	private final    static LogMsg firstLogMsg;
	static{
		consolAppender = new LogAppenderJavaKonsole();

		final LogAppenderBuffer logAppenderBuffer = new LogAppenderBuffer(LoggerManager.getSystemBufferInitialCapacity());
		logAppenderBuffer.doFilter(new HauptLogFilter(FilterModus.ALWAYS_PASS));
		systemAppender = logAppenderBuffer;

		firstLogMsg = new LogMsg(LogLvl.info, Logger.SWFW_, "Start der Applikation");
		Logger.doLog(firstLogMsg);	//ErstHier(systemAppender da)

		xLogHandler = new XLogHandler(systemAppender);
		final String name = "";
		final java.util.logging.Logger xLogger = java.util.logging.Logger.getLogger(name);
		xLogger.setUseParentHandlers(false);
		xLogger.addHandler(xLogHandler);

		fillAndRegister(consolAppender);	//sofort

		manageEvtlVerzoegertesApplicationInfo();
	}
	/**
	 * Falls Logging vor Main.main(...) aufgerufen wird kenn ich die AppInfo noch
	 * nicht. Also warte ich maximal 60s oder 80% der maximalen Eintraeg in
	 * den Buffer bevor es auf ein NotInfo geschrieben wird.
	 * <br> Ruft sobald es soweit ist asynchron
	 * onApplicationInfo(final ApplicationInfo info).
	 */
	private static final void manageEvtlVerzoegertesApplicationInfo(){
		Main.getExecutorService().submit(new Callable<Void>(){
			@Override public Void call() throws Exception {
				while(!Main.hasApplicationInfo()){	//warte auf gutes info
					if(systemAppenderIsNearlyFull() || timeOut()){
						Logger.SWFW.warn("LoggerImpl: Kann den Applikationsnamen"
									   + " nicht rechtzeitig bestimmen."
									   + " Schreibe die Datei Loggings in die"
									   + " Default Stelle.");
						break;	//Notausgang, besser schlechtes als gar kein info
					}
					Sleep.sleep(100);
				}
				if(!Main.hasApplicationInfo())
					onApplicationInfo(getApplicationInfo());
				else 
					/**already handled by {@link Main#myRunChoreography(MstFrame)}*/;
				return null;
			}
		});
	}
	/**wird maximal einmal aufgerufen wenn valides Info, oder es pressiert*/
	private static final void onApplicationInfo(final ApplicationInfo info){
//		System.out.println(LoggerImpl.class.getSimpleName() 
//				        + ".onApplicationInfo(...) on thread: " 
//				        + Thread.currentThread().getName()
//				        + " at " + System.currentTimeMillis());
		initDateiAppender(info);
		initTempDateiAppender(info);
	}
	private static void initDateiAppender(final ApplicationInfo info) {
		setDateiAppender(LocalHome.noLocalHomeWanted() ?
						  null
						: LogAppenderFile.Factory.createStdDateiAppender(info));
	}
	private static void initTempDateiAppender(final ApplicationInfo info) {
		if(noLogFiles()){
			Logger.SWFW.warn("IM JUNIT DEBUG MODUS. Setze keinen Temp Datei Appender");
			return;
		} else if(LoggerImpl.tempDateiAppender != null){
			final String msg = "LoggerImpl.initTempDateiAppender(info)"
							 + " TempDateiAppender schon gesetzt. Tue nichts.";
			Logger.SWFW.warn(msg);
			return;
		}
		LoggerImpl.tempDateiAppender = LogAppenderFile.Factory.createTempDateiAppender(info);
		for (final LogAppenderFile logAppenderFile : LoggerImpl.tempDateiAppender)
			fillAndRegister(logAppenderFile);

		final String titel = "Temp Datei Appender";
		final String kommentar = "Hier knnen deren Filter gesetzt werden (Windows only Feature).";
		registerPreference(getTempDateiAppender(), createParentPref(titel, kommentar));
	}
	private static void fillAndRegister(final ILogAppender logAppender) {
		if(logAppender == null)
			return;
		logAppender.doLog(getSystemAppender().getMessages());
		getSystemAppender().register(logAppender);
	}
	@SuppressWarnings("deprecation")
	private static ApplicationInfo getApplicationInfo() {
		return Main.getApplicationInfo();
	}
	private static boolean systemAppenderIsNearlyFull() {
		return      getSystemAppender().getSize()
		>= .8 * getSystemAppender().getCapacity();
	}
	private static boolean timeOut() {
		return (System.currentTimeMillis() - START_MS) > TIMEOUT_MS;
	}

//=============================================================================

	/**basic interface im technischen Sinn*/
	static LogMsg doLog(final LogMsg msg){
		final LogMsg checkedMsg = ((msg == null) ? createNewNullMsg() : msg);
		getSystemAppender().doLog(checkedMsg);
		if(checkedMsg.getLevel().istErnsterAls(LogLvl.error))	//	">="
			flush();	//echte Fehler sofort ausdrucken etc.
		return checkedMsg;
	}
	static void flush() {getSystemAppender().flush();}
	static ILogProvider getSystemLogProvider() {return getSystemAppender();}
	static void __initTineLogExport() {
		final MstLog4TineLog_Facade facade = MstLog4TineLog_Facade.getInstance();
		setTineLogAppender(facade.getTineLogAppender(getFirstLogMsg(), null));
	}
	static LogMsg       getFirstLogMsg() {return LoggerImpl.firstLogMsg;}
	static boolean _hasTineLogAppender() {return tineLogAppender != null;}
//
	private static void setTineLogAppender(final ILogAppender tineLogAppender) {
		if(getTineLogAppender() != null){
			final String msg = "LoggerImpl.setTineLogAppender(...) berschreibe"
							 + " den tineLogAppender nicht, da schon gesetzt.";
			Logger.SWFW.warn(msg);
			return;
		}else if(tineLogAppender == null)
			return;
		Logger.SWFW.info("Logging zum Tine Protokoll und Logging Server ist initialisiert.");
		LoggerImpl.tineLogAppender = tineLogAppender;
		fillAndRegister(getTineLogAppender());
		registerPreference(getTineLogAppender(), "Tine Protokoll Server");
	}

	private /*synchronized*/ static final MstApp getMstApp() {return mstApp;}
	public  /*synchronized*/ static void setMstApp(final MstApp mstApp) {
		if(getMstApp() != null){
			Logger.SWFW.warn("LoggerImpl.setMstApp(...) getMstApp() != null ignore.");
			return;
		}

		LoggerImpl.mstApp = mstApp;

		setPreference(createPreference());
		getPreferencesManager().register(getPreference());

		registerPreference(getSystemAppender());
		registerPreference(getConsolAppender(), null);
		
		if(getDateiAppender() == null && getApplicationInfo() != null)
			onApplicationInfo(getApplicationInfo());
	}

	private static IPrefManager getPreferencesManager() {
		final MstApp mstApp = getMstApp();
		return mstApp == null ? null : mstApp.getPreferencesManager();
	}
//=============================================================================

	static final ILogAppender getConsolAppender() {return consolAppender;}
	/**maybe null at beginning*/
	static final LogAppenderFile 		getDateiAppender()		{return dateiAppender;}
	static final List<LogAppenderFile>	getTempDateiAppender()	{return tempDateiAppender;}
	private static final void setDateiAppender(final LogAppenderFile dateiAppender){
		if(noLogFiles()){
			Logger.SWFW.warn("IM JUNIT DEBUG MODUS. Setze keinen Datei Appender");
			return;
		}else if(LoggerImpl.dateiAppender != null){
			final String msg = "LoggerImpl.setDateiAppender(...) berschreibe"
							 + " den dateiAppender nicht, da schon gesetzt.";
			Logger.SWFW.warn(msg);
			return;
		}else if(dateiAppender == null){
			Logger.SWFW.warn("LoggerImpl.setDateiAppender(null) HUCH und ignoriere.");
			return;
		}

		LoggerImpl.dateiAppender = dateiAppender;
		fillAndRegister(dateiAppender);
		registerPreference(getDateiAppender(), "Standard Protokoll Datei");
	}
	/**may be null*/
	static final ILogAppender getTineLogAppender() {return tineLogAppender;}
	/**>=0*/
	final static int getSystemBufferCapacity() {
		return getSystemAppender().getCapacity();
	}
	/**>=0*/
	final static void setSystemBufferCapacity(final int systemBufferCapacity) {
		if(systemBufferCapacity < 0){
			final String msg = "Ignoriere negative systemBufferCapacity: "
							 + systemBufferCapacity;
			final String msgError = "Negative systemBufferCapacity.";
			Logger.SWFW.warn(msg, new IllegalArgumentException(msgError));
			return;
		}
		getSystemAppender().setCapacity(systemBufferCapacity);
	}

//=============================================================================

	private final static LogMsg createNewNullMsg(){
		final NullPointerException ex = new NullPointerException("Tried to log with a null msg.");
		return new LogMsg(LogLvl.error, Logger.SW, ex);
	}
	private static LogAppenderBuffer getSystemAppender() {return systemAppender;}

	private /*synchronized*/ static final IPreferences getPreference() {return preference;}
	private /*synchronized*/ static final void setPreference(final IPreferences preference) {
		LoggerImpl.preference = preference;
	}
	private static PrefGenericParent createPreference() {
		final String title = "Logging und Protokoll Einstellungen";
		final String kommentar = "Hier knnen die Filter diverer Logging und Protokollappender konfiguriert werden.";
		final PrefGenericParent pref = new PrefGenericParent(title, kommentar);
		return pref;
	}
	
	private static void registerPreference(final LogAppenderBuffer appender) {
		final IPrefManager prefManager = getPreferencesManager();
		if(appender != null && prefManager != null)
			appender.registerPreference(prefManager, getPreference());
	}
	
	private static void registerPreference(final ILogAppender appender, final String appenderName) {
		registerPreference(appender, appenderName, getPreference());
	}
	private static void registerPreference(final ILogAppender appender, final String appenderName, final IPreferences parent) {
		final IPrefManager prefManager = getPreferencesManager();
		if(appender != null && prefManager != null)
			appender.registerPreference(prefManager, parent, appenderName == null ? appender.getName() : appenderName);
	}
	private static void registerPreference(final List<LogAppenderFile> dieAppender, final IPreferences parent) {
		for (final LogAppenderFile appender : dieAppender)
			if(appender != null)
				registerPreference(appender, "Temp Datei " + appender.getFilter().getTag(TagState.ON).getName(), parent);
	}
	private static IPreferences createParentPref(final String titel, final String kommentar) {
		final IPreferences parent = new PrefGenericParent(titel, kommentar, getPreference());
		final IPrefManager preferencesManager = getPreferencesManager();
		if(preferencesManager != null)
			preferencesManager.register(parent);
		return parent;
	}

	/**only for junit tests*/
	@SuppressWarnings("deprecation") private static boolean noLogFiles() {
		return ParamDecoder._ImplementationSwitches.DEBUGSWITCHES.NOFILELOGGING.isOn();
	}
}
