/**
 * Oct 23, 2008 - Nov 24, 2008	ws:	Creation from PrefLoggingPanel
 */
package de.desy.mst.libs.framework.mstapp.logging.appender;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextField;

import de.desy.mst.libs.framework.mstapp.ini.PrefStorage;
import de.desy.mst.libs.framework.mstapp.logging.Logger;
import de.desy.mst.libs.framework.mstapp.preferences.IPrefContainer;
import de.desy.mst.libs.framework.mstapp.preferences.helper.APrefClientPanel2;
import de.desy.mst.libs.framework.mstapp.util.gui.AppColor;

/**
 * Main Panel for Preferences
 * @author ws
 * @version 1.0.0
 */
class PrefLoggingPanelAppenderBuffer extends APrefClientPanel2 {
	private static final long serialVersionUID = 1L;
 //=============================================================================
	private final LogAppenderBuffer buffer;
	private final JTextField jCapacity;
//=============================================================================
	/**
	 * @param prefContainer	not null
	 * @param logAppender	not null
	 * @throws NullPointerException falls container oder appendex Werte null ist
	 */
	public PrefLoggingPanelAppenderBuffer(final IPrefContainer prefContainer, final LogAppenderBuffer buffer) {
		super(prefContainer);
		if(buffer == null )
			throw new NullPointerException("Der LogAppenderBuffer darf nicht null sein.");

		this.buffer = buffer;

		this.setLayout(new BorderLayout());

		this.jCapacity = createJCapacity();
		final String txt = " Kapazitt des System Puffers (Default: "
						 + getCapacitySystemDefault() + ").";
		this.add(createPanel(getJCapacity(), new JLabel(txt)), BorderLayout.NORTH);

		systemToGui();
	}

//=============================================================================

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.framework.mstapp.preferences.IPrefClient#getSaveString()
	 */
	@Override public String getSaveString() {
		final int capacity = getCapacityGUI();
		final int defCapac = getCapacitySystemDefault();
		if(capacity == defCapac)
			return "";	//do store explicitly nothing
		final String value = Integer.toString(capacity);
		final String key = PrefLoggingAppenderBuffer.KEY_SYSTEM_LOG_CAPACITY;
		return PrefStorage.serialize(super.getSaveString(), key, value);
	}

	//------------------------------------------------------------------------

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.framework.mstapp.preferences.helper.APrefClientPanel#onDispose()
	 */
	@Override public void onDispose() {}

//=============================================================================
	private LogAppenderBuffer getLogAppenderBuffer() {return this.buffer;}
	private JTextField getJCapacity() {return this.jCapacity;}

	private int getCapacitySystem() 			{return getLogAppenderBuffer().getCapacity();}
	private int getCapacitySystemDefault()		{return getLogAppenderBuffer().getCapacityDefault();}
	private int getCapacityMinimumSystem() 		{return getLogAppenderBuffer().getCapacityMinimum();}
	private int getCapacityRiskyLargeSystem()	{return getLogAppenderBuffer().getCapacityRiskyLarge();}
	private int getCapacityMaximumSystem()		{return getLogAppenderBuffer().getCapacityMaximum();}
	private int getCapacityGUI   ()	{
			final Integer c = getCapacityGUI_RAW();
			return c == null ? getCapacitySystem() : c.intValue();
	}
	/**null falls illegal*/
	private Integer getCapacityGUI_RAW() {
		int capCandidate;
		try {
			capCandidate = Integer.parseInt(getJCapacity().getText());
		} catch (NumberFormatException ex) {
			return null;
		}

		if(capCandidate < getCapacityMinimumSystem())
			return null;

		if(capCandidate > getCapacityMaximumSystem())
			return null;

		return capCandidate ;
	}

	private void setCapacityGUI   (final int c) {getJCapacity().setText("" + c);}
	private void setCapacitySystem(final int c) {
		if(!riskyLargeCapacityInGUI())
			getLogAppenderBuffer().setCapacity(c);
		else if(isUserConfirmed(c))
			getLogAppenderBuffer().setCapacity(c);
		else
			/*canceled by user*/;
	}
	private boolean riskyLargeCapacityInGUI() {
		return getCapacityGUI() >= getCapacityRiskyLargeSystem();
	}
	private boolean isUserConfirmed(final int c) {
		final String pre = "Die gewnschte maximale Kapazitt von '";
		final String suf = "' ist extrem hoch und kann die Applikation im Laufe"
						 + " der Zeit zum Absturz bringen."
						 + " Soll sie wirklich so gesetzt werden?";
		final String title = "Extrem hohe Logging Kapazitt";
		final boolean isUserConfirmed =
			(JOptionPane.OK_OPTION
					==
			 JOptionPane.showConfirmDialog(this, pre + c + suf, title, JOptionPane.WARNING_MESSAGE));
		return isUserConfirmed;
	}

	//-------------------------------------------------------------------------

	@Override protected void guiToSystem(){
		if(!isChanged())
			return;

		setCapacitySystem(getCapacityGUI());
		Logger.SWFW.info("PrefLoggingAppenderBufferPanel just updated '"
						 + getPrefContainer().getTitle() + "' memory depth to "
						 + getCapacitySystem() + ".");
		updateIsChanged(false);
	}
	@Override protected  void systemToGui() {
		setCapacityGUI(getCapacitySystem());
		updateIsChanged(false);
	}

	@Override protected boolean isChanged() {
		final boolean diffCap = (getCapacitySystem() != getCapacityGUI());
		return diffCap;
	}

	//-------------------------------------------------------------------------
	private JTextField createJCapacity() {
		final JTextField cmp = new JTextField(6);
//		cmp.setMaximumSize(new JLabel("654321").getPreferredSize());
		cmp.addKeyListener(new KeyAdapter(){
			@Override public void keyReleased(final KeyEvent evt){onNewText();}
			private void onNewText() {
				if(getCapacityGUI_RAW() == null) {
					AppColor._FEHLER.set(cmp);
					cmp.setToolTipText("Nur Positive ganze Zahlen sind erlaubt im Bereich "
							+ getCapacityMinimumSystem() + " bis "
							+ getCapacityRiskyLargeSystem() + "/"
							+ getCapacityMaximumSystem() + ".");
				}else if(riskyLargeCapacityInGUI()){
					AppColor._WARNUNG.set(cmp);
					cmp.setToolTipText("Die maximale Anzahl der zu speichernden"
							+ " Logging/Protokoll Meldungen ist"
							+ " gefhrlich hoch ( >= "
							+ getCapacityRiskyLargeSystem()+ ").");
				}else{
					AppColor._EIN_UND_OK.set(cmp);
					cmp.setToolTipText("Maximale Anzahl von gespeicherten Logging/Protokoll Meldungen.");
				}
				updateIsChanged();
			}
		});

		return cmp;
	}

	private JPanel createPanel(final Component west, final Component center) {
		final JPanel panel = new JPanel(new BorderLayout());
		if(west != null)
			panel.add(west,   BorderLayout.WEST);
		if(center != null)
			panel.add(center, BorderLayout.CENTER);
		return panel;
	}
}
