/**
 * ws:	Jun 14, 2007 - Jul 11, 2007 creation from scratch
 * ws:	Nov  7, 2007 - Nov  7, 2007 Names start with numbers so sort ok in Explo
 */
package de.desy.mst.libs.framework.mstapp.logging.appender.file;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.Collections;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.List;

import de.desy.mst.libs.framework.mstapp.logging.Logger;

/**
 * Berechnet FileTargets nach Wochentag (Montag bis Sonntag)<br>
 * Explizit und nicht mit SimpleDateFormat Trick wegen Internationalisierung
 * @author ws
 * @version 1.0.0
 */
/**
 * MAIN_COMMENT_HERE
 * @author ws
 * @version 1.0.0
 */
public class FTWochentag implements IFileTargetsByDate {
	/**pp*/ static final int ILLEGAL_DATE_INDEX = 0;
	private static final String DEFAULT_BASENAME = "Protokoll";
	private static final String DATE_SEPARATOR = "_";
	private static final long IST_ALT_ZEIT_IN_MS = 5l * 24 * 3600 * 1000;//5Tage
	/**Explizite Liste von einheitlichen Namen*/
	private final static List<String> targets;	
//-----------------------------------------------------------------------------	
	static{
		final List<String> t = new ArrayList<String>();
		t.add(ILLEGAL_DATE_INDEX,	"0XXX");	//0
		t.add(Calendar.SUNDAY, 		"1Son");	//1
		t.add(Calendar.MONDAY, 		"2Mon");	//2
		t.add(Calendar.TUESDAY, 	"3Die");	//3
		t.add(Calendar.WEDNESDAY, 	"4Mit");	//4
		t.add(Calendar.THURSDAY, 	"5Don");	//5
		t.add(Calendar.FRIDAY, 		"6Fre");	//6
		t.add(Calendar.SATURDAY , 	"7Sam");	//7
		targets = Collections.unmodifiableList(t); 
	}
//-----------------------------------------------------------------------------	
	private final String baseName;	//never null
	/**extra Klasse um Kalenderzugriff zu synchronisieren*/
	private class IndexCalculator {
		private final Calendar calendar;
		private IndexCalculator(){ this.calendar = new GregorianCalendar();}
		/**synchronized, da nur ein Kalendar pro Objekt*/
		public synchronized int calcIndex(final Date date) {	
			this.calendar.setTime(date);
			return this.calendar.get(Calendar.DAY_OF_WEEK);
		}
	}; 
	private final IndexCalculator indexCalculator;
	
	private boolean hasChanged = true;	
	private int index = ILLEGAL_DATE_INDEX;
//=============================================================================
	/**@param baseName Filename ohne pfad, ohne .log suffix, ohne Datumsdekor*/
	public FTWochentag(final String baseName) {
		super();
		this.baseName = (baseName == null) ? DEFAULT_BASENAME : baseName;
		this.indexCalculator = new IndexCalculator();
	}
//=============================================================================
	/* (non-Javadoc)
	 * @see de.desy.mst.libs.framework.mstapp.logging.appender.file.IFileTargetsByDate#getBaseName()
	 */
	@Override public String getBaseName() {return this.baseName;}
	/* (non-Javadoc)
	 * @see de.desy.mst.libs.framework.mstapp.logging.appender.file.IFileTargetsByDate#getName()
	 */
	@Override public final String getName(){
		return getBaseName() + getDateSeparator() + getDateSuffix();
	}
	/* (non-Javadoc)
	 * @see de.desy.mst.libs.framework.mstapp.logging.appender.file.IFileTargetsByDate#setDate(java.util.Date)
	 */
	@Override public final boolean setDate(final Date date){
		setIndex(calcIndex(date));
		return hasChanged();
	}
	/* (non-Javadoc)
	 * @see de.desy.mst.libs.framework.mstapp.logging.appender.file.IFileTargetsByDate#istAlt(java.util.Date)
	 */
	@Override public final boolean istAlt(final Date date){
		final long jetztInMS = System.currentTimeMillis();
		final long dateInMS = date.getTime();
		final long dateDiffInMS = (jetztInMS - dateInMS);
		final boolean b = (dateDiffInMS > IST_ALT_ZEIT_IN_MS);
		return b;
	}
	/* (non-Javadoc)
	 * @see de.desy.mst.libs.framework.mstapp.logging.appender.file.IFileTargetsByDate#needNewOutput(java.util.Date)
	 */
	@Override public boolean needNewOutput(final Date msgDate) {
		return calcIndex(msgDate) != getIndex();
	}
	//=============================================================================
	/*pp*/ String getDateSeparator()	{return DATE_SEPARATOR;}
	/*pp*/ String getDateSuffix() 		{return targets.get(getIndex());}

	/*pp*/ final boolean hasChanged()	{return this.hasChanged;}
	/*pp*/ final int getIndex() 		{return this.index;}
	/*pp*/ final void setIndex(final int index) {
		if(isLegal(index)){
			this.hasChanged = (this.index != index);
			this.index = index;
		}else{
			final String msg = "FTWochenTag.setCurrentIndex(" 
							 + index 
							 + ") has illegal index. Set to Illegal Date.";
			Logger.SWFW.warn(msg);
			setIndex(ILLEGAL_DATE_INDEX);
		}
			
	}
	/*pp*/ boolean isLegal(final int index) {
		return 0 <= index && index < targets.size();
	}
	/*pp*/ int calcIndex(final Date msgDate) {
		return this.indexCalculator.calcIndex(msgDate);
	}
}
