/**
 * ws:	Jun 26, 2007 - Jul 11, 2007 creation from FTWochentagTest (Xtract Inte.)
 */

package de.desy.mst.libs.framework.mstapp.logging.appender.file;

import java.util.Date;

/**
 * Abstrahiert die Logig fuer einen Protokollwechsel nach Logging Datum<br>
 * Beispiel Implementation: <li> 
 * &see de.desy.mst.libs.framework.mstapp.logging.appender.file.FTWochentag<br>
 * Die Implementation kennt das zuletzt verwandte Date. Wird setDate(msgDate)
 * aufgerufen erfaehrt man, ob eine neue Datei benoetigt wird. Es laesst sich
 * bestimmen ob eine evtuell vorhandene Datei des neuen Namens aus 
 * Altersgruenden geloescht werden soll (istAlt(dateiDate)). Den neuen Namen
 * bekommt man mit getName(). Er wird aus dem msgDate-Intervall und dem im
 * Konstruktor uebergebenen BaseName berechnet.
 * 
 * @author ws
 * @version 1.0.0
 */
public interface IFileTargetsByDate {
	/**never null (ohne Datums-Dekor, ohne Pfad, ohne .log Suffix*/
	public abstract String getBaseName();

	/**
	 * @return Filename never null
	 * 		ohne Pfad, mit Datums-Dekor und ohne .log suffix, Target eindeutig
	 */
	public abstract String getName();

	/**
	 * @param msgDate date of current message, not null	
	 * @return	hasChanged due to date change (other File)
	 * @throws NullPointerException, falls date == null
	 */
	public abstract boolean setDate(final Date msgDate);

	/**
	 * Bei rollierenden Dateien muss man wissen ob Datei geloescht 
	 * oder appended werden soll. Alte Dateien werden geloescht bei relativ
	 * neueren wird hinzugefuegt.
	 * @param  dateiDate Datei Date
	 * @return date wird fuer alt gehalten
	 */
	public abstract boolean istAlt(final Date dateiDate);

	/**
	 * @param msgDate Date der aktuellen Logging Message  
	 * @return true falls z.B. eine andere Output Datei bebraucht wird
	 */
	public abstract boolean needNewOutput(final Date msgDate);
}