/**
 * Apr 24, 2009 - Apr 24, 2009	ws:	Creation from scratch
 */
package de.desy.mst.libs.framework.mstapp.logging.appender.tinelogsrv;

import java.util.Collections;
import java.util.LinkedList;
import java.util.List;
import de.desy.mst.libs.framework.mstapp.logging.EinfacherLogFormatter;
import de.desy.mst.libs.framework.mstapp.logging.LogMsg;

/**
 * Extrahiert aus LogMsg die Elemente fuer das Tine Clog Text Element
 * <br> Nr, Date, Kategorie, Eintrag
 * @author ws
 * @version 1.0.0
 */
class ClogTextFormatter extends EinfacherLogFormatter {
	public  static final String SEPARATOR_InMSG	= "|";	//\t wird in label schlecht angezeigt (Clog Rohdarstellung)

	private static final List<StringPair> escapedSubStrings;
	private static final List<StringPair> reverseEscapedSubStrings;
	static {
		final LinkedList<StringPair> list = new LinkedList<StringPair>();
		final LinkedList<StringPair> reverseList = new LinkedList<StringPair>();
		add(list, reverseList, "&", "&amp;");
		add(list, reverseList, "\t", "&tab;" );	//(key, value)
	  //add(list, reverseList, "\n", "&nl;"  );	// not anymore "|" due to Consolewindow direct jump to errors feature in eclipse
		add(list, reverseList, "\"", "&quot;");
		add(list, reverseList, SEPARATOR_InMSG, "&bar;");
		add(list, reverseList, NEWLINEPAIR);	//nur temporaer
		list.remove(NEWLINEPAIR);	//nur zum dekodieren
		escapedSubStrings        = Collections.unmodifiableList(list);
		reverseEscapedSubStrings = Collections.unmodifiableList(reverseList);
	}
	/**Subklassen wie ClogTextFormatter haben eigene escapedSubStrings*/
	@Override protected List<StringPair> getEscapedSubStrings() {
		return ClogTextFormatter.escapedSubStrings;
	}
	/**Subklassen wie ClogTextFormatter haben eigene escapedSubStrings*/
	@Override protected List<StringPair> getReverseEscapedSubStrings() {
		return ClogTextFormatter.reverseEscapedSubStrings;
	}
//=============================================================================
	@Override public String getHeader(boolean withMultiMessageSeparator) {
		return 					  HEADER_NR			//nrFormat(logMsg.nr)
		 + getSEPARATOR_InMSG() + HEADER_DATUM		//getFormatDate().format(logMsg.date)
		 + getSEPARATOR_InMSG() + HEADER_KATEGORIE	//kategorieFormat(logMsg.kategorie)
		 + getSEPARATOR_InMSG() + HEADER_MELDUNG	//msgAndThrowableFormat(logMsg.message, logMsg.throwable)
		 + (withMultiMessageSeparator ? getMultiMessageSeparator() : "")
		 ;
	}
	@Override public String format(final LogMsg logMsg) {
		final String msg =
			nrFormat(logMsg.nr)
		  + getSEPARATOR_InMSG() + getFormatDate().format(logMsg.date)
		  + getSEPARATOR_InMSG() + kategorieFormat(logMsg.kategorie)
		  + getSEPARATOR_InMSG() + msgAndThrowableFormat(logMsg.message, logMsg.throwable)
		  ;
		return getReduceThrowable() ? encodeTheNewLine(msg) : msg;
	}

	/**never pad*/
	@Override public String pad(final String string) {return string;}

	/**wird ignoriert, da immer true*/
	@Override public void setReduceThrowable(boolean reduceThrowable) {
		super.setReduceThrowable(reduceThrowable);
	}
	@Override public boolean getReduceThrowable() {return true;}
	@Override public String  getSEPARATOR_InMSG() {return ClogTextFormatter.SEPARATOR_InMSG;}
}
