/**
 * Apr 23, 2009 - Apr 24, 2009	ws:	Creation from scratch
 */
package de.desy.mst.libs.framework.mstapp.logging.appender.tinelogsrv;

import static de.desy.tine.server.logger.ClsLog.ClogPriority.*;
import static de.desy.tine.server.logger.ClsLog.ClogStatus.*;

import java.util.HashMap;
import de.desy.mst.libs.framework.mstapp.logging.LogLvl;
import de.desy.mst.libs.framework.mstapp.logging.Logger;
import de.desy.tine.server.logger.ClsLog.ClogPriority;
import de.desy.tine.server.logger.ClsLog.ClogStatus;

/**
 * bildet LogLvl streng monoton auf Clog (Status, Prioritaet) ab
 * @author ws
 * @version 1.0.0
 */
class LogLvlMap{
	/**pp*/ static class Pair{
		private final ClogStatus 	status;
		private final ClogPriority 	priority;
		public Pair(ClogStatus status, ClogPriority priority) {
			checkNonNull(status);
			checkNonNull(priority);
			this.status = status;
			this.priority = priority;
		}
		/**not null*/ public ClogPriority getPriority() {return this.priority;}
		/**not null*/ public ClogStatus   getStatus()   {return this.status;}
		@Override public int hashCode() {
			final int prime = 31;
			int result = 1;
			result = prime * result + getStatus()  .hashCode();
			result = prime * result + getPriority().hashCode();
			return result;
		}
		@Override public boolean equals(Object obj) {
			if (this == obj)
				return true;
			if (obj == null || this.getClass() != obj.getClass())
				return false;
			final Pair other = (Pair) obj;
			return this.getPriority().equals(other.getPriority())
				&& this.getStatus()  .equals(other.getStatus());
		}
		@Override public String toString() {
			return "(" + getStatus() + ", " + getPriority() + ")";
		}
	}
	private static class MyHashMap extends HashMap<LogLvl, LogLvlMap>{
		private static final long serialVersionUID = 1L;
		public MyHashMap() {super(LogLvl.values().length);}
		void put(final LogLvl logLvl, final ClogStatus status, final ClogPriority priority) throws NullPointerException{
			checkNonNull(logLvl);checkNonNull(status);checkNonNull(priority);
			final LogLvlMap logLvlMap = new LogLvlMap(logLvl, status, priority);
			if(containsValue(logLvlMap))
				Logger.SWFW.warn("LogLvlMap class init mit doppelter LogLvlMap: " + logLvlMap);
			if(containsKey(logLvl))
				Logger.SWFW.warn("LogLvlMap class init mit doppeltem LogLvl: " + logLvlMap);
			put(logLvl, logLvlMap);
		}
	};
	//-------------------------------------------------------------------------
	private final static MyHashMap theMap = createTheMap();
	private static MyHashMap createTheMap(){
		//---------+-----------+-----------------------+------------------------
		//ordinal()	LogLvl{		ClogStatus{				ClogPriority{
		//---------+-----------+-----------------------+------------------------
		//0				debug    	CLOG_STATUS_NONE,	   	CLOG_PRIORITY_NONE,
		//1				info	    CLOG_STATUS_INFO,	    CLOG_PRIORITY_USEFUL,
		//2				warn	    CLOG_STATUS_WARN,	    CLOG_PRIORITY_IMPORTANT,
		//3				error    	CLOG_STATUS_ERR	    	CLOG_PRIORITY_URGENT
		//4				fatal		};	  					};
		//	  		};
		//---------+-----------+-----------------------+------------------------
		final MyHashMap hashMap = new MyHashMap();
		hashMap.put(LogLvl.debug, CLOG_STATUS_NONE, CLOG_PRIORITY_NONE);
		hashMap.put(LogLvl.info,  CLOG_STATUS_INFO, CLOG_PRIORITY_USEFUL);
		hashMap.put(LogLvl.warn,  CLOG_STATUS_WARN, CLOG_PRIORITY_USEFUL);
		hashMap.put(LogLvl.error, CLOG_STATUS_ERR,  CLOG_PRIORITY_USEFUL);
		hashMap.put(LogLvl.fatal, CLOG_STATUS_ERR,  CLOG_PRIORITY_IMPORTANT);
		return hashMap;
	}
	private static void createNotfallMap(final LogLvl logLvl) {
		//sollte hier niemals hinkommen, denn createTheMap() sollte vollstaendig
		//sein
		theMap.put(logLvl,  CLOG_STATUS_ERR,  CLOG_PRIORITY_URGENT);
		Logger.SWFW.warn("LogLvlMap.createNotMap(..): " + theMap.get(logLvl));
	}
	private static void checkNonNull(final Object object) {
		if(object == null)
			throw new NullPointerException();
	}
	//-------------------------------------------------------------------------
	public static LogLvlMap get(final LogLvl logLvl){
		if(!theMap.containsKey(logLvl))
			createNotfallMap(logLvl);
		return theMap.get(logLvl);
	}
//=============================================================================
	private final LogLvl logLvl;
	private final Pair   pair;
//=============================================================================
	/**
	 * @param 	logLvl, not null
	 * @param 	priority, not null
	 * @param 	status, not null
	 * @throws 	NullPointerException
	 */
	private LogLvlMap(final LogLvl logLvl, final ClogStatus status, final ClogPriority priority) throws NullPointerException{
		checkNonNull(logLvl);
		this.logLvl 	= logLvl;
		this.pair 	= new Pair(status, priority);
	}
//=============================================================================
	/**not null*/ public LogLvl       getLogLvl()   {return this.logLvl;}
	/**not null*/ public Pair getPair() {return this.pair;}
	/**not null*/ public ClogPriority getPriority() {return getPair().getPriority();}
	/**not null*/ public ClogStatus   getStatus()   {return getPair().getStatus();}
//-----------------------------------------------------------------------------
	@Override public String toString() {
		return "" + getLogLvl() + " -> " + getPair();
	}
}
