/**
 * May  4, 2009 - May  7, 2009	ws:	Creation from scratch
 */
package de.desy.mst.libs.framework.mstapp.logging.appender.tinelogsrv;

import java.util.UUID;

import de.desy.mst.libs.framework.mstapp.logging.ILogFormatter;
import de.desy.mst.libs.framework.mstapp.logging.LogMsg;
import de.desy.tine.server.logger.ClsLog.ClogPriority;
import de.desy.tine.server.logger.ClsLog.ClogStatus;

/**
 * Manages Logging To Tine for a specific AppName and RunId<br>
 * Maximal ein TineLogMgr pro AppName and RunID
 * @author ws
 * @version 1.0.0
 */
/**pp*/ class TineLogMgr {
	private static final String CTX = "MstLog";
	private static final ILogFormatter formatter = new ClogTextFormatter();
	//-------------------------------------------------------------------------
	public  static final String getTCaller(final UUID runId) {
		return runId.toString().replace("-", "");
	}
//=============================================================================
	private final String applikation;
	private final String pcName;
	private final UUID   runId;	//UUID dieser Message
	private final HeartBeat hb;	//intern geaendert im Aufruf: createHeartBeat()
	//=============================================================================
	/**entries must be nontrivial */
	public TineLogMgr(final String applikation, final String pcName, final UUID runId, final long msStartzeit)
								throws NullPointerException, IllegalArgumentException{
		check(applikation);
		check(pcName);
		check(runId);
		this.applikation = applikation;
		this.pcName 	 = pcName;
		this.runId 		 = runId;
		this.hb = new HeartBeat(msStartzeit);
	}
//=============================================================================
	public final String		getSystemContext()	{return CTX;}
	public final HeartBeat  getHeartBeat()		{return this.hb;}

	public final String	getApplikation(){return this.applikation;}
	public final String	getPcName() 	{return this.pcName;}
	public final UUID   getRunId() 		{return this.runId;}
//=============================================================================
	public ITineLogInfos createTInfo(final LogMsg logMsg) {return new LogMsgMap(logMsg);}
	public ITineLogInfos createHeartBeat(final long currentTimeMillis) {
		final HeartBeat hb = getHeartBeat();
		hb.updateForNextHeartBeat(currentTimeMillis);
		return new HeartBeatTLInfo(hb);
	}
	public ITineLogInfos createTerminatorInfo() {return null;}
	public final String 		getTTag() 	 {return getApplikation();}
	public final String 		getTCaller() {return getTCaller(getRunId());}
	public final ILogFormatter 	getFmter()	 {return TineLogMgr.formatter;}
//=============================================================================

	private void check(final String text)throws NullPointerException, IllegalArgumentException{
		check((Object) text);
		if(text.length() == 0)
			throw new IllegalArgumentException();
	}
	private void check(final Object obj)throws NullPointerException{
		if(obj == null)
			throw new NullPointerException();
	}
//=============================================================================
	private abstract class AITineLogInfos implements ITineLogInfos{
		@Override public final String getTTag() 	 {return TineLogMgr.this.getTTag();}
		@Override public final String getTCaller() {return TineLogMgr.this.getTCaller();}
	}
	private abstract class AITineLogInfos_NonSystem extends AITineLogInfos {
		@Override public final boolean isSystemMsg()	{return false;}
		@Override public final String	 getTContext()	{return null;}	//TINE will ultimately determine value
	}
	private abstract class AITineLogInfos_System extends AITineLogInfos {
		@Override public final boolean isSystemMsg()	{return true;}
		@Override public final String	 getTContext()	{return getSystemContext();}
	}
	/**pp*/ class HeartBeatTLInfo extends AITineLogInfos_System{
		private static final String EQ = " = ";
		private static final String SEP = "; ";
		private static final String KEY_HEADER    = "Header";
		private static final String KEY_STARTZEIT = "Startzeit";
		private static final String KEY_LAUFZEIT  = "Laufzeit";
		private static final String KEY_PCNAME    = "PCName";
		private static final String KEY_HEARTBEAT = "HeartBeat";
		//---------------------------------------------------------------------
		private final HeartBeat hb;
		//---------------------------------------------------------------------
		public HeartBeatTLInfo(final HeartBeat hb) {this.hb = hb;}
		//---------------------------------------------------------------------
		@Override public final ClogPriority getTPriority(){return ClogPriority.CLOG_PRIORITY_USEFUL;}
		@Override public final ClogStatus   getTStatus()  {return ClogStatus.CLOG_STATUS_INFO;}
		@Override public final String getTText() {
			return getTText(System.currentTimeMillis());
			}
		public final String getTText(final long currentTimeMillis) {
			final HeartBeat hb = getHeartBeat();
			final long nHB = hb.getHBCount();
			final String sta = hb.getTxStartzeit();
			final String dau = hb.getTxDauer(currentTimeMillis);
			final String pcN = getPcName();
			final String txt = getFmter().getHeader(false);

			final StringBuilder sb = new StringBuilder();
			sb.append(KEY_HEARTBEAT).append(EQ).append(nHB).append(SEP)
			  .append(KEY_STARTZEIT).append(EQ).append(sta).append(SEP)
			  .append(KEY_LAUFZEIT ).append(EQ).append(dau).append(SEP)
			  .append(KEY_PCNAME   ).append(EQ).append(pcN).append(SEP)
			  .append(KEY_HEADER   ).append(EQ).append(txt).append(SEP)
			   ;
			return sb.toString().trim();
		}
		private HeartBeat getHeartBeat() {return this.hb;}
	}
	private class LogMsgMap extends AITineLogInfos_NonSystem {
	//=========================================================================
		private final LogMsg    logMsg;
		private final LogLvlMap lvlMap;
	//-------------------------------------------------------------------------
		public LogMsgMap(final LogMsg logMsg) throws NullPointerException, IllegalArgumentException{
			if(logMsg == null)
				throw new NullPointerException("LogMsg may not be null.");
			check("Applikation",getApplikation(),	logMsg.getApplikation());
			check("PC Name",	getPcName(),		logMsg.getPcName());
			check("RunId", 		getRunId(), 		logMsg.getRunId());
			this.logMsg = logMsg;
			this.lvlMap = LogLvlMap.get(logMsg.getLevel());
		}
	//-------------------------------------------------------------------------
		@Override public final String getTText() {return getFmter().format(getLogMsg());}
		@Override public final ClogStatus   getTStatus()	{return getLvlMap().getStatus()  ;}
		@Override public final ClogPriority getTPriority(){return getLvlMap().getPriority();}
	//-------------------------------------------------------------------------
		private final LogMsg  	getLogMsg() {return this.logMsg;}
		private final LogLvlMap getLvlMap()	{return this.lvlMap;}
		//---------------------------------------------------------------------
		private void check(final String was, final Object expected, final Object actual) {
			if(expected.equals(actual))
				return;

			final String msg = "Inkompatible " + was + " in LogMsg. Expected "
							 + expected + ", habe " + actual;
			throw new IllegalArgumentException(msg);
		}
	}//END class: LogMsgMap
}



//	private static class SystemLogs implements ITineLogInfos{
//
//	};
//	private final SystemLogs systemLogs;