/**
 * 	Mar 20, 2009 - Mar 23, 2009	ws:	extracted from AField
 * 	Jul 14, 2009 - Jul 14, 2009	ws:	fixed some Eclipse 3.5 warnings
 */
package de.desy.mst.libs.framework.mstapp.logging.gui;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.regex.Pattern;
import java.util.regex.PatternSyntaxException;

import de.desy.mst.libs.framework.mstapp.logging.LogMsg;
import de.desy.mst.libs.framework.mstapp.logging.Logger;

/**
 * Ein einzel Filter fuer ein bestimmtes Feld;br:
 * AFieldFilterElement
 * @author ws
 * @version 1.0.0
 */

@SuppressWarnings({"rawtypes", "unchecked"})
abstract class AFFE<E extends Serializable, L> implements Cloneable{
	/**AFieldFilterElementFactory*/
	abstract static class AFFEFactory<E extends Serializable, L>{
		/**might return null, if not possible to create*/
		public abstract AFFE<E,L> create();
		public String asString(){
			final AFFE<E, L> factory = create();
			return (factory == null) ? "" : factory.asString();
		}
	}
//=============================================================================
	static final String INVERTED_TOKEN   = "nicht ";
	static final String INFIX_TERMINATOR = ": ";
	//---------------------------------------------------------------------
	private final AField<E, L> field;
	//---------------------------------------------------------------------
	AFFE(final AField<E, L> field){this.field=field;}
	//---------------------------------------------------------------------
	          public abstract String asString();
	@Override public abstract boolean equals(Object obj);
	@Override public abstract int hashCode();
	@Override        abstract protected Object clone();
	/**pp*/          abstract boolean passesForInvertedIsFalse(final E value);
	/**not null, but maybe empty <br> kann ffes erzeugen in denen dies konvertierbar ist*/
	//---------------------------------------------------------------------
	public final AField<E, L> getField() {return this.field;}
	public       boolean   getInverted() {return false;}
	public       String    getProblems() {return "";}
	public final boolean passes(final E value) {
		if(getInverted())
			return !passesForInvertedIsFalse(value);
		return passesForInvertedIsFalse(value);
	}
	/**pp*/ final String calcInvertedPrefix() {
		return getInverted() ? INVERTED_TOKEN : "";
	}
	public List<AFFEFactory<E, L>> getFactories(){
		return Collections.emptyList();
	}
	/**might be null*/
	public <F extends AFFE<E,L>> F getFFE(final Class<F> clazz) {
		final List<AFFEFactory<E, L>> factories = getFactories();
		for (final AFFEFactory<E, L> factory : factories) {
			if(null != createFfeCandidate(clazz, factory))
				return createFfeCandidate(clazz, factory);
		}
		return null;
	}
//	@SuppressWarnings("unchecked")
	private <F> F createFfeCandidate(final Class<F> clazz, final AFFEFactory factory) {
		if(factory == null)
			return null;
		final F ffeCandidate = (F) factory.create();
		if(ffeCandidate != null && ffeCandidate.getClass().equals(clazz))
			return ffeCandidate;
		return null;
	}

}
/**supports refValue and inverted<br>AFieldFilterElementExtended*/
@SuppressWarnings({"rawtypes"})
abstract class AFFE_Ext<E extends Serializable, L, R> extends AFFE<E, L>{
	private final R 	  refValue;
	private final boolean inverted;
	public AFFE_Ext(final AField<E, L> field, final R refValue, final boolean inverted) {
		super(field);
		this.refValue = refValue;
		this.inverted = inverted;
	}
	public abstract String   asStringInfix();
	public final R 		 getRefValue() {return this.refValue;}
	@Override public final boolean getInverted() {return this.inverted;}
	@Override public final String asString() {
		return calcInvertedPrefix() + asStringInfix()
			 + getField().asStringLong(getRefValue());
	}
	@Override public int hashCode() {
		final int prime = 31;
		final int result =  prime * getField().hashCode()
				+ ((getRefValue() == null) ? 0 : getRefValue().hashCode());
		return result;
	}
//	@SuppressWarnings("unchecked")
	@Override public boolean equals(final Object obj) {
		if (this == obj)
			return true;
		if (!(obj instanceof AFFE_Ext))
			return false;
		final AFFE_Ext other = (AFFE_Ext) obj;
		if (!getField().equals(other.getField()))
			return false;
		if (getRefValue() == null) {
			if (other.getRefValue()!= null)
				return false;
		} else if (!getRefValue().equals(other.getRefValue()))
			return false;
		return getInverted() == other.getInverted();
	}
}
class FFEValueEquality<E extends Serializable, L> extends AFFE_Ext<E, L, E>
												  implements Serializable{
	/**pp*/ static final String INFIX = "gleich" + INFIX_TERMINATOR;
	private static final long serialVersionUID = 1L;
	/**not null*/
	public FFEValueEquality(final AField<E, L> field, final E refValue, final boolean inverted) {
		super(field, refValue, inverted);
	}
	/**Bei Strings auch noch RegEx und SubString, sonst dies oder Range default*/
	@Override public String asStringInfix() {
		return (getField().getClazz() == String.class ? INFIX
				  						: "");
		}
	@Override boolean passesForInvertedIsFalse(final E value) {
		final E refValue = getRefValue();
		return (refValue == value)
			|| (refValue != null && refValue.equals(value));
	}
	@Override protected FFEValueEquality<E, L> clone() {
		return new FFEValueEquality<E, L>(getField(), getRefValue(), getInverted());
	}
	@Override public String toString() {
		return "FFEValueEquality: " + calcInvertedPrefix()
			 + getRefValue().toString();
	}
	@Override public List<AFFEFactory<E, L>> getFactories() {
		final ArrayList<AFFEFactory<E, L>> list = new ArrayList<AFFEFactory<E, L>>(3);
		if(getField().supportsInverted())
			list.add(createInverterFactory());
		list.add(createSubStringFactory());
		list.add(createRegexFactory());
		return list;
	}
	private AFFEFactory<E, L> createSubStringFactory() {
		return new AFFEFactory<E, L>(){
			@Override public FFEHasSubstring<E, L> create() {
				return new FFEHasSubstring<E, L>(FFEValueEquality.this);
			}
		};
	}
	private AFFEFactory<E, L> createRegexFactory() {
		return new AFFEFactory<E, L>(){
			@Override public FFEValueRegEx<E, L> create() {
				final FFEHasSubstring<E, L> ffeSubstring = new FFEHasSubstring<E, L>(FFEValueEquality.this);
				return FFEValueRegEx.createFromSubstring(ffeSubstring);
			}
		};
	}
	private AFFEFactory<E, L> createInverterFactory() {
		return new AFFEFactory<E, L>(){
			@Override public FFEValueEquality<E, L> create() {
				return new FFEValueEquality<E,L>(getField(), getRefValue(), !getInverted());
			}
		};
	}
}
class FFEHasSubstring<E extends Serializable, L> extends AFFE_Ext<E, L, String>{
	/**pp*/  static final String INFIX = "enthlt"  + INFIX_TERMINATOR;
	/**pp*/  static <E extends Serializable, L> FFEHasSubstring<E, L> createEmptyFFE(final AField<E, L> field, final boolean inverted) {
		return new FFEHasSubstring<E, L>(field, "", inverted);
	}
//=============================================================================
	public FFEHasSubstring(final AField<E, L> field, final String subString, final boolean inverted){
		super(field, subString, inverted);
	}
	public FFEHasSubstring(final FFEValueEquality<E, L> ve) {
		this(ve.getField(), ve.getRefValue().toString(), ve.getInverted());
	}
	public final String getSubString(){return getRefValue();}
	@Override public boolean equals(final Object obj) {
		return (obj instanceof FFEHasSubstring<?, ?> && super.equals(obj));
	}
	@Override public int hashCode() {return super.hashCode();}
	@Override protected FFEHasSubstring<E, L> clone() {
		return new FFEHasSubstring<E, L>(getField(), getSubString(), getInverted());
	}
	@Override public boolean passesForInvertedIsFalse(final E superStringObject) {
		final String superString = "" + getField().asString(superStringObject);
		final String superString_NonNull_UpperCase = superString.toUpperCase();
		return superString_NonNull_UpperCase.contains(getSubString_NonNull_UpperCase());
	}
	private String getSubString_NonNull_UpperCase() {
		return ("" + getSubString()).toUpperCase();
	}
	@Override public String asStringInfix() {return INFIX;}
	@Override public List<AFFEFactory<E, L>> getFactories() {
		final ArrayList<AFFEFactory<E, L>> list = new ArrayList<AFFEFactory<E, L>>(3);
		if(getField().supportsInverted())
			list.add(createInverterFactory());
		list.add(createRegexFactory());
		return list;
	}
	private AFFEFactory<E, L> createInverterFactory() {
		return new AFFEFactory<E, L>(){
			@Override public FFEHasSubstring<E, L> create() {
				return new FFEHasSubstring<E,L>(getField(), getRefValue(), !getInverted());
			}
		};
	}
	private AFFEFactory<E, L> createRegexFactory() {
		return new AFFEFactory<E, L>(){
			@Override public FFEValueRegEx<E, L> create() {
				return FFEValueRegEx.createFromSubstring(FFEHasSubstring.this);
			}
		};
	}
}
class FFEKategorie extends AFFE<Logger, LogMsg>{
	/**pp*/  static final String INFIX = "Kategorie"  + INFIX_TERMINATOR;
	final FFEValueEquality<Logger, LogMsg> ffeEqual;
	/**@throws IllegalArgumentException, falls inverted*/
	public FFEKategorie(final FFEValueEquality<Logger, LogMsg> ffeEqual) {
		super(ffeEqual.getField());
		this.ffeEqual = ffeEqual;
		if(getInverted())
			throw new IllegalArgumentException("inverted ist nicht unterstuetzt");
	}
	public FFEKategorie(final Logger kategorie) {
		this(new FFEValueEquality<Logger, LogMsg>(MsgModel_MstLog.getInstance().getKategorieField(), kategorie, false));
	}
//	@Override public String asStringInfix() {return INFIX;}
	@Override protected Object clone() {
		return new FFEKategorie(this.ffeEqual); 
	}
	@Override public boolean equals(final Object obj) {
		return (obj instanceof FFEKategorie)
			&& (this.ffeEqual.equals(((FFEKategorie)obj).ffeEqual));
	}
	@Override public int    hashCode() {return this.ffeEqual.hashCode();}
	/**kein inverted, kein infix*/
	@Override public String asString() {
		return getField().asStringLong(getRefValue());
	}

	@Override public boolean passesForInvertedIsFalse(final Logger value) {
		final Logger refValue = getRefValue();
		return (refValue == value)
			|| (refValue != null && value.isKategorie(refValue));
	}
	public Logger getRefValue() {return this.ffeEqual.getRefValue();}
	@Override public String toString() {
		return "FFEKategorie: " + calcInvertedPrefix()
								+ getRefValue().toString();
	}
	@Override public boolean getInverted() {return this.ffeEqual.getInverted();}
}
/**case insensitive fuer beliebige objekte via toString()*/
class FFEValueRegEx<E extends Serializable, L> extends AFFE_Ext<E, L, String>{
	/**pp*/ static final String ANY_STRING = ".*";
	/**pp*/ static final String INFIX = "matches" + INFIX_TERMINATOR;
//	private static final long serialVersionUID = 1L;
	//--------------------------------------------------------------------
	public static <E extends Serializable, L> FFEValueRegEx<E, L> createFromValueEquality(final FFEValueEquality<E, L> ffeEquality){
		return new FFEValueRegEx<E, L>(ffeEquality.getField(), ffeEquality.getRefValue().toString(), ffeEquality.getInverted());
	}
	public static <E extends Serializable, L> FFEValueRegEx<E, L> createFromSubstring(final FFEHasSubstring<E, L> ffeHasSubstring){
		final String regex = toSubStringRegex(ffeHasSubstring.getSubString());
		return new FFEValueRegEx<E, L>(ffeHasSubstring.getField(), regex, ffeHasSubstring.getInverted());
	}
	private static String toSubStringRegex(final String subString) {
		return ANY_STRING + escape(subString) + ANY_STRING;
	}
	/**pp*/ static String escape(final String subString) {
		final char[] chars = subString.toCharArray();
		final StringBuilder sb = new StringBuilder(chars.length);
		for (int i = 0; i < chars.length; i++) {
			final char c = chars[i];
			if(isAlphaNumeric(c) || ' ' == c || '-' == c)
				sb.append(c);
			else if('\\' == c || '[' == c || ']' == c || '^' == c)
				escape(sb, c);
			else
				escapeByBoxing(sb, c);
		}
		return sb.toString();
	}
	/**pp*/ static boolean isAlphaNumeric(final char c) {
		return (('a' <= c) && (c <= 'z'))
			|| (('A' <= c) && (c <= 'Z'))
			|| (('0' <= c) && (c <= '9'));
	}
	private static void escape(final StringBuilder sb, final char c) {
		sb.append("\\" ).append(c);
	}
	private static void escapeByBoxing(final StringBuilder sb, final char c) {
		sb.append("[" ).append(c).append("]");
	}
	//---------------------------------------------------------------------
	private final Pattern pattern;
	private final String compileException;
	/**pp*/ FFEValueRegEx(final AField<E, L> field, final String regex, final boolean inverted) {
		super(field, "" + regex, inverted);
		Pattern compile;
		String compileException;
		try {
			compile = Pattern.compile(getRegex().toUpperCase());
			compileException = "";
		} catch (final PatternSyntaxException e) {
			compile = null;
			compileException = e.getMessage();
		}
		this.pattern = compile;
		this.compileException = compileException;
	}
	public final String getRegex() {return getRefValue();}
	@Override public boolean equals(final Object obj) {
		return (obj instanceof FFEValueRegEx<?, ?> && super.equals(obj));
	}
	@Override protected FFEValueRegEx<E, L> clone() {
		return new FFEValueRegEx<E, L>(getField(), getRegex(), getInverted());
	}
	@Override public String asStringInfix() {return INFIX;}
	@Override boolean passesForInvertedIsFalse(final E value) {
		if(value != null && this.pattern != null){
			final String sValue = getField().asStringLong(value).toUpperCase();
			try {return this.pattern.matcher(sValue).matches();}
			catch (final Exception ex) {
				final String msg = "FFEValueRegEx.matches '" + sValue
								 + "' '" + this.pattern + "'.";
				Logger.SWFW.warn(msg);
			}
		}
		return false;
	}
	@Override public String getProblems() {return this.compileException;}
	@Override public List<AFFEFactory<E, L>> getFactories() {
		final ArrayList<AFFEFactory<E, L>> list = new ArrayList<AFFEFactory<E, L>>(2);
		if(getField().supportsInverted())
			list.add(createInverterFactory());
		list.add(createSubStringFactory());
		return list;
	}
	private AFFEFactory<E, L> createSubStringFactory() {
		return new AFFEFactory<E, L>(){
			/**Conversion only if trivial*/
			@Override public FFEHasSubstring<E, L> create() {
				final String regex = getRegex();
				if(regex == null || regex.isEmpty()
								 || !regex.startsWith(ANY_STRING)
								 || !regex.endsWith(ANY_STRING))
					return null;
				if(ANY_STRING.equals(regex))
					return new FFEHasSubstring<E, L>(getField(), "", getInverted());

				final int len = ANY_STRING.length();
				final String stripped = regex.substring(len, regex.length() - len);
				final FFEHasSubstring<E, L> candidateFFE = new FFEHasSubstring<E, L>(getField(), stripped, getInverted());
				final FFEValueRegEx<E, L> checkFFERegex = FFEValueRegEx.createFromSubstring(candidateFFE);
				if(regex.equals(checkFFERegex.getRegex()))
						return candidateFFE;
				return null;
			}
			@Override public String asString() {
				final FFEHasSubstring<E, L> ffe = create();
				if(ffe != null)
					return ffe.asString();

				final AFFE<E, L> emptyFFE = FFEHasSubstring.createEmptyFFE(getField(), getInverted());
				return emptyFFE.asString();
			}
		};
	}
	private AFFEFactory<E, L> createInverterFactory() {
		return new AFFEFactory<E, L>(){
			@Override public FFEValueRegEx<E, L> create() {
				return new FFEValueRegEx<E,L>(getField(), getRefValue(), !getInverted());
			}
		};
	}
}

final class FFERange<E extends Serializable, L> extends AFFE<E, L>{
//	/**pp*/  static final String INFIX = "im Bereich"  + INFIX_TERMINATOR;
	public final E von;
	public final E bis;
	/**field != null, von or bis != null*/
	public FFERange(final AField<E, L> field, final E von, final E bis) {
		super(field);
		if(von == null && bis == null)
			throw new IllegalArgumentException("von UND bis sind null.");
		this.von = (von == null) ? field.getDefaultVon() : von;
		this.bis = (bis == null) ? field.getDefaultBis() : bis;
	}
//	@Override public String asStringInfix() {return INFIX;}
	@Override public String asString() {
		return getField().asString(this.von)
			 + getField().getRangeToken()
			 + getField().asString(this.bis);
	}
	@Override public boolean passesForInvertedIsFalse(final E value) {
		return isInRange(value);
	}
	@Override public int hashCode() {
		final int prime = 31;
		int result =  prime
					* getField().hashCode()
					+ ((this.bis == null) ? 0 : this.bis
							.hashCode());
		result = prime * result
		+ ((this.von == null) ? 0 : this.von.hashCode());
		return result;
	}
	@SuppressWarnings("unchecked")
	@Override public boolean equals(final Object obj) {
		if (this == obj)
			return true;
		if (!(obj instanceof FFERange))
			return false;
		final FFERange<E, L> other = (FFERange<E, L>) obj;
		if (this.bis == null) {
			if (other.bis != null)
				return false;
		} else if (!this.bis.equals(other.bis))
			return false;
		if (this.von == null) {
			if (other.von != null)
				return false;
		} else if (!this.von.equals(other.von))
			return false;
		return true;
	}
	@Override public String toString() {
		return "FFEValueRange: " + calcInvertedPrefix() + asString();
	}
	@Override protected FFERange<E, L> clone() {
		return new FFERange<E, L>(getField(), this.von, this.bis);
	}

	private boolean isInRange(final E value){
		if(value instanceof Comparable<?>)
			return lhsIsSmallerRhs(this.von, value)
				&& lhsIsSmallerRhs(value, this.bis);
		else
			return false;
	}
	@SuppressWarnings("unchecked")
	private boolean lhsIsSmallerRhs(final E lhs, final E rhs) {
		return  (lhs == null)
			||  (rhs == null)
			||  (((Comparable<E>) lhs).compareTo(rhs) <= 0);
	}
}
