/**
 * 	May 13, 2008 - Jun  p, 2008	ws:	Creation from scratch
 * 	Jun 12, 2008	- Jun 12, 2008	ws: added class AsJTextAreaDate
 * 	Jul  2, 2008 - Jul  2, 2008	ws:	added filterNotSelect support
 * 	Mar 18, 2009 - Apr 16, 2009	ws:	AsJTextArea for String AFields support for different Filter Types (==, SubString, RegEx)
 * 	Jul 14, 2009 - Jul 14, 2009	ws:	fixed some Eclipse 3.5 warnings
 */
package de.desy.mst.libs.framework.mstapp.logging.gui;

import java.awt.Component;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.Serializable;
import java.util.Date;
import java.util.EventObject;
import java.util.LinkedList;

import javax.swing.AbstractButton;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.JTextArea;
import javax.swing.JTree;
import javax.swing.ListModel;
import javax.swing.event.CellEditorListener;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.event.PopupMenuEvent;
import javax.swing.event.PopupMenuListener;
import javax.swing.tree.DefaultTreeCellRenderer;
import javax.swing.tree.TreeCellEditor;
import javax.swing.tree.TreeCellRenderer;
import javax.swing.tree.TreePath;

import de.desy.mst.libs.framework.mstapp.logging.LogMsg;
import de.desy.mst.libs.framework.mstapp.logging.Logger;
import de.desy.mst.libs.framework.mstapp.logging.gui.AFFE.AFFEFactory;
import de.desy.mst.libs.framework.mstapp.logging.gui.AField.FieldFilter;
import de.desy.mst.libs.framework.mstapp.logging.gui.AField.FieldFilterSized;
import de.desy.mst.libs.framework.mstapp.logging.gui.FilterHandler.FilterPanel.EditorPanel;
import de.desy.mst.libs.framework.mstapp.logging.gui.KategorieTreeModel.Kategorie;
import de.desy.mst.libs.framework.mstapp.logging.gui.LogMsgAuswertung.FieldInfo;
import de.desy.mst.libs.framework.mstapp.util.MstDateFormat;
import de.desy.mst.libs.framework.mstapp.util.gui.AppColor;

/**
 * Gui Editoren fuer die Filter fuer die Felder sind hier definiert.<br>
 * L Typ des Logging Entrys (CLog, MstLog, ...)<br>
 * M Typ des AMsgModel's<br>
 * @author ws
 * @version 1.0.0
 */
@SuppressWarnings({"rawtypes", "unchecked"})
abstract class AFieldFilterGUIFactory<E extends Serializable, L, M extends AMsgModel<L>> implements Serializable{
	private static final long serialVersionUID = 1L;
	private final String text;
	private final String toolTip;
	private final boolean needsEntryMap;
	private AFieldFilterGUIFactory(final String text
							  	 , final String toolTip
							  	 , final boolean needsEntryMap) {
		super();
		this.text = text;
		this.toolTip = toolTip;
		this.needsEntryMap = needsEntryMap;
	}
	//-------------------------------------------------------------------------
	public boolean isWideEditor()	 {return false;}
	public final String getText() 	 {return this.text;}
	public final String getToolTip() {return this.toolTip;}
	public boolean needsEntryMap()	 {return this.needsEntryMap;}
	//-------------------------------------------------------------------------
	public abstract JPanel create(final AField<E, L> field
								, final EditorPanel editorPanel);
	//-------------------------------------------------------------------------
	/**
	 * Implementation Note: alle Textaenderungen werden durch MSTEVENT_TEXT_CHANGED geschliffen.
	 * Da sind UI Changes via Key.
	 * Das sind UI Changes via Kontext Menue.
	 */
	static class AsJTextArea<E extends Serializable, L, M extends AMsgModel<L>> extends AFieldFilterGUIFactory<E, L, M>{
		private static final long serialVersionUID = 1L;
		private static final String MSTEVENT_TEXT_CHANGED = "MSTEVENT_TEXT_CHANGED";
		//---------------------------------------------------------------------
		private final boolean isWideEditor;
		private int stackDepth = 0;
		//---------------------------------------------------------------------
		public AsJTextArea(final String text
				  		 , final String toolTip
				  		 , final boolean isWideEditor) {
			super(text, toolTip, false);
			this.isWideEditor = isWideEditor;
		}
		//---------------------------------------------------------------------
		@Override public boolean isWideEditor() {return this.isWideEditor;}
		@Override public JPanel create(final AField<E, L> field
									 , final EditorPanel editorPanel){
			final JPanel panel = new JPanel();
			panel.setLayout(new BoxLayout(panel, BoxLayout.Y_AXIS));

				final JLabel label = new JLabel(getText());
				label.setToolTipText(getToolTip());
				label.setAlignmentX(Component.LEFT_ALIGNMENT);
			panel.add(label);

				final JTextArea textArea = createTextArea(field, editorPanel);
				textArea.setPreferredSize(label.getPreferredSize());
				extendPanelAndTextArea(panel, field, editorPanel, textArea);
			panel.add(textArea);

			panel.add(Box.createVerticalGlue());
			return panel;
		}
		private JTextArea createTextArea(final AField<E, L> field, final EditorPanel editorPanel) {
			final AndOfFieldsFilter editFilter = editorPanel.getEditFilter();
			final JTextArea textArea = new JTextArea();
			textArea.setAlignmentX(Component.LEFT_ALIGNMENT);
			textArea.setToolTipText(editFilter.getBeschreibung());

			updateTextAreaFromFilter(textArea, editFilter, field);

			if(field.isStringField()){
				final JPopupMenu popupMenu = new JPopupMenu("Mein JPopupMenu AsJTextArea fuer StringField");
				popupMenu.addPopupMenuListener(new PopupMenuListener(){
					@Override public void popupMenuCanceled(final PopupMenuEvent evt) {
						popupMenu.removeAll();
					}
					@Override public void popupMenuWillBecomeInvisible(final PopupMenuEvent evt) {
						popupMenu.removeAll();
					}
					@Override public void popupMenuWillBecomeVisible(final PopupMenuEvent evt) {
						rebuildPopupMenu(popupMenu, field, editFilter, textArea, editorPanel.getTable_());
					}
				});
				textArea.addMouseListener(createMouseListenerForPopup(textArea, popupMenu));
				//textArea.add(popupMenu);	//geht nicht
			}

			textArea.addPropertyChangeListener(new PropertyChangeListener(){
				@Override public void propertyChange(final PropertyChangeEvent evt) {
					if(!MSTEVENT_TEXT_CHANGED.equals(evt.getPropertyName()))
							return;
					onNewText(field, editorPanel, editFilter, textArea);
				}
			});
			textArea.addKeyListener(new KeyAdapter(){
				@Override public void keyReleased(final KeyEvent evt)	{
					if(userPressedEnterAtEndOfText(textArea, evt))
						addDefaultFFE(field, editorPanel, editFilter, textArea);
					else
						fireTextChange(textArea);
				}
				private boolean userPressedEnterAtEndOfText(final JTextArea textArea, final KeyEvent evt) {
					if(KeyEvent.VK_ENTER != evt.getKeyCode())
						return false;
					final int endPosition = textArea.getText().length();
					final int caretPosition = textArea.getCaretPosition();
					return caretPosition == endPosition;
				}
				private void addDefaultFFE(final AField<E, L> field, final EditorPanel editorPanel, final AndOfFieldsFilter editFilter, final JTextArea textArea) {
					final AFFEFactory<E, L> defaultFactory = field.createDefaultFFEFactory();
					if(defaultFactory == null)
						return;
					final AFFE<E, L> targetFFE = defaultFactory.create();
					if(targetFFE == null)
						return;
					editFilter.add(field, targetFFE, editorPanel.getTable_());
					updateTextAreaFromFilter(textArea, editFilter, field);
				}
			});

			return textArea;
		}
		private void fireTextChange(final JTextArea textArea) {
			final int MAXSTACKDEPTH = 100;
			final String newText = "" + textArea.getText();
			final Object oldText = getPreviousText(textArea);
			if(newText.equals(oldText))
				resetStackDepth();
			else if(incAndGetStackDepth() <= MAXSTACKDEPTH)
				putNewText(textArea, newText);
			else
				Logger.SWFW.warn("AFieldFilterGUIFactory.AsJTextArea.fireTextChange exceeded MAXSTACKDEPTH of " + MAXSTACKDEPTH, new IllegalStateException());
			//System.out.println("AFieldFilterGUIFactory.AsJTextArea.fireTextChange(\n\n" + textArea.getText() + " \n\n) stackDepth = " + stackDepth + ".");
		}
		private int  incAndGetStackDepth()	{return ++this.stackDepth;}
		private void resetStackDepth()		{this.stackDepth  = 0;}
		private void updateTextAreaFromFilter(final JTextArea textArea, final AndOfFieldsFilter editFilter, final AField<E, L> field) {
			final String text = editFilter.getStringEncoded(field);
			if(("" + text).equals(textArea.getText()))
				return;	//keine Aenderung im Filter
			textArea.setRows(countRows(text));
			textArea.setText(text);
			fireTextChange(textArea);
		}
		/**for subclasses to add more controls to panel*/
		void extendPanelAndTextArea(final JPanel panel
								  , final AField<E, L> field
								  , final EditorPanel  editorPanel
								  , final JTextArea textArea){}

		void onNewText(final AField<E, L> field
					 , final EditorPanel  editorPanel
					 , final AndOfFieldsFilter editFilter
					 , final JTextArea textArea) {
			final String newText = "" + textArea.getText();
			final LoggingTable table = editorPanel.getTable_();
			final FieldFilter filter = field.createFieldFilter(table);
			try {
				final int caretPosition = textArea.getCaretPosition();
				textArea.setRows(countRows(newText));
				filter.setStringEncoded(newText);

				AppColor._EIN_UND_OK.set(textArea);
				textArea.setToolTipText(editFilter.getBeschreibung());
				textArea.setCaretPosition(Math.min(caretPosition, textArea.getText().length()));

				editFilter.add(filter);
				updateTextAreaFromFilter(textArea, editFilter, field);
				editorPanel.onEditFilterChange();
			} catch (final IllegalArgumentException ex) {
				AppColor._FEHLER.set(textArea);
				final String problems = "Problem in '"
				+ ex.getMessage()
				+ "'.";
				textArea.setToolTipText(problems);
				editorPanel.updateTAEditFilterWarning(problems);
			}
		}
		private void rebuildPopupMenu(final JPopupMenu popupMenu
									, final AField<E,L> field
									, final AndOfFieldsFilter editFilter
									, final JTextArea textArea
									, final LoggingTable<L, M> tableIn) {
			popupMenu.removeAll();

			popupMenu.add(createMenuItemTitle(field));
			popupMenu.addSeparator();
			popupMenu.add(createMenuItemAlleEntfernen(field, editFilter, textArea));
			popupMenu.addSeparator();
			popupMenu.add(createMenuItemFFEAddDefault(field, editFilter, textArea, tableIn));
			popupMenu.addSeparator();
			popupMenu.add(createMenuItemZuruecksetzen(field, editFilter, textArea));

			if(editFilter.contains(field)){
				final LinkedList<AFFE<E,L>> ffes = editFilter.get(field).getFFEs();
				for (final AFFE<E,L> ffe : ffes)
					popupMenu.add(createMenuFFEConverters(textArea, ffes, ffe, editFilter));
			}
		}
		private JMenuItem createMenuItemFFEAddDefault(
										   final AField<E, L> field
										 , final AndOfFieldsFilter editFilter
										 , final JTextArea textArea
										 , final LoggingTable<L, M> tableIn) {
			final JMenuItem menuItem = new JMenuItem("//Kein Default Filter definiert//");
			final AFFEFactory<E, L> defaultFactory = field.createDefaultFFEFactory();
			menuItem.setEnabled(defaultFactory != null);
			if(defaultFactory != null){
				menuItem.setText("Fge hinzu: " + defaultFactory.asString());
				final AFFE<E, L> targetFFE = defaultFactory.create();
				menuItem.setEnabled(!editFilter.contains(field, targetFFE));
				if(targetFFE != null){
					menuItem.addActionListener(new ActionListener(){
						@Override public void actionPerformed(final ActionEvent e) {
							editFilter.add(field, targetFFE, tableIn);
							updateTextAreaFromFilter(textArea, editFilter, field);
						}
					});
				}
			}
			return menuItem;
		}
		private JMenu createMenuFFEConverters(final JTextArea textArea
											, final LinkedList<AFFE<E, L>> ffes
											, final AFFE<E, L> ffe
											, final AndOfFieldsFilter editFilter) {
			final JMenu menu = new JMenu("Editiere den Filter: " + ffe.asString());
			for(final AFFEFactory<E, L> factory : ffe.getFactories())
				menu.add(createMenuItemFFEConverter(textArea, ffes, ffe, editFilter, factory));
			menu.setEnabled(menu.getMenuComponentCount() > 0);
			return menu;
		}
		private JMenuItem createMenuItemFFEConverter(final JTextArea textArea
												   , final LinkedList<AFFE<E, L>> ffes
												   , final AFFE<E, L> ffe
												   , final AndOfFieldsFilter editFilter
												   , final AFFEFactory<E, L> factory) {
			final JMenuItem menuItem = new JMenuItem("Konvertieren zu: " + factory.asString());
			final AFFE<E, L> targetFFE = factory.create();
			menuItem.setEnabled(targetFFE != null);
			if(targetFFE != null){
				menuItem.addActionListener(new ActionListener(){
					@Override public void actionPerformed(final ActionEvent e) {
						final int indexOf = ffes.indexOf(ffe);
						ffes.removeFirstOccurrence(ffe);
						ffes.add(indexOf, targetFFE);
						updateTextAreaFromFilter(textArea, editFilter, ffe.getField());
					}
				});
			}
			return menuItem;
		}
		private JMenuItem createMenuItemTitle(final AField<E, L> field) {
			final String title = "Editier Hilfe fr die Filterung von " + field.getName() + ".";
			final JMenuItem titleMenuItem = new JMenuItem(title);
			titleMenuItem.setEnabled(false);
			return titleMenuItem;
		}
		private JMenuItem createMenuItemAlleEntfernen(final AField<E, L> field, final AndOfFieldsFilter editFilter, final JTextArea textArea) {
			final JMenuItem alleEntfernenMenuItem = new JMenuItem("Alle Filter von '" + field.getName() + "' entfernen.");
			alleEntfernenMenuItem.setEnabled(editFilter.filtert());
			alleEntfernenMenuItem.addActionListener(new ActionListener(){
				@Override public void actionPerformed(final ActionEvent e) {
					editFilter.get(field).removeAll();
					updateTextAreaFromFilter(textArea, editFilter, field);
				}
			});
			return alleEntfernenMenuItem;
		}
		private JMenuItem createMenuItemZuruecksetzen(final AField<E, L> field, final AndOfFieldsFilter editFilter, final JTextArea textArea) {
			final JMenuItem item = new JMenuItem();
			updateMenuItemZuruecksetzenText(item, field, getPreviousText(textArea));
			item.addActionListener(new ActionListener(){
				@Override public void actionPerformed(final ActionEvent e) {
					final String discardedText = textArea.getText();
					final Object prop = getPreviousText(textArea);
					final String restoredText = prop == null ? "" : prop.toString();
					putNewText(textArea, discardedText);	//so can toggle
					updateMenuItemZuruecksetzenText(item, field, discardedText);
					textArea.setText(restoredText);	//so text is different to stored previous text
				}

			});
			return item;
		}
		private String getPreviousText(final JTextArea textArea) {
			return (String)textArea.getClientProperty(MSTEVENT_TEXT_CHANGED);
		}
		private void putNewText(final JTextArea textArea, final String discardedText) {
			textArea.putClientProperty(MSTEVENT_TEXT_CHANGED, discardedText);
		}
		private void updateMenuItemZuruecksetzenText(final JMenuItem item, final AField<E, L> field, final String textAreaText) {
			final String text = textAreaText == null ? "" : textAreaText;
			item.setText("Die Filterung von " + field.getName() + " zurcksetzen auf: " + text);
		}
		private void removeOldListenerAndRememberNewOne(final JTextArea textArea, final MouseListener listener) {
			final String key = "KEY_OLD_LISTENER";
			final Object oldListener = textArea.getClientProperty(key);
			if(oldListener instanceof MouseListener)
				textArea.removeMouseListener((MouseListener)oldListener);
			textArea.putClientProperty(key, listener);
		}
		private int countRows(final String text) {
			int n = 0;
			int fromIndex = -1;
			do {
				n++;
				fromIndex = text.indexOf("\n", fromIndex + 1);
			} while (fromIndex >= 0);
			return n;
		}
		private MouseAdapter createMouseListenerForPopup(final JTextArea textArea, final JPopupMenu popupMenu) {
			final MouseAdapter listener = new MouseAdapter(){
				@Override public void mouseReleased(final MouseEvent evt) {
					handlePossiblePopup(evt);
				}
				@Override public void mousePressed(final MouseEvent evt) {
					handlePossiblePopup(evt);
				}
				private void handlePossiblePopup(final MouseEvent evt) {
					if(evt.isPopupTrigger())
						popupMenu.show(textArea, evt.getX(), evt.getY());
				}
			};
			removeOldListenerAndRememberNewOne(textArea, listener);
			return listener;
		}
	}//END AsJTextArea

	static class AsJTextAreaDate<E extends Serializable, L, M extends AMsgModel<L>> 	extends AsJTextArea<E, L, M>
																implements Serializable{
		private static final long serialVersionUID = 1L;
		private static final long   ONE_DAY_IN_MS      = 24L * 3600L * 1000L;
		private static final String MAX_YEAR = "2035";	//!Tine Limit beachten
		public AsJTextAreaDate(final String text, final String toolTip, final boolean isWideEditor) {
			super(text, toolTip, isWideEditor);
		}
		@Override void extendPanelAndTextArea(final JPanel panel
											, final AField<E, L> field
											, final EditorPanel editorPanel
											, final JTextArea textArea) {
			panel.add(Box.createVerticalStrut(20));

			/**Format: immer Zahl der Tage blank dann beliebig*/
			final String[] listData = {"0 Tage", "1 Tag", "3 Tage", "7 Tage", "14 Tage", "21 Tage", "28 Tage"};
			final JList list = new JList(listData);
			list.setAlignmentX(Component.CENTER_ALIGNMENT);
			list.getSelectionModel().addListSelectionListener(
					new ListSelectionListener(){
						@Override public void valueChanged(final ListSelectionEvent e) {
							final int index = list.getSelectedIndex();
							final ListModel model = list.getModel();
							if(index >= 0 && index < model.getSize())
								handle((String) model.getElementAt(index));
						}
						private void handle(final String selText) {
							final String sNDays = selText.substring(0, selText.indexOf(" "));
							final long nDays = Long.valueOf(sNDays);
							final long time = System.currentTimeMillis() - nDays * ONE_DAY_IN_MS;
							final String sBeginRoh = MstDateFormat.ISO.format(new Date(time));
							final String sBegin = sBeginRoh.substring(0, 10);
							// 2008-01-23
							// 0123456789
							//Monat und Tag von SBegin wegen ungefaehrer Offset Gleichtheit
							final String sEnd = MAX_YEAR + sBegin.substring(MAX_YEAR.length());
							final String sDateRange = sBegin + field.getRangeToken() + sEnd;
							textArea.setText(sDateRange);
							onNewText(field, editorPanel, editorPanel.getEditFilter(), textArea);
						}
					}
			);
			panel.add(list);
		}


	}//END AsJTextAreaDate

	static class AsJList<E extends Serializable, L, M extends AMsgModel<L>>	extends AFieldFilterGUIFactory<E, L, M>
																			implements Serializable {
		private static final long serialVersionUID = 1L;
		private static final boolean NEEDS_ENTRY_MAP = true;
		public AsJList(final String text, final String toolTip) {
			super(text, toolTip, NEEDS_ENTRY_MAP);
		}
		@Override public JPanel create(final AField<E, L> field
									 , final EditorPanel editorPanel){
			final JPanel panel = new JPanel();
			final FieldInfo<E, L> fieldInfo = editorPanel.getTable_()
														 .getModel()
														 .getLogSource()
														 .getAuswertung()
														 .getFieldInfo(field);
			if(!fieldInfo.hasInfo())
				return panel;

			panel.setLayout(new BoxLayout(panel, BoxLayout.Y_AXIS));

			final long anzahl = fieldInfo.getAnzahl();
			final JLabel label = new JLabel(getText() + "   (" + anzahl + ")");
			label.setToolTipText(getToolTip());
			panel.add(label);

			//$ANALYSIS-IGNORE
			final int maxVisibleRows = (int) Math.min(8, anzahl);
			final JList jList = createJListEditor_theJList(field, maxVisibleRows, editorPanel);
			jList.setToolTipText(label.getToolTipText());
			if(anzahl <= maxVisibleRows)
				panel.add(jList);
			else{
				final JScrollPane scrollPane = new JScrollPane();
				scrollPane.getViewport().add(jList);
				panel.add(scrollPane);
			}

			panel.add(createJListEditor_theButtonPanel(field, jList, editorPanel));

			panel.add(Box.createVerticalGlue());

			return panel;
		}
		private JPanel createJListEditor_theButtonPanel(
						  final AField<E, L> field
						, final JList jList
						, final FilterHandler<L ,M>.FilterPanel.EditorPanel editorPanel) {

			final JPanel buttonPanel = new JPanel();
			buttonPanel.setLayout(new BoxLayout(buttonPanel, BoxLayout.X_AXIS));

			buttonPanel.add(Box.createHorizontalGlue());

			final JButton jbKeine = new JButton("Keine");
			jbKeine.setAlignmentX(Component.RIGHT_ALIGNMENT);
			jbKeine.setToolTipText("Keine auswhlen");
			jbKeine.addActionListener(new ActionListener(){
				@Override public void actionPerformed(final ActionEvent e) {
					(editorPanel.getEditFilter().get(field)).removeAll();
					jList.repaint();
					editorPanel.onEditFilterChange();
				}
			});
			buttonPanel.add(jbKeine);

			final JButton jbAlle = new JButton("Alle");
			jbAlle.setAlignmentX(Component.RIGHT_ALIGNMENT);
			jbAlle.setToolTipText("Alle auswhlen");
			jbAlle.addActionListener(new ActionListener(){
				@Override public void actionPerformed(final ActionEvent e) {
					final FieldFilter filter = editorPanel.getEditFilter().get(field);
					((FieldFilterSized) filter).addAll();
					jList.repaint();
					editorPanel.onEditFilterChange();
				}
			});
			buttonPanel.add(jbAlle);
			return buttonPanel;
		}

		private JList createJListEditor_theJList(
				 final AField<E, L> field
			   , final int maxVisibleRows
			   , final FilterHandler<L, M>.FilterPanel.EditorPanel editorPanel) {

			final LoggingTable<L, M> table = (LoggingTable<L, M>)editorPanel.getTable_();
			final Object[] elements = table.getModel()
										   .getLogSource()
										   .getAuswertung()
										   .getFieldInfo(field)
										   .getElementsAsSortedArray();
			final AField<E, L>.FieldFilterSized filter = getFilterSized(field, editorPanel);
			if(filter.size() == 0)
				filter.addAll();
			final JList list = new JList(elements);
			list.setVisibleRowCount(maxVisibleRows);
			final TableCheckBoxRenderer<E, L, M> renderer =
				new TableCheckBoxRenderer<E, L, M>(filter, table.getModel(), field);
			list.setCellRenderer(renderer);
			list.addMouseListener(new MouseAdapter(){
				@Override public void mouseClicked(final MouseEvent e) {
					if(e.getX() < 60)	// in der Nhe der Checkbox
						handleCheck(list, field, editorPanel);
				}
			});
			list.addKeyListener(new KeyAdapter(){
				@Override public void keyPressed(final KeyEvent e) {
					if(e.getKeyChar() == ' ')//space means checkbox clicked
						handleCheck(list, field, editorPanel);
				}
			});
			return list;
		}
		private void handleCheck(final JList jList
								   , final AField<E, L> field
								   , final FilterHandler<L, M>.FilterPanel.EditorPanel editorPanel) {
			final int index = jList.getSelectedIndex();
			if(index < 0)
				return;
			final AField<E, L>.FieldFilter filter = getFilterSized(field, editorPanel);
			final E refValue = (E) editorPanel.getValue(jList, index);
			if(filter.passes(refValue)) {
				filter.remove(new FFEValueEquality<E, L>(field, refValue, false));
			} else
				filter.add	 (new FFEValueEquality<E, L>(field, refValue, false));
			jList.repaint();
			editorPanel.onEditFilterChange();
		}
		private AField<E, L>.FieldFilterSized getFilterSized(
											  final AField<E, L> field
											, final FilterHandler<L, M>.FilterPanel.EditorPanel editorPanel) {
			final FieldFilter filter = editorPanel.getEditFilter().get(field);
			if(filter instanceof FieldFilterSized)
				return (FieldFilterSized) filter;
			
			final FieldFilter newFilter = field.createFieldFilter(editorPanel.getTable_());
			if(newFilter instanceof FieldFilterSized){
				editorPanel.getEditFilter().add(newFilter);
				return (FieldFilterSized) newFilter;
			}
			Logger.SWFW.error("LoggingTableDisplay...getFilterSized("
							 + field + ") hat keinen solchen Filter");
			return null;
		}
	}//END AsJList

	static class ForFixedValues<E extends Serializable, L, M extends AMsgModel<L>>	extends AFieldFilterGUIFactory<E, L, M>
																					implements Serializable {
		private static final long serialVersionUID = 1L;
		private static final boolean NEEDS_ENTRY_MAP = true;
		//---------------------------------------------------------------------
		private final E[] theValues;
		//---------------------------------------------------------------------
		public ForFixedValues(final String text, final String toolTip, final E[] theValues) {
			super(text, toolTip, NEEDS_ENTRY_MAP);
			if(theValues == null)
				throw new NullPointerException();
			this.theValues = theValues;
		}
		//---------------------------------------------------------------------
		public E[] getTheValues() {return this.theValues;}
		@Override
		public JPanel create (final AField<E, L> field, final EditorPanel editorPanel){
			final JPanel panel = new JPanel();
			panel.setLayout(new BoxLayout(panel, BoxLayout.Y_AXIS));

			final JLabel label = new JLabel(getText());
			label.setToolTipText(getToolTip());
			panel.add(label);

			final E[] values = getTheValues();
			final AndOfFieldsFilter editFilter = editorPanel.getEditFilter();
			final AField<E, L>.FieldFilter levelFilter = editFilter.get(field);
			for (int i = 0; i < values.length; i++) {
				final E logLvl = values[i];
				final E e = logLvl;
				final boolean selected = (levelFilter == null)
									  || (levelFilter.passes(e));
				final long anzahl = editorPanel.getTable_()
											   .getModel()
											   .getLogSource()
											   .getAuswertung()
											   .getFieldInfo(field)
											   .getAnzahl(e);
				final String text = field.asString(e) + " (" + anzahl + ")";
				final JCheckBox checkBox = new JCheckBox(text, selected);
				checkBox.setToolTipText(getText() + " (Anzahl der Eintrge)");
				checkBox.addItemListener(new ItemListener(){
					@Override public void itemStateChanged(final ItemEvent evt) {
						final FieldFilter levelFilter = editFilter.get(field);
						if(levelFilter == null){
							final FieldFilter filter =
								field.createFieldFilter(editorPanel.getTable_());
							for (final E logLvl : values) {
								filter.add(new FFEValueEquality<E, L>(field, logLvl, false));
							}
							editFilter.add(filter);
							itemStateChanged(evt);//jetzt nochmal mit filter
							return;
						}
						final boolean isSelected = checkBox.isSelected();
						if(isSelected)
							levelFilter.add(new FFEValueEquality<E, L>(field, e, false));
						else
							levelFilter.remove(new FFEValueEquality<E, L>(field, e, false));
						editorPanel.onEditFilterChange();
					}
				});
				panel.add(checkBox);
			}

			panel.add(Box.createVerticalGlue());
			return panel;
		}

	}//END ForFixedValues

	static class ForMstKat	extends AFieldFilterGUIFactory<Logger, LogMsg, MsgModel_MstLog>
							implements Serializable {
		private static final long serialVersionUID = 1L;
		public ForMstKat(final String text, final String toolTip) {
			super(text, toolTip, true);
		}
		@Override
		public JPanel create (final AField<Logger, LogMsg> field, final EditorPanel editorPanel) {
			final JPanel panel = new JPanel();
			final LoggingTable<LogMsg, MsgModel_MstLog> table = editorPanel.getTable_();
			final FieldInfo<Logger, LogMsg> fieldInfo = table.getModel()
															 .getLogSource()
															 .getAuswertung()
															 .getFieldInfo(field);
			if(fieldInfo.getAnzahl() < 2)
				return panel;

			final AndOfFieldsFilter editFilter = editorPanel.getEditFilter();
			final AField<Logger, LogMsg>.FieldFilter filter = editFilter.get(field);
			if(filter == null){
				editFilter.add(field.createFieldFilter(table));
				return create(field, editorPanel);//nun ist filter != null
			}

			panel.setLayout(new BoxLayout(panel, BoxLayout.Y_AXIS));

			final JLabel label = new JLabel("Erlaubte Kategorien.  ");
			label.setToolTipText( "Welche Kategorie hat der Eintrag?"
								+ " (Anzahl:exakt/inklusive Unterkategorien)");
			panel.add(label);

			final KategorieTreeModel model = new KategorieTreeModel(fieldInfo);
			final JTree jTree = new JTree(model);
			for (int i = 0; i < jTree.getRowCount(); i++) {
				jTree.expandRow(i);
			}
			jTree.setVisibleRowCount(5);
			jTree.setCellRenderer(new TreeCheckBoxRenderer(filter));
			jTree.setCellEditor  (new TreeCheckBoxEditor  (table));
			jTree.setEditable(true);
			jTree.getCellEditor().addCellEditorListener(new CellEditorListener(){
				@Override public void editingCanceled(final ChangeEvent e) {/*NOP*/}
				@Override public void editingStopped(final ChangeEvent e) {
					handleCheck(jTree, filter, editorPanel);
				}
			});

			final JScrollPane scrollPane = new JScrollPane();
			scrollPane.getViewport().add(jTree);
			panel.add(scrollPane);

			panel.add(Box.createVerticalGlue());
			return panel;
		}
		private void handleCheck(final JTree tree
							   , final AField<Logger, LogMsg>.FieldFilter filter
							   , final FilterHandler<LogMsg, MsgModel_MstLog>.FilterPanel.EditorPanel editorPanel) {
			final TreePath selectionPath = tree.getSelectionPath();
			if(selectionPath == null)
				return;

			final Object[] path = selectionPath.getPath();
			if(path.length == 0)
				return;

			final Kategorie value = (Kategorie) path[path.length - 1];
			final FFEValueEquality<Logger, LogMsg> ffe =
				new FFEValueEquality<Logger, LogMsg>(filter.getField(), value.kat, false);
			if(filter.passes(value.kat)) {
				filter.remove(ffe);
			} else
				filter.add	 (ffe);
			tree.repaint();
			editorPanel.onEditFilterChange();
		}

		private static class TreeCheckBoxRenderer extends JCheckBox implements TreeCellRenderer {
			private static final long serialVersionUID = 1L;
			private final AField<Logger, LogMsg>.FieldFilter filter;
			private final DefaultTreeCellRenderer defaultRenderer;
			public TreeCheckBoxRenderer(final AField<Logger, LogMsg>.FieldFilter filter) {
				if(filter == null)
					throw new NullPointerException("Der filter darf nicht null sein.");
				this.filter = filter;
				this.defaultRenderer = new DefaultTreeCellRenderer();
			}
			/* (non-Javadoc)
			 * @see javax.swing.tree.TreeCellRenderer#getTreeCellRendererComponent(javax.swing.JTree, java.lang.Object, boolean, boolean, boolean, int, boolean)
			 */
			@Override public Component getTreeCellRendererComponent(
					  final JTree tree
					, final Object value
					, final boolean sel
					, final boolean expanded
					, final boolean leaf
					, final int row
					, final boolean hasFocus) {
				final Kategorie kategorie = (Kategorie)value;
				setText(kategorie.toString());
				setSelected(this.filter.passes(kategorie.kat));

				this.defaultRenderer.getTreeCellRendererComponent(tree, value, sel, expanded, leaf, row, hasFocus);
				setForeground(this.defaultRenderer.getForeground());
				setBackground(this.defaultRenderer.getBackground());
				return this;
			}
		}	//END TreeCheckBoxRenderer
		private static class TreeCheckBoxEditor   implements TreeCellEditor{
			final TreeCellEditor defaultEditor;
			private TreeCheckBoxEditor(final JTable table) {
				super();
				/**Hack um Boolean Editor i.E. Checkbox zu bekommen*/
				this.defaultEditor = (TreeCellEditor)
				table.getDefaultEditor(Boolean.class);
			}
			@Override public Component getTreeCellEditorComponent(final JTree                              tree, final Object value, final boolean isSelected, final boolean expanded, final boolean leaf, final int row) {
				final Component cmp = this.defaultEditor.getTreeCellEditorComponent(tree,        value,         isSelected,         expanded,         leaf,     row);
				if(cmp instanceof AbstractButton)	//sollte immer war sei
					((AbstractButton)cmp).setText(value.toString());
				return cmp; 
			}
			@Override public void addCellEditorListener(final CellEditorListener l) {
				this.defaultEditor.addCellEditorListener(l);
			}
			@Override public void cancelCellEditing() {
				this.defaultEditor.cancelCellEditing();
			}
			@Override public Object getCellEditorValue() {
				return this.defaultEditor.getCellEditorValue();
			}
			@Override public boolean isCellEditable(final EventObject anEvent) {
				return this.defaultEditor.isCellEditable(anEvent);
			}
			@Override public void removeCellEditorListener(final CellEditorListener l) {
				this.defaultEditor.removeCellEditorListener(l);
			}
			@Override public boolean shouldSelectCell(final EventObject anEvent) {
				return this.defaultEditor.shouldSelectCell(anEvent);
			}
			@Override public boolean stopCellEditing() {
				return this.defaultEditor.stopCellEditing();
			}
		}	//END TreeCheckBoxEditor
	}
	//END ForMstKat
}
