/**
 * 	Feb 25, 2008 - Mar 28, 2008	ws:	Creation from scratch
 * 	Apr  7, 2008 - Apr  7, 2008	ws: getEntryMap(..) mit <E> Param
 * 	Apr 14, 2008 - Apr 17, 2008	ws: added class	Kategorie mit TreeModell
 * 	May 19, 2008 - Jun  5, 2008	ws: now 	Generisch mit AMsgModel
 * 									added:	getFieldInfo
 * 									keep old Auswertungsinfo
 * 	Jun 17, 2008 - Jun 17, 2008	ws: added: hideColumnByDefault
 * 	Aug 28, 2008	- Aug 28, 2008	ws: Kommentare unberuecksichtigt
 *	Jan 15, 2009 - Jan 15, 2009	ws:	serializable
 *	Jul 23, 2009 - Jul 23, 2009	ws:	changed List<L> getMessages() -> IRList<L> ... 
 *	Sep 10, 2009 - Sep 10, 2009	ws:	added: public E[] getElementsAsSortedArray(final E[] a)  
 */

package de.desy.mst.libs.framework.mstapp.logging.gui;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.swing.event.TreeModelListener;
import javax.swing.tree.TreeModel;
import javax.swing.tree.TreePath;

import de.desy.mst.libs.framework.mstapp.logging.gui.LogMsgAuswertung.FieldInfo;
import de.desy.mst.libs.framework.mstapp.logging.LogMsg;
import de.desy.mst.libs.framework.mstapp.logging.Logger;

/**
 * Das GUI braucht ein paar elementare Informationen ueber die Messages eines
 * ILogProvider's. Diese werden hier geholt.
 * @author ws
 * @version 1.0.0
 */
class LogMsgAuswertung<L, M extends AMsgModel<L>> {
	private static final Map<Object, Integer> ENTRYMAP_NON_NULL =
								Collections.unmodifiableMap(createEntryMap());
//=============================================================================
	/**fuer einige Felder Werden die verschiedenen Entries gezaehlt*/
	private final Map<AField<?, L>, Map<Object, Integer>> entriesMap;
	private final ALogSource<L, M> logSource;
//=============================================================================
	public LogMsgAuswertung(final ALogSource<L, M> logSource
						  , final LogMsgAuswertung<L, M> oldAuswertung) {
		super();
		this.logSource = logSource;
		this.entriesMap = Collections.unmodifiableMap(createEntriesMap(oldAuswertung));
	}
//=============================================================================
	public final ALogSource<L, M> getLogSource() {return this.logSource;}
	public <E extends Serializable> FieldInfo<E, L> getFieldInfo(final AField<E, L> field){
		return new FieldInfo<E, L>(field, getLogSource(), getEntryMap(field));
	}
//=============================================================================
	/**@return Map mit (Key, Value) (Wert des Feldes, Anzahl) in msgs*/
	@SuppressWarnings("unchecked")
	private <E extends Serializable> Map<E, Integer> getEntryMap(final AField<E, L> field) {
		final Map<AField<?, L>, Map<Object, Integer>> entriesMap = getEntriesMap();
		final Object entryMap = (entriesMap.containsKey(field))
							  ? entriesMap.get(field) : null;
		final Object entryMapNonNull = (entryMap == null) ? ENTRYMAP_NON_NULL
														  : entryMap;
		return (Map<E, Integer>) entryMapNonNull;
	}
	private Map<AField<?, L>, Map<Object, Integer>> getEntriesMap(){
		return this.entriesMap;
	}
	/**only for XTor
	 * @param oldAuswertung */
	private Map<AField<?, L>, Map<Object, Integer>> createEntriesMap(final LogMsgAuswertung<L, M> oldAuswertung) {
		final Map<AField<?, L>, Map<Object, Integer>> entriesMap = new HashMap<AField<?, L>, Map<Object,Integer>>(4);
		initEntriesMap(entriesMap, oldAuswertung);
		fillEntriesMap(entriesMap);
		sealEntriesMap(entriesMap);
		return entriesMap;
	}
	private void initEntriesMap(final Map<AField<?, L>, Map<Object, Integer>> entriesMap
							  , final LogMsgAuswertung<L, M> oldAuswertung) {
		if(oldAuswertung == null)
			initEntriesMap(entriesMap, getLogSource().getMsgModel().getDiscreteFields());
		else
			initEntriesMap(entriesMap, oldAuswertung.getEntriesMap());
	}
	private void initEntriesMap(final Map<AField<?, L>, Map<Object, Integer>> entriesMap
			, final Collection<AField<?, L>> fields) {
		for (final AField<?, L> field : fields)
			entriesMap.put(field, createEntryMap());
	}
	private void initEntriesMap(final Map<AField<?, L>, Map<Object, Integer>> newEntriesMap
							  , final Map<AField<?, L>, Map<Object, Integer>> oldEntriesMap) {
		if(oldEntriesMap.size() == 0)
			return;
		final Set<AField<?, L>> oldFields = oldEntriesMap.keySet();
		for (final AField<?, L> field : oldFields) {
			final Collection<Object> oldFieldEntries = oldEntriesMap.get(field).keySet();
			if(oldFieldEntries.size() == 0)
				continue;
			final HashMap<Object, Integer> newMap = createEntryMap();
			newEntriesMap.put(field, newMap);
			for (final Object entry : oldFieldEntries) {
				newMap.put(entry, 0);	//alle Eintraege werden mit 0 gefuehrt
			}
		}
	}
	private static HashMap<Object, Integer> createEntryMap() {
		return new HashMap<Object, Integer>();
	}
	private void fillEntriesMap(final Map<AField<?, L>, Map<Object, Integer>> entriesMap) {
		final __IRList<L> messages = getLogSource().getMessages();
		final M msgModel = getLogSource().getMsgModel();
		final Set<AField<?, L>> fields = entriesMap.keySet();
		for (final L logMsg : messages) {
			if(msgModel.isComment(logMsg))
				continue;
			for (final AField<?, L> field : fields) {
				final Object key  = field.get(logMsg);
				final Map<Object, Integer> map = entriesMap.get(field);
				final int    anzahl = map.containsKey(key) ? map.get(key) + 1
															 : 1;
				map.put(key, anzahl);
			}
		}
	}
	/**makes the entryMap's in entriesMap readonly*/
	private void sealEntriesMap(
			final Map<AField<?, L>, Map<Object, Integer>> entriesMap) {

		final Set<AField<?, L>> keys = entriesMap.keySet();
		for (final AField<?, L> field : keys) {
			final Map<Object, Integer> entryMap = entriesMap.get(field);
			entriesMap.put(field, Collections.unmodifiableMap(entryMap));
		}
	}
//=============================================================================
	static class FieldInfo<E extends Serializable, L> {
		private final AField<E, L> 	field;
		private final ALogSource<L, ? extends AMsgModel<L>> logSource;
		private final Map<E, Integer> entryMap;
		//---------------------------------------------------------------------
		FieldInfo(final AField<E, L> field
				, final ALogSource<L, ? extends AMsgModel<L>> logSource
				, final Map<E, Integer> entryMap){
			super();
			this.field     = field;
			this.logSource = logSource;
			this.entryMap  = entryMap;
		}
		//---------------------------------------------------------------------
		public AField<E, L> getField() {return this.field;}

		/**zB anzahl > 1, keine duplicate entry column*/
		public boolean hasInfo()	{
			return (getField().isDiscrete() && getAnzahl() > 1)
				||!(getField().isDiscrete() || getField().hideColumnByDefault());
		}
		public String getToolTipText(){
			return this.hasInfo() ? getField().getDescription()
								  : "Wert: " + getElementOfFirstMsg();
		}
		/**Zahl der verschiedenen Eintraege falls bekannt sonst alle Eintraege*/
		public long getAnzahl()	{
			return hasEntryMap() ? getEntryMap().size()
								 : getLogSource().getSize();
		}
		/**Anzahl des Elements e*/
		public int getAnzahl(final E e) {
			if(hasEntryMap()){
				final Map<E, Integer> entryMap = getEntryMap();
				return (entryMap.containsKey(e)) ? entryMap.get(e) : 0;
			}
			return 0;
		}
		public Set<E> getElements() {
			return hasEntryMap() ? getEntryMap().keySet()
								 : Collections.<E>emptySet();
		}
		public E[] getElementsAsSortedArray(final E[] a) {
			final E[] array = getElements().toArray(a);
			Arrays.sort(array, new Comparator<E>(){
				@Override public int compare(final E o1, final E o2) {
					return getAnzahl(o2) - getAnzahl(o1);	//Descend.
				}
			});
			return array;
		}
		public Object[] getElementsAsSortedArray() {
			final Object[] array = getElements().toArray();
			Arrays.sort(array, new Comparator<Object>(){
				@SuppressWarnings("unchecked")
				@Override public int compare(final Object o1, final Object o2) {
					return getAnzahl((E) o2) - getAnzahl((E)o1);	//Descend.
				}
			});
			return array;
		}

		public boolean hasEntryMap() {return getField().isDiscrete();}

		//---------------------------------------------------------------------
		private final ALogSource<L, ? extends AMsgModel<L>> getLogSource() {
			return this.logSource;
		}
		private String getElementOfFirstMsg() {
			return (getLogSource().getSize() <= 0)
					? " (es existieren keine Meldungen)"
					: getField().asString
								(getField().get(getLogSource().getMessage(0)));
		}
		private Map<E, Integer> getEntryMap() {return this.entryMap;}
	}
}//END class LogMsgAuswertung

//=============================================================================
//=============================================================================
//=============================================================================


class KategorieTreeModel implements TreeModel{
	final private Kategorie root;
//=============================================================================
	public KategorieTreeModel(final FieldInfo<Logger, LogMsg> fieldInfo) {
		super();
		final Map<Logger, Kategorie> map = calcKategorieMap(fieldInfo);
		fillKategorieMapWithAllParents(map);
		fillKategorieChildren(map);
		final Kategorie root = map.get(Logger.Any);
		fillKategorieKumulierteAnzahl(root);
		this.root = root;
	}
	private Map<Logger, Kategorie> calcKategorieMap(final FieldInfo<Logger, LogMsg> fieldInfo) {
		final Set<Logger> values = fieldInfo.getElements();
		final Map<Logger, Kategorie> map = new HashMap<Logger, Kategorie>(11);
		for (final Logger logger : values) {
			map.put(logger, new Kategorie(logger, fieldInfo.getAnzahl(logger)));
		}
		return map;
	}
	private void fillKategorieMapWithAllParents(final Map<Logger, Kategorie> map) {
		final Logger[] keySet = map.keySet().toArray(new Logger[0]);//need copy
		for (final Logger key : keySet) {
			Logger parent = key.parent;
			while(!map.containsKey(parent)){
				map.put(parent, new Kategorie(parent, 0));
				parent = parent.parent;
			}
		}
	}
	private void fillKategorieChildren(final Map<Logger, Kategorie> map) {
		final Set<Logger> keySet = map.keySet();//need copy
		for (final Logger key : keySet) {
			if( key == Logger.Any)
				continue;
			final Kategorie value = map.get(key);
			final Kategorie parent = map.get(key.parent);
			parent.children.add(value);
		}
	}
	private void fillKategorieKumulierteAnzahl(final Kategorie kat) {
		final List<Kategorie> children = kat.children;
		for (final Kategorie child : children) {
			fillKategorieKumulierteAnzahl(child);
			kat.anzahlKum += child.anzahlKum;
		}
	}
//=============================================================================
	@Override public Kategorie getChild(final Object parent, final int index) {
		if(index < 0 || parent == null)
			return null;
		try {
			return ((Kategorie)parent).children.get(index);
		} catch (final Exception e) {
			return null;
		}
	}
	@Override public int getChildCount(final Object parent) {
		return ((Kategorie)parent).children.size();
	}
	@Override public int getIndexOfChild(final Object parent, final Object child) {
		if(parent == null || child == null)
			return -1;
		final List<Kategorie> children = ((Kategorie)parent).children;
		for (int i = 0; i < children.size(); i++) {
			if(children.get(i).equals(child))
				return i;
		}
		return -1;
	}
	@Override public Kategorie getRoot() {return this.root;}
	@Override public boolean isLeaf(final Object node) {return false;}
	@Override public void valueForPathChanged(final TreePath path, final Object newValue){/**ignore*/}
	@Override public void removeTreeModelListener	(final TreeModelListener l)	   {/**ignore*/}
	@Override public void addTreeModelListener	(final TreeModelListener l)	   {/**ignore*/}
	//-------------------------------------------------------------------------
	/**Kategorie Hierarchie wie in Auswertung gefunden*/
	class Kategorie{
		final Logger kat;
		final int    anzahl;
		final List<Kategorie> children = new ArrayList<Kategorie>(0);
		int   anzahlKum;
		Kategorie(final Logger kat, final int anzahl) {
			super();
			this.kat = kat;
			this.anzahl = anzahl;
			this.anzahlKum = anzahl;	//die anderen kommen spaeter dazu
		}
		@Override public String toString() {
			return this.kat.toString() + " (" + this.anzahl + " / " + this.anzahlKum + ")";
		} 
	}
}