/**
 * 	Jan 18, 2008 - Feb 11, 2008	ws:	Creation extraction of MstLogViewer
 * 	May 19, 2008 - May 21, 2008	ws: ILogSourceListener jetzt generisch
 * 	Jun 24, 2008 - Jun 26, 2008	ws: added afterCreated for LoggingTableDisplay
 *	Jul 14, 2008 - Jul 14, 2008	ws: add support for Protokoll Menu
 *	Jul 29, 2008 - Jul 29, 2008	ws: loggingDisplay renewed at structure changed
 *	Aug 26, 2008 - Aug 26, 2008	ws: added: externalOnNewLogSource
 *	Sep  4, 2008 - Sep  4, 2008	ws:	removed call to loggingDisplay.afterCreated
 *	Sep  5, 2008 - Sep  5, 2008	ws: added: onNewDisplayText(...)
 *	Feb  4, 2009 - Feb  5, 2009	ws:	support Ctrl+F4 to close a data tab (>0)
 *	Dec 22, 2009 - Dec 22, 2009	ws: added applyStandardMouseListener(final Component cmp)
 */
package de.desy.mst.libs.framework.mstapp.logging.gui;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Container;
import java.awt.KeyEventDispatcher;
import java.awt.KeyboardFocusManager;
import java.awt.event.KeyEvent;

import javax.swing.Icon;
import javax.swing.JEditorPane;
import javax.swing.JMenu;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTabbedPane;
import javax.swing.event.TableModelEvent;
import javax.swing.event.TableModelListener;

import de.desy.mst.libs.framework.mstapp.ApplicationInfo;
import de.desy.mst.libs.framework.mstapp.IMstAppKernel;
import de.desy.mst.libs.framework.mstapp.logging.Logger;
import de.desy.mst.libs.framework.mstapp.util.gui.GUIFactory;
import de.desy.mst.libs.framework.mstapp.util.gui.JTabbedPaneButton;

/**
 * Tabbed Pain mit Logging Darstellung und Auswahl
 * <br><b>Must call dispose() after use (memory leaks otherwise)<\b>
 * <br>===================================
 * @author ws
 * @version 1.0.0
 */
@SuppressWarnings({"rawtypes", "unchecked"})
public class LoggingJTabbedPane extends JTabbedPane
							    implements ILogSourceListener{
	private static final long serialVersionUID = 1L;
	//-------------------------------------------------------------------------
	private final IMstAppKernel mstApp;
	private final LoggingJFileChooser loggingJFileChooser;
	private final KeyEventDispatcher keyEventDispatcher = new KeyEventDispatcher(){
		@Override public boolean dispatchKeyEvent(final KeyEvent evt) {
			//System.out.println(" keyCode = " + evt.getKeyCode() + ", Source = " + evt.getSource());
			if( evt.getKeyCode() == KeyEvent.VK_F4 && evt.isControlDown()
			 && evt.getID()      == KeyEvent.KEY_RELEASED
			 && evt.getSource() instanceof Component){
				removeTabContainingTheSource((Component)evt.getSource());
				return true;
			}
			return false;
		}
	};

	//=============================================================================
	public LoggingJTabbedPane(final IMstAppKernel mstApp) {
		super();
		this.mstApp = mstApp;

		this.loggingJFileChooser = createLoggingJFileChooser(mstApp);
		this.addTab("Auswahl des Logs"
				  , (Icon) null
				  ,  getLoggingJFileChooser()
				  , "Hier kann ein Log (Protokoll) ausgesucht werden.");
		KeyboardFocusManager.getCurrentKeyboardFocusManager()
							.addKeyEventDispatcher(this.keyEventDispatcher);
	}
//=============================================================================
	/**must be called after finished using*/
	public void dispose() {
		KeyboardFocusManager.getCurrentKeyboardFocusManager()
							.removeKeyEventDispatcher(this.keyEventDispatcher);
	}

	/* (non-Javadoc)
	 * @see javax.swing.JTabbedPane#removeTabAt(int)
	 */
	@Override public void removeTabAt(final int index) {
		final Component cmp = getComponentAt(index);
		if(cmp instanceof LoggingTableDisplay) {
			final LoggingTableDisplay tableDisplay = (LoggingTableDisplay) cmp;
			if(tableDisplay.getTable().isDirty()){
				removeTabAt_InDirtyTableDisplay(index, tableDisplay);
				return;	//special treatment in removeTabAt_InDirtyTableDisplay
			}
		}
		super.removeTabAt(index);
	}
	/**Der User bekommt Gelegenheit: mit/ohne zu speichern den Tab zu entfernen
	 * oder weiter zu editieren*/
	private void removeTabAt_InDirtyTableDisplay(final int index, final LoggingTableDisplay tableDisplay) {
		final String msg = "Der Reiter hat editierte Felder. Sollen diese gespeichert werden?";
		switch (JOptionPane.showConfirmDialog(this, msg)) {
		case JOptionPane.OK_OPTION:
			final LoggingTableModel tableModel = tableDisplay.getLoggingTable().getModel();
			if(JOptionPane.OK_OPTION == LoggingTable.saveViaGui(tableDisplay, tableModel))
				super.removeTabAt(index);
			return;
		case JOptionPane.CANCEL_OPTION: /*Den Tab NICHT entfernen*/  return;
		case JOptionPane.NO_OPTION:     super.removeTabAt(index);    return;
		}
	}

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.framework.mstapp.logging.gui.ILogSourceListener#onNewLogSource(de.desy.mst.libs.framework.mstapp.logging.gui.ALogSource, java.lang.String, java.lang.String)
	 */
	@Override public void onNewLogSource(final ALogSource logSource, final String title, final String tip) {
		externalOnNewLogSource(logSource, title, tip);
	}
	/* (non-Javadoc)
	 * @see de.desy.mst.libs.framework.mstapp.logging.gui.ILogSourceListener#onNewDisplayText(java.lang.String, java.lang.String, java.lang.String, java.lang.String)
	 */
	@Override public void onNewDisplayText(final String contents, final String textType, final String title, final String tip) {
		createDisplayTab(contents, textType, title, tip);
	}
	/**pp
	 * @return relevant LoggingTableDisplay*/
	<L, M extends AMsgModel<L>> LoggingTableDisplay<L, M> externalOnNewLogSource(
											  final ALogSource<L, M> logSource
											, final String title
											, final String tip) {
		return createDisplayTab(logSource, title, tip);
	}

//=============================================================================
	/**@return JMenu zur Protokoll Auswahl*/
	public JMenu getJMenuLog() {return getLoggingJFileChooser().getJMenuLog();}
	/**@return JMenu zur CSV Datei Auswahl*/
	public JMenu getJMenuCSV() {return getLoggingJFileChooser().getJMenuCSV();}
//=============================================================================
	private LoggingJFileChooser createLoggingJFileChooser(final IMstAppKernel mstApp) {
		final ILogSourceListener logSourceListener = this;
		final ApplicationInfo appInfo = mstApp.getStartParameter()
											  .applicationInfo;
		final LoggingJFileChooser loggingJFileChooser =
						new LoggingJFileChooser(appInfo, logSourceListener);
		return loggingJFileChooser;
	}

	private void createDisplayTab(final String text, final String type, final String title, final String tip){
		final JPanel panel = new JPanel(new BorderLayout());
		JEditorPane editorPane;
		try {
			editorPane = new JEditorPane(type, text);
		} catch (final Exception ex) {
			Logger.SWFW.warn("LoggingJTabbedPane.createDisplayTab: Kann den text nicht darstellen als typ " + type, ex);
			final String conservativeType = "text/text";
			if(type.equals(conservativeType))
					editorPane = new JEditorPane(conservativeType, "Kann den original Text nicht darstellen.");
			else{
				createDisplayTab(text, conservativeType, title, tip);
				return;
			}
			editorPane = new JEditorPane(conservativeType, text);
		}
		editorPane.setFont(GUIFactory.createFixedWidthFont());
		editorPane.setEditable(false);	//nur darstellen
		panel.add(new JScrollPane(editorPane), BorderLayout.CENTER);

		this.addTab(title, (Icon) null, panel, tip);
		final int index = this.getTabCount() - 1;
		final JTabbedPaneButton component = new JTabbedPaneButton(this);
		this.setTabComponentAt(index, component);
		this.setSelectedIndex(index);
		applyStandardMouseListener(panel);
	}
	private void applyStandardMouseListener(final Component cmp) {
//		final boolean recursive = true;
//		getMstApp().getPopUpMenuMgr().applyStandardMouseListener(this, recursive);
//		if(cmp == null)
//			return;
//		final MouseListener[] mouseListeners = cmp.getMouseListeners();
//		if(cmp instanceof Container)
//			for (int iCmp = 0; iCmp < ((Container)cmp).getComponentCount(); iCmp++)
//				if(((Container)cmp).getComponent(iCmp) != cmp)
//					applyStandardMouseListener(((Container)cmp).getComponent(iCmp));
	}

	private <L, M extends AMsgModel<L>> LoggingTableDisplay<L, M> createDisplayTab(
											  final ALogSource<L, M> logSource
											, final String title
											, final String tip) {
		final LoggingTableDisplay<L, M> loggingDisplay = new LoggingTableDisplay<L, M>(getMstApp(), logSource, getLoggingJFileChooser().getMenuCSVMgr());
		this.addTab(title, (Icon) null, loggingDisplay, tip);
		final int index = this.getTabCount() - 1;
		final JTabbedPaneButton component = new JTabbedPaneButton(this);
		this.setTabComponentAt(index, component);
		this.setSelectedIndex(index);
		replaceDisplayTabOnLogSourceStructureChange(title, tip, loggingDisplay);
		applyStandardMouseListener(loggingDisplay);
		return loggingDisplay;
	}
	private void replaceDisplayTabOnLogSourceStructureChange(final String title, final String tip, final LoggingTableDisplay oldLoggingDisplay) {
		final LoggingTable table = oldLoggingDisplay.getTable();
		final AMsgModel oldMsgModel = table.getLogMsgModel();
		final TableModelListener tml = new TableModelListener(){
			@Override public void tableChanged(final TableModelEvent evt) {
				final AMsgModel newMsgModel = oldLoggingDisplay.getTable().getLogMsgModel();
				if(oldMsgModel.equalsInStructure(newMsgModel))
					return;
				final LoggingTableModel tableModel = table.getModel();
				tableModel.removeTableModelListener(this);
				final ALogSource newLogSource = tableModel.getLogSource();
				LoggingJTabbedPane.this.remove(oldLoggingDisplay);
				createDisplayTab(newLogSource, title, tip);
			}

		};
		table.getModel().addTableModelListener(tml);
	}
	private IMstAppKernel getMstApp() {return this.mstApp;}
	private LoggingJFileChooser getLoggingJFileChooser() {
		return this.loggingJFileChooser;
	}
	private void removeTabContainingTheSource(final Component source) {
		final int selectedIndex = getSelectedIndex();
		if(selectedIndex > 0 && isInParentChainOfSource(getSelectedComponent(), source))
			removeTabAt(selectedIndex);
	}
	private boolean isInParentChainOfSource(final Component removalCandidate, final Component source) {
		if(removalCandidate == source)
			return true;

		if(source == null)
			return false;

		final Container parent = source.getParent();
		if(source == parent)
			return false;

		return isInParentChainOfSource(removalCandidate, parent);
	}
}
