/**
 * 	Aug 12, 2008 - Aug 12, 2008	ws:	Creation from scratch/TindStdLogSource
 * 	Oct  6, 2008 - Oct  6, 2008	ws:	added getBezeichnung
 *	Jan 15, 2009 - Jan 15, 2009	ws:	serializable
 *	Jul 23, 2009 - Jul 23, 2009	ws:	changed List<L> getMessages() -> IRList<L> ... 
 */
package de.desy.mst.libs.framework.mstapp.logging.gui;

import java.io.File;
import java.io.IOException;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import de.desy.mst.libs.framework.mstapp.logging.gui.AFieldFilterGUIFactory.AsJTextArea;
import de.desy.mst.libs.framework.mstapp.util.MstParser;

/**
 * Fuer Watch Dog Text Logs
 * @author ws
 * @version 1.0.0
 */
class WDogStdLogSource extends ALogSource_File<WDogStdLogRow, WDogStdLogMsgModel> {
	public static WDogStdLogSource create(final File file) throws IOException {
		return create(ALogSource_File.calcInputLines(file), file);
	}
	public static WDogStdLogSource create(final String[] inputLines) {
		return create(inputLines, (File) null);
	}
	private static WDogStdLogSource create(final String[] inputLines, final File file) {
		return new WDogStdLogSource(WDogStdLogRow.createList(inputLines), file);
	}

//-----------------------------------------------------------------------------
	private List<WDogStdLogRow> msgs;
//-----------------------------------------------------------------------------
	private WDogStdLogSource(final List<WDogStdLogRow> msgs, final File file) {
		super(new WDogStdLogMsgModel(), file);
		this.msgs = Collections.unmodifiableList(msgs);
	}
//-----------------------------------------------------------------------------
	/* (non-Javadoc)
	 * @see de.desy.mst.libs.framework.mstapp.logging.gui.ALogSource#getMessages()
	 */
	@Override public __IRList<WDogStdLogRow> getMessages() {return new __ListAdapt<WDogStdLogRow>(this.msgs);}
	/* (non-Javadoc)
	 * @see de.desy.mst.libs.framework.mstapp.logging.gui.ALogSource#hasMassiveDecodingProblems()
	 */
	@Override public boolean hasMassiveDecodingProblems() {
		final __IRList<WDogStdLogRow> messages = getMessages();
		int countBad = 0;
		for (final SplitTextRow row : messages) {
			if(row.hasDecodingProblem())
				countBad++;
		}
		return countBad != 0 && countBad > 0.1 * messages.size();
	}
	/* (non-Javadoc)
	 * @see de.desy.mst.libs.framework.mstapp.logging.gui.ALogSource#reloadAllData()
	 */
	@Override public void reloadAllData() throws IOException { 
		final File file = getFile();
		if(file != null) {
			reloadAllData(file);
		}
	}

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.framework.mstapp.logging.gui.ALogSource#toString()
	 */
	@Override public String toString() {
		final File file = getFile();
		return "WDog Std Log Source ("
		 + ((file == null) ? "" : "', Datei = '" + file.getAbsolutePath())
		 + "') " + System.identityHashCode(this);
	}
	@Override public String getBezeichnung() {
		return getBezeichnung("WDog Std Log");
	}
//-----------------------------------------------------------------------------
	/**@throws NullPointerException for null file*/
	private void reloadAllData(final File file) throws IOException {
		this.msgs = WDogStdLogRow.createList(calcInputLines(file));
	}
}//END class WDogStdLogRow

//=============================================================================
//=============================================================================
//=============================================================================

class WDogStdLogRow extends SplitTextRow{
	public static List<WDogStdLogRow> createList(final String[] inputLines) {
		final List<WDogStdLogRow> list = new ArrayList<WDogStdLogRow>(inputLines.length);
		for (final String line : inputLines) list.add(new WDogStdLogRow(line));
		return list;
	}
	private static final String[] split(final String line) {
	//MINXPL3IPLRF 2008-08-08 17:17:00  wdog > process: V:\MCA\Bins32\L3Timer\TimerSrv.exe   3 started. Heart beat will not  be checked, according to cfg file.
		final String[] retVal = new String[3];
		if(line != null){
			final int begin1 = line.indexOf(" ");
			if(begin1 > -1){
				retVal[0] = line.substring(0, begin1);
				final int end1 = line.indexOf("w", begin1);
				if(end1 > 0){
					retVal[1] = line.substring(begin1, end1);
					retVal[2] = line.substring(end1);
				}else{
					retVal[1] = line.substring(begin1);
					retVal[2] = "";
				}
			}
		}
		return MstParser.normalize(retVal);
	}
//-----------------------------------------------------------------------------
	public WDogStdLogRow(final String line) {super(WDogStdLogRow.split(line));}
	//-----------------------------------------------------------------------------
	@Override public String getName() {return "WDogStdLogRow";}
}//END class WDogStdLogRow

//=============================================================================

class WDogStdLogMsgModel extends AMsgModel<WDogStdLogRow>{
	private static Map<String, ? extends AField<?, WDogStdLogRow>> createMap() {
		final Map<String, WDogStdLogField<?>> map = new HashMap<String,  WDogStdLogField<?>>();
		map.put(WDogStdLogField.PCNAME_FIELD.getName(), WDogStdLogField.PCNAME_FIELD);
		map.put(WDogStdLogField.  DATE_FIELD.getName(), WDogStdLogField. DATE_FIELD);
		map.put(WDogStdLogField.   MSG_FIELD.getName(), WDogStdLogField.  MSG_FIELD);
		return Collections.unmodifiableMap(map);
	}
//-----------------------------------------------------------------------------
	WDogStdLogMsgModel() {super(createMap());}
//-----------------------------------------------------------------------------
	@Override public AField<?, WDogStdLogRow> getCoreInfoField() {
		return WDogStdLogField.MSG_FIELD;
	}

	@Override public AField<Date, WDogStdLogRow> getDateField() {
		return null;	//im Augenblick zweites Feld nicht konvertiert
	}
}//END class WDogStdLogMsgModel

//=============================================================================

class WDogStdLogField<E extends Serializable> extends AField<E, WDogStdLogRow >{
	private static final long serialVersionUID = 1L;
	static final WDogStdLogField<String> PCNAME_FIELD;
	static final WDogStdLogField<String>   DATE_FIELD;
	static final WDogStdLogField<String>   MSG_FIELD;
	static{
		final boolean isWideEditor = true;
		PCNAME_FIELD = new WDogStdLogField<String>(0, "PC Name", "Name des PCs auf dem der Watchdog lief", 	String.class, "MSTDOALARM",					!isWideEditor);
		  DATE_FIELD = new WDogStdLogField<String>(1, "Datum",   "Datum im Original Textformat", 			String.class, "22.07.08 16:41:49", 			!isWideEditor);
		   MSG_FIELD = new WDogStdLogField<String>(2, "Meldung", "Watchdog Meldung", 						String.class, "Eine Meldung wwwwwwwwwwwwww", isWideEditor);
	}
	private static AsJTextArea<String, WDogStdLogRow, WDogStdLogMsgModel> createFFFactory(final String name, final String description, final boolean isWideEditor) {
		return new AFieldFilterGUIFactory.AsJTextArea<String, WDogStdLogRow, WDogStdLogMsgModel>(name, description, isWideEditor);
	}
	//-----------------------------------------------------------------------------
	public WDogStdLogField(final int index, final String name, final String description, final Class<E> clazz, final E valueForPrefSize, final boolean isWideEditor) {
		this(index, name, description, clazz, valueForPrefSize, createFFFactory(name, description, isWideEditor));
	}
	@SuppressWarnings({ "rawtypes", "unchecked" })
	private WDogStdLogField(final int index, final String name, final String description, final Class<E> clazz, final E valueForPrefSize, final AFieldFilterGUIFactory filterFactory) {
		super(index, name, description, clazz, valueForPrefSize, filterFactory);
	}
//-----------------------------------------------------------------------------
	@SuppressWarnings("unchecked")
	@Override E get(final WDogStdLogRow logRow) {return (E)logRow.get(getIndex());}
}//END class WDogStdLogRow


