/**
 *	Aug 18, 2009 - Sep 22, 2009	ws:	creation from scratch 
 */
package de.desy.mst.libs.framework.mstapp.logging.gui;

import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Window;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Date;
import java.util.concurrent.Callable;
import java.util.concurrent.ExecutionException;
import java.util.concurrent.Future;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.atomic.AtomicReference;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.ButtonGroup;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.SwingConstants;

import de.desy.mst.libs.framework.mstapp.Main;
import de.desy.mst.libs.framework.mstapp.logging.LogMsg;
import de.desy.mst.libs.framework.mstapp.logging.Logger;
import de.desy.mst.libs.framework.mstapp.logging.LoggerManager;
import de.desy.mst.libs.framework.mstapp.logging.gui.__LSMstLogInTineSrv.LSInfo;
import de.desy.mst.libs.framework.mstapp.logging.map_tinelogsrv.MstLog4TineLog_Facade;
import de.desy.mst.libs.framework.mstapp.logging.map_tinelogsrv.MstLog4TineLog_Facade.SystemInfoSummary;
import de.desy.mst.libs.framework.mstapp.logging.map_tinelogsrv.MstLog4TineLog_Facade.SystemInfoSummaryApp;
import de.desy.mst.libs.framework.mstapp.util.gui.AJPanelForUserInput;
import de.desy.mst.libs.framework.mstapp.util.gui.GUIFactory;
import de.desy.mst.libs.framework.mstapp.util.gui.JDialogWithCloseOnEscape;
import de.desy.tine.server.logger.Clog;

/**
 * Delivers a GUI  via get() for the Specification of an 
 * "ALogSource for MstLog In Tine Logging"
 * ILogSourceListener.onNewLogSource(...) might be fired
 * 
 * @author ws
 * @version 1.0.0
 */
@SuppressWarnings({"rawtypes", "unchecked"})
class __LSMstLogInTineSrv_SpecGUIFactory {
	/**
	 * Bridge zwischen SPECCASE Instanzen und AExekutor Instanzen pro __LSMstLogInTineSrv_SpecGUIFactory
	 * <br> Vorteil: lokales model, CASE weniger konvoluted
	 */
	enum SPECCASE{
		  Uebersicht (MstLog4TineLog_Facade.SYSTEM_KONTEXT) 
		, Applikation(MstLog4TineLog_Facade.SYSTEM_KONTEXT)
		, Run		 (null);
	//.........................................................................
	    private final String context;
	//.........................................................................
		private SPECCASE(final String context) {this.context = context;}
		public String getContext() {return this.context;}
	}//END: enum SPECCASE
	//=========================================================================
	enum ModelPart{SeitTagen, AppName, SystemInfoSummary}	//END: enum ModelPart
	//=========================================================================
	/**exakt ein Model Objekt pro enclosing __LSMstLogInTineSrv_SpecGUIFactory (need: onModelChange()*/
	class Model{
		private final ILogSourceListener lsListener;
		private int seitTagen;
		private String appName;
		private SystemInfoSummary sis;
		//.....................................................................
		Model(final ILogSourceListener lsListener, final __LSMstLogInTineSrv systemLS) {
			this.lsListener = lsListener;
			final SystemInfoSummaryApp defaultModelData = calcDefaultModelData(systemLS);
			this.seitTagen	= DEFAULT_SEIT_TAGEN;
			this.appName	= defaultModelData.appName;
			this.sis = new SystemInfoSummary(defaultModelData);//wegen polymorphic toString() need true sis
		}
		//.....................................................................
		final ILogSourceListener getLSListener(){return this.lsListener;}
		
		final int  getSeitTagen() {return this.seitTagen;}
		final void setSeitTagen(final int seitTagen){
			if(getSeitTagen() == seitTagen)
				return;
			this.seitTagen = seitTagen; 
			onModelChange(ModelPart.SeitTagen);
		}
		final String calcTageSuffix(){return " letzte " + getSeitTagen() + " Tage";}
		
		final String getAppName() {return this.appName;}
		final void setAppName(final String appName) {
			if(("" + getAppName()).equals(appName))
				return;
			this.appName = appName; 
			onModelChange(ModelPart.AppName);
		}
		
		final SystemInfoSummary getSysInfoSummary(){return this.sis;}
		final void setSystemInfoSum(final SystemInfoSummary sis) {
			if(sis != null && sis.equals(getSysInfoSummary()))
				return;
			this.sis = sis;
			onModelChange(ModelPart.SystemInfoSummary);
		}
		//.....................................................................
		private SystemInfoSummaryApp calcDefaultModelData(final __LSMstLogInTineSrv systemLS) {
			final boolean useThisApp = Logger._hasTineLogAppender() || systemLS == null || systemLS.getSize() <= 0;
			return useThisApp ? extractDefaultModelDataFromThisApp() : extractDefaultModelDataFromLastEntry(systemLS);
		}
		private SystemInfoSummaryApp extractDefaultModelDataFromThisApp() {
			return extractDefaultModelDataFromLogMsg(LoggerManager.getFirstLogMsg());
		}
		private SystemInfoSummaryApp extractDefaultModelDataFromLogMsg(final LogMsg msg) {
			return new SystemInfoSummaryApp(msg.runId, msg.date, msg.pcName, msg.applikation); 
		}
		private SystemInfoSummaryApp extractDefaultModelDataFromLastEntry(final __LSMstLogInTineSrv systemLS) {
			final LogSource_Clog lsTine = systemLS.getLSTine();
			final Clog msg = lsTine.getMessage(lsTine.getSize() -1);
			final MstLog4TineLog_Facade facade = MstLog4TineLog_Facade.getInstance();
			final LogMsg mappedMsg = facade.createClogMapper(msg).map(msg);
			
			return	new SystemInfoSummaryApp(facade.getSystemInfoSummary_FromSystemClog(msg), mappedMsg.applikation); 
		}
	}//END: class Model
	//=========================================================================
	/**zu Bridge: siehe CASE Kommentar<br> leider non static da XTor in CASE gebraucht*/
	abstract static class AExekutor{
		static final String TITLE_SUFFIX = " (Tine Log Server)";
		//.....................................................................
		private final Model model;
		//.....................................................................
		AExekutor(final Model model) {this.model = model;}
		//.....................................................................
		final Model getModel() {return this.model;}
		final ILogSourceListener getLSListener() 	{return getModel().getLSListener();}
		final int  				 getSeitTagen()  	{return getModel().getSeitTagen();}
		final String 			 getAppName()    	{return getModel().getAppName();}
		final SystemInfoSummary  getSysInfoSummary(){return getModel().getSysInfoSummary();}
		final String calcTageSuffix()	{return "letzte " + getSeitTagen() + " Tage";}
		/**concatinated mit " " dazwischen*/
		final String composeInfoText(final String ... txte){
			if(txte == null || txte.length == 0)
				return "";
			final StringBuilder sb = new StringBuilder(txte[0]);
			for (int i = 1; i < txte.length; i++)
				sb.append(" ").append(txte[i]);
			return sb.toString();
		}
		//.....................................................................
		abstract String calcInfoText();
		abstract void doAnzeigen();
		abstract void doEdit();
		void onModelChange(final ModelPart modelPart) {}
	}//END: class AExekutor
	//-------------------------------------------------------------------------
	static class ExekutorUeb extends AExekutor{
		private final AtomicReference <Future<__LSMstLogInTineSrv>> lsFutureRef;
		//.....................................................................
		ExekutorUeb(final Model model, final __LSMstLogInTineSrv systemLS) {
			super(model);
			this.lsFutureRef = createRefLSFuture(systemLS);
		}
		//.....................................................................
		@Override String calcInfoText() {return composeInfoText("bersicht", calcTageSuffix());}
		@Override void doAnzeigen() {
			getLSListener().onNewLogSource(
				  getLS()
				, "Alle MstApp Heartbeat Logs" + TITLE_SUFFIX	//Title
				, ""											//Tip
			);
		}
		@Override void doEdit() {
			final JDialog dlg = TageJPanel.createDialog(SPECCASE.Uebersicht, getModel());//new TageDialog(SPECCASE.Uebersicht, getModel());
			dlg.pack();
			dlg.setVisible(true);
		}
		@Override void onModelChange(final ModelPart modelPart) {
			if(ModelPart.SeitTagen != modelPart)
				return;
			Logger.SWFW.info("__LSMstLogInTineSrv_SpecGUIFactory.ExekutorUeb.onModelChange(" + modelPart + ").");
			updateCashedLS_asynch();
		}
		//.....................................................................
		private AtomicReference<Future<__LSMstLogInTineSrv>> createRefLSFuture(final __LSMstLogInTineSrv systemLS) {
			return new AtomicReference <Future<__LSMstLogInTineSrv>> (createLSFuture(systemLS));
		}
		private Future<__LSMstLogInTineSrv> createLSFuture(final __LSMstLogInTineSrv systemLS) {
				return systemLS == null ? createLSFuture() : createLSFutureDummy(systemLS);
		}
		private Future<__LSMstLogInTineSrv> createLSFuture() {
			return Main.getExecutorService().submit(new Callable<__LSMstLogInTineSrv>(){
				@Override public __LSMstLogInTineSrv call(){return createLS();}
			});
		}
		private Future<__LSMstLogInTineSrv> createLSFutureDummy(final __LSMstLogInTineSrv systemLS) {
			return new Future<__LSMstLogInTineSrv>() {
				@Override public boolean cancel(final boolean mayInterruptIfRunning)	{return false;}
				@Override public __LSMstLogInTineSrv get()	{return systemLS;}
				@Override public __LSMstLogInTineSrv get(final long timeout, final TimeUnit unit){return systemLS;}
				@Override public boolean isCancelled() 		{return false;}
				@Override public boolean isDone() 			{return true;}
			};
		}
		private __LSMstLogInTineSrv createLS() {
			return __LSMstLogInTineSrv.createSystemLS(getSeitTagen(), getLSListener());
		}
		private void updateCashedLS_asynch() {
			final Future<__LSMstLogInTineSrv> oldFuture = this.lsFutureRef.get();
			this.lsFutureRef.set(createLSFuture(null));
			if(oldFuture == null || oldFuture.isDone() || oldFuture.isCancelled())
				return;
			oldFuture.cancel(true);
		}
		public __LSMstLogInTineSrv getLS() {
			try {return this.lsFutureRef.get().get();} 
			catch (final InterruptedException	ex) {} 
			catch (final ExecutionException 	ex) {}
			//falls Threading Probleme mache es einfach synchron			
			return createLS();
		}
	}//END: class ExekutorUeb
	//-------------------------------------------------------------------------
	abstract static class AExekutor2 extends AExekutor{
		private final ExekutorUeb exUeb;
		AExekutor2(final Model model, final ExekutorUeb exUeb) {super(model); this.exUeb = exUeb;}
		//fresh from exUeb:
		final __LSMstLogInTineSrv getLSSystem() {return this.exUeb.getLS();}
	}//END: class AExekutor2
	//-------------------------------------------------------------------------
	static class ExekutorApp extends AExekutor2{
		ExekutorApp(final Model model, final ExekutorUeb exUeb) {super(model, exUeb);}
		//.....................................................................
		@Override String calcInfoText() {return composeInfoText(getAppName(), calcTageSuffix());}
		@Override void doAnzeigen() {
			getLSListener().onNewLogSource(__LSMstLogInTineSrv.createApplicationLS(getSeitTagen(), getAppName(), getLSListener(), getLSSystem())
					, getAppName() + TITLE_SUFFIX	//Title
					, "");							//Tip
		}
		@Override void doEdit() {
			final JDialog dlg = TageAppJPanel.createDialog(SPECCASE.Applikation, getModel(), getLSSystem());
			dlg.pack();
			dlg.setVisible(true);
		}
	}//END: class ExekutorApp
	//-------------------------------------------------------------------------
	static class ExekutorRun extends AExekutor2{
		ExekutorRun(final Model model, final ExekutorUeb exUeb){super(model, exUeb);}
		//.....................................................................
		@Override String calcInfoText() {return composeInfoText(getAppName(), getSysInfoSummary().toString(), calcTageSuffix());}
		@Override void doAnzeigen() {
			final String sRunId = getSysInfoSummary().sRunId;
			getLSListener().onNewLogSource(
					__LSMstLogInTineSrv.createRunLS(getSeitTagen(), getAppName(), sRunId, getLSListener(), getLSSystem())
				  , getAppName() + " " + sRunId + TITLE_SUFFIX		//Title
				  , getSysInfoSummary().toString());		//Tip
		}
		@Override void doEdit() {
			final JDialog dlg = TageAppRunJPanel.createDialog(SPECCASE.Run, getModel(), getLSSystem());
			dlg.pack();
			dlg.setVisible(true);
		}
	}//END: class ExekutorRun
//=============================================================================
	private static final String TITLE_MAIN	     =  "MstLog im Tine Server";
	private static final String TITLE_NAVIGATION =  "In neuem Tab anzeigen:";
	private static final int ABSTAND = 10;

	/**pp*/ static final int DEFAULT_SEIT_TAGEN = 7;
	
//=============================================================================
	private final LSInfo lsInfo;
	private final Model model;
	private final AExekutor[] exekutoren;
	
	private final JButton 	jbAnzeigen;
	private final JButton 	jbEditieren;
	private final JRadioButton[] rbCases;
	private final JLabel    jlInfo;
	private final Component instance;
	
//-----------------------------------------------------------------------------
	__LSMstLogInTineSrv_SpecGUIFactory(final __LSMstLogInTineSrv ls){
		this.lsInfo = ls.getLsInfo();
		this.model = new Model(getLsInfo().lsListener, getLsInfo().getOrCreateSystemLS(ls));

		this.exekutoren	 = createExekutoren(getModel(), getLsInfo().getOrCreateSystemLS(ls));

		this.jbAnzeigen  = createJBAnzeigen();
		this.jbEditieren = createJBEdit();
		this.rbCases	 = createRBCases();
		this.jlInfo		 = createJLInfo();
		this.instance 	 = createGUI();

		final SPECCASE iniCase = getLsInfo().speccase;
		getRbCase(iniCase).setSelected(true);
		onNewCaseSelection(iniCase);
	}
	//-----------------------------------------------------------------------------
	/**pp*/ final Component get() {return this.instance;}
	/**pp*/ final Model getModel() {return this.model;}
//-----------------------------------------------------------------------------
	
	private final LSInfo      getLsInfo    () {return this.lsInfo;} 			
	private final AExekutor[] getExekutoren() {return this.exekutoren;}
	private final AExekutor   getExekutor(final SPECCASE case_) {
		return getExekutoren()[case_.ordinal()];
	}
	private AExekutor getExekutor() {return getExekutor(getSelectedCase());}

	private final JButton 		getJbAnzeigen()	{return this.jbAnzeigen;}
	private final JButton 		getJbEditieren(){return this.jbEditieren;}
	private final JRadioButton[]getRbCases() 	{return this.rbCases;};
	private final JLabel 		getJlInfo() 	{return this.jlInfo;}
	private JRadioButton getRbCase(final SPECCASE case_) {
		return getRbCases()[case_.ordinal()];
	}
	//.........................................................................
	
	private SPECCASE getSelectedCase() {
		for (final SPECCASE cas : SPECCASE.values()) 
			if(getRbCase(cas).isSelected())
				return cas;
		return SPECCASE.Uebersicht;	//just in case
	}
	private void onNewCaseSelection(final SPECCASE case_) {
		updateGUIToNewModel();
	}
	
	private void onModelChange(final ModelPart modelPart){
		updateGUIToNewModel();
		notifyExekutoren(modelPart);
	}
	private void updateGUIToNewModel() {
		final SPECCASE selectedCase = getSelectedCase();
		for (final SPECCASE specCase : SPECCASE.values()) {
			final String txt = getExekutor(specCase).calcInfoText();
			getRbCase(specCase).setToolTipText(txt);
			if(specCase.equals(selectedCase)){
				getJlInfo().setText(txt);
				getJlInfo().setToolTipText(txt);
			}
		}
	}
	private void notifyExekutoren(final ModelPart modelPart) {
		for (final AExekutor exec : getExekutoren()) 
			exec.onModelChange(modelPart);
	}
	
	//.........................................................................
	

	private AExekutor[] createExekutoren(final Model model, final __LSMstLogInTineSrv systemLS) {
		final ExekutorUeb exekutorUeb = new ExekutorUeb(model, systemLS);
		final ExekutorApp exekutorApp = new ExekutorApp(model, exekutorUeb);
		final AExekutor exekutorRun = new ExekutorRun(model, exekutorUeb);
		
		final AExekutor[]  exeks = new AExekutor[SPECCASE.values().length];
		exeks[SPECCASE.Uebersicht	.ordinal()] = exekutorUeb;
		exeks[SPECCASE.Applikation	.ordinal()] = exekutorApp;
		exeks[SPECCASE.Run			.ordinal()] = exekutorRun;
		return exeks;
	}

	private JButton createJBAnzeigen() {
		final JButton jButton = new JButton("Anzeigen:");
		jButton.addActionListener(new ActionListener() {
			@Override public void actionPerformed(final ActionEvent e) {
				getExekutor().doAnzeigen();
			}
		});
		return jButton;
	}
	private JButton createJBEdit() {
		final JButton jButton = new JButton("Editieren der Anzeige-Parameter:");
		jButton.addActionListener(new ActionListener() {
			@Override public void actionPerformed(final ActionEvent e) {
				getExekutor().doEdit();
			}
		});
		return jButton;
	}
	private JRadioButton[] createRBCases() {
		final ButtonGroup bGr = new ButtonGroup();
		final SPECCASE[] cases = SPECCASE.values();
		final int nCases = cases.length;
		final JRadioButton[]  rbs = new JRadioButton[nCases];
		for (int i = 0; i < nCases; i++) {
			final SPECCASE case_ = cases[i];
			final JRadioButton rb = new JRadioButton(case_.name());
			bGr.add(rb);
			rb.setToolTipText(getExekutor(case_).calcInfoText());
			rb.addActionListener(new ActionListener() {
				@Override public void actionPerformed(final ActionEvent evt) {
					onNewCaseSelection(case_);
				}
			});
			rbs[i] =  rb;
		}
		return rbs;
	}
	private JLabel createJLInfo() {
		final JLabel lbl = new JLabel("xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx");
		lbl.setBorder(GUIFactory.createTitledBorder(""));
		return lbl;
	}
	private JPanel createGUI() {
		final JPanel pnl = new JPanel();
		pnl.setLayout(new BoxLayout(pnl, BoxLayout.Y_AXIS));
		pnl.setBorder(GUIFactory.createTitledBorder(TITLE_MAIN));
	
		final JPanel navigationPanel = createNavigationPanel();
		final JPanel topPanel        = createTopPanel(navigationPanel);
		pnl.add(topPanel);
		pnl.add(navigationPanel);
		return pnl;
	}
	private JPanel createTopPanel(final JPanel navigationPanel) {
		final JPanel pnl = new JPanel();
		pnl.setLayout(new BoxLayout(pnl, BoxLayout.X_AXIS));
		final JButton jButton = createButtonDetails(navigationPanel, "Mehr");
		final JLabel  jLabel = new JLabel(getLsInfo().speccase + ": " + getLsInfo().toString());
		
		pnl.add(jButton);
		pnl.add(Box.createHorizontalStrut(2*ABSTAND));

		pnl.add(jLabel);
		pnl.add(Box.createHorizontalGlue());
		
		return pnl;
	}
	private enum DetailState{SHOW(true,  createIcon("/resources/io/Up16.gif"  ))
						   , HIDE(false, createIcon("/resources/io/Down16.gif"));
		private final boolean show; private final ImageIcon icon;
		private DetailState(final boolean showDetailState, final ImageIcon icon) {
			this.show = showDetailState; this.icon = icon;
		}
		private void updateToState(final JComponent detailCmp, final JButton jButton){
			detailCmp.setVisible(this.show); jButton  .setIcon   (this.icon);
		}
		private static ImageIcon createIcon(final String name){
			try {return new ImageIcon(DetailState.class.getResource(name));} 
			catch (final Exception ex) {
				Logger.SWFW.warn("Can not create icon from \"" + name + "\"", ex);
				return null;
			}
		}
	}
	private JButton createButtonDetails(final JComponent detailCmp, final String title) {
		final JButton jButton = new JButton(title){
			private static final long serialVersionUID = 1L;
			@Override public Dimension getMaximumSize(){
				final Dimension preferredSize = getPreferredSize();
				return new Dimension(preferredSize.width, Math.min(preferredSize.height, 14));
			}
		};
		DetailState.HIDE.updateToState(detailCmp, jButton);
		jButton.addActionListener(new ActionListener() {
			@Override public void actionPerformed(final ActionEvent evt) {
				nextState(detailCmp).updateToState(detailCmp, jButton);
			}
			private DetailState nextState(final JComponent detailCmp) {
				return detailCmp.isVisible() ? DetailState.HIDE : DetailState.SHOW;
			}
		});
		return jButton;
	}
	private JPanel createNavigationPanel() {
		final JPanel pnl = new JPanel();
		pnl.setLayout(new BoxLayout(pnl, BoxLayout.X_AXIS));
		pnl.setBorder(GUIFactory.createTitledBorder(TITLE_NAVIGATION));
	
		pnl.add(getJbAnzeigen());
		pnl.add(Box.createHorizontalStrut(ABSTAND));
		
		pnl.add(getJbEditieren());
		pnl.add(Box.createHorizontalStrut(2*ABSTAND));
	
		for (final JRadioButton jRadioButton : getRbCases())
			pnl.add(jRadioButton);
		pnl.add(Box.createHorizontalStrut(ABSTAND));
	
		pnl.add(getJlInfo());
		pnl.add(Box.createHorizontalGlue());
		
		return pnl;
	}
	
//=============================================================================
	static class TageJPanel extends AJPanelForUserInput{
		public static JDialog createDialog(final SPECCASE sCase, final Model model){
			final String title = sCase.name() + " editieren";
			final JDialog dlg = new JDialogWithCloseOnEscape((JFrame)null, title, true);
			final TageJPanel pnl = new TageJPanel(dlg, sCase, model);
			pnl.model2GUI();	//nicht im XTor
			dlg.add(pnl);
			return dlg;
		}
		private static final long serialVersionUID = 1L;
		/**pp*/ static final Integer[] TAGE = {0, 1, 2, 3, 7, 14, 31};
		//.....................................................................
		private final Model model;
		private final JLabel lbl;
		private final JComboBox cbTage;
		//.....................................................................
		public TageJPanel(final Window wnd, final SPECCASE sCase, final Model model) {
			super(wnd);
			this.setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));

			this.model = model;
			
			this.lbl    = createAndRegisterLbl   (this);
			this.cbTage = createAndRegisterCBTage(this);
		}
		//.....................................................................
		@Override public final void onSet  (final ActionEvent evt) {gui2Model();}
		@Override public final void onReset(final ActionEvent evt) {model2GUI();}
		//.....................................................................
		/**pp*/ final Model 	getModel ()	{return this.model;}
		/**pp*/ final JComboBox getCbTage()	{return this.cbTage;}
		/**pp*/ void model2GUI() {onNewSpec(getModel().getSeitTagen());}
		/**pp*/ void gui2Model() {
			final Integer nTage = (Integer) getCbTage().getSelectedItem();
			getModel().setSeitTagen(nTage);
			onNewSpec(nTage);	//fuer different Indikator
		}
		//.....................................................................
		private final JLabel getLbl() {return this.lbl;}
		private final void onNewSpec(final Integer nTage) {
			toCBTage(nTage);
			getCbTage().setSelectedItem(nTage);
			final boolean nTageGeaendert = !((Integer)this.model.getSeitTagen()).equals(nTage);
			getLbl().setForeground(nTageGeaendert ? Color.BLACK : Color.GRAY);
			getLbl().setText("Seit " + nTage + " Tagen.");
			if(nTageGeaendert)
				setDirty();
		}
		private void toCBTage(final int seitTagen) {
			if(getCbTage().getSelectedItem().equals(seitTagen))
				return;	//vorhanden und gesetzt
			 
			getCbTage().setSelectedItem(seitTagen);//versuche es
			if(getCbTage().getSelectedItem().equals(seitTagen))
				return;	//erfolg seitTage existierte
			
			getCbTage().addItem(seitTagen);	//erzeuge seitTagen neu
			getCbTage().setSelectedItem(seitTagen);
		}
		private JLabel createAndRegisterLbl(final JPanel pnl) {
			final JLabel lbl = new JLabel("Platzhalter fuer Tage Auswahl.");
			lbl.setAlignmentX(Component.LEFT_ALIGNMENT);		//Comp. itself
			lbl.setHorizontalAlignment(SwingConstants.LEFT);	//Text in Comp.
			pnl.add(lbl);
			return lbl;
		}
		private JComboBox createAndRegisterCBTage(final JPanel jPanel) {
			final JComboBox cb = new JComboBox(TAGE);
			cb.setEditable(true);
			cb.addActionListener(new ActionListener() {
				final Color oriBackground = cb.getBackground();
				@Override public void actionPerformed(final ActionEvent evt) {
					final Object selectedItem = getCbTage().getSelectedItem();
					markValid(true);	//default
					if(selectedItem instanceof Integer)
						onNewSpec((Integer)selectedItem);
					else if(selectedItem instanceof String)
						try {onNewSpec(convert((String)selectedItem));} 
						catch (final NumberFormatException ex) {markValid(false);}
					else
						markValid(false);
				}
				private void markValid(final boolean isValid) {
					getCbTage().setBackground(isValid? this.oriBackground : Color.RED);
				}
			});
			jPanel.add(cb);
			return cb;
		}
		private int convert(final String intCandidat) throws NumberFormatException{
			return Integer.parseInt(intCandidat);
		}
	}//END class: TageJPanel
	//-------------------------------------------------------------------------
	static class TageAppJPanel extends TageJPanel{
		public static JDialog createDialog(final SPECCASE sCase, final Model model, final IAppNamesProvider appNameProvider){
			final String title = sCase.name() + " editieren";
			final JDialog dlg = new JDialogWithCloseOnEscape((JFrame)null, title, true);
			final TageAppJPanel pnl = new TageAppJPanel(dlg, sCase, model, appNameProvider);
			dlg.add(pnl);
			pnl.model2GUI();	//nicht im XTor
			return dlg;
		}
		private static final long serialVersionUID = 1L;
		//.....................................................................
		private final IAppNamesProvider appNameProvider;
		private final JLabel lbl;
		private final JComboBox cbAppName;
		//.....................................................................
		private TageAppJPanel(final Window wnd, final SPECCASE sCase, final Model model, final IAppNamesProvider appNameProvider) {
			super(wnd, sCase, model);
			this.appNameProvider = appNameProvider;
			this.add(Box.createVerticalStrut(ABSTAND));
			this.lbl = createAndRegisterLbl(this);
			this.cbAppName = createAndRegisterCBAppName(this, getAppNameProvider().getAppNames());
		}
		//.....................................................................
		@Override void gui2Model() {
			super.gui2Model();
			final String appName = "" + getCbAppName().getSelectedItem();
			getModel().setAppName(appName);
			onNewSpec(appName);	//fuer different Indikator
		}
		@Override void model2GUI() {super.model2GUI(); onNewSpec(getModel().getAppName());}
		//.....................................................................
		private final IAppNamesProvider getAppNameProvider() {return this.appNameProvider;}
		/**pp*/ final JComboBox getCbAppName()	{return this.cbAppName;}
		private final JLabel 	getLbl() 		{return this.lbl;}
		private void onNewSpec(final String appName) {
			toCBAppName(appName);
			getCbAppName().setSelectedItem(appName);
			final boolean nAppNameGeaendert = !(getModel().getAppName()).equals(appName);
			getLbl().setForeground(nAppNameGeaendert ? Color.BLACK : Color.GRAY);
			getLbl().setText("Applikation: \"" + appName + "\"");
			if(nAppNameGeaendert)
				setDirty();
		}
		private void toCBAppName(final String appName) {
			if(("" + getCbAppName().getSelectedItem()).equals(appName))
				return;	//vorhanden und gesetzt

			getCbAppName().setSelectedItem(appName);//versuche es
			if(getCbAppName().getSelectedItem().equals(appName))
				return;	//erfolg seitAppName existierte
 
			getCbAppName().addItem(appName);	//erzeuge appName neu
			getCbAppName().setSelectedItem(appName);
		}
		private JLabel createAndRegisterLbl(final JPanel pnl) {
			final JLabel lbl = new JLabel("Platzhalter fuer AppName Auswahl.");
			lbl.setAlignmentX(Component.LEFT_ALIGNMENT);		//Comp. itself
			lbl.setHorizontalAlignment(SwingConstants.LEFT);	//Text in Comp.
			pnl.add(lbl);
			return lbl;
		}
		private JComboBox createAndRegisterCBAppName(final JPanel jPanel, final String ... appNames) {
			final JComboBox cb = new JComboBox(appNames);
			cb.setEditable(true);
			cb.addActionListener(new ActionListener() {
				@Override public void actionPerformed(final ActionEvent evt) {
					onNewSpec("" + getCbAppName().getSelectedItem());
				}
			});
			jPanel.add(cb);
			return cb;
		}
	}//END class: TageJPanelApp
	//-------------------------------------------------------------------------
	static class TageAppRunJPanel extends TageAppJPanel{
		public static JDialog createDialog(final SPECCASE sCase, final Model model, final ISysInfosProvider sisProvider){
			final String title = sCase.name() + " editieren";
			final JDialog dlg = new JDialogWithCloseOnEscape((JFrame)null, title, true);
			final TageAppRunJPanel pnl = new TageAppRunJPanel(dlg, sCase, model, sisProvider);
			dlg.add(pnl);
			pnl.model2GUI();	//nicht im XTor
			return dlg;
		}
		private static final long serialVersionUID = 1L;
		//.....................................................................
		private final ISysInfosProvider sisProvider;
		private final JLabel lbl;
		private final JComboBox cbSysInfo;
		private       String oldAppName = null;
		//.....................................................................
		private TageAppRunJPanel(final Window wnd, final SPECCASE sCase, final Model model, final ISysInfosProvider sisProvider) {
			super(wnd, sCase, model, sisProvider);
			this.sisProvider = sisProvider;
			this.lbl = createAndRegisterLbl(this);
			this.cbSysInfo = createAndRegisterCBSysInfo(this);
			this.oldAppName = model.getAppName();
			getCbAppName().addActionListener(new ActionListener() {
				@Override public void actionPerformed(final ActionEvent evt) {
					final Object newAppName = getCbAppName().getSelectedItem();
					if(newAppName instanceof String && !newAppName.equals(getOldAppName()))
						updateCBSysInfo("" + newAppName);
				}
			});
		}
		//.....................................................................
		@Override void gui2Model() {
			final Object selectedItem = getCbSysInfo().getSelectedItem();
			if(selectedItem instanceof SystemInfoSummary){
				final SystemInfoSummary sis = (SystemInfoSummary)selectedItem;
				getModel().setSystemInfoSum(sis);
				onNewSpec(sis);
			}
		}
		@Override void model2GUI(){super.model2GUI();onNewSpec(getModel().getSysInfoSummary());}
		//.....................................................................
		/**pp*/ final JComboBox getCbSysInfo()	{return this.cbSysInfo;}
		private final String getOldAppName() 			   {return this.oldAppName;}
		private final void setOldAppName(final String oldAppName) {this.oldAppName = oldAppName;}
		private final JLabel 	getLbl() 		{return this.lbl;}
		private void onNewSpec(final SystemInfoSummary sis) {
			toCbSysInfoSum(sis);
			getCbSysInfo().setSelectedItem(sis);
			final boolean sisGeaendert = !(getModel().getSysInfoSummary()).equals(sis);
			getLbl().setForeground(sisGeaendert ? Color.BLACK : Color.GRAY);
			getLbl().setText("" + sis);
			if(sisGeaendert)
				setDirty();
		}
		private void updateCBSysInfo(final String appName) {
			final JComboBox cb = getCbSysInfo();
			cb.removeAllItems();
			fillSystemInfoSummaries(cb, getSisProvider().getSISs(appName));
			selectLastItem(cb);
			setOldAppName(appName);
		}
		private void fillSystemInfoSummaries(final JComboBox cb, final SystemInfoSummary[] siSs) {
			for (final SystemInfoSummary sis : siSs) 
				cb.addItem(sis);
		}
		private void selectLastItem(final JComboBox cb) {
			final int size = cb.getModel().getSize();
			if(size > 0)
				cb.setSelectedIndex(size - 1);
		}
		
		private void toCbSysInfoSum(final SystemInfoSummary sis) {
			if(sis == null || sis.equals(getCbSysInfo().getSelectedItem()))
				return;	//HUCH || vorhanden und gesetzt

			getCbSysInfo().setSelectedItem(sis);//versuche es
			if(sis.equals(getCbSysInfo().getSelectedItem()))
				return;	//erfolg seitAppName existierte

			getCbSysInfo().addItem(sis);	//erzeuge spec neu
			getCbSysInfo().setSelectedItem(sis);
		}
		private JLabel createAndRegisterLbl(final JPanel pnl) {
			final JLabel lbl = new JLabel("Platzhalter fuer SystemInfoSummary Auswahl.");
			lbl.setAlignmentX(Component.LEFT_ALIGNMENT);		//Comp. itself
			lbl.setHorizontalAlignment(SwingConstants.LEFT);	//Text in Comp.
			pnl.add(lbl);
			return lbl;
		}
		private JComboBox createAndRegisterCBSysInfo(final JPanel jPanel) {
			final SystemInfoSummary[] sisArray = getSisProvider().getSISs(getModel().getAppName());
			final JComboBox cb = new JComboBox(sisArray);
			cb.setEditable(true);
			cb.addActionListener(new ActionListener() {
				@Override public void actionPerformed(final ActionEvent evt) {
					final Object selectedItem = getCbSysInfo().getSelectedItem();
					if(selectedItem instanceof SystemInfoSummary)
						onNewSpec((SystemInfoSummary)selectedItem);
					else if(("" + selectedItem).isEmpty())
						/*NoOp*/;
					else
						onNewSpec(createSystemInfoSummary("" + selectedItem));
				}
				private SystemInfoSummary createSystemInfoSummary(final String sRunID) {
					return new SystemInfoSummary(sRunID, new Date(0), "?PCNAME?");
				}
			});
			jPanel.add(cb);
			return cb;
		}
		private final ISysInfosProvider getSisProvider() {return this.sisProvider;}
	}//END class: TageJPanelApp
}
