/**
 *	Jul 30, 2009 - Jul 30, 2009	ws:	creation from scratch 
 */
package de.desy.mst.libs.framework.mstapp.logging.gui;

import java.lang.ref.SoftReference;
import java.util.ArrayList;
import java.util.Collections;
import java.util.LinkedList;
import java.util.List;

import de.desy.mst.libs.framework.mstapp.logging.ILogProvider;
import de.desy.mst.libs.framework.mstapp.logging.LogMsg;
import de.desy.mst.libs.framework.mstapp.logging.LoggerManager;
import de.desy.mst.libs.framework.mstapp.logging.appender.ILogAppenderKern;
import de.desy.mst.libs.framework.mstapp.logging.gui.__IAutoAppend.IAAClient;
import de.desy.mst.libs.framework.mstapp.logging.gui.__IAutoAppend.IAAServer;

/**
 * Singleton ALogSource for System Logger
 * <br>(de)registration via SoftReference (Otherwise danger of memory leak) 
 * @author ws
 * @version 1.0.0
 */
final class __LSSystem extends ALogSourceILogProvider {
	private static final String NAME = "SystemLogProvider";
	private static final __LSSystem instance = createInstance();
	private static __LSSystem createInstance() {
		final ILogProvider systemLogProvider = LoggerManager.getSystemLogProvider();
		final AAServer4LogSourceSystem autoAppendServer = new AAServer4LogSourceSystem(systemLogProvider);
		final MsgModel_MstLog msgModel = MsgModel_MstLog.getInstance();
		return new __LSSystem(systemLogProvider, autoAppendServer, msgModel);
	}
//-----------------------------------------------------------------------------	
	static __LSSystem getInstance() {return __LSSystem.instance;}
//=============================================================================	
	__LSSystem(final ILogProvider logProvider,
			final IAAServer<LogMsg> autoAppendServer, final MsgModel_MstLog logMsgModel) {
		super(logProvider, autoAppendServer, logMsgModel);
	}
//-----------------------------------------------------------------------------
	
	@Override public void    reloadAllData()	{/*SystemLogProvider immer frisch*/}
	@Override public String  toString() 		{return __LSSystem.NAME + " " + System.identityHashCode(this);}
	@Override public String  getBezeichnung()	{return getBezeichnung(__LSSystem.NAME);}
	@Override public boolean hasMassiveDecodingProblems() {return false;}
	
//=============================================================================	
	private static final class AAServer4LogSourceSystem implements IAAServer<LogMsg>, ILogAppenderKern {
		private final List<SoftReference<IAAClient<LogMsg>>> refClients = Collections.synchronizedList(new LinkedList<SoftReference<IAAClient<LogMsg>>>());
		private final ILogProvider logProvider;
		//---------------------------------------------------------------------
		public AAServer4LogSourceSystem(final ILogProvider logProvider) {
			this.logProvider = logProvider;
		}
		//---------------------------------------------------------------------
		@Override public boolean deregister(final Object client) {
			if(client == null)
				return false;
			
			for (final SoftReference<IAAClient<LogMsg>> refClt : getRefClients()){
				if(client.equals(refClt.get())){
					getRefClients().remove(refClt);
					if(getRefClients().size() == 0)
						getLogProvider().deregister(this);
					return true;	//removed first occurence
				}
			}
			
			return false;
		}
		@Override public boolean register(final IAAClient<LogMsg> client)	{
			cleanRefClients();	//not too many dead clients in list
			final boolean changed = getRefClients().add(new SoftReference<IAAClient<LogMsg>>(client));
			if(changed && getRefClients().size() == 1)
				getLogProvider().register(this);
			return changed;
		}
		@Override public boolean supportsAutoAppend() {return true;}
		//---------------------------------------------------------------------
		@Override public void doLog(final LogMsg msg) 	{for (final IAAClient<LogMsg> clt : getClients()) clt.onNewEntry  (msg    );}
		@Override public void doLog(final List<LogMsg> msgList) {for (final IAAClient<LogMsg> clt : getClients()) clt.onNewEntries(msgList);}
		@Override public void flush() {}
		//---------------------------------------------------------------------
		private ILogProvider 			getLogProvider(){return this.logProvider;}
		private List<SoftReference<IAAClient<LogMsg>>> getRefClients()	{return this.refClients;}
		private List<IAAClient<LogMsg>> getClients() {
			final ArrayList<IAAClient<LogMsg>> list = new ArrayList<IAAClient<LogMsg>>(getRefClients().size());
			for (final SoftReference<IAAClient<LogMsg>> refClt : getRefClients()){
				final IAAClient<LogMsg> clt = refClt.get();
				if(clt != null)
					list.add(clt);
			}
			return list;
		}
		private void cleanRefClients() {
			try {for (final SoftReference<IAAClient<LogMsg>> refClt : getRefClients()){
					if(refClt.get() == null){
						getRefClients().remove(refClt);
						cleanRefClients();
						return;	//Iterator invalid due to removal
					}
				 } 
			}catch(final Exception e) {}//just in case
		}
	}//END class: AAServer4LogSourceSystem
}