/**
 *	Jan 15, 2008 - Jan 15, 2008	ws:	serializable
 *	Dec 15, 2009 - Dec 17, 2009	ws:	add getClickCounter() for debugging
 *									add applyStandardMouseListener(...)
 */
package de.desy.mst.libs.framework.mstapp.menu;

import java.awt.Component;
import java.awt.event.MouseListener;
import java.io.Serializable;
import java.util.List;
import java.util.concurrent.atomic.AtomicInteger;

import javax.swing.JComponent;
import javax.swing.JList;
import javax.swing.JMenuItem;

import de.desy.mst.libs.framework.mstapp.mode.MaschineMgr;

public interface IPopupMenuMgr extends Serializable{
	/**
	 * Alle registrierten JMenuItem's werden in der Reihenfolge der Registrierung angezeigt.<br>
	 * Die Schlssel-Properties der Items werden bernommen und synchronisiert.<br>
	 * Usersynchronisationen erfordern GuiSelectedMgr oder abgeleitete Klasse.<br>
	 * JMenuItem's sind:
	 * <li>JMenuItem
	 * <li>JMenu
	 * <li>JCheckBoxMenuItem
	 * <li>JRadioButtonMenuItem
	 */
	public void register(JMenuItem menu);
	/**
	 * Komfort Routine um viele Items auf einmal zu registrieren.
	 * Gut um echte Kontextnderungen zu managen.
	 */
	public void register(List<JMenuItem> items);
	/**
	 * Registriert Componenten fr extra Contextmenus
	 * Alle registrierten JMenuItem's werden in der Reihenfolge der Registrierung angezeigt.<br>
	 * Die Schlssel-Properties der Items werden bernommen und synchronisiert.<br>
	 * Usersynchronisationen erfordern GuiSelectedMgr oder abgeleitete Klasse.<br>
	 * JMenuItem's sind:
	 * <li>JMenuItem
	 * <li>JMenu
	 * <li>JCheckBoxMenuItem
	 * <li>JRadioButtonMenuItem
	 */
	public void registerCopmponentForAdditionalPopup(final JComponent component, final JMenuItem item);
	/**
	 * Registriert Componenten fr extra Contextmenus
	 * Komfort Routine um viele Items auf einmal zu registrieren.
	 * Gut um echte Kontextnderungen zu managen.
	 */
	public void registerCopmponentForAdditionalPopup(final JComponent component, final List<JMenuItem> items);
	/**
	 * Registriert eine JListe zum Drucken des Inhalts.
	 * Erfordert setzen der Daten ber das Model der Liste.
	 */
	@SuppressWarnings({"rawtypes"})
	public void registerListForPrintContent(final JList jList, final MaschineMgr mgr, final String title);
	/**Fgt einen Separator ans Ende.*/
	public void addSeparator();
	/**
	 * Degeristriert in beliebiger Reihenfolge. Wichtig um nichtbenutzte Controls wieder loswerden
	 * zu koennen. Separatoren am Ende oder hintereinander werden automatisch aufgerumt.
	 * Nur was registriert wurde, kann wieder entfernt werden (Referenz merken)
	 */
	public void deRegister(JMenuItem menu);
	/**
	 * Komfort Routine um viele Items auf einmal zu deregistrieren.
	 * Gut um echte Kontextnderungen zu managen.
	 */
	public void deRegister(List<JMenuItem> items);
	/**
	 * Degeristriert in beliebiger Reihenfolge Componenten mit extra ContextMenus. Wichtig um nichtbenutzte Controls wieder loswerden
	 * zu koennen.
	 */
	public void deRegisterCopmponentForAdditionalPopup(final JComponent component);
	/**
	 * Deregistriert eine JListe zum Drucken des Inhalts
	 */
	@SuppressWarnings({"rawtypes"})
	public void deRegisterListForPrintContent(final JList jList);
	/**
	 * Fr Komponenten mit eigenem MouseListener.
	 * Argumente vom MousListener.mouseReleased/mousePressed event (e.getComponent(), e.getX(), e.getY())
	 */
	public void showMenu(Component invoker, int x, int y);
	/**
	 *  Zeigt das Popup Menu an.
	 */
	public MouseListener getStandardPopUpMouseListener();
	/**
	 * @return a counter with the actual number of clicks (show menu invoked)
	 * 		   for debugging only
	 */
	public AtomicInteger getClickCounter();
	/**
	 * Setzt den getStandardPopUpMouseListener() auf cmp, falls es noch keinen
	 * MouseListener hat.<br>
	 * Falls recursive wird die gleiche Funktion auf alle Unterkomponenten
	 * angewandt.
	 * @param cmp
	 * @param recursive
	 */
	public void applyStandardMouseListener(final Component cmp, final boolean recursive);
}
