/*
 *	Created on Feb 4-18, 2005
 *	Feb 14, 2007 - Feb 14, 2007	ws: added text Feld fuer Preferences
 *	Nov 20, 2007 - Nov 20, 2007	ws: OpMode[] getValues() statt public VALUES
 *	Feb 15, 2008 - Feb 15, 2008 ws: added "@Override"es
 *	Mar 14, 2008 - Mar 14, 2008	ws: valueOf  == -> equals
 *	Jan 15, 2009 - Jan 15, 2009	ws:	added equals
 */
package de.desy.mst.libs.framework.mstapp.mode;

import java.util.Arrays;

/**
 * @author schuette
 * @version 1.0.0
 *
 * Reine Datenklasse fr die verschiedenen Operations Modes
 * @link http://mstweb/PETRA3/Maass/Operation_Modes.doc
 *
 * Vorsicht bei Aenderungen: Die Klasse ist so weit als moeglich schon an
 * Java 1.5 enum's angepasst
 *
 * Kollaboriert mit: OpModeMgr "OpModeManager"
 * @see de.desy.mst.libs.framework.mstapp.mode.OpModeMgr
 *
 * Benutzung:
 * kann direkt die Objekte mit == vergleichen, da nur genau diese drei Objekte
 * pro Applikation erzeugt werden koennen. Siehe folgendes Beispiel:
 *
 * 		OpMode mode = OpMode.BETRIEB;
 *		if (mode == OpMode.BETRIEB){ ... ;}
 *
 */
final public class OpMode {
	public static final OpMode BETRIEB;
	public static final OpMode BETRACHTEN;
	public static final OpMode EXPERT;
	/**/   static final OpMode[] VALUES;

	static{
		BETRACHTEN = new OpMode("Betrachten", 0, "Modus fr Betrachter ");
		BETRIEB    = new OpMode("Betrieb",    1, "Modus fr den Betrieb");
		EXPERT     = new OpMode("Expert",     2, "Modus fr Experten   ");

		//Beachte: Index(Array) = index(Objekt)
		VALUES = (new OpMode[]{BETRACHTEN, BETRIEB, EXPERT});
	}

	public static OpMode[] getValues() {
		return Arrays.copyOf(VALUES, VALUES.length);
	}
//=============================================================================
	public final String name;
	public final int index;
	public final String text;

	/**
	 * @param name
	 * @param index
	 *
	 * Konstruktor muss privat da enum emulation
	 */
	private OpMode(final String name, final int index, final String text) {
		super();
		this.name = name;
		this.index = index;
		this.text = text;
	}

//	may return null
	public final static OpMode valueOf(final String name){
		for(int index = 0; index < VALUES.length; index++)
			if(name.equals(VALUES[index].name))
				return VALUES[index];
		return null;	//Maybe through an error instead?
	}
//	may return null
	public static OpMode valueOf(final int index) {
		for(int i = 0; i < VALUES.length; i++){
			if(index == VALUES[i].index){return VALUES[i];}
		}
		return null;	//Maybe through an error instead?
	}

	/* (non-Javadoc)
	 * @see java.lang.Object#hashCode()
	 */
	@Override public int hashCode() {return toString().hashCode();}
	/* (non-Javadoc)
	 * @see java.lang.Object#equals(java.lang.Object)
	 */
	@Override public boolean equals(final Object obj) {return this == obj;}
	/* (non-Javadoc)
	 * @see java.lang.Object#toString()
	 */
	@Override public String toString() {return "Operations Mode [" + this.name + "]";}
}
 