/**
 * ?????????  - ??????????  Wst: creation from scratch
 * 2007-02-13 - 2007-02-19	ws: Anschluss an das System
 * 2009-05-11 - 2009-05-11	ws: Fix usability issue on bad or cancelled user supplied password
 */

package de.desy.mst.libs.framework.mstapp.mode;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.security.AccessControlException;

import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.ButtonGroup;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.border.TitledBorder;

import de.desy.mst.libs.framework.mstapp.logging.Logger;
import de.desy.mst.libs.framework.mstapp.preferences.IPrefContainer;
import de.desy.mst.libs.framework.mstapp.preferences.helper.APrefClientPanelEinfach;
import de.desy.mst.libs.framework.mstapp.util.CanceledByUserException;
import de.desy.mst.libs.framework.mstapp.util.gui.GUIFactory;

/**
 * Preferences fuer OpModes grafischer Teil
 *
 * @author wst, ws version 1.0.0
 */
public class PrefOperationModePanel extends APrefClientPanelEinfach {
	private static final long serialVersionUID = 5052867769787200829L;

	private final OpModeMgr       opModeMgr;
	private final ServerAnwahlMgr serverAnwahlMgr;
	private JPanel jPnlOperationMode = null;
	private JRadioButton[] jRButtons = null;
	private ButtonGroup  buttonGroup = null;

	/** Der aktuell vom user eingestellte OpMode (nicht notwendig committed) */
	private OpMode opMode;

	private final IServerAnwahlObserver anwahlObserver = new IServerAnwahlObserver(){
		@Override public void onServerAnwahlChange(final ServerAnwahl anwahl) {
			initServerAnwahlFromSystem();
		}
	};
	private final IOpModeObserver opModeObserver = new IOpModeObserver(){
		@Override public void onOpModeChange(final OpMode opMode) {
			initServerAnwahlFromSystem();
		}
	};

//=============================================================================

	public PrefOperationModePanel(final IPrefContainer prefContainer
								, final OpModeMgr      opModeMgr
								, final ServerAnwahlMgr serverAnwahlMgr) {
		super(prefContainer);
		this.opModeMgr = opModeMgr;
		this.serverAnwahlMgr = serverAnwahlMgr;
		initialize();
		registerForFrameworkinfo();
	}

//=============================================================================

	/*
	 * (non-Javadoc)
	 *
	 * @see de.desy.mst.libs.framework.mstapp.preferences.IPrefClient#onReset()
	 */
	@Override public void onReset() {
		initServerAnwahlFromSystem();
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see de.desy.mst.libs.framework.mstapp.preferences.IPrefClient#onSet()
	 */
	@Override public void onSet() throws AccessControlException, CanceledByUserException {
		setOriginalOpMode(getOpMode());
	}

	@Override public void onDispose() {deregisterForFrameworkinfo();}

//=============================================================================

	private void initialize() {
		this.setSize(126, 200);
		this.add(getJPnlOperationMode(), null);
		initServerAnwahlFromSystem();
	}

	private final JRadioButton[] getJRButtons() {
		if (this.jRButtons == null) {
			final OpMode[] opModes = OpMode.VALUES;
			this.jRButtons = new JRadioButton[opModes.length];
			for (int i = 0; i < opModes.length; i++) {
				this.jRButtons[i] = createJRButton(opModes[i]);
			}
		}
		return this.jRButtons;
	}
	private JRadioButton createJRButton(final OpMode opMode) {
		final OpModeKonfiguration opModeKonfiguration
									= getOpModeMgr().getOpModeKonfiguration();
		final JRadioButton jRButton = new JRadioButton();
		jRButton.setText(opMode.text + calcSuffix(opMode));
		jRButton.setEnabled(opModeKonfiguration.getImplementierteModes()
											   .isInList(opMode));
		jRButton.setFont(GUIFactory.createButtonFont());
		jRButton.addActionListener(new ActionListener() {
			@Override public void actionPerformed(final ActionEvent e) {
				setOpMode(opMode);
			}
		});
		return jRButton;
	}
	private String calcSuffix(final OpMode opMode) {
		return 	isPasswortBenoetigt(opMode) ? " (Passwort bentigt.)" : "";
	}
	private boolean isPasswortBenoetigt(final OpMode opMode) {
		return getOpModeMgr().getOpModeKonfiguration()
							 .getGeschuetzteModes()
		                     .isInList(opMode);
	}

	private JPanel getJPnlOperationMode() {
		if (this.jPnlOperationMode == null) {
			this.jPnlOperationMode = new JPanel();
			this.jPnlOperationMode.setLayout(new BoxLayout(this.jPnlOperationMode,
					BoxLayout.Y_AXIS));
			this.jPnlOperationMode.setBorder(createBorder());

			this.buttonGroup = new ButtonGroup();
			final JRadioButton[] butArr = getJRButtons();
			for (final JRadioButton button : butArr) {
				this.buttonGroup.add(button);
				this.jPnlOperationMode.add(button);
			}

			this.jPnlOperationMode.setPreferredSize(new Dimension(300, 100));
		}
		return this.jPnlOperationMode;
	}

	private TitledBorder createBorder() {
		return BorderFactory.createTitledBorder(
				BorderFactory.createLineBorder(Color.gray, 1),
				getBetriebsart(), TitledBorder.DEFAULT_JUSTIFICATION,
				TitledBorder.DEFAULT_POSITION, null, null);
	}
 
	private String getBetriebsart() {
		return "Betriebsart (ServerAnwahl: " + getServerAnwahlMgr().getAnwahl().name + ")";
	}
	private void updateBetriebsart(){
		getJPnlOperationMode().setBorder(createBorder());
	}

	public void setButtonGroupToSelectedValue() {
		final OpMode opMode = getOpMode();
		try {
			(getJRButtons()[opMode.index]).setSelected(true);
		} catch (final Exception e) {
			final String msg = "PrefOperationModePanel: Kann den OpMode '"
				             + opMode + "' Index = '" + opMode.index
				             + "' nicht setzen. Ignorier die Aktion.";
			Logger.SWFW.error(msg, e);
		}
	}

//=============================================================================

	private final OpModeMgr        getOpModeMgr     () {return this.opModeMgr;}
	private final ServerAnwahlMgr getServerAnwahlMgr() {return this.serverAnwahlMgr;}

	private final OpMode getOriginalOpMode() {
		return getOpModeMgr().getOpMode();
	}
	private final void setOriginalOpMode(final OpMode opMode) throws CanceledByUserException, AccessControlException{
		try {
			final String passwort = getPasswort(isPasswortBenoetigt(opMode));
			if(dialogIsCanceled(passwort))
				throw new CanceledByUserException();

			getOpModeMgr().setOpMode(opMode, passwort);
			initServerAnwahlFromSystem();
		} catch (final AccessControlException e) {
			final String msg = "Das Passwort ist nicht ausreichend mchtig.";
//			JOptionPane.showMessageDialog(this, msg);
			throw new AccessControlException(msg);
		}
	}

	private boolean dialogIsCanceled(final String passwort) {
		return passwort == null;
	}

	private final OpMode getOpMode() {return this.opMode;}
	private final void setOpMode(final OpMode opMode) {
		this.opMode = opMode;
		getPrefContainer().setChanged(isChanged());
	}
	private boolean isChanged() {return (getOriginalOpMode() != getOpMode());}

	private void initServerAnwahlFromSystem() {
		updateBetriebsart();
		setOpMode(getOriginalOpMode());
		setButtonGroupToSelectedValue();
	}

	private String getPasswort(final boolean required) {
		if(required){
			final String msg = "Bitte ein gltiges Passwort eingeben.";
			return JOptionPane.showInputDialog(msg);
		}else{
			return "";
		}
	}


	private void registerForFrameworkinfo() {
		getServerAnwahlMgr().registerAnwahlObserver(this.anwahlObserver);
		getOpModeMgr().registerIOpModeObserver(this.opModeObserver);
	}
	private void deregisterForFrameworkinfo() {
		getOpModeMgr().deregisterIOpModeObserver(this.opModeObserver);
		getServerAnwahlMgr().deregisterAnwahlObserver(this.anwahlObserver);
	}
}
