/*
 * Created on Apr 4, 2005 - Apr 4, 2005
 *	Feb 15, 2008 - Feb 15, 2008 ws: added "@Override"es
 */
package de.desy.mst.libs.framework.mstapp.mode;

import java.util.Arrays;

/**
 * Tells which ServerAnwahl'en are allowed.
 * Value equality == Object equality
 * They can not be changed.
 *
 * Naming convention of constants:
 * 	Betrieb,Betrachten,Expert
 * 	Capital letter: mode is allowed mode
 *  "_": 			mode is not allowed
 *  Im Detail:
 *  <li><b>ServerAnwahlListe.DS</b> DefaultSimulationsserver</li>
 *  <li><b>ServerAnwahlListe._S</b> 		_Simulationsserver		</li>
 *  <li><b>ServerAnwahlListe.D_</b> 		Default_				</li>
 *  <li><b>ServerAnwahlListe.EMPTY_LISTE</b> ___					2</li>
 *
 * @author schuette
 *
 */
public final class ServerAnwahlListe {
	//DefaultSimulationsserver:
	public static final ServerAnwahlListe DS;
	//Simulationsserver:
	public static final ServerAnwahlListe _S;
	//Default:
	public static final ServerAnwahlListe D_;
	//EmptyList:
	public static final ServerAnwahlListe EMPTY_LISTE;

	public static final ServerAnwahlListe[] VALUES;

	static{
		DS = new ServerAnwahlListe( "DS", true,  true);
		_S = new ServerAnwahlListe("_S", false, true );
		D_ = new ServerAnwahlListe("D_", true,  false);
		EMPTY_LISTE = new ServerAnwahlListe("EMPTY_LISTE", false, false);

//		DEFAULTLIST = BB_;

		VALUES = (new ServerAnwahlListe[] {DS, _S, D_, EMPTY_LISTE});
	}


	public final String name;
	//Index of  == index of ServerAnwahl
	private final boolean[] anwahlArray;	//might later internally be represented by a bitarray

//=============================================================================


	//true if not null and at least one true entry
	public static boolean isGefuellterAnwahlArray(final boolean[] anwahlArray) {
		if(anwahlArray == null){return false;}
		for(int index = 0; index < anwahlArray.length; index++){
			if(anwahlArray[index]){return true;}
		}
		return false;
	}


	/**Ergibt eine ServerAnwahlListe die nur ServerAnwahln aus beiden Listen enthaelt*/
	public static ServerAnwahlListe and(final ServerAnwahlListe list1, final ServerAnwahlListe list2){
		final boolean[] a1  = list1.getAnwahlArray();
		final boolean[] a2  = list2.getAnwahlArray();
		final boolean[] a12 = new boolean[a1.length];

		for (int i = 0; i < a1.length; i++) {
			a12[i] = a1[i] && a2[i];
		}
		return serverAnwahlFromBooleanArray(a12);
	}

	/**Ergibt die logisch invertierte ServerAnwahlListe*/
	public static ServerAnwahlListe not(final ServerAnwahlListe list){
		final boolean[] a1  = list.getAnwahlArray();
		final boolean[] a2 = new boolean[a1.length];

		for (int i = 0; i < a1.length; i++) {
			a2[i] = ! a1[i];
		}
		return serverAnwahlFromBooleanArray(a2);
	}

//=============================================================================

	private ServerAnwahlListe(final String name, final boolean default_, final boolean simulationsServer) {
		super();
		this.name = name;
		final boolean[] anwahlArray = new boolean[ServerAnwahl.VALUES.length];
		anwahlArray[ServerAnwahl.DEFAULT.index] = default_;
		anwahlArray[ServerAnwahl.SIMULATIONSSERVER.index] = simulationsServer;
		this.anwahlArray = anwahlArray;
	}



//=============================================================================

	public boolean isInList(final int index){
		if(index >= 0 && index < this.anwahlArray.length){
			return this.anwahlArray[index];
		}
		return false;
	}
	public boolean isInList(final ServerAnwahl serverAnwahl){
		if(serverAnwahl != null){
			return isInList(serverAnwahl.index);
		}
		return false;
	}
	/**
	 * @return 	Returns a clone of the anwahlArray. Always a valid non null array.
	 * 			must be a clone, so user can not change values in an unchecked fashion
	 */
	public boolean[] getAnwahlArray() {
		final boolean[] cloneAnwahlArray = new boolean[this.anwahlArray.length];
		System.arraycopy(this.anwahlArray, 0, cloneAnwahlArray, 0, this.anwahlArray.length);
		return cloneAnwahlArray;
	} 

	//ServerAnwahl mit kleinstem Index oder null
	public ServerAnwahl getDefaultServerAnwahl() {
		for(int index = 0; index < this.anwahlArray.length; index++){
			if(this.anwahlArray[index]){return ServerAnwahl.valueOf(index);}
		}
		return null;
	}

	/* (non-Javadoc)
	 * @see java.lang.Object#toString()
	 */
	@Override public String toString() {
		return this.name;
	}

//=============================================================================

	//true if not null and at least one true entry
	public static boolean isGefuellterModeArray(final boolean[] anwahlArray) {
		if(anwahlArray == null){return false;}
		for(int index = 0; index < anwahlArray.length; index++){
			if(anwahlArray[index]){return true;}
		}
		return false;
	}

//	=============================================================================

	private static ServerAnwahlListe serverAnwahlFromBooleanArray(final boolean[] a12) {
		for (int i = 0; i < VALUES.length; i++) {
			if(Arrays.equals(VALUES[i].getAnwahlArray(), a12)){
				return VALUES[i];
			}
		}
		throw new IllegalArgumentException("Bad AnwahlArray");	//sollte nie passieren
	}
}
