/**
 * 	Nov  3, 2008 - Nov 24, 2008	ws:	Creation from scratch/APrefClientPanel
 *	Aug 10, 2009 - Aug 10, 2009	ws:	make getJCBAbspeichern() public for tests
 */
package de.desy.mst.libs.framework.mstapp.preferences.helper;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.LayoutManager;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JCheckBox;
import javax.swing.JPanel;

import de.desy.mst.libs.framework.mstapp.preferences.IPrefContainer;
import de.desy.mst.libs.framework.mstapp.preferences.PrefSpeicherMode;

/**
 * APrefClientPanel mit GUI Support (Checkbox at bottom (SOUTH))
 * zum Speichern von Preferences
 * <br> - add(...) actually puts things in subpanel (CENTER)
 * <br> - setLayout(..) ebenso
 * <br> - updateIsChanged(...) bei potentiellen Aenderungen rufen
 * <br> - isChanged(), systemToGUI(), GuiToSystem() implementieren fuer eigenes Zeugs
 * <br> - getSpeicherMode(), onReset(), onSet() is managed here
 * @author ws
 * @version 1.0.0
 */
public abstract class APrefClientPanel2 extends APrefClientPanel {
	private static final long serialVersionUID = 1L;
	private final JCheckBox	 jCBAbspeichern;
	private final JPanel panel;
//=============================================================================
	public APrefClientPanel2(final IPrefContainer prefContainer) {
		super(prefContainer);
		super.setLayout(new BorderLayout());	//!! super not this
		this.jCBAbspeichern = createJCBAbspeichern();
		this.panel = createPanel();
	}
	@SuppressWarnings("deprecation") public APrefClientPanel2()  {
		super();
		super.setLayout(new BorderLayout());	//!! super not this
		this.jCBAbspeichern = createJCBAbspeichern();
		this.panel = createPanel();
	}
//=============================================================================
	protected abstract boolean isChanged();
	protected abstract void systemToGui();
	protected abstract void guiToSystem();

	protected final void updateIsChanged(){
		updateIsChanged(sollenDieWerteAbgespeichertWerden() || isChanged());
	}
	protected final void updateIsChanged(final boolean isChanged){
		getPrefContainer().setChanged(isChanged);
	}

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.framework.mstapp.preferences.IPrefClient#onReset()
	 */
	@Override public final void onReset() {systemToGui(); getJCBAbspeichern().setSelected(false);}
	/* (non-Javadoc)
	 * @see de.desy.mst.libs.framework.mstapp.preferences.IPrefClient#onSet()
	 */
	@Override public final void onSet()   {guiToSystem(); updateIsChanged();}
	/* (non-Javadoc)
	 * @see de.desy.mst.libs.framework.mstapp.preferences.IPrefClient#getSpeicherMode()
	 */
	@Override public final PrefSpeicherMode getSpeicherMode() {
		return sollenDieWerteAbgespeichertWerden() ? PrefSpeicherMode.APPLOCAL
												   : PrefSpeicherMode.MEMORYONLY;
	}

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.framework.mstapp.preferences.IPrefClient#getSaveString()
	 */
	@Override public String getSaveString() {
		getJCBAbspeichern().setSelected(false);
		updateIsChanged();
		return "";
	}
	//=============================================================================
	private JCheckBox createJCBAbspeichern() {
		final String text = "Sollen diese Werte abgespeichert werden";
		final boolean selected = false;
		final JCheckBox cb = new JCheckBox(text, selected);
		super.add(cb, BorderLayout.SOUTH);	//!! super not this
		cb.addActionListener(new ActionListener(){
			@Override public void actionPerformed(final ActionEvent evt) {
				updateIsChanged();
			}
		});
		return cb;
	}
	@Deprecated /**public zum Testen only*/
	public JCheckBox  getJCBAbspeichern() {return this.jCBAbspeichern;}
	private boolean sollenDieWerteAbgespeichertWerden() {
		return getJCBAbspeichern().isSelected();
	}

	private JPanel createPanel() {
		final JPanel panel = new JPanel(new BorderLayout());
		super.add(panel, BorderLayout.CENTER);	//!! super not this
		return panel;
	}
	private JPanel getPanel(){return this.panel;}
//=============================================================================

	/* (non-Javadoc)
	 * @see java.awt.Container#setLayout(java.awt.LayoutManager)
	 */
	@Override public void setLayout(final LayoutManager mgr) {
		if(this.panel == null)
			super.setLayout(mgr);	//waehrend der Konstruktion von JPanel
		else
			getPanel().setLayout(mgr);
	}

	// JETZT alle add von Component abfangen und an internen panel delegieren

	/* (non-Javadoc)
	 * @see java.awt.Container#add(java.awt.Component, int)
	 */
	@Override public Component add(final Component comp, final int index) {
		return getPanel().add(comp, index);
	}
	/* (non-Javadoc)
	 * @see java.awt.Container#add(java.awt.Component, final java.lang.Object, final int)
	 */
	@Override public void add(final Component comp, final Object constraints, final int index) {
		getPanel().add(comp, constraints, index);
	}
	/* (non-Javadoc)
	 * @see java.awt.Container#add(java.awt.Component, final java.lang.Object)
	 */
	@Override public void add(final Component comp, final Object constraints) {
		getPanel().add(comp, constraints);
	}
	/* (non-Javadoc)
	 * @see java.awt.Container#add(java.awt.Component)
	 */
	@Override public Component add(final Component comp) {
		return getPanel().add(comp);
	}
	/* (non-Javadoc)
	 * @see java.awt.Container#add(java.lang.String, final java.awt.Component)
	 */
	@Override public Component add(final String name, final Component comp) {
		return getPanel().add(name, comp);
	}
}
