/*
 * 	ws:	Feb 13, 2007 - Feb 19, 2007 creation from Basisentwurf von Wst
 * 	ws:	Apr 19, 2007 - Apr 19, 2007	added:	besserer Kommentar fuer
 * 											createPrefClientPanel()
 * 	ws:	May 14, 2007 - May 15, 2007	changed:createPrefClientPanel() better Signature
 * 											onNewPrefString not final
 * 	May 16, 2007 - May 16, 2007	ws:	rename PrefString -> SaveString
 * 									(Konsistenz zu IPrefClient)
 * 	Oct 15, 2008 - Oct 15, 2008	ws: setParent(...) mit throws IllegalArgumentException
 *	Aug  7, 2009 - Aug  7, 2009	ws:	hunt KiHi Problem 
 */
package de.desy.mst.libs.framework.mstapp.preferences.helper;

import java.awt.Component;

import de.desy.mst.libs.framework.mstapp.logging.Logger;
import de.desy.mst.libs.framework.mstapp.preferences.IPrefClient;
import de.desy.mst.libs.framework.mstapp.preferences.IPrefContainer;
import de.desy.mst.libs.framework.mstapp.preferences.IPreferences;

/**
 * Basis Klasse fuer IPreferences ohne Speichern von PrefStrings
 * (nicht persistent).<br>
 * Wie im Wrapper ist toString() == getTitle()<br><br>
 * Implementationsdetail:<br>
 * Das Framework garantiert folgende call Reihenfolge:
 * <li> getPrefComponent(final IPrefContainer prefContainer)
 * <li> getPrefClient();
 * <li> onDispose() <br>
 * D.H. es existiert ein prefContainer bei
 * createPrefClientPanel(IPrefContainer prefContainer)
 *
 * @author ws (Abstraktion von Wst Entwurf)
 * version 1.0.0
 */
public abstract class APreferences implements IPreferences {
	private APrefClientPanel prefClientPanel;
	private IPreferences     parent;
	private final String     title;
	private final String[]   keyWords;
	private IPrefContainer   prefContainer;

//=============================================================================

	/**
	 * @param title		Titel der gewaehlten Preferences (steht so im Tree)
	 * @param parent    null oder hierarchisch uebergeordnete Preference
	 * @param keyWords  Schluesselwoerter zum Suchen dieser Preference
	 */
	public APreferences(final String title,
			            final IPreferences parent,
			            final String ... keyWords) {
		super();
		this.title  = ((title == null) ? this.getClass().getName() : title);
		this.parent = parent;
		this.keyWords = keyWords;
	}

//=============================================================================
	/**
	 * Erzeugt ein neues APrefClientPanel. Diese Funktion wird gerufen,
	 * falls kein lokales vorhanden ist. Einfach ein neues durchreichen.
	 * Es lokal zu speichern ist schaedlich. Statt dessen holen via
	 * getPrefClient() und nur lokal benutzen.
	 */
	public abstract APrefClientPanel
					createPrefClientPanel(final IPrefContainer prefContainer);

//=============================================================================

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.framework.mstapp.preferences.IPreferences#getKeyWords()
	 */
	@Override public final String[] getKeyWords() {return this.keyWords;}

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.framework.mstapp.preferences.IPreferences#getParent()
	 */
	@Override public final IPreferences getParent() {return this.parent;}

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.framework.mstapp.preferences.IPreferences#getPrefClient()
	 */
	@Override public final IPrefClient getPrefClient() {return getOrCreatePrefClient();}

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.framework.mstapp.preferences.IPreferences#getPrefComponent(de.desy.mst.libs.framework.mstapp.preferences.IPrefContainer)
	 */
	@Override public final Component getPrefComponent(final IPrefContainer prefContainer) {
		setPrefContainer(prefContainer);
		return getOrCreatePrefClient();
	}

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.framework.mstapp.preferences.IPreferences#getTitle()
	 */
	@Override public final String getTitle() {return this.title;}

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.framework.mstapp.preferences.IPreferences#onDispose()
	 */
	@Override public void onDispose(){
		if (this.prefClientPanel != null) {
			this.prefClientPanel.onDisposeForSystemUseOnly();
			this.prefClientPanel  = null; // meine einzige Referenz
			setPrefContainer(null);
		}
		//Logger.SWFW.info("APreferences onDispose ");
	}

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.framework.mstapp.preferences.IPreferences#onNewPrefString(java.lang.String)
	 */
	@Override public void onNewSaveString(final String saveString) {
		final String msg = "Einen SaveString bekommen, " +
				     "obwohl keine Persistenz vorgesehen ist.";
		Logger.SWFW.warn(msg, new IllegalStateException());
	}

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.framework.mstapp.preferences.IPreferences#setParent(de.desy.mst.libs.framework.mstapp.preferences.IPreferences)
	 */
	@Override public final void setParent(final IPreferences parent) {
		if(parent == this)
			throw new IllegalArgumentException("Der Parent ist identisch mit sich selbst.");
		this.parent = parent;
	}

//=============================================================================

	final APrefClientPanel getAPrefClientPanelForTestingOnly(){
		return this.prefClientPanel;
	}
//=============================================================================

	/**Liefert nach moeglichkeit ein nicht null APrefClientPanel*/
	private APrefClientPanel getOrCreatePrefClient() {
		if(this.prefClientPanel == null){
			this.prefClientPanel = createPrefClientPanel(getPrefContainer());
		}
		return this.prefClientPanel;
	}

	private void setPrefContainer(final IPrefContainer prefContainer) {
		this.prefContainer = prefContainer;
	}
	public final IPrefContainer getPrefContainer() {return this.prefContainer;}

	/**getTitle wie der Wrapper im Frame*/ 
	@Override public String toString() {return this.getTitle();}
}
