/**
 * 	Apr 20, 2015 - Apr 21, 2015	ws:	created from scratch
 */
package de.desy.mst.libs.framework.mstapp.security;

import de.desy.mst.libs.framework.mstapp.tabellendaten.entry.OPModeACLEntry;

/**
 * @author ws
 */
public enum EAclEntryType { 
	  Experte		(new __Experte())
	, Developer		(new __Developer())
	, BKR			(new __BKR())
	, DeviceServer	(new __DeviceServer())
	 ;
   	//---------------------------------------------------------------------------------------------  
	public static boolean stringStartsWith(final String theString, final String ... startCandidates) {
    	for (final String candidate : startCandidates) 
    		if(theString.startsWith(candidate))
    			return true;
    	return false;
    }
	private static boolean matchesThePattern(final String pcName, final String pattern) {
		return pcName != null && pcName.matches(pattern);
	}
	//=============================================================================================  
	private final __Impl impl;
	//---------------------------------------------------------------------------------------------  
	private EAclEntryType(final __Impl impl) {this.impl = impl;}
	//---------------------------------------------------------------------------------------------  
	public final boolean is(final OPModeACLEntry entry, final String user, final String hostName, final String appName) {
		return this.impl.is(entry, user, hostName, appName);
	}
	public final boolean is(final OPModeACLEntry entry) {
		return this.impl.is(entry, entry.pcUser, entry.pcName, entry.appName);
	}
	//=============================================================================================  
	private static abstract class __Impl {
		private final String aclNameTag;
		//.........................................................................................
		public __Impl(final String aclNameTag) {this.aclNameTag = aclNameTag;}
		//.........................................................................................
		abstract boolean _is(final OPModeACLEntry entry, final String user, final String hostName, final String appName);
		//.........................................................................................
		String getAclNameTag() {return this.aclNameTag;}
		final boolean is(final OPModeACLEntry entry, final String user, final String hostName, final String appName) {
			return aclNameContains(entry, getAclNameTag())
				|| _is(entry, user, hostName, appName); 
		}
		//.........................................................................................
		private boolean aclNameContains(final OPModeACLEntry acl, final String refString) {
			return acl != null
				&& acl.getAclName() != null
				&& acl.getAclName().contains(refString);
		}
	}
	//---------------------------------------------------------------------------------------------  
	private static class __Experte extends __Impl { public __Experte() {super("Experte");}
		@Override boolean _is(final OPModeACLEntry entry, final String user, final String hostName, final String appName) {
			return false; 
		}
	}
	private static class __Developer extends __Impl { public __Developer() {super("Entwickler");}
		@Override boolean _is(final OPModeACLEntry entry, final String user, final String hostName, final String appName) {
			return                   isDeveloperHost(hostName)
				|| (entry != null && isDeveloperHost(entry.pcName)); 
		}
		private boolean isDeveloperHost(final String pcName) {
			return pcName != null 
			   && (stringStartsWith(pcName.toUpperCase(), "MCS", "MST"));
		}
	}
	private static class __BKR extends __Impl { public __BKR() {super("BKR");}
		private static final String BKR_PC_PATTERN		= "BKR.*CON.*";			//BKRXPD2CON15
		@Override boolean _is(final OPModeACLEntry entry, final String user, final String hostName, final String appName) {
			return                   matchesThePattern(hostName,     BKR_PC_PATTERN)
			    || (entry != null && matchesThePattern(entry.pcName, BKR_PC_PATTERN))
			    || isBKRUser(user)
			    || (entry != null && isBKRUser(entry.pcUser)); 
		}
		private boolean isBKRUser(final String user) {
			return user != null && user.toLowerCase().endsWith("con");
		}
	}
	private static class __DeviceServer extends __Impl { public __DeviceServer() {super("DeviceServer");}
	private static final String DEVSRV_PC_PATTERN	= "ACC.*R[0-9]*[A-N]";	//ACCXPL2R1E
		@Override boolean _is(final OPModeACLEntry entry, final String user, final String hostName, final String appName) {
 			return                   matchesThePattern(hostName,     DEVSRV_PC_PATTERN)
			    || (entry != null && matchesThePattern(entry.pcName, DEVSRV_PC_PATTERN)) 
			    || isDeviceServerUser(user)
			    || (entry != null && isDeviceServerUser(entry.pcUser)); 
		}
		private boolean isDeviceServerUser(final String user) {
			return user != null && user.toLowerCase().endsWith("dev");
		}
	}
}
