/**
 * 	Nov 27, 2008 - Dec 12, 2008	ws:	Creation from scratch
 * 	Apr 22, 2015 - Apr 28, 2015	ws:	umstellen von is*(...) auf EAclEntryType.is*(...) wg Abschaltung SQL server
 */
package de.desy.mst.libs.framework.mstapp.security;

import static de.desy.mst.libs.framework.mstapp.security.EAclEntryType.*;

import java.net.InetAddress;
import java.net.UnknownHostException;
import java.util.Arrays;
import java.util.concurrent.Callable;
import java.util.concurrent.ExecutionException;
import java.util.concurrent.Future;

import de.desy.mst.libs.framework.mstapp.Main;
import de.desy.mst.libs.framework.mstapp.logging.Logger;
import de.desy.mst.libs.framework.mstapp.tabellendaten.entry.OPModeACLEntry;
import de.desy.mst.libs.framework.mstapp.tabellendaten.entry.OPModeACLEntryFactory;
import de.desy.mst.libs.framework.mstapp.tabellendaten.entry.OPModeACLEntryFactory_Default;

/**
 * Ersetzt MstSecurityManager
 * <br> ausser Factory Methoden und prepare(...): nicht mehr statisch
 * <br> ohne obsolete SecurityRole
 * <br><br> future basiert fuer bessere elapsed Performance
 * <br> PCName und HostName sind Synonyme
 * @author ws
 * @version 1.0.0
 */
public final class MstSecurity {
	/**pp*/ static final OPModeACLEntryFactory OPMODEACLFACTORY = OPModeACLEntryFactory_Default.INSTANCE;
	private static final String UNKNOWN_USER = "UNKNOWN_USER";
//	private static final String DEFAULT_ACL_ENTW_USR   = "Default1: User: Entwickler";
//	private static final String DEFAULT_ACL_ENTW_PC    = "Default1: PC: Entwickler"	 ;
//	private static final String DEFAULT_ACL_EXP_USR    = "Default1: User: Experte"	 ;
//	private static final String DEFAULT_ACL_BKR_USR    = "Default1: User: BKRUser"	 ;
//	private static final String DEFAULT_ACL_DEVSRV_USR = "Default1: User: DeviceServer";
//	private static final String DEFAULT_ACL_DEVSRV_PC  = "Default1: PC: DeviceServer";
//	private static final String DEFAULT_ACL_BKR_PC_USR = "Default2: PC & User: BKRKonsole & BKRUser";

//	//Fuer (OPModeACLEntry Object).getAclName()
//	private static final String ACL_ENTW 	= "Entwickler";
//	private static final String ACL_EXP		= "Experte";
//	private static final String ACL_BKR 	= "BKR";
//	private static final String ACL_DEVSRV	= "DeviceServer";
//
//	//Fuer Heuristic (do not need a complete acl)
//	private static final String BKR_USER	= "DesyCon";
//	private static final String DEVSRV_USER	= "DesyDev";
//	private static final String BKR_PC_PATTERN		= "BKR.*CON.*";			//BKRXPD2CON15
//	private static final String DEVSRV_PC_PATTERN	= "ACC.*R[0-9]*[A-N]";	//ACCXPL2R1E

	/**the security context of this appName on this PC with this user*/
	volatile private static MstSecurity mstSecurity = create(Main.getAppName());
	//-------------------------------------------------------------------------
	public static MstSecurity get() {return MstSecurity.mstSecurity;}
	//-------------------------------------------------------------------------
	/**for system use only: preloads infos as early as possible, checks appName*/
	public static String prepare() {
		final String appName = Main.getAppName();

		if(get().getAppName().equals(appName))	//found already correct appName
			return " future finished for '" + get().toString() + "'";

		MstSecurity.mstSecurity = create(appName);//redo with (fresh) appName
		return " created a future for '" + appName + "'.";
	}

	static MstSecurity create(final String appName) throws NullPointerException {
		return new MstSecurity(appName);
	}
	static MstSecurity create(final String appName, final String hostName, final String hostAddress, final String user) throws NullPointerException {
		return new MstSecurity(appName, hostName, hostAddress, user);
	}
	//-------------------------------------------------------------------------
	private static String calcHostName(){
		try {return InetAddress.getLocalHost().getHostName();}
		catch (final UnknownHostException ex)
		{Logger.Netzwerk.error("Can not determine Hostname", ex);	 return "?HostName?";}
	}
	private static String calcHostAddress(){
		try {return InetAddress.getLocalHost().getHostAddress();}
		catch (final UnknownHostException ex)
		{Logger.Netzwerk.error("Can not determine HostAddress", ex); return "?HostAddress?";}
	}
	private static String calcUser(){
		final String user = loadStringByKey("user.name", UNKNOWN_USER);;
		if(user == null || user.isEmpty())
			return UNKNOWN_USER;
		return user;
	}
	/**
	 * equivalent to System.getProperty(key, defaultValue)
	 * but all Exception's go to logger and the default will be returned
	 */
	private static final String loadStringByKey(final String key, final String defaultValue) {
		try {
			return System.getProperty(key, defaultValue);
		} catch (final Exception e) {
			Logger.SWFW.warn("can not load '" + key + "'", e);
			return defaultValue;
		}
	}

//=============================================================================
	private final String appName;
	private final String hostName;
	private final String hostAddress;
	private final String user;
	private final Future<OPModeACLEntry> acl;
//=============================================================================
	private MstSecurity(final String appName) throws NullPointerException {
		this(appName, calcHostName(), calcHostAddress(), calcUser());
	}
	private MstSecurity(final String appName, final String hostName, final String hostAddress, final String user) throws NullPointerException{
		super();
		check(appName);

		this.appName     = appName;
		this.hostName    = hostName;
		this.hostAddress = hostAddress;
		this.user        = user;
		this.acl 		 = createACLFuture(appName, hostName, user);
	}
	//-------------------------------------------------------------------------
	private Future<OPModeACLEntry> createACLFuture(final String appName, final String hostName, final String user) {
		final Callable<OPModeACLEntry> task = new Callable<OPModeACLEntry>(){
			@Override public OPModeACLEntry call() throws Exception {
				return OPMODEACLFACTORY.getOPModeACLEntry(user, hostName, appName);
			}
		};
		final Future<OPModeACLEntry> acl = Main.getExecutorService().submit(task);
		return acl;
	}
	private void check(final Object obj) throws NullPointerException{
		if(obj == null)
			throw new NullPointerException();
	}
//=============================================================================
	public String getAppName()		{return this.appName;}
	/**same as PCName*/
	public String getHostName()		{return this.hostName;}
	public String getHostAddress()	{return this.hostAddress;}
	public String getUser() 		{return this.user;}
	//-------------------------------------------------------------------------
	/**anyvalid user*/
	@Deprecated public boolean isDeveloper(final String user) {
		return isDeveloper(user, null, null);
	}
	/**user oder appName kann null sein*/
	@Deprecated public boolean isExperte(final String user, final String appName) {
		return isExperte(user, null, appName);
	}
	/**user oder pcName/hostName kann null sein*/
	@Deprecated public boolean isBKR(final String user, final String pcName) {
		return isBKR(user, pcName, null);
	}
	/**user oder pcName/hostName kann null sein*/
	public boolean isDeviceServer(final String user, final String pcName) {
		return isDeviceServer(user, pcName, null);
	}

	public boolean isDeveloper()	{return isDeveloper   (getUser(), getHostName(), getAppName());}
	public boolean isExperte()		{return isExperte     (getUser(), getHostName(), getAppName());}
	public boolean isBKR() 			{return isBKR         (getUser(), getHostName(), getAppName());}
	public boolean isDeviceServer() {return isDeviceServer(getUser(), getHostName(), getAppName());}

	public OPModeACLEntry getOPModeACLEntry() {
		try {
			return this.acl.get();
		} catch (final InterruptedException e) {
			return getOPModeACLEntry();	//just redo ad nauseum
		} catch (final ExecutionException e) {
			return OPMODEACLFACTORY.getOPModeACLEntry(getUser(), getHostName(), getAppName());	//versuch es direkt
		}
	}

	//-------------------------------------------------------------------------
	/* (non-Javadoc)
	 * @see java.lang.Object#equals(java.lang.Object)
	 */
	@Override public boolean equals(final Object obj) {
		if(this == obj)
			return true;
		if( !(obj instanceof MstSecurity) )
			return false;
		final MstSecurity other = (MstSecurity) obj;
		return equalString(this.user,        other.user)
			&& equalString(this.hostName,    other.hostName)
			&& equalString(this.hostAddress, other.hostAddress)
			&& equalString(this.appName,     other.appName)
			;
	}
	/* (non-Javadoc)
	 * @see java.lang.Object#hashCode()
	 */
	@Override public int hashCode() {return getAppName().hashCode();}
	/* (non-Javadoc)
	 * @see java.lang.Object#toString()
	 */
	@Override public String toString() {
		return "MstSecurity: " + Arrays.asList(getUser(), getHostName(), getHostAddress(), getAppName());
	}

//=============================================================================

	private boolean isDeveloper(final String user, final String hostName, final String appName) {
		return Developer	.is(retrieveACL(user, hostName, appName), user, hostName, appName);
	}
	private boolean isExperte(final String user, final String hostName, final String appName) {
		return Experte		.is(retrieveACL(user, hostName, appName), user, hostName, appName);
	}
	private boolean isBKR(final String user, final String pcName, final String appName) {
		return BKR			.is(retrieveACL(user, pcName, appName), user, pcName, appName);
	}
	private boolean isDeviceServer(final String user, final String pcName, final String appName) {
		return DeviceServer	.is(retrieveACL(user, pcName, appName), user, pcName, appName);
	}
//
//	/**pp*/ boolean matchesPCName(final String pcName, final String bkrPcPattern) {
//		return pcName != null && pcName.matches(bkrPcPattern);
//	}
//	/**pp*/ boolean equalsUser(final String actualUser, final String expectedUser) {
//		return expectedUser.equalsIgnoreCase(actualUser);
//	}
//
////=============================================================================
//
//	private boolean aclNameContains(final String user, final String pcName, final String appName, final String subString) {
//		return aclNameContains(retrieveACL(user, pcName, appName), subString);
//	}
//	private boolean aclNameContains(final OPModeACLEntry acl, final String refString) {
//		return acl != null
//		&& acl.getAclName() != null
//		&& acl.getAclName().contains(refString);
//	}
	private OPModeACLEntry retrieveACL(final String user, final String pcName, final String appName) {
		if(isThisContext(user, pcName, appName))
			return getOPModeACLEntry();
		final OPModeACLEntry entry = OPMODEACLFACTORY.getOPModeACLEntry(user, pcName, appName);
		return entry;
	}
	private boolean isThisContext(final String user, final String pcName, final String appName) {
		return equalString(user, getUser())
			&& equalString(pcName, getHostName())
			&& equalString(appName, getAppName());
	}
	private boolean equalString(final String lhs, final String rhs) {
		return (lhs == rhs)
			|| (lhs != null && lhs.equalsIgnoreCase(rhs));
	}
}
