/*
 * Created on Mar 30, 2005 - Mar 30, 2005
 *	Feb 15, 2008 - Feb 15, 2008 ws: added "@Override"es
 *	Jan 15, 2009 - Jan 15, 2009	ws:	added equals
 */
package de.desy.mst.libs.framework.mstapp.security;

/**
 * Reine Datenklasse fr die verschiedenen Sicherheitsrollen
 *
 * Vorsicht bei Aenderungen: Die Klasse ist so weit als moeglich schon an
 * Java 1.5 enum's angepasst
 *
 * Benutzung:
 * kann direkt die Objekte mit == vergleichen, da nur genau diese drei Objekte
 * pro Applikation erzeugt werden koennen. Siehe folgendes Beispiel:
 *
 * 		OpMode mode = OpMode.BETRIEB;
 *		if (mode == OpMode.BETRIEB){ ... ;}
 *
 * @author schuette
 * @version 1.0.0
 */
public class SecurityRole {

	public static final SecurityRole GUEST;
	public static final SecurityRole OPERATEUR;
	public static final SecurityRole DEVELOPER;

	public static final SecurityRole[] VALUES;

	static{
		GUEST = new SecurityRole("Guest",0);
		OPERATEUR = new SecurityRole("Operateur",1);
		DEVELOPER = new SecurityRole("Developer",2);

		//Beachte: Index(Array) = index(Objekt)
		VALUES = (new SecurityRole[]{GUEST, OPERATEUR, DEVELOPER});
	}

	public final String name;
	public final int index;

	/**
	 * @param name
	 * @param index
	 *
	 * Konstruktor muss privat sein um == Abfrage zu ermoeglichen
	 */
	private SecurityRole(final String name, final int index) {
		super();
		this.name = name;
		this.index = index;
	}

//	may return null
	public final static SecurityRole valueOf(final String name){
		for(int index = 0; index < VALUES.length; index++){
			if(name == VALUES[index].name){return VALUES[index];}
		}
		return null;	//Maybe through an error instead?
	}

	/* (non-Javadoc)
	 * @see java.lang.Object#hashCode()
	 */
	@Override public int hashCode() {return this.name.hashCode();}
	/* (non-Javadoc)
	 * @see java.lang.Object#equals(java.lang.Object)
	 */
	@Override public boolean equals(final Object obj) {return this == obj;}
	/* (non-Javadoc)
	 * @see java.lang.Object#toString()
	 */
	@Override public String toString() {return "Sicherheits Rolle [" + this.name + "]";}
} 
