/**
 *  	Jul 21, 2005 - Jul 21, 2005	ws: Created on 
 *		Aug  8, 2005 - Sep  2, 2005	ws: 2) an DB anpassen, Iterable, ...
 *		Sep  8, 2005 - Sep 14, 2005 ws: 3) Spalten Support
 * 		Oct 25, 2005 - ...			ws: new logging system
 * 		Dec 13, 2005 - ...			ws: added: getSpalteAsStringArray(...)
 * 		Jun 02, 2005 - ...			ws: better error message on null adding
 * 		Sep 13, 2006 - ...			ws: getFirstEntry() protected -> public
 * 		Oct 25, 2006 - Oct 26, 2006	ws: ISpaltenInfo statt direct PublicFields
 *  Jun 25, 2015 - Jun 25, 2015	ws:	added equals & hash
 */

package de.desy.mst.libs.framework.mstapp.tabellendaten;

import java.lang.ref.SoftReference;
import java.util.Collections;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.TreeMap;

import de.desy.mst.libs.framework.mstapp.ini.IIniFileRoot;
import de.desy.mst.libs.framework.mstapp.ini.XMLUtil;
import de.desy.mst.libs.framework.mstapp.logging.Logger;
import de.desy.mst.libs.framework.mstapp.tabellendaten.entry.AEntry;
import de.desy.mst.libs.framework.mstapp.util.reflection.ISpaltenInfo;
import de.desy.mst.libs.framework.mstapp.util.reflection.PublicFields;

/**
 * Einfache Tabelle (als (Hash)Map implementiert)
 * <li>mit Zeilen des Typs E (Entries mit Interface IEntry<K>)</li>
 * <li>mit Key K</li>
 * DeFacto Immutable after JavaXMLDecoder Construction
 * 
 * @author ws
 * @version 1.0.0
 */
@SuppressWarnings({"rawtypes"})
public class Tabelle   <K extends Comparable<K>,  E extends AEntry<K>> 
		     implements Iterable<E>, IIniFileRoot{

	private static final String NO_TABID_PREFIX = "NO ID SET. ";
	private SoftReference<ISpaltenInfo<E>> spaltenRef;
	
	/**Kontrakt: never Null, muss aber im XTor gesetzt werden*/
	protected Map<K, E>	entries;	//pseudoFinal
	protected String 	tabId;		//pseudoFinal, Unique Id fuer Tabelle (App weit, eigentlich sogar Systemweit)
	protected E			firstEntry; //pseudoFinal, erstes Entry != null fuer analyse Zwecke

//=============================================================================

	/**
	 * 	Kontrakt:
	 * 	<li>super wird nicht gerufen
	 * 	<li>entries wird im XTor gesetzt
	 * 	<li>tabId   wird im XTor gesetzt != null und != ""
	 * @throws NullPointerException, 		tabId == null
	 * @throws IllegalArgumentException, 	tabId == ""
	 */
	public Tabelle() {
		//IDEE Abschaffen!!!! und in DB XML direkt unterstuetzen
		this(NO_TABID_PREFIX + AlleTabellen.size());	
	}
	/**
	 * @throws NullPointerException, 		tabId == null
	 * @throws IllegalArgumentException, 	tabId == ""
	 */
	public Tabelle(final String tabId) {
		if(tabId == null){throw new NullPointerException("TabellenId ist null.");}
		if(tabId.length() == 0){throw new IllegalArgumentException("TabellenId ist der leere String.");}
		this.entries = new TreeMap<K, E>(); // never null
		this.tabId = tabId;
	}

//=============================================================================

	@Override public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((this.entries == null) ? 0 : this.entries.hashCode());
		result = prime * result + ((this.tabId == null) ? 0 : this.tabId.hashCode());
		return result;
	}
	@Override public boolean equals(final Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		final Tabelle other = (Tabelle) obj;
		if (this.tabId == null) {
			if (other.tabId != null)
				return false;
		} else if (!this.tabId.equals(other.tabId))
			return false;
		if (this.entries == null) {
			if (other.entries != null)
				return false;
		} else if (!this.entries.equals(other.entries))
			return false;
		return true;
	}

//.................................................................................................	
	/**eindeutiger Tabellen Identifizierer, Schluessel in AlleTabellen*/
	public final String getTabId() {return this.tabId;}
	public final E getEntry(final K key) {return this.entries.get(key);}
	public final boolean containsKey(final K key) 		{return this.entries.containsKey(key);}
	public final boolean containsValue(final E entry)	{return this.entries.containsValue(entry);}
	public final int size(){return this.entries.size();}

    /**
     * @return an Iterator over the values as value type.
     */ 
	@Override public final Iterator<E> iterator() {
		final Map<K, E> unmodifiableMap = Collections.unmodifiableMap(this.entries);
		return unmodifiableMap.values().iterator();
	}

	/**never null, but possibly empty (nur public nonstatic fields, insbesondere nicht 'unsupportedExtraParams'*/
	public final List<String> getSpaltenNamen(){
		return this.getSpaltenInfo().getSpaltenNamen(); 
	}

//	/**never null, but possibly empty (nur public nonstatic fields, insbesondere nicht 'unsupportedExtraParams'*/
//	public final List<String> getSpaltenNamen(){
//		PublicFields publicFields = getPublicFields();
//		return (publicFields == null) ? new ArrayList<String>(1) : publicFields.getSpaltenNamen();  
//	}	
	/**never null, but possibly empty*/
	public final List<Object> getSpalte(final String spaltenName){
		return this.getSpaltenInfo()
				   .getSpalte(this.getTabId(), spaltenName, this);
	}
//	/**never null, but possibly empty*/
//	public final List<Object> getSpalte(final String spaltenName){
//		Field field = getPublicFields().getField(spaltenName);
//		List<Object> spalte = new ArrayList<Object>(size());
//		for (E entry : this) {
//			Object object = null;
//			try {
//				if(entry != null && field != null){object = field.get(entry);}
//			} catch (IllegalArgumentException e) {
//				String msg = createErrMsgForFieldRetrieval(spaltenName, entry);
//				Logger.SW.error(msg);
//			} catch (IllegalAccessException e) {
//				String msg = createErrMsgForFieldRetrieval(spaltenName, entry);
//				Logger.SW.error(msg);
//			}
//			spalte.add(object);
//		}
//		return spalte;  
//	}
	/**never null, but possibly empty*/
	public final String[] getSpalteAsStringArray(final String spaltenName){
		final Object[] oArray = (this.getSpalte(spaltenName)).toArray();
		final String[] sArray = new String[oArray.length];
		for (int i = 0; i < sArray.length; i++) {
			final Object oEntry = oArray[i];
			sArray[i] = (oEntry == null) ? ("") : (oEntry.toString());
		}
		return sArray;
	}
	
	/**true, falls Tabelle im Augenblick inSeinenWertenGleich'e Entries enthaelt*/
	public final boolean inSeinenWertenGleich(final Tabelle<K,E> otherTabelle){
		if(this == otherTabelle)	{return true;}
		if(otherTabelle == null)	{return false;}
		if(this.size() != otherTabelle.size())	{return false;}
		
		if(!inSeinenWertenGleich(otherTabelle, this)){
			return false;
		}
		if(!inSeinenWertenGleich(this, otherTabelle)){
			return false;
		}
		return true;
	}

	private boolean inSeinenWertenGleich(final Tabelle<K, E> otherTabelle, final Tabelle<K, E> thisTabelle) {
		for (final AEntry<K> entry : thisTabelle) {
			final K key = entry.getKey();
			if(!otherTabelle.containsKey(key))	{
				return false;
			}
			final E otherEntry = otherTabelle.getEntry(key);
			if(!entry.inSeinenWertenGleich(otherEntry))	{
				return false;
			}
		}
		return true;
	}
	
	public final boolean hasTrivialTabId(){
		final String id = getTabId();
		return id == null 
		    || id.length() == 0
		    || id.startsWith(NO_TABID_PREFIX);
	}

	/* (non-Javadoc)
	 * @see de.desy.mst.mstapp.ini.IIniFileRoot#getContentRoot()
	 */
	@Override public final Object getContentRoot() {
		return this;
	}
	/* (non-Javadoc)
	 * @see de.desy.mst.mstapp.ini.IIniFileRoot#setContentRoot(java.lang.Object)
	 */
	/**@throws UnsupportedOperationException*/
	@Override public void setContentRoot(final Object contentRoot) {
		throw new UnsupportedOperationException("Tabelle ist seine eigene Content Root"); 
	}
	
//-----------------------------------------------------------------------------

	/**
	 * only for XMLDecoder use 
	 * 
	 * @throws NullPointerException, 	 if the specified element is null
	 * @throws IllegalArgumentException, if the specified element already exists
	 */
	@Deprecated
	public boolean add(final E zeile) {
		if (zeile == null) {
			final String msg = "Tabellendaten '" + this.getClass().getName() + "' haben keine null Zeilen.";
			final NullPointerException e = new NullPointerException(msg);
			Logger.SW.error(msg, e);
			throw e;
		}
		final K key = zeile.getKey();
		if (containsKey(key)) {
			final String msg = "Key " + key + " existiert schon in der Tabelle,"
 				                      + " neuer Eintrag wird ignoriert.";
			final IllegalArgumentException e = new IllegalArgumentException(msg);
			Logger.SW.error(msg, e);
			throw e;
		}
		this.entries.put(key, zeile);
		if(this.firstEntry == null){this.firstEntry = zeile;}
		return true;
	}
	
//=============================================================================

	public final E getFirstEntry()	{return this.firstEntry;}
	
//=============================================================================

	/**
	 * Darf nur von java.beans.Encoder aufgerufen werden (runtime enforced!).
	 * @return Returns the entries == interne Repraesentation der Reihen.
	 * @throws UnsupportedOperationException, bei Aufruf (Ausnahme java.beans.Encoder)
	 */
	@Deprecated
	public final Map<K, E> getEntries() {
		XMLUtil.checkCalledByXMLDecoder();
		return this.entries;
	}

	/**
	 * Darf nur von java.beans.Encoder aufgerufen werden (runtime enforced!).
	 * @param entries The entries to set.  entries == interne Repraesentation der Reihen.
	 * @throws UnsupportedOperationException, bei Aufruf (Ausnahme java.beans.Encoder)
	 */
	@Deprecated
	public final void setEntries(final Map<K, E> entries) {
		XMLUtil.checkCalledByXMLDecoder();
		this.entries = entries;
	}
	/**
	 * Darf nur von java.beans.Encoder aufgerufen werden (runtime enforced!).
	 * @param tabId == eindeutige Id der Tabelle.
	 * @throws UnsupportedOperationException, bei Aufruf (Ausnahme java.beans.Encoder)
	 */
	@Deprecated
	public final void setTabId(final String tabId) {
		XMLUtil.checkCalledByXMLDecoder();
		this.tabId = tabId;
	}
	/**
	 * Darf nur von java.beans.Encoder aufgerufen werden (runtime enforced!).
	 * @throws UnsupportedOperationException, bei Aufruf (Ausnahme java.beans.Encoder)
	 */
	@Deprecated
	protected final void setFirstEntry(final E firstEntry) {
		XMLUtil.checkCalledByXMLDecoder();
		this.firstEntry = firstEntry;
	}
	protected static String getNO_TABID_PREFIX() {
		return NO_TABID_PREFIX;
	}
	
	/**
	 * Hier wird die fuer diese Tabelle relevante SpaltenInfo erzeugt.
	 * @param 	theClass	typ der Entries der Tabelle oder falls leer null
	 * @return  SpaltenInfo fuer die Entries dieser Tabelle (vom Typ thClass)
	 */
//	@SuppressWarnings("unchecked") 
	protected ISpaltenInfo<E> createSpaltenInfo(final Class<? extends AEntry> theClass) {
		return new PublicFields<E>(theClass);
	}
	
//=============================================================================
	
	private ISpaltenInfo<E> getSpaltenInfo() {
		return this.getSpaltenRef().get();
	}
	
	/**never null*/
	private SoftReference<ISpaltenInfo<E>> getSpaltenRef() {
		if	(		this.spaltenRef == null 
				|| 	this.spaltenRef.get() == null 
				|| 	this.spaltenRef.get().isTrivial()
			){
			this.spaltenRef = createSpaltenReference(this.getFirstEntry());
		}
		return this.spaltenRef;
	}
	/**package privat: hier koennen spezielle ISpaltenInfos eingegeben werden*/
//	@SuppressWarnings("unchecked") 
	private SoftReference<ISpaltenInfo<E>> createSpaltenReference(final E firstEntry){
		Class<? extends AEntry> theClass = null;
		if(firstEntry != null) {theClass = firstEntry.getClass();}
		final ISpaltenInfo<E> spaltenInfo = createSpaltenInfo(theClass);
		return new SoftReference<ISpaltenInfo<E>>(spaltenInfo);
	}
}
