/**
 *	Sep 13, 2006 - Sep 21, 2006	ws: creation from Status entry
 *	Jun 12, 2008 - Jun 12, 2008	ws: small fix
 *	Dec  2, 2008 - Dec  2, 2008	ws:	getTabelle(..) handle appName == null
 *	Dec 15, 2009 - Dec 15, 2009	ws:	implemented: useLocalSecurity debug switch semantics
 *	Mar 30, 2015 - Apr 13, 2015	ws:	start migration of OPModeACLEntry retrieval from Web to AnLa TineSrv
 *	Jun 22, 2015 - Jun 25, 2015	ws:	extracted for caching the OpModeACLEntrys
 *									(1) calcTabId(...)
 *									(2) extractOPModeACLEntry(tabelle)
 *									added: packOPModeACLEntry(entry)
 *	Jul 14, 2015 - Jul 14, 2015	ws:	added statics (de-)serialize(...)
 * 	Jul 14, 2015 - Jul 17, 2015	ws:	added equals(...), hashCode() for JUnit tests
 * 									added calcFileName() for filename calculation in object cache
 *	Aug 24, 2015 - Aug 24, 2015	ws:	expanded info in toLoggerInfo()
 */
package de.desy.mst.libs.framework.mstapp.tabellendaten.entry;

import java.beans.XMLDecoder;
import java.io.IOException;
import java.io.InputStream;

import de.desy.mst.libs.framework.mstapp.logging.Logger;
import de.desy.mst.libs.framework.mstapp.mode.OpMode;
import de.desy.mst.libs.framework.mstapp.tabellendaten.AlleTabellen;
import de.desy.mst.libs.framework.mstapp.tabellendaten.Tabelle;
import de.desy.mst.libs.framework.mstapp.util.xml.XMLCoder;

/**
 * @author  ws
 * @version 1.0.0
 */
public class OPModeACLEntry  extends AEntry<Integer>{
	public final static OpMode DEFAULT_OPMODE = OpMode.BETRACHTEN;	//im Zweifel harmlos
	       final static int    DEFAULT_OAZ_NR = -999;				//
	public final static String TAB_ID_PREFIX  = "OpModeACLXML0+";

	public final int	oazNr	;
	public final String	pcUser	;
	public final String	pcName	;
	public final String	appName	;
	public final OpMode	maxOpMode	;
	public final OpMode	defaultOpMode	;
	public final OpMode	maxTestServerOpMode	;
	public final String	aclName	;


//=============================================================================
	public final static String SERIALIZATION_TEMPLATE =
			"<java class=\"java.beans.XMLDecoder\">"
					+ "<object class=\"%s\">" 
						+ "<int>%d</int>" 
						+ "<string>%s</string>" 
						+ "<string>%s</string>" 
						+ "<string>%s</string>" 
						+ "<int>%d</int>" 
						+ "<int>%d</int>" 
						+ "<int>%d</int>" 
						+ "<string>%s</string>" 
					+ "</object>" 
				+ "</java>"
				;
//			"<java class=\"java.beans.XMLDecoder\">"
//				+ "<object class=\"de.desy.mst.libs.framework.mstapp.tabellendaten.entry.OPModeACLEntry\">" 
//					+ "<int>35</int>" 
//					+ "<string>schuette</string>" 
//					+ "<string></string>" 
//					+ "<string></string>" 
//					+ "<int>2</int>" 
//					+ "<int>1</int>" 
//					+ "<int>2</int>" 
//					+ "<string>Default1: User: Entwickler</string>" 
//				+ "</object>" 
//			+ "</java>"
//			;
			
	public static String serialize(final OPModeACLEntry entry) { 
//		public OPModeACLEntry(final int oazNr
//						    , final String pcUser, final String pcName, final String appName
//						    , final int maxOpMode, final int defaultOpMode, final int maxTestServerOpMode
//						    , final String aclName
//				) {
		return String.format(SERIALIZATION_TEMPLATE
				           , entry.getClass().getName()
				           , entry.oazNr
				           , enc(entry.pcUser),     enc(entry.pcName),         enc(entry.appName)
				           , entry.maxOpMode.index, entry.defaultOpMode.index, entry.maxTestServerOpMode.index
				           , enc(entry.aclName)
				            );
	}
	private static String enc(final String s) {return XMLCoder.encode(s);}

	public static OPModeACLEntry deSerialize(final String sEntry) throws IOException{
		final XMLDecoder xmlDecoder = new XMLDecoder(createInputStreamFrom(sEntry));
        try {return (OPModeACLEntry) xmlDecoder.readObject();}
        catch (final Exception e) {throw new IOException(e.toString());}
        finally{xmlDecoder.close();}
	}

	@SuppressWarnings("deprecation")
	private static InputStream createInputStreamFrom(final String readString) {
		return new java.io.StringBufferInputStream(readString);
	}
	
//-------------------------------------------------------------------------------------------------

	public final static OpMode createOPMode(final int opModeNr){
		final OpMode[] opModes = OpMode.getValues();
		if(0 <= opModeNr && opModeNr < opModes.length){
			return opModes[opModeNr];
		}else{
			final String msg = OPModeACLEntry.class.getName()
							 + ".createOPMode: OPModeNr "
							 + opModeNr
							 + " existiert nicht. Nehme Default OPMode '"
							 + DEFAULT_OPMODE
							 + "' an.";
			Logger.SWFW.error(msg);
			return DEFAULT_OPMODE;
		}
	}

	@Deprecated public static OPModeACLEntry getOPModeACLEntry(final String pcUser, final String pcName, final String appName) {
		return OPModeACLEntryFactory_Default.INSTANCE.getOPModeACLEntry(pcUser, pcName, appName);
//		if(useLocalSecurity()){
//			Logger.SWFW.warn("IM JUNIT DEBUG MODUS. Use getDefaultOPModeACLEntry(...)");
//			return OPModeACLEntry.getNullOPModeACLEntry(pcUser, pcName, appName);
//		}
//		final OPModeACLEntry entry = getOPModeACLEntry_from_TabellenDaten(pcUser, pcName, appName);
//		return entry == null ? OPModeACLEntry.getNullOPModeACLEntry(pcUser, pcName, appName)
//							 : entry;
	}
	@Deprecated static OPModeACLEntry getOPModeACLEntry_from_TabellenDaten(final String pcUser, final String pcName, final String appName) {
		final Tabelle<Integer, OPModeACLEntry> tabelle = getTabelle(pcUser, pcName, appName);
		return extractOPModeACLEntry(tabelle);
	}
//	private static OPModeACLEntry getNullOPModeACLEntry(final String pcUser, final String pcName, final String appName) {
//		return _OPModeACLEntryFactory_NullObject._INSTANCE
//				                                .getOPModeACLEntry(pcUser, pcName, appName);
//		final int opModeNr = DEFAULT_OPMODE.index;
//		final String msg = " - getDefaultOPModeACLEntry("  + toString(pcUser, pcName, appName + ")");
//		return new OPModeACLEntry(DEFAULT_OAZ_NR
//				                , pcUser, pcName, appName
//				                , opModeNr, opModeNr , opModeNr
//				                , "X"+opModeNr+opModeNr+opModeNr + msg);
//	}

	@SuppressWarnings("deprecation")
	public static Tabelle<Integer, OPModeACLEntry> packOPModeACLEntry(final OPModeACLEntry entry) {
//		calcTabId(String pcUser, String pcName, String appName)
		final String tabId = calcTabId(entry.pcUser, entry.pcName, entry.appName);
		final Tabelle<Integer, OPModeACLEntry> tab = new Tabelle<Integer, OPModeACLEntry> (tabId);
		tab.add(entry);
		return tab;
	}

	public static OPModeACLEntry extractOPModeACLEntry(final Tabelle<Integer, OPModeACLEntry> tabelle) {
		final int anzahl = tabelle.size();
		if(anzahl <= 0)
			return null;
		if(anzahl > 1)
			Logger.SWFW.error("Zu viele Entries in 'OPModeACLEntry getOPModeACLEntry' bekommen");
		return tabelle.getFirstEntry();
	}

	private static String toString(final String ...strings) {
		final String separator = ", ";
		String retVal = "";
		final int maxIndex = strings.length - 1;
		for (int i = 0; i <= maxIndex; i++) {
			final String string = strings[i];
			retVal += string;
			if(i != maxIndex) retVal += separator;
		}
		return retVal;
	}
	/**null oder in SQL "'" eingepackt*/
	private static String normalizeString(final String string) {
		if(string == null || string .length() == 0)
			return null;
		return "'" + string + "'";
	}

	private static Tabelle<Integer, OPModeACLEntry> getTabelle(final String pcUser, final String pcName, final String appName) {
		final String tabId = calcTabId(pcUser, pcName, appName);
		return AlleTabellen.getTabelle(tabId);
	}
	public static String calcTabId(String pcUser, String pcName, String appName) {
		pcUser = normalizeString(pcUser);
		pcName = normalizeString(pcName);
		appName = normalizeString(appName);
		appName = (appName == null ? appName : appName.replace(" ", "+"));
		final String entry = normalizeString(OPModeACLEntry.class.getName());
		//		PROC OpModeACLXML0
//		  @pcUser		nvarchar( 25)	= null
//		, @pcName		nvarchar( 25)	= null
//		, @appName		nvarchar( 25)	= null
//		, @entry		nvarchar(255)	= 'de.desy.mst.libs.framework.mstapp.tabellendaten.entry.OPModeACLEntry'
//		, @collection	nvarchar(255)	= 'de.desy.mst.libs.framework.mstapp.tabellendaten.Tabelle'
//		, @decoder		nvarchar(255)	= 'java.beans.XMLDecoder'
		final String tabId = TAB_ID_PREFIX +	 "@pcUser="		+ pcUser
										   +	",@pcName="		+ pcName
										   +	",@appName="	+ appName
										   +	",@entry="		+ entry;
		return tabId;
	}
//=============================================================================

	public OPModeACLEntry(final int oazNr
			            , final String pcUser, final String pcName, final String appName
			            , final int maxOpMode, final int defaultOpMode, final int maxTestServerOpMode
			            , final String aclName
			             ) {
		super();
		this.oazNr		= oazNr;

		this.pcUser		= pcUser;
		this.pcName		= pcName;
		this.appName	= appName;

		this.maxOpMode				= createOPMode(maxOpMode);
		this.defaultOpMode			= createOPMode(defaultOpMode);
		this.maxTestServerOpMode	= createOPMode(maxTestServerOpMode);

		this.aclName	= aclName;
	}
	public OPModeACLEntry(final int oazNr
						, final String pcUser, final String pcName, final String appName
						, final int maxOpMode, final int defaultOpMode, final int maxTestServerOpMode
			            , final String aclName
						, final Object ... unsupportedExtraParams) {
		this(oazNr, pcUser, pcName, appName, maxOpMode, defaultOpMode, maxTestServerOpMode, aclName);
		handleUnsupportedExtraParams(this, unsupportedExtraParams);
	}
//	/**Verzweiflungs XTor*/
//	public OPModeACLEntry(final Object ...objects){
//		this(0, "", "", "", 0, 0, 0);
//		Logger.SWFW.warn("Bad OPModeACLEntry XTOR");
//		Logger.SWFW.info(Arrays.asList(objects).toString());
//	}

//=============================================================================
	/* (non-Javadoc)
	 * @see de.desy.mst.mstapp.tabellendaten.entry.AEntry#getKey()
	 */
	@Override
	public final Integer 	getKey() 	{return Integer.valueOf(getOAZNr());}

	public final int getOAZNr()		{return this.oazNr;}

	public final String getPcUser()		{return this.pcUser;}
	public final String getPcName()		{return this.pcName;}
	public final String getAppName() 	{return this.appName;}

	public final OpMode getMaxOpMode()			{return this.maxOpMode;}
	public final OpMode getDefaultOpMode()		{return this.defaultOpMode;}
	public final OpMode getMaxTestServerOpMode(){return this.maxTestServerOpMode;}
	public String getAclName() 					{return this.aclName;}

	public String calcFileName() {return calcFileName(this.pcUser, this.pcName, this.appName);}
	public static String calcFileName(String pcUser, String pcName, String appName) {
		pcUser  =  pcUser.replaceAll("<", "0").replaceAll(">", "1");
		pcName  =  pcName.replaceAll("<", "0").replaceAll(">", "1");;
		appName = appName.replaceAll("<", "0").replaceAll(">", "1");;
		appName = (appName == null ? appName : appName.replace(" ", "+"));
		return "ACL(" + pcUser + ", " + pcName + ", "+ appName + ")";
	}

	@Override public String toString() {
		return	"OPModeACLEntry"
			+ 	"("
			+ 	OPModeACLEntry.toString(
						"oazNr = "   + getOAZNr()

					,	"pcUser = "  + getPcUser()
					,	"pcName = "  + getPcName()
					,	"appName = " + getAppName()

					,	"maxOpMode = "			+ getMaxOpMode().name
					,	"defaultOpMode = " 		+ getDefaultOpMode().name
					,	"maxTestServerOpMode = "+ getMaxTestServerOpMode().name
					,	"aclName = " + getAclName()
                       					)
           + ")"
           ; 
	}

//=============================================================================

	/**statt this.equals zu definieren, muesste dann auch HashCode neu definieren*/
	@Override public boolean inSeinenWertenGleich(final AEntry<Integer> aEntry){
		if( this.equals(aEntry)) {return true;}

		if(!(aEntry instanceof OPModeACLEntry)){return false;}
		final OPModeACLEntry other = (OPModeACLEntry) aEntry;
		return 	this.oazNr == other.oazNr
			&&	this.pcUser.equalsIgnoreCase(other.pcUser)
			&&	this.pcName.equalsIgnoreCase(other.pcName)
			&&	this.appName.equalsIgnoreCase(other.appName)
			&&	this.maxOpMode 				== other.maxOpMode
			&&	this.defaultOpMode 			== other.defaultOpMode
			&&	this.maxTestServerOpMode 	== other.maxTestServerOpMode
			&&	this.aclName.equalsIgnoreCase(other.aclName)
			;
	}

	public String toLoggerInfo() {
		return	"Benutze folgende OpMode Einstellungen: "
		+	" " + getAclName() + " "
		+	"(" + getMaxOpMode().name
		+	"," + getDefaultOpMode().name
		+	"," + getMaxTestServerOpMode().name
		+	"," + getOAZNr()
		+	") (Max,Default,MaxTestServer,oazNr)."
		+ "\n auf Basis von (" +  this.pcUser + ", " +  this.pcName + ", " +  this.appName + ")"
		+ " (pcUser, pcName, appName)"
		;
	}
//-------------------------------------------------------------------------------------------------
	@Override public int hashCode() {
		final int prime = 31;
		int result = super.hashCode();
		result = prime * result + ((this.aclName == null) ? 0 : this.aclName.hashCode());
		result = prime * result + ((this.appName == null) ? 0 : this.appName.hashCode());
		result = prime * result
				+ ((this.defaultOpMode == null) ? 0 : this.defaultOpMode.hashCode());
		result = prime * result
				+ ((this.maxOpMode == null) ? 0 : this.maxOpMode.hashCode());
		result = prime
				* result
				+ ((this.maxTestServerOpMode == null) ? 0 : this.maxTestServerOpMode
						.hashCode());
		result = prime * result + this.oazNr;
		result = prime * result + ((this.pcName == null) ? 0 : this.pcName.hashCode());
		result = prime * result + ((this.pcUser == null) ? 0 : this.pcUser.hashCode());
		return result;
	}

	@Override public boolean equals(final Object obj) {
		if (this == obj)
			return true;
		if (!super.equals(obj))
			return false;
		if (getClass() != obj.getClass())
			return false;
		final OPModeACLEntry other = (OPModeACLEntry) obj;
		if (this.aclName == null) {
			if (other.aclName != null)
				return false;
		} else if (!this.aclName.equals(other.aclName))
			return false;
		if (this.appName == null) {
			if (other.appName != null)
				return false;
		} else if (!this.appName.equals(other.appName))
			return false;
		if (this.defaultOpMode == null) {
			if (other.defaultOpMode != null)
				return false;
		} else if (!this.defaultOpMode.equals(other.defaultOpMode))
			return false;
		if (this.maxOpMode == null) {
			if (other.maxOpMode != null)
				return false;
		} else if (!this.maxOpMode.equals(other.maxOpMode))
			return false;
		if (this.maxTestServerOpMode == null) {
			if (other.maxTestServerOpMode != null)
				return false;
		} else if (!this.maxTestServerOpMode.equals(other.maxTestServerOpMode))
			return false;
		if (this.oazNr != other.oazNr)
			return false;
		if (this.pcName == null) {
			if (other.pcName != null)
				return false;
		} else if (!this.pcName.equals(other.pcName))
			return false;
		if (this.pcUser == null) {
			if (other.pcUser != null)
				return false;
		} else if (!this.pcUser.equals(other.pcUser))
			return false;
		return true;
	}
}
