/**
 *  Created on  Sep  2, 2005 - Sep 14, 2005
 * 				Dec 14, 2005 - ...	Konstanten FIELD_*
 */
package de.desy.mst.libs.framework.mstapp.tabellendaten.entry;

/**
 * Basisklasse fuer alle SedacDevice'
 * @author ws
 * @version 1.0.0
 */
public class SedacDevice extends Device {
	public static final String FIELD_SED_LN	= "sedLn";	//KEEP IN SYNCH WITH ACTUAL FIELD NAME
	public static final String FIELD_SED_CR	= "sedCr";	//KEEP IN SYNCH WITH ACTUAL FIELD NAME
	public static final String FIELD_SED_SU	= "sedSu";	//KEEP IN SYNCH WITH ACTUAL FIELD NAME
	
	public final int sedLn;	// >= 0
	public final int sedCr;	// >= 0
	public final int sedSu;	// >= 0
	
//=============================================================================
	
	/**
	 * @param devGrNr 			Device Gruppen Nummer
	 * @param devNo   >= 0		Device Nummer 	(eindeutig innerhalb einer Device Gruppe)
	 * @param devName <> null	Device Name 	(eindeutig innerhalb einer Device Gruppe)
	 * @param sedLn   >= 0		SEDAC Line
	 * @param sedCr   >= 0      SEDAC Crate
	 * @param sedSu   >= 0		Sedac SubAddress
	 * @throws NullPointerException,     if devName == null
	 * @throws IllegalArgumentException, if (devNo, sedCr, sedLn, sedSu) < 0
	 */
	public SedacDevice(final int devGrNr, final int devNo, final String devName
			         , final int sedLn,   final int sedCr, final int sedSu) {
		super(devGrNr, devNo, devName);
		this.sedLn = sedLn;
		this.sedCr = sedCr;
		this.sedSu = sedSu;
	}

//=============================================================================
	public int getSedCr() {return this.sedCr;}
	public int getSedLn() {return this.sedLn;}
	public int getSedSu() {return this.sedSu;}

	@Override public boolean inSeinenWertenGleich(final AEntry<Integer> otherEntry) {
		if(!(otherEntry instanceof SedacDevice)) {return false;}
		final SedacDevice other = (SedacDevice) otherEntry;
		return super.inSeinenWertenGleich(otherEntry)
			&& this.sedCr == other.sedCr 
			&& this.sedLn == other.sedLn 
			&& this.sedSu == other.sedSu 
			;
	}
}
 