/**
 *  Created on 	Jul 19, 2005 - Jul 20, 2005	ws:	1) erster Entwurf
 *  			Aug 08, 2005 - Sep 07, 2005	ws:	2) an DB anpassen und mit AEntry impl.
 * 				Oct 25, 2005 - ...			ws:	new logging system
 * 				Oct 31, 2007 - Nov 20, 2007	ws:	Color durch AppColor ersetzt
 * 				Feb 14, 2008 - Feb 14, 2008	ws:	farbNr 2 AppColor._AUS (nicht RED)
 *	Feb 15, 2008 - Feb 15, 2008 ws: added "@Override"es
 */
package de.desy.mst.libs.framework.mstapp.tabellendaten.entry;

import de.desy.mst.libs.framework.mstapp.ini.XMLUtil;
import de.desy.mst.libs.framework.mstapp.logging.Logger;
import de.desy.mst.libs.framework.mstapp.util.gui.AppColor;

/**
 * Zeile der Status Tabelle
 * Immutable Object (kann nach Erzeugung nicht mehr geaendert werden)
 * @author ws
 * @version 1.0.0
 */
public class Status extends AEntry<Integer>{
	public final static int DEFAULT_FARB_NR = 0;
	
	public final int      aufgNr;	//Aufgaben Nummer
	public final int      nr;		//Nummer des Entries 
	public final String   msg;		//Bedeutung: kurz
	public final String   dscr;		//Bedeutung: lang
	public final AppColor farbe;	//Farbe im Device Mgr Display und evtl. auch anderswo	   

//=============================================================================
	
	public final static AppColor createFarbe(final int farbNr){
		switch (farbNr) {	//alles 'fall through statements', wegen unmittelbarem 'return'
		case -2:	return AppColor.RED;
		case -1:	return AppColor.YELLOW;
		case  0:	return AppColor.GREEN;
		case +1:	return AppColor.WHITE;
		case +2:	return AppColor._AUS;
		
		default:	
			Logger.SWFW.error(badFarbNrMsg(farbNr));
			return createFarbe(DEFAULT_FARB_NR);	
		}
	}

	private final static String badFarbNrMsg(final int farbNr){
		return Status.class.getName()
			 + ".createFarbe: FarbNr "
			 + farbNr 
			 + " existiert nicht. Nehme Default Farb Nummer "
			 + DEFAULT_FARB_NR 
			 + " an.";
	}

//=============================================================================
	
	//<object long="de.desy.mst.mstapp.tabellendaten.tabellen.StatusEntry">
	//	<int>0</int> 
	//	<int>0</int> 
	//	<string>OK</string> 
	//	<string>.</string> 
	//	<int>0</int> 
	//</object>
	public Status(final int aufgNr, final int nr, final String msg, final String dscr, final int farbNr) {
		this(aufgNr, nr, msg, dscr, createFarbe(farbNr));
	}

	public Status(final int aufgNr, final int nr, final String msg, final String dscr, final AppColor farbe) {
		super();
		this.aufgNr = aufgNr;
		this.nr 	= nr; 
		this.msg 	= msg;
		this.dscr 	= dscr;
		this.farbe  = farbe;
	}
	/**DO NOT USE, ONLY DYSFUNCTIONAL DUMMY FOR XMLEncoder*/
	@Deprecated public Status()	{
		this(-99, -999, "TestStatus", "This gets created as a test for XMLEncoding only by Default XTor.", 2);
		XMLUtil.checkCalledByXMLDecoder();
	}

//=============================================================================
	/* (non-Javadoc)
	 * @see de.desy.mst.mstapp.tabellendaten.entry.AEntry#getKey()
	 */
	@Override
	public final Integer 	getKey() 	{return Integer.valueOf(getNr());}
	//strenggenommen (aufgNr, Nr), per Konvention aber applikationsweit eindeutig
	
	public final int 		getAufgNr()	{return this.aufgNr;}
	public final Aufgabe	getAufgabe(){return Aufgabe.getByAufgNr(getAufgNr());}
	public final int		getNr()		{return this.nr;}
	public final String 	getMsg()	{return this.msg;}
	public final String 	getDscr()	{return this.dscr;}
	public final AppColor	getFarbe()	{return this.farbe;}
	
//=============================================================================
	
	/**statt this.equals zu definieren, muesste dann auch HashCode neu definieren*/
	@Override public boolean inSeinenWertenGleich(AEntry<Integer> aEntry){
		if( this.equals(aEntry)) {return true;}
		
		if(!(aEntry instanceof Status)){return false;}
		Status entry = (Status) aEntry;
		if( this.getAufgNr() != entry.getAufgNr())	{return false;}
		if( this.getNr() != entry.getNr())	{return false;}
		if(!objectEquals(this.getMsg(), entry.getMsg())) {return false;}
		if(!objectEquals(this.getDscr(), entry.getDscr())) {return false;}
		if(this.getFarbe() != entry.getFarbe())	{return false;}
		return true;
	}
	protected final boolean objectEquals(Object lhs, Object rhs) {
		return ((lhs == rhs) || ((lhs != null) && (lhs.equals(rhs))));
	}
	
//=============================================================================
	/**DO NOT USE, ONLY DYSFUNCTIONAL DUMMY FOR XMLEncoder*/
	@Deprecated public final void setAufgNr(int aufgNr)	{XMLUtil.checkCalledByXMLDecoder();}
	/**DO NOT USE, ONLY DYSFUNCTIONAL DUMMY FOR XMLEncoder*/
	@Deprecated public final void setDscr(String dscr)	{XMLUtil.checkCalledByXMLDecoder();}
	/**DO NOT USE, ONLY DYSFUNCTIONAL DUMMY FOR XMLEncoder*/
	@Deprecated public final void setFarbe(AppColor farbe)	{XMLUtil.checkCalledByXMLDecoder();}
	/**DO NOT USE, ONLY DYSFUNCTIONAL DUMMY FOR XMLEncoder*/
	@Deprecated public final void setMsg(String msg)	{XMLUtil.checkCalledByXMLDecoder();}
	/**DO NOT USE, ONLY DYSFUNCTIONAL DUMMY FOR XMLEncoder*/
	@Deprecated public final void setNr(int nr)			{XMLUtil.checkCalledByXMLDecoder();}
}
