/**
 * Oct 19, 2006 - Nov  1, 2006	ws:	created from scratch
 * Nov 06, 2006 - ...			ws: guessTabName(tabId) as static method extrac.
 */
package de.desy.mst.libs.framework.mstapp.tabellendaten.generics;

import de.desy.mst.libs.framework.mstapp.logging.Logger;
import de.desy.mst.libs.framework.mstapp.tabellendaten.AlleTabellen;
import de.desy.mst.libs.framework.mstapp.tabellendaten.Tabelle;
import de.desy.mst.libs.framework.mstapp.tabellendaten.entry.AEntry;
import de.desy.mst.libs.framework.mstapp.util.reflection.ISpaltenInfo;

/**
 * Spezial Tabelle fuer GEntry's (Generische Entries).
 * @author ws
 */
@SuppressWarnings({"rawtypes"})
public class GEntryTabelle extends Tabelle<Long, GEntry> {
	public static final String ENTRY_PARAM		= "@entry";
	public static final String ENTRY_CLASS		= GEntry.class.getName();
	public static final String COLLECTION_PARAM = "@collection";
	public static final String COLLECTION_CLASS = GEntryTabelle.class.getName();
	
	private String tabName = null;	//TabellenName (geraten oder set von aussen)
//=============================================================================
		
	public GEntryTabelle(final String tabId)	{super(tabId);	}	
	public GEntryTabelle() 				{super();		}
	
//=============================================================================
	
	/**
	 * Versucht aus einer normalen TabId eine Generische Tabelle zu Konstruieren
	 * Geht nur falls die stored Procedure in der Konvention<br> 
	 * entry 		= NAME_DES_ENTRY <br>
	 * Collection 	= NAME_DER_TABELLE <br>
	 * geschrieben ist. Explizites setzen dieser beiden Parameter 
	 * wird honoriert.
	 */
	public static GEntryTabelle getTabelle(final String tabId) {
		final Object tabelle = AlleTabellen.getTabelleAsObject(modifyTabId(tabId));	 
		final GEntryTabelle gEntryTabelle = (GEntryTabelle)tabelle;
		return gEntryTabelle;			
	}
	
	/**
	 * @return the tabName, falls nicht gesetzt wird es geraten aus tabId
	 */
	public String getTabName() {
		if(this.tabName == null){
			this.setTabName(guessTabName());
		}
		return this.tabName;
	}
	/**
	 * @param tabName the tabName to set
	 */
	public void setTabName(final String tabName) {
		this.tabName = tabName;
	}
	
//=============================================================================
	
	/* (non-Javadoc)
	 * @see de.desy.mst.libs.framework.mstapp.tabellendaten.Tabelle#createSpaltenInfo(java.lang.Class)
	 */
	@Override protected ISpaltenInfo<GEntry> 
						createSpaltenInfo(final Class<? extends AEntry> theClass) {
		return new GEntrySpaltenInfo(getTabName());
	}
	
	protected String guessTabName() {
		final String tabId	= this.getTabId();
		return guessTabName(tabId);
	}
//=============================================================================
	static String guessTabName(final String tabId) {
		final String TABID  = tabId.toUpperCase();
		final String sXML   = "XML";
		final String sPLUS  = "+";
		final String sBLANK = " ";
		int index = - 1;
		if(TABID.contains(sXML)){
			index = TABID.indexOf(sXML);
		}else if(TABID.contains(sPLUS)){
			index = TABID.indexOf(sPLUS);
		}else if(TABID.contains(sBLANK)){
			index = TABID.indexOf(sBLANK);
		}else{
			index = TABID.length() - 1;
		}
		final String tabNameRaw = tabId.substring(0, index);
		final String suffix = "Verz";
		final String tabName = tabNameRaw 
							 + (tabNameRaw.contains(suffix) ? "" : suffix);
		final String msg = GEntryTabelle.class.getName() 
						 + ".guessTabName() Guessed the TabName to be '"
						 + tabName
						 + " from the tabId: '"
						 + tabId
						 + "', falls nicht korrekt bitte explizit via" 
						 + " setTabName(...) setzen.";
		Logger.SWFW.info(msg);
		return tabName;
	}
		
	private static String modifyTabId(String tabId) {
		tabId = conditionallyAppend(tabId, ENTRY_PARAM, ENTRY_CLASS);
		tabId = conditionallyAppend(tabId, COLLECTION_PARAM, COLLECTION_CLASS);
		return tabId;
	}
	private static String conditionallyAppend(final String tabId
											, final String paramName
											, final String paramValue) {
		if(tabId.contains(paramName)) {
			return tabId;
		}else{
			final String separator = 
				(tabId.contains("+") || tabId.contains(" ")) ? "," : "";
			return tabId + separator + "+" + paramName + "=" + decorate(paramValue);
		}
	}
	private static String decorate(final String paramValue) {
		if(paramValue.contains("'")) {
			return paramValue;
		}else{
			return "'" + paramValue.trim() + "'";
		}
	}
}
