/**
 * 								ws:	Creation unknown
 *	Jul 28, 2008 - Jul 28, 2008	ws:	added countSubstrings
 */
package de.desy.mst.libs.framework.mstapp.util;

public class MstParser {
	private static final String QUOTE = "\"";
	
	public static final double	DEFAULT_DOUBLE 	= Double.NaN;
	public static final float	DEFAULT_FLOAT 	= Float.NaN;
	public static final int		DEFAULT_INTEGER	= 0;
	public static final byte	DEFAULT_BYTE 	= 0;
	public static final short	DEFAULT_SHORT 	= 0;

	//---------------------------------------------------------------------------
	/**
	 * @param string
	 * @param myDefault
	 * &author W. Schuette -mst1-
	 *
	 * @throws NullPointerException, falls myDefault == null
	 */
	/*
	 * Vers		Date				Description
	 *  01	Jun 8, 2006	Methode erstellt
	 */
	//---------------------------------------------------------------------------
	public static Double parseDouble (final String string, final Double myDefault){
		if(myDefault == null) {throw new NullPointerException();}
		try {
			return new Double(string);
		} catch (NumberFormatException e) {
			return myDefault;
		}
	}
	public static Double parseDouble(final String string){
		return parseDouble(string, Double.valueOf(DEFAULT_DOUBLE));
	}

	//---------------------------------------------------------------------------
	/**
	 * @param string
	 * @param pDefault
	 * &author J. Maass -mst1-
	 *
	 * @throws NullPointerException, falls pDefault == null
	 */
	/*
	 * Vers		Date				Description
	 *  01	Jun 8, 2006	Methode erstellt
	 */
	//---------------------------------------------------------------------------
	public static Float parseFloat (final String string, final Float pDefault){
		if(pDefault == null) {throw new NullPointerException();}
		try {
			return new Float(string);
		} catch (NumberFormatException e) {
			return pDefault;
		}
	}
	public static Float parseFloat(final String string){
		return parseFloat(string, Float.valueOf(DEFAULT_FLOAT));
	}

	//---------------------------------------------------------------------------
	/**
	 * @param pString
	 * @param pDefault
	 * &author J. Maa -mst1-
	 *
	 * @throws NullPointerException, falls myDefault == null
	 */
	/*
	 * Vers		Date				Description
	 *  01	Jun 8, 2006	Methode erstellt
	 */
	//---------------------------------------------------------------------------
	public static Integer parseInteger(final String pString, final Integer pDefault) {
		if(pDefault == null) {throw new NullPointerException();}
		String valString = cutFractional(pString);
		try {
			return new Integer(valString);
		} catch (NumberFormatException e) {
			return pDefault;
		}
	}
	public static Integer parseInteger(final String pString){
		return parseInteger(pString, Integer.valueOf(DEFAULT_INTEGER));
	}

	//---------------------------------------------------------------------------
	/**
	 * @param pString
	 * @param pDefault
	 * &author J. Maa -mst1-
	 *
	 * @throws NullPointerException, falls myDefault == null
	 */
	/*
	 * Vers		Date				Description
	 *  01	Jun 8, 2006	Methode erstellt
	 */
	//---------------------------------------------------------------------------
	public static Byte parseByte(final String pString, final Byte pDefault) {
		if(pDefault == null) {throw new NullPointerException();}
		String valString = cutFractional(pString);
		try {
			return new Byte(valString);
		} catch (NumberFormatException e) {
			return pDefault;
		}
	}
	public static Byte parseByte(final String pString){
		return parseByte(pString, Byte.valueOf(DEFAULT_BYTE));
	}

	//---------------------------------------------------------------------------
	/**
	 * @param pString
	 * @param pDefault
	 * &author J. Maa -mst1-
	 *
	 * @throws NullPointerException, falls myDefault == null
	 */
	/*
	 * Vers		Date				Description
	 *  01	Jun 8, 2006	Methode erstellt
	 */
	//---------------------------------------------------------------------------
	public static Short parseShort(final String pString, final Short pDefault) {
		if(pDefault == null) {throw new NullPointerException();}
		String valString = cutFractional(pString);
		try {
			return new Short(valString);
		} catch (NumberFormatException e) {
			return pDefault;
		}
	}
	public static Short parseShort(final String pString){
		return parseShort(pString, Short.valueOf(DEFAULT_SHORT));
	}

	//---------------------------------------------------------------------------
	/**
	 * @param pString
	 * &author J. Maa -mst1-
	 */
	/*
	 * Vers		Date				Description
	 *  01	Jun 8, 2006	Methode erstellt
	 */
	//---------------------------------------------------------------------------
	private static String cutFractional(String pString) {
		String intString = pString;
  	int k = pString.indexOf(".");
  	if (k != -1) {
  		//Anteil hinter dem Dezimalpunkt abschneiden
  		intString = pString.substring(0,k);
  	}
  	return intString;
	}

//=============================================================================

	public static int countSubstringsIn(final String subString, final String line) {
		if(		line == null 	 || subString == null
		   || 	line.isEmpty() || subString.isEmpty()
		   ||  !line.contains(subString))
			return 0;

		final int skip = subString.length(); int count = -1, fromIndex = 0;
		do{count++;  fromIndex = line.indexOf(subString, fromIndex) + skip;
		}while(fromIndex >= skip);
		return count;
	}
	/**robust String.split in nCols mit entries, '"' and ,,, is respected*/
	public static String[] split(final String separator, final String line, final int nCol){
		if(line == null)
			return split(separator, "", nCol);
		if(separator == null)
			return split("", line, nCol);

		final int skip = separator.length();
		final String [] columns = new String[nCol];
		for (int iCol = 0, fromIndex = 0; iCol < columns.length; iCol++) {
			final int endIndex = calcEndIndex(separator, line, fromIndex);
			final String substring = line.substring(fromIndex, endIndex);
			columns[iCol] = extractQuotedSubstring(substring);
			fromIndex = endIndex + skip;
			if(endIndex >= line.length())
				break;
			fromIndex = endIndex + skip;
		}
		return columns;
	}
	/**if at least two quotes: Contents of Quotes including the quotes, else line*/
	static String extractQuotedSubstring(final String line) {
		if(countSubstringsIn(QUOTE, line) < 2)
			return line;
		final int  firstQuote = line.indexOf(QUOTE, 0);
		final int secondQuote = line.indexOf(QUOTE, firstQuote + 1);
		final String substring = line.substring(firstQuote, secondQuote + 1);
		return substring;
	}
	/**Hilfsfunktion fuer split beruecksichtigt QUOTES, pp zu testen only*/
	static int calcEndIndex(final String subString, final String line, final int fromIndex) {
		final int indexOf = line.indexOf(subString, fromIndex);
		final int firstQuote  = line.indexOf(QUOTE, fromIndex);
		if(firstQuote < 0 || indexOf < firstQuote)
			return indexOf < 0 ? line.length() : indexOf;
		final int secondQuote = line.indexOf(QUOTE, firstQuote + 1);
		if(secondQuote < 0)
			return indexOf < 0 ? line.length() : indexOf;
		return calcEndIndex(subString, line, secondQuote + 1);	//naechtste sep
	}
	/**getrimmed und non null*/
	public static String[] normalize(final String[] input) {
		if(input == null)
			return new String[0];
		for (int i = 0; i < input.length; i++) {
			input[i] = (input[i] == null) ? "" : input[i].trim();
		}
		return input;
	}
}
