/**
 * 	Aug  6, 2009 - Aug  6, 2009	ws:	Eclipse asked for removing unnessary casts, so I did
 */
package de.desy.mst.libs.framework.mstapp.util.applicationresizemanager.managers;

import java.awt.*;
import java.beans.PropertyChangeEvent;

import javax.swing.*;

import de.desy.mst.libs.framework.mstapp.util.applicationresizemanager.listeners.ComponentResizeListener;
import de.desy.mst.libs.framework.mstapp.util.applicationresizemanager.listeners.IconChangeListener;
import de.desy.mst.libs.framework.mstapp.util.applicationresizemanager.resizers.*;

/**
 * Determines the number of Swing components in the container and instantiates the corresponding component resize manager objects.
 * 
 * @author R.Bacher
 * @version 3.0.0
 */	
public class ContainerManager {
	
	private ApplicationResizeManager myApplicationResizeManager = null;
	
	private Component myReferenceComponent = null;
	
	private Container myContainer = null;
	private int myNumberOfComponents = 0;
	private Component[] myContainerComponents;  
	private JPopupMenu[] myContainerComponentsPopupMenues;
	
	private ComponentResizer[] myComponentResizer = null;
	private ComponentResizer[] myComponentPopupMenuResizer = null;
	
	/**
	 * 
	 * Constructor
	 * 
	 * @param referenceComponent Frame object of the resized visual application 
	 * @param container Container object containing the components to be resized
	 *
	 */
	public ContainerManager (final ApplicationResizeManager applicationResizeManager, final Component referenceComponent, final Component container) {	
		
		this.myApplicationResizeManager = applicationResizeManager;
		this.myContainer = (Container)container;
  		this.myReferenceComponent = referenceComponent;
  		findComponents();	
		addListeners();
		analyzeContainer();
	}
	
	/**
	 * Determines the number of Swing components in the container and instantiates the corresponding component resize manger objects.
	 * 
	 */	
	private void findComponents() {
		
		int i;
		
		if (this.myContainer instanceof JMenu) {
			this.myNumberOfComponents = ((JMenu)this.myContainer).getMenuComponentCount();
			this.myContainerComponents = new Component[this.myNumberOfComponents];
			this.myContainerComponentsPopupMenues = new JPopupMenu[this.myNumberOfComponents];
			for (i = 0; i < this.myNumberOfComponents; i++) {
				this.myContainerComponents[i] = (((JMenu)this.myContainer).getMenuComponent(i)); 
				try {
					this.myContainerComponentsPopupMenues[i] = ((JComponent)(this.myContainerComponents[i])).getComponentPopupMenu();
				}
				catch (final ClassCastException e) {}
			}
		}
		else {
			this.myNumberOfComponents = this.myContainer.getComponentCount();
			this.myContainerComponents = this.myContainer.getComponents(); 
			this.myContainerComponentsPopupMenues = new JPopupMenu[this.myNumberOfComponents];
			for (i = 0; i < this.myNumberOfComponents; i++) {
				try {
					this.myContainerComponentsPopupMenues[i] = ((JComponent)(this.myContainerComponents[i])).getComponentPopupMenu();
				}
			    catch (final ClassCastException e) {}
			}
		}
		
		this.myComponentResizer = new ComponentResizer[this.myNumberOfComponents];
		this.myComponentPopupMenuResizer = new ComponentResizer[this.myNumberOfComponents];
        
   	    for (i = 0; i < this.myNumberOfComponents; i++) {
   	       	try {
	   	       	this.myComponentResizer[i] = new ComponentResizer(this.myReferenceComponent, this.myContainerComponents[i], this.myApplicationResizeManager);
	   	       	if (this.myContainerComponentsPopupMenues[i] != null) {this.myComponentPopupMenuResizer[i] = new ComponentResizer(this.myReferenceComponent, this.myContainerComponentsPopupMenues[i], this.myApplicationResizeManager);}
   	       	}
   	       	catch (final ClassCastException e) {
   	       		System.out.println("Caught ClassCastException: Casting " + this.myContainerComponents[i].getClass().getName());
   	        }
   	    }     
   	    
	}
	
	/** 
	 * 
	 * Adds the listener objects to the components found
	 *
	 */
	private void addListeners() {
		
		int i = 0;
		
		IconChangeListener myComponentIconChangeListener = null;
		final PropertyChangeEvent myComponentIconPropertyChangeEvent = null;
		
   	    for (i = 0; i < this.myNumberOfComponents; i++) {	
			if (this.myContainerComponents[i] instanceof JLabel) {
				myComponentIconChangeListener = new IconChangeListener(this.myApplicationResizeManager, this.myReferenceComponent, this.myContainerComponents[i]);
				this.myContainerComponents[i].addPropertyChangeListener("icon", myComponentIconChangeListener);
				myComponentIconChangeListener.propertyChange(myComponentIconPropertyChangeEvent);
			}
			
			new ComponentResizeListener(this.myReferenceComponent, this.myComponentResizer[i]);
			if (this.myContainerComponentsPopupMenues[i] != null) {new ComponentResizeListener(this.myReferenceComponent, this.myComponentPopupMenuResizer[i]);}
   	    }
		
	}
	
	/**
	 * 
	 * Adds the manager objects to analyze the next graphical level
	 *
	 */
	private void analyzeContainer() {

		int i = 0;
		
  	    for (i = 0; i < this.myNumberOfComponents; i++) {	
			if (!( (this.myContainerComponents[i].getClass().getName() == "de.desy.acop.displayers.AcopTable") || (this.myContainerComponents[i] instanceof JTable) || (this.myContainerComponents[i] instanceof Button) || (this.myContainerComponents[i] instanceof Checkbox) || (this.myContainerComponents[i] instanceof Label) || (this.myContainerComponents[i] instanceof List) || (this.myContainerComponents[i] instanceof Scrollbar) || (this.myContainerComponents[i] instanceof TextArea) || (this.myContainerComponents[i] instanceof TextField))) {
				new ContainerManager(this.myApplicationResizeManager, this.myReferenceComponent, this.myContainerComponents[i]);
				if (this.myContainerComponentsPopupMenues[i] != null) {new ContainerManager(this.myApplicationResizeManager, this.myReferenceComponent, this.myContainerComponentsPopupMenues[i]);}
			}
  	    }
   		
	}  

}
