/**
 * ws:	May 22, 2007 - May 31, 2007 creation from WST Original
 */
package de.desy.mst.libs.framework.mstapp.util.gui;

import java.awt.Component;
import java.awt.Rectangle;
import java.util.HashMap;
import java.util.Map;

import de.desy.mst.libs.framework.mstapp.logging.Logger;
import de.desy.mst.libs.framework.mstapp.preferences.IPrefContainer;
import de.desy.mst.libs.framework.mstapp.preferences.IPreferences;
import de.desy.mst.libs.framework.mstapp.preferences.PrefSpeicherMode;
import de.desy.mst.libs.framework.mstapp.preferences.helper.APrefClientPanel;
import de.desy.mst.libs.framework.mstapp.preferences.helper.APreferences;
import de.desy.mst.libs.framework.mstapp.util.xml.XMLCoder;

/**
 * Oberklasse fuer die Positionierung von Components mit lokalem Abspeichern.
 * @author ws
 * @version 1.0.0
 */
public class PrefPosition extends APreferences {
	private static final String   TITLE    = "Programm Position und Gre";
	private static final String[] KEYWORDS = {"Position", "Gre", "Resize"};
	private final Component component;
	private final String    name;
//=============================================================================
	/**
	 * @param name 			z.B "Hauptform" Applikations-spezifisch und 
	 * 													-eindeutig
	 * 							, fest, nicht null
	 * @param component 	z.B. Hauptframe (Komponente deren Groesse gesetzt wird, nicht null
	 * @param parent 		Optionale Parent Preference (kein Parent dann null)
	 * @throws NullPointerException  falls component == null oder name == null
	 */
	public PrefPosition(final String name, final Component component, final IPreferences parent) {
		super(name.trim() + " " + TITLE, parent, KEYWORDS);
		if(component == null){
			throw new NullPointerException("Die component darf nicht null sein.");
		}
		this.component = component;
		this.name = name;
	}
//=============================================================================
	/* (non-Javadoc)
	 * @see de.desy.mst.libs.framework.mstapp.preferences.helper.APreferences#createPrefClientPanel(de.desy.mst.libs.framework.mstapp.preferences.IPrefContainer)
	 */
	@Override public APrefClientPanel createPrefClientPanel(final IPrefContainer prefContainer) {
		PrefPosition.Encoder.setRectangle(this.name, this.component);
		return new PrefPositionPanel(this.component, prefContainer);
	}
	/* (non-Javadoc)
	 * @see de.desy.mst.libs.framework.mstapp.preferences.helper.APreferences#onNewSaveString(java.lang.String)
	 */
	@Override public void onNewSaveString(final String saveString) {
		PrefPosition.Encoder.decode(saveString);
		final Rectangle rectangle = Encoder.getRectangle(this.name, this.component);
		this.component.setBounds(rectangle);
	}
//=============================================================================
	/**
	 * Alle PrefPositon's teilen genau einen Encoder: Haelt die saemtliche 
	 * Positions in map nach Key = name
	 */ 
	private static class Encoder{
		private static Map<String, Rectangle> map 
											= new HashMap<String, Rectangle>();
		/**succeeds or returns rectangle of this component*/
		@SuppressWarnings("unchecked") //(Map<String, Rectangle>)object
		static void decode(final String saveString) {
			final String msgPref = "PrefPosition: Kann den gespeicherten Wert"
								 + "nicht dekodieren. Ignorier ihn.";
			final String msgSuff = " Original String: '" + saveString + "'";
			try {
			final Object object = XMLCoder.stringToBean(saveString);//<<<<<<<<<
				if(!(object instanceof Map)){
					final String msg = msgPref 
									 + " Erwarte ein Objekt vom Typ Map."
									 + msgSuff;
					Logger.SWFW.warn(msg);
					return;
				}
				PrefPosition.Encoder.map = (Map<String, Rectangle>) object;//<<
			} catch (final Exception e) {
				final String msg = msgPref + msgSuff;
				Logger.SWFW.warn(msg, e);
			}
		}
		static void setRectangle(final String name, final Component component) {
			PrefPosition.Encoder.map.put(name, component.getBounds());
		}
		static String encode() {
			final Map<String, Rectangle> map = PrefPosition.Encoder.map;
			final String s = XMLCoder.beanToStringStripped(map);
			return s;
		}
		static Rectangle getRectangle(final String name, final Component component) {
			try {
				final Rectangle rectangle = map.get(name);
				return (rectangle == null) ? component.getBounds() : rectangle;
			} catch (final Exception e) {
				final String msg = "PrefPosition.Encoder:"
								 + " Problem mit Rectangle von '" + name +"'.";
				Logger.SWFW.warn(msg, e);
				return component.getBounds();
			}
		}
	}
//=============================================================================	
	/**DUMMY TO GET GOING*/
	private class PrefPositionPanel extends APrefClientPanel{
		private static final long serialVersionUID = 1L;
		public PrefPositionPanel(final Component component, final IPrefContainer prefContainer) {
			super(prefContainer);
			prefContainer.setChanged(true);
		}
		//---------------------------------------------------------------------
		
		/* (non-Javadoc)
		 * @see de.desy.mst.libs.framework.mstapp.preferences.helper.APrefClientPanel#onDispose()
		 */
		@Override public void onDispose() 			{/**nothing to do*/}
		
		/* (non-Javadoc)
		 * @see de.desy.mst.libs.framework.mstapp.preferences.IPrefClient#getSaveString()
		 */
		@Override public String getSaveString(){
			return PrefPosition.Encoder.encode();
			}
		
		/* (non-Javadoc)
		 * @see de.desy.mst.libs.framework.mstapp.preferences.IPrefClient#getSpeicherMode()
		 */
		@Override public PrefSpeicherMode getSpeicherMode() 	{
			return PrefSpeicherMode.APPLOCAL;
		}
		
		/* (non-Javadoc)
		 * @see de.desy.mst.libs.framework.mstapp.preferences.IPrefClient#onReset()
		 */
		@Override public void onReset() 						{}
		
		/* (non-Javadoc)
		 * @see de.desy.mst.libs.framework.mstapp.preferences.IPrefClient#onSet()
		 */
		@Override public void onSet() throws Exception 		{}
	}
}
