/**
 * Sep 29, 2009 - Oct  6, 2009	pb:	received from Piotr Bartkiewicz [piotr.bartkiewicz@desy.de]
 * 								ws:	adopted to mstApp conventions (same semantics, ideas as pb)
 * 									kludged a small timer bug, added isRunning()
 */

package de.desy.mst.libs.framework.mstapp.util.gui;

import java.awt.AWTEvent;
import java.awt.Toolkit;
import java.awt.event.AWTEventListener;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.LinkedList;
import java.util.List;

import javax.swing.Timer;


/**
 * Idea: Monitors Session Idleness (no activity) and fires an actionEvent
 * after a set Timeout.
 * <br>
 * Thread Policy: not threadsafe, must be used in Swing EDT
 * <br>
 * Careful: might throw on construction an exception if not privileged
 * enough to monitor activity
 *
 * @author karol
 */
public class SessionActivityMonitor{
	final private Object eventSource;
	final private Timer timer;
	final private List<ActionListener> actionListeners;
	private int maxIdleTimeMs;
	private long lastStartTime;
//=============================================================================
	/**@throws Exception if not enought previleges to monitor awt activity*/
	public SessionActivityMonitor() throws Exception {
		this.eventSource = this;
		this.actionListeners = new LinkedList<ActionListener>();
		this.timer = createTimer();

		final long mask = AWTEvent.MOUSE_EVENT_MASK + AWTEvent.KEY_EVENT_MASK;
		final Toolkit toolkit = Toolkit.getDefaultToolkit();
		toolkit.addAWTEventListener(new AwtEventListener(), mask);
	}
//=============================================================================
	public void stop(){this.timer.stop();}

	/**sets MaxIdleTime AND starts the timer*/
	public void setMaxIdleTimeMs(final int maxIdleTimeMs){
		//System.out.println("" + System.currentTimeMillis() + " setMaxIdleTimeMs(" + maxIdleTimeMs + " [ms])");
		this.maxIdleTimeMs = maxIdleTimeMs;
		this.timer.setInitialDelay(maxIdleTimeMs);
		this.timer.start();
		this.lastStartTime = System.currentTimeMillis();
	}
	public int getMaxIdleTimeMs(){return this.maxIdleTimeMs;}
	public boolean isRunning()	 {return this.timer.isRunning();}

	public int getMinRestTimeMS(){
		return isRunning() ? toInt(this.lastStartTime - System.currentTimeMillis() + this.maxIdleTimeMs) : 0;
	}

	public synchronized void addActionListener   (final ActionListener lst){this.actionListeners.add   (lst);}
	public synchronized void removeActionListener(final ActionListener lst){this.actionListeners.remove(lst);}
//=============================================================================
	private Timer createTimer() {
		final Timer timer = new Timer(999000, new ActionListener(){
			@Override public void actionPerformed(final ActionEvent evt){
				//System.out.println("" + System.currentTimeMillis() + " Timer expired");
				fireActionEvent(new ActionEvent(SessionActivityMonitor.this.eventSource, evt.getID(), "session expired"));
			}
		});
		timer.stop();
		timer.setRepeats(false);
		return timer;
	}
	private synchronized void fireActionEvent(final ActionEvent evt){
		for(final ActionListener lst : this.actionListeners )
			lst.actionPerformed(evt);
	} 
	/**save converion*/ private int toInt(final long l) {
		try
			{return (int)(l);}
		catch (final Exception ex)
			{return l > 0 ? Integer.MAX_VALUE : Integer.MIN_VALUE;}
	}

//=============================================================================
	private class AwtEventListener implements AWTEventListener{
		@Override public void eventDispatched(final AWTEvent evt){
			if( evt.toString().contains("PRESSED")){
				//System.out.println("" + System.currentTimeMillis() + "SessionActivityMonitor.AwtEventListener.eventDispatched(" + evt +")");
				if(isRunning()){
					stop();
					setMaxIdleTimeMs(SessionActivityMonitor.this.maxIdleTimeMs);	//restart has wrong maxIdleTime
					//System.out.println("" + System.currentTimeMillis() + "SessionActivityMonitor.AwtEventListener initialDelay: " + maxIdleTime);
				}
			}
		}
	}
}
