/**
 *  Jul 15, 2009 - Jul 27, 2009 ws:	creation from scratch
 * 	Nov  2, 2009 - Nov  4, 2009	ws: instrument to see if part of longActionDeadlock
 */

package de.desy.mst.libs.framework.mstapp.util.gui;

import java.awt.Adjustable;
import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Dialog;
import java.awt.Window;
import java.util.List;
import java.util.concurrent.atomic.AtomicBoolean;

import javax.swing.BorderFactory;
import javax.swing.BoundedRangeModel;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollBar;
import javax.swing.SwingConstants;
import javax.swing.SwingUtilities;
import javax.swing.SwingWorker;
import javax.swing.WindowConstants;

import de.desy.mst.libs.framework.mstapp.Main;
import de.desy.mst.libs.framework.mstapp.logging.Logger;
import de.desy.mst.libs.framework.mstapp.util.StringPadding;
import de.desy.mst.libs.framework.mstapp.util.threads.Sleep;
import de.desy.mst.libs.framework.mstapp.util.threads.ThreadHelper;

/**
 * 	Wenn eine GUI Action etwas laenger dauert kann man mit Hilfe dieser
 * 	Klasse einen GUI Indikator, dass etwas passiert anzeigen.
 * 
 * 	@author ws
 *	@version 1.0.0
 */
class __LongActionIndication {
	private static final boolean DEBUG_ON = false;
	private final AtomicBoolean refShow = new AtomicBoolean(false);
//-----------------------------------------------------------------------------
	public __LongActionIndication() {}
//-----------------------------------------------------------------------------
		
	/**
	 * @param runnable, not null (will be executed in new Thread) runnable should
	 * 					finish only after all other relevant and timeconsuming 
	 * 					Threads spawned within are finished
	 * @param refCmp	may be null (for centering the dialog Box)
	 * @param title		displayed in dialog
	 * @param modal		dialog is on Top and other app GUI is disabled for User
	 * @return the executing thread of runnable (in case you want to do a join)
	 */
	public final Thread runInExtraThread_showingBusyDialog(final Runnable runnable
														 , final Component refCmp
														 , final String title
														 , final boolean modal){
		final String   name = "MstDialogWorkerThread-" + title;
		final Runnable wrap = wrapWithDialogShowLogic(runnable, refCmp, title, modal);
		final Thread thread = new Thread(wrap, name);
		thread.start();
		return thread;
	}

	private Runnable wrapWithDialogShowLogic(final Runnable runnable, final Component refCmp, final String title, final boolean modal) {
		return new Runnable() {
			@Override public void run() {
				JDialog dialog = null;
				try {
					dialog = show(refCmp, title, modal);
					runnable.run();
				} finally {
					if(dialog != null) dialog.dispose();
				}
			}
		};
	}
	/**call dispose if your are finisched with it*/
	public final JDialog show(final Component refCmp, final String title, final boolean modal){
		this.refShow.set(true);
		
		final JLabel   jLabelTitle = new JLabel(title);
		final JScrollBar srcollBar = new JScrollBar( Adjustable.HORIZONTAL);
		final JLabel     jLabelMS  = createLabelMS(title);
		
		final JPanel  panel = new JPanel(new BorderLayout());
		panel.add(jLabelTitle, BorderLayout.NORTH );
		panel.add(srcollBar,   BorderLayout.CENTER);
		panel.add(jLabelMS ,   BorderLayout.SOUTH );
		final Window windowAncestor = refCmp == null ? null : SwingUtilities.getWindowAncestor(refCmp);
		final JDialog dialog = createDialog(windowAncestor, title, false /*modal*/);
		dialog.add(panel);
		
		dialog.pack();
		//now the dialog is constructed completely
		
		setVisibleAndCenter_LaterInEDT(dialog, refCmp);//might lock or the wrong Thread
		animate(srcollBar.getModel(), jLabelMS);	   //at end (after complete construction)
		
		return dialog;
	}

	private JLabel createLabelMS(final String prototyp) {
		final JLabel jLabel = new JLabel(prototyp); 
		jLabel.setHorizontalAlignment(SwingConstants.RIGHT);
		jLabel.setAlignmentX(Component.RIGHT_ALIGNMENT);
		jLabel.setFont(GUIFactory.createFixedWidthFont());
		jLabel.setBorder(BorderFactory.createEtchedBorder());
		return jLabel;
	}

//-----------------------------------------------------------------------------	
	private JDialog createDialog(final Component refCmp, final String title, final boolean modal) {
		class DisposingDialog extends JDialogWithCloseOnEscape{
			private static final long serialVersionUID = 1L;
			//-----------------------------------------------------------------
			public DisposingDialog(final Dialog owner, final String title, final boolean modal) {
				super(owner, title, modal);
			}
			public DisposingDialog(final JFrame owner, final String title, final boolean modal) {
				super(owner, title, modal);
			}
			//-----------------------------------------------------------------
			@Override public void dispose() {__LongActionIndication.this.refShow.set(false); super.dispose();}
		}
		
		final JDialog dialog = (refCmp instanceof JFrame ) ? new DisposingDialog((JFrame)refCmp, title, modal)
		: (  (refCmp instanceof Dialog ) ? new DisposingDialog((Dialog)refCmp, title, modal) 
		                                 : new DisposingDialog((JFrame)null, title, modal));
		dialog.setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);
		return dialog;
	}
	private void setVisibleAndCenter_LaterInEDT(final Window wnd, final Component refCmp) {
		SwingUtilities.invokeLater(new Runnable() {
			@Override public void run() {
				if(!__LongActionIndication.this.refShow.get())
					return;	//nichts mehr zu tun
				try{
					wnd.setVisible(true);
					//wnd.setAlwaysOnTop(true);
					if(refCmp == null)
						Main.centerToMstFrameMain(wnd);
					else
						wnd.setLocationRelativeTo(refCmp);
				}catch(final Exception ex){Logger.SWFW.warn("Unexpected Exception.", ex);}
			}
		});
	}	
	private void animate(final BoundedRangeModel model, final JLabel jLabel) {
		new MySwingWorker(model, jLabel).execute();
	}
//=============================================================================	
	private class MySwingWorker extends SwingWorker<Void, Long>{
		private static final String LABEL_SUFFIX = " [ms]";
		//---------------------------------------------------------------------
		private final BoundedRangeModel model;
		private final JLabel jLabel;
		private final long   start;
		private final StringPadding sPad;
		//---------------------------------------------------------------------
		public MySwingWorker(final BoundedRangeModel model, final JLabel jLabel){
			this.model  = model;
			this.jLabel = jLabel;
			this.start  = System.currentTimeMillis();
			this.sPad   = new StringPadding(Math.max(8, jLabel.getText().length()));
		}
		//---------------------------------------------------------------------
		@Override protected Void doInBackground() throws Exception {
			while(__LongActionIndication.this.refShow.get() && !isCancelled()){
				if(DEBUG_ON) Logger.SWFW.info("__LongActionIndication.MySwingworker.doInBackground() now " + System.currentTimeMillis());
				Sleep.sleep(100);
				publish(1L);
				if((System.currentTimeMillis() - this.start) > 10000){
					if(DEBUG_ON) ThreadHelper.get().printAllStackTraces();
					return null;	//Notausgang zum debuggen bei Deadlock
				}
			}
			return null;
		}
		@Override protected void done() {super.done();}
		@Override protected void process(final List<Long> lst) {
			if(DEBUG_ON) Logger.SWFW.info("__LongActionIndication.MySwingworker.process() now " + System.currentTimeMillis());
			cycleModel();
			updateLabel();
			super.process(lst);
		}
		//---------------------------------------------------------------------
		private final BoundedRangeModel getModel() 	{return this.model;}
		private final JLabel            getLabel()	{return this.jLabel;}
		private final long				getStart()	{return this.start;}
		private final void cycleModel()				{cycleModel(getModel());}
		private final void cycleModel(final BoundedRangeModel model) {
			final int oldValue = model.getValue();
			final int extent   = model.getExtent();
			final int maxValue = model.getMaximum() - extent;
			final int proposal = oldValue + extent/2 + 1;
			final int newValue = (proposal <= maxValue) ? proposal : model.getMinimum();
			model.setValue(newValue);
		}
		private final void updateLabel(){
			final long laufzeit = System.currentTimeMillis() - getStart();
			final String   text = pad("" + laufzeit + LABEL_SUFFIX);
			getLabel().setText(text);
		}
		private String pad(final String text) {return this.sPad.format(text);}
	}//END CLASS: MySwingWorker
}
 