/**
 *	Nov 2, 2009 - Nov 9, 2009	ws:	creation from scratch 
 */
package de.desy.mst.libs.framework.mstapp.util.threads;

import java.io.PrintStream;
import java.util.Arrays;
import java.util.Comparator;
import java.util.Map;
import java.util.Set;

/**
 * Utility class to support threading like printAllStackTraces()
 * <br> see also JMa's Sleep in same package
 * @author ws
 * @version 1.0.0
 */
public class ThreadHelper {
	private static final String TITLE = "ThreadHelper.printAllStackTraces(...) --- ";
	private static final String TITLE_START = ThreadHelper.TITLE + "START Anzahl = ";
	private static final String TITLE_END   = ThreadHelper.TITLE + "END   Anzahl = ";
	
	private static final ThreadHelper instance = new ThreadHelper();
//-----------------------------------------------------------------------------	
	public static ThreadHelper get(){return ThreadHelper.instance;}
	
//=============================================================================	
	private ThreadHelper(){;}
//-----------------------------------------------------------------------------
	/**
	 * printAllStackTraces() after msDelay ms in a demon thread
	 * @return the delay handling thread in case a join is needed 
	 * */
	public final Thread printAllStackTracesAfterMS(final long msDelay){
		final Runnable runnable = new Runnable() {
			@Override public void run() {
				Sleep.sleep(msDelay);
				printAllStackTraces();
			}
		};
		final Thread thread = new Thread(runnable);
		thread.setDaemon(true);
		thread.start();
		return thread;
	}
    public final void printAllStackTraces() {printAllStackTraces(System.err);}
    public final void printAllStackTraces(final PrintStream s) {
		synchronized (s) {
	    	final Map<Thread, StackTraceElement[]> allStackTraces = Thread.getAllStackTraces();
			final Set<Thread> keySet = allStackTraces.keySet();
			s.println(ThreadHelper.TITLE_START + keySet.size());
			final Thread[] keys = keySet.toArray(new Thread[0]);
			Arrays.sort(keys, new Comparator<Thread>() {
				@Override public int compare(final Thread o1, final Thread o2) {
					return o1.getName().compareTo(o2.getName());
				}
				
			});
			for (final Thread thread : keys) 
				printStackTrace(s, thread, allStackTraces.get(thread));
			
			s.println(ThreadHelper.TITLE_END + keySet.size());
		}
    }
    public void printStackTrace(final PrintStream s, final Thread thread, final StackTraceElement[] stackTraceElements) {
    	if(s == null){
    		printStackTrace(System.err, thread, stackTraceElements);
    		return;
    	}
		synchronized (s) {
			s.println();
			s.println("\tThreadName: " + thread.getName());
			for (final StackTraceElement stackTraceElement : stackTraceElements) 
				s.println("\tat " + stackTraceElement);
		}
	}
//-----------------------------------------------------------------------------	
}
