/**
 *  ws:	May 29, 2006 - May 29, 2006	Created on	
 *  ws:	May 29, 2007 - May 31, 2007	added: beanToString, stringToBean
 */
package de.desy.mst.libs.framework.mstapp.util.xml;

import java.beans.XMLDecoder;
import java.beans.XMLEncoder;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;

/**
 * Utility Klasse um XML reserved Characters zu Codieren/Dekodieren<br>
 *   <li>  '"' <=> '&quot;',
 *   <li>  '&' <=> '&amp;',
 *   <li>  '>' <=> '&gt;',
 *   <li>  '<' <=> '&lt;',
 *
 * @author ws
 * @version 1.0.0
 */
public class XMLCoder {
	public final static class Entry{
		public final String reservedChar;
		public final String encodedChar;
		public Entry(final String reservedChar, final String encodedChar) {
			super();
			this.reservedChar = reservedChar;
			this.encodedChar = encodedChar;
		}
	}
	public static final Entry[] entries = {
		  new Entry("&",  "&amp;")
		, new Entry("\"", "&quot;")
		, new Entry(">",  "&gt;")
		, new Entry("<",  "&lt;")
	};

	final static String XMLDECODER_END_TAG   = "</java>";
	final static String XMLDECODER_BEG_TAG   = "<java class=\"" 
											 + XMLDecoder.class.getName() 
											 + "\">";
//=============================================================================	
	/**reserved xml chars to escaped versions*/
	public static final String encode(final String stringMitReservedChars){
		String retVal = stringMitReservedChars;
		for (int i = 0; i < entries.length; i++) {
			Entry entry = entries[i];
			retVal = retVal.replace(entry.reservedChar, entry.encodedChar);
			
		}
		return retVal;
	}
	/**escaped xml chars back to their reserverd chars (NICHT rekursiv)*/
	public static final String decode(final String stringMitKodiertenReservedChars){
		String retVal = stringMitKodiertenReservedChars;
		for (int i = entries.length - 1; 0 <= i; i--) {
			Entry entry = entries[i];
			retVal = retVal.replace(entry.encodedChar, entry.reservedChar);
		}
		return retVal;
	}
	
	
	/**Object must be beans xml encodable*/
	public static String beanToString(final Object javaBean) {
		ByteArrayOutputStream arrayOutputStream = new ByteArrayOutputStream();
		XMLEncoder xmlEncoder = new XMLEncoder(arrayOutputStream);
		xmlEncoder.writeObject(javaBean);
		xmlEncoder.close();
		final String string = arrayOutputStream.toString();
		return string;
	}
	/**String must be Java Bean encoded*/
	public static Object stringToBean(final String javaBeanEncodedString) throws Exception{
		final byte[] bytes = decorate(javaBeanEncodedString).getBytes();
		final ByteArrayInputStream is = new ByteArrayInputStream(bytes);
        XMLDecoder xmlDecoder = new XMLDecoder(is);
        final Object object = xmlDecoder.readObject();
        xmlDecoder.close();
        return object;
	}
	/**beanToString without XML Decl and Decoder class wrapper*/
	public static String beanToStringStripped(final Object javaBean) {
		return strip(XMLCoder.beanToString(javaBean));
	}
//==============================================================================	
	/**removes newlines, XML Declaration, XMLDecoder declarations*/
	private static String strip(final String javaBeanEncodedString) {
		String string = javaBeanEncodedString;
		
		//(1) Newline removal
		final String NEWLINE = "\n";
		string = string.replace(NEWLINE, "");
		
		//2) "> <" Compaction
		final String oldS1 = "> <";
		final String oldS2 = ">  <";
		final String oldS3 = ">   <";
		final String oldS4 = ">   <";
		final String oldS5 = ">   <";
		final String newS = "><";
		string = string.replace(oldS1, newS);
		string = string.replace(oldS2, newS);
		string = string.replace(oldS3, newS);
		string = string.replace(oldS4, newS);
		string = string.replace(oldS5, newS);
		
		//(2) XML Declaration Removal
		final String XMLDECL_START = "<?xml";
		final String XMLDECL_END   =    "?>";
		if(string.startsWith(XMLDECL_START)){
			final int beginIndex = string.indexOf(XMLDECL_END) 
								 + XMLDECL_END.length();
			string = string.substring(beginIndex);
		}
		
		//(3) XML Decoder declarationS removal
		final String XMLDECODER_START = XMLDecoder.class.getName()+"\">";
		final int indexOf = string.indexOf(XMLDECODER_START) 
						  + XMLDECODER_START.length();
		if(indexOf > -1){
			final int endIndex = string.lastIndexOf(XMLDECODER_END_TAG);
			string = string.substring(indexOf, endIndex);
		}
		
		return string.trim();
	}
	/**adds - if necessary - XMLDecoder declarations*/
	private static String decorate(final String javaBeanEncodedString) {
		if(javaBeanEncodedString.contains(XMLDECODER_END_TAG)){
			return javaBeanEncodedString;	//nothing to do
		}
		return XMLDECODER_BEG_TAG + javaBeanEncodedString + XMLDECODER_END_TAG;
	}
}
