/**
 * 	Jul  9, 2010 - Jul 13, 2010	ws:	created from scratch
 */
package de.desy.mst.libs.info.appadmin;

import java.util.Enumeration;
import java.util.HashSet;
import java.util.LinkedList;
import java.util.Set;
import java.util.logging.Level;

import javax.jms.JMSException;
import javax.jms.Message;

/**
 * Wrapper for a "Java Messaging Service" Message to make those handy and
 * JMS independent for the rest of the application.
 * @author ws
 */
class _JMSMessageWrapper {
	/**pp for testing only*/
	private final class _WrapperEntry extends _StringEntry{
		_WrapperEntry(final String key, final String value) {
			super(key, value);
		}
	}
	
	/**
	 * @param jmsMsg nonnull JMS Message
	 * @return the wrapper || if problems do _Logging and return null
	 */
	public static _JMSMessageWrapper createOrNull(final Message jmsMsg){
		try {
			return new _JMSMessageWrapper(jmsMsg);
		} catch (final JMSException ex) {
			final String msg = "Can not decode the incoming message " + jmsMsg;
			_Logging.log(Level.WARNING, msg, ex);
			return null;
		}
	}
//=============================================================================
	
	private final Message jmsMsg;
	private final _Msg    msg;
	//-------------------------------------------------------------------------
	_JMSMessageWrapper(final Message jmsMsg) throws JMSException {
		this.jmsMsg = jmsMsg;
		final Set<String>    propertyNameSet = extractPropertyNames();
		final Set<_Entry<?>> entrySet = extractEntries(propertyNameSet);
		this.msg = new _Msg(_ExternalSystemEntry.FROM_JMS_MSG, entrySet);
	}
	//-------------------------------------------------------------------------
	Message getJmsMsg() {return this.jmsMsg;}
	_Msg    getMsg   () {return this.msg   ;}
	//-------------------------------------------------------------------------
	/**pp for testing only*/
	Set<String> extractPropertyNames() throws JMSException {
		final LinkedList<String> lst = new LinkedList<String>();
		final Enumeration<String> propertyNames = getPropertyNames(getJmsMsg());
		while(propertyNames.hasMoreElements())
			lst.add(propertyNames.nextElement());
		return new HashSet<String>(lst);
	}
	@SuppressWarnings("unchecked") /**encapsulation of forced generic type assignment*/
	private Enumeration<String> getPropertyNames(final Message jmsMsg) throws JMSException {
		return jmsMsg.getPropertyNames();
	}
	/**pp for testing only*/
	Set<_Entry<?>> extractEntries(final Set<String> propertyNameSet) {
		final LinkedList<_Entry<?>> lst = new LinkedList<_Entry<?>>();
		for (final String propertyName : propertyNameSet)
			addEntry(lst, propertyName);
		return new HashSet<_Entry<?>>(lst);
	}
	private void addEntry(final LinkedList<_Entry<?>> lst, final String key) {
		final String value = getStringProperty(key);
		if(value != null)
			lst.add(new _WrapperEntry(key, value));
	}
	private String getStringProperty(final String propertyName) {
		try {
			return getJmsMsg().getStringProperty(propertyName);
		} catch (final JMSException ex) {
			final String msg = "Problem extracting value for: " + propertyName;
			_Logging.log(Level.WARNING, msg, ex);
			return null;
		}
	}
}

