/**
 * 	Aug  5, 2010 - Aug  6, 2010	ws:	created from scratch
 *	Aug 16, 2010 - Aug 16, 2010	ws:	logging ohne stack trace
 *	Aug 29, 2011 - Aug 29, 2011	ws:	respects AppAdmincontext.isOffline()
 */

package de.desy.mst.libs.info.appadmin;

import java.util.concurrent.atomic.AtomicInteger;
import java.util.logging.Level;

import javax.jms.JMSException;
import javax.jms.QueueSender;
import javax.jms.Session;
import javax.jms.QueueConnection;
import javax.jms.QueueSession;

import com.sun.messaging.QueueConnectionFactory;

/**
 * one shot sending 
 * <br> intent: sending error messages etc which should be rare
 * @author ws
 */
class _MsgSenderJMSQueue implements _IMsgSender {
	private static final AtomicInteger refCountOpenConnections = new AtomicInteger(0);
	//=========================================================================
	private final String queueName;
	//-------------------------------------------------------------------------
	_MsgSenderJMSQueue(final String queueName) {this.queueName = queueName;}
	//-------------------------------------------------------------------------
	public final String getQueueName() {return this.queueName;}
	@Override public void send(final _Msg msg) {
		if(AppAdminContext.isOffline())
			return;	//silently ignore
		
		try {
			final QueueConnection connection = createConnection();
			final QueueSession    session    = connection.createQueueSession(false, Session.AUTO_ACKNOWLEDGE);
			final QueueSender     sender = session.createSender(session.createQueue(getQueueName()));
			
			_Logging.log(Level.FINE, calcMsg(refCountOpenConnections.incrementAndGet(), "Created"), null);
			
			sender.send(_MsgSenderJMS.fillJMSMsgWithMsg(session.createMessage(), msg));
			
			sender    .close();
			session   .close();
			connection.close();
			_Logging.log(Level.FINE, calcMsg(refCountOpenConnections.decrementAndGet(), "Closed "), null);
		} catch (final Exception ex) {
			final String ms = "_MsgSenderJMSQueue can not sent to '" 
				            + getQueueName() + "' ignore request.";
			_Logging.log(Level.SEVERE, ms, ex);
		}
	}
	//-------------------------------------------------------------------------
	private String calcMsg(final int cnt, final String what) {
		return "_MsgSenderJMSQueue: " + what + " (" + cnt + ") JMS connection, session, sender on queue '" + getQueueName() + "'.";
	}
	private QueueConnection createConnection() throws JMSException {
		final QueueConnectionFactory qcf = new QueueConnectionFactory();

		final AppAdminContext instance = AppAdminContext.getInstance();
		try {
			qcf.setProperty(instance.getJMSMQ_URL_KEY(), instance.getJMSMQ_URL());
			final QueueConnection queueConnection = qcf.createQueueConnection();
			queueConnection.start();
			return queueConnection;
		} catch (final Throwable th) {
			final String msg = "Can not create a topicConnection. Original Message: " + th.getMessage();
			_Logging.log(Level.SEVERE, msg, null);
			throw new JMSException("Can not create JMSConnection: " + th);
		}
	}
}
 