/**
 *	Feb  3, 2010 - Feb  4, 2010	ws:	creation from scratch 
 *	Apr 12, 2010 - Apr 12, 2010	ws:	_IMsgSender replaced by _IMsgSenderFactory
 */
package de.desy.mst.libs.info.appadmin;

import java.util.concurrent.BlockingQueue;
import java.util.concurrent.LinkedBlockingQueue;
import java.util.logging.Level;

/**
 * holds all the messages and exports to the _IMsgSender
 * <br> capacity from AppAdminContext can not be changed after class creation
 * @author ws
 * @version 1.0.0
 */
class _Queue {
//=============================================================================
	private final BlockingQueue<_Msg> queue;
	//-------------------------------------------------------------------------
	/**
	 * @param capacity must be >0 and can't be changed later
	 * @param msgSender sink for this queue, must not be null
	 * @throws IllegalArgumentException, if capacity <= 0
	 * @throws NullPointerException, if msgSender == null
	 */
	_Queue(final int capacity, final _IMsgSenderFactory msgSenderFactory) throws IllegalArgumentException, NullPointerException{
		if(msgSenderFactory == null)
			throw new NullPointerException("The _IMsgSenderFactory must not be null.");
		this.queue = new LinkedBlockingQueue<_Msg>(capacity);
		final Runnable runnable = new Runnable() {
			@SuppressWarnings("synthetic-access") 
			public void run() {
				final _IMsgSender msgSender = msgSenderFactory.create();
				while(true){
					try{
						msgSender.send(getQueue().take());
					} catch(final Throwable th) {
						final String msg = "AppInfo._Queue.XTOR SendThread.run()."
										 + " Problem sinking the queue"
										 + " or sending the msg.";
						_Logging.log(Level.WARNING, msg, th);
					}
				}
			}
		};
		new Thread(runnable, "AppAdminQueue SendThread").start();
	}
	//-------------------------------------------------------------------------
	/**pp <br> @return true if msg is accepted in app*/ 
	final boolean offer(final _Msg msg) {
		if(AppAdminContext.getInstance().logEveryMessage())
			_Logging.log(Level.FINE, "_Queue.off(" + msg + ").", null);
		return (msg == null) ? false : getQueue().offer(msg);
	}
	//-------------------------------------------------------------------------
	private final BlockingQueue<_Msg> 	getQueue()		{return this.queue;}
}
