/**
 * 
 * Last changed: $Date$
 * By: $Author$
 * URL: $HeadURL$
 * $Revision$ 
 *
 */
package de.desy.mst.libs.ser.launcher.cache;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.nio.channels.Channels;
import java.nio.channels.ReadableByteChannel;

/**
 * @author labudda
 *
 */
class ResourceStream implements IResource {
	private final static java.util.logging.Logger log = java.util.logging.Logger.getLogger(ResourceStream.class.getName()); 
	/**
	 * Datasource, might be null on case of error
	 */
	private final InputStream src;
	private final long lastModified;
	
	/**
	 * reads resource from stream cache time stamp will be now
	 * @param stream
	 */
	public ResourceStream(final InputStream stream) {
		this(stream, Long.MIN_VALUE);
	}

	/**
	 * reads resource from stream cache 
	 * @param stream
	 * @param lastModified of cache item
	 */
	public ResourceStream(final InputStream stream, final long lastModified) {
		this.src = stream;
		this.lastModified = lastModified;
	}
	
	/**
	 * Reads resource from URL, 
	 * Intern wird der Stream (openStream) der url genutzt. 
	 * @param url
	 */
	public ResourceStream(java.net.URL url) {
		InputStream stream;
		long lastModified;
		try {
			stream = url.openStream(); 
			lastModified = url.openConnection().getLastModified();
		} catch (java.io.IOException e) {
			ResourceStream.log.warning("Unable to get stream from " + url.toExternalForm());
			stream = null;
			lastModified = Long.MIN_VALUE;
		}
		this.src = stream;
		this.lastModified = lastModified;
	}
	

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.ser.launcher.cache.IResource#copyResourcesTo(java.lang.String)
	 */
	public void copyResourcesTo(String destination) throws IOException {
		this.copyResourcesTo(new java.io.File(destination));
	}

	/* (non-Javadoc)
	 * @see de.desy.mst.libs.ser.launcher.cache.IResource#copyResourcesTo(java.io.File)
	 */
	@Override
	public void copyResourcesTo(File destFile) throws IOException {
		final java.util.logging.Logger log = java.util.logging.Logger.getLogger(ResourceStream.class.getName());
		if (this.src == null) {
			throw new IOException("No valid source was defined at constructor");
		} else {
			log.info("Copy to " + destFile.getAbsolutePath());
			final java.io.File destFolder = destFile.getParentFile();
			/* Es ist moeglich, dass eine Datei mit dem Namen des Verzeichnisses existiert.
			 * (Alte Version des Caches)
			 * In diesem Fall muss die existierende Datei entfernt werden.
			 */
			if (destFolder.exists() && destFolder.isFile()) {
				destFolder.delete();
			} else {
				// after all no file with foldername exists
			}
			destFile.getParentFile().mkdirs();
			
	        ReadableByteChannel rbc = Channels.newChannel(this.src);
	        final FileOutputStream fos = new FileOutputStream(destFile);
	        fos.getChannel().transferFrom(rbc, 0, Long.MAX_VALUE);
	        fos.close();
			
			if (this.lastModified >= 0) {
				destFile.setLastModified(this.lastModified);
			} else {
				// there's no date of last modification defined
			}
		}
	}

}
