//: TLink.java
package de.desy.tine.headers;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import java.util.Arrays;

import de.desy.tine.definitions.TErrorList;
import de.desy.tine.endianUtils.Swap;
import de.desy.tine.server.logger.MsgLog;
import de.desy.tine.startup.TInitializerFactory;
public final class TReqHdr
{
  private String username = null; 
  private short tineProtocol;
  private short revisionId;
  static public final short hdrSizeInBytes = 2 + 16 + 2 + 2; // network header size
  private short totalSizeInBytes;
  private short msgSizeInBytes;
  ByteArrayOutputStream dBuffer;
  public TReqHdr(String u,int protocolLevel)
  {
    username = u.toUpperCase();
    tineProtocol = (short)protocolLevel;
    revisionId = (short)TInitializerFactory.getInstance().getInitializer().getRevisionId();
    dBuffer = new ByteArrayOutputStream(hdrSizeInBytes);
  }
  public TReqHdr(String u,int protocolLevel,short protocolId)
  {
    username = u.toUpperCase();
    tineProtocol = (short)protocolLevel;
    revisionId = protocolId;
    dBuffer = new ByteArrayOutputStream(hdrSizeInBytes);
  }
  public short getRevisionId() { return revisionId; }
  public String getUserName() { return username; }
  public void setUserName(String name) 
  { 
    if (username != name && username.compareToIgnoreCase(name) != 0)
    {
      username = name.toUpperCase();
    }
  }
  public short getTotalSizeInBytes() { return totalSizeInBytes; }
  public short getMsgSizeInBytes() { return msgSizeInBytes;}
  public void setMsgSizeInBytes(short sizeInBytes) 
  { 
    msgSizeInBytes = sizeInBytes;
  }
  private byte[] bstr = new byte[16];
  public byte[] toByteArray()
  {
    totalSizeInBytes = (short)(msgSizeInBytes + hdrSizeInBytes);
    try
    {
      dBuffer.reset();
      DataOutputStream ds = new DataOutputStream(dBuffer);
      ds.writeShort(Swap.Short(totalSizeInBytes));
      Arrays.fill(bstr, (byte)0);
      byte[] tmp  = username.getBytes();
      for (int i=0; i<16 && i<tmp.length; i++) bstr[i] = tmp[i];
      ds.write(bstr,0,16);
      ds.writeShort(Swap.Short(tineProtocol));
      ds.writeShort(Swap.Short(revisionId));
      ds.close();
      return dBuffer.toByteArray();
    }
    catch (IOException e)
    {
      e.printStackTrace();
      MsgLog.log("TReqHdr.toByteArray",e.toString(),TErrorList.code_failure,e,0);
    }
    return null;  
  }
  public TReqHdr(byte[] data,int off, int len)  
  { // prepare incoming ...
    int bytesread = 0;
    try
    {
      ByteArrayInputStream dinBuffer = new ByteArrayInputStream(data,off,len);
      DataInputStream ds = new DataInputStream(dinBuffer);
      // overall message length:
      totalSizeInBytes = Swap.Short(ds.readShort());
      bytesread = 2;
      Arrays.fill(bstr, (byte)0);
      while (bytesread < hdrSizeInBytes)
      {
        ds.read(bstr,0,16);
        username = (new String(bstr)).trim();
        tineProtocol = Swap.Short(ds.readShort());
        revisionId = Swap.Short(ds.readShort());
        bytesread += 20;
      }
      ds.close();
      dinBuffer.close();
    }
    catch (IOException e)
    {
      e.printStackTrace();
      MsgLog.log("TReqHdr",e.toString(),TErrorList.code_failure,e,0);
    }
  }      
  public String toString()
  {
    return "User : " + username + "\nTotal size : " + totalSizeInBytes + "\nTotal number : " + revisionId;
  }
  public short getProtocol()
  {
    return tineProtocol;
  }
  public short getTineProtocol()
  {
    return tineProtocol;
  }
  public void setTineProtocol(int protocolLevel)
  {
    tineProtocol = (short)protocolLevel;
  }
}